var MapMatch = {
    calculateSimpleName: function (geoArea) {
        if (MapConstants.ABBREVIATION_NAME_MAP[geoArea]) {
            return MapConstants.ABBREVIATION_NAME_MAP[geoArea];
        }
        var subAreaName;

        if (geoArea.endsWith(MapConstants.REGION_SUFFIX.specialAdminRegion)) {
            subAreaName = geoArea.substring(0, geoArea.length - MapConstants.REGION_SUFFIX.specialAdminRegion.length);
        } else if (geoArea.endsWith(MapConstants.REGION_SUFFIX.region)) {
            subAreaName = geoArea.substring(0, geoArea.length - MapConstants.REGION_SUFFIX.region.length);
        } else if (geoArea.endsWith(MapConstants.REGION_SUFFIX.province) || geoArea.endsWith(MapConstants.REGION_SUFFIX.city)
            || geoArea.endsWith(MapConstants.REGION_SUFFIX.league) || geoArea.endsWith(MapConstants.REGION_SUFFIX.adminArea)
            || geoArea.endsWith(MapConstants.REGION_SUFFIX.county) || geoArea.endsWith(MapConstants.REGION_SUFFIX.flag)) {
            subAreaName = geoArea.substring(0, geoArea.length - MapConstants.REGION_SUFFIX.province.length);
        } else {
            subAreaName = geoArea;
        }
        MapConstants.ABBREVIATION_NAME_MAP[geoArea] = subAreaName;
        return subAreaName;
    },

    matchOtherName: function (areaName, geoArea) {
        var simpleName = MapConstants.ABBREVIATION_NAME_MAP[geoArea];
        if (!simpleName) {
            return false;
        }
        //地区换成市，自治县换成县，自治洲换成洲
        if (geoArea.endsWith(MapConstants.REGION_SUFFIX.region)) {
            return areaName.endsWith(simpleName + MapConstants.REGION_SUFFIX.city);
        } else if (geoArea.endsWith(MapConstants.REGION_SUFFIX.autonomousCounty)) {
            return areaName.endsWith(simpleName + MapConstants.REGION_SUFFIX.county);
        } else if (geoArea.endsWith(MapConstants.REGION_SUFFIX.autonomousPrefecture)) {
            return areaName.endsWith(simpleName + MapConstants.REGION_SUFFIX.prefecture);
        }
    },

    matchArea: function (seriesData, geoAreas, customResult) {
        //先匹配自定义，在匹配同名、然后匹配简称，最后按规则匹配
        if (customResult && customResult[seriesData] && geoAreas.indexOf(customResult[seriesData]) >= 0) {
            return customResult[seriesData];
        }
        if (geoAreas.indexOf(seriesData) >= 0) {
            return seriesData;
        }
        // 省、直辖市、自治区、特区简称直接转换为对应全称
        if (MapConstants.PROVINCE_SHORT_NAME_MAP[seriesData]) {
            seriesData = MapConstants.PROVINCE_SHORT_NAME_MAP[seriesData];
        }
        for (var j = 0, geoLen = geoAreas.length; j < geoLen; j++) {
            if (geoAreas[j] == '') {
                continue;
            }
            var geoArea = geoAreas[j];
            //1.包含匹配，区域名需要最后几位匹配到行政区域名，例如江苏省南京市，只能匹配南京市，但是不能匹配江苏省
            //2.用简称包含匹配，计算简称后用最后几位匹配。
            //3.部分地区也叫市，比如毕节地区，有时也叫毕业市，部分自治州和自治县，有时候也叫洲和县，比如恩施土家族苗族自治州也叫恩施州，互助土族自治县也叫互助县。
            if (seriesData.endsWith(geoArea) || seriesData.endsWith(MapMatch.calculateSimpleName(geoArea)) || MapMatch.matchOtherName(seriesData, geoArea)) {
                return geoArea;
            }
        }
    },

    matchAreaList: function (seriesDataList, geoAreas, customResult) {
        var resultMap = {};
        for (var i = 0, seriesLen = seriesDataList.length; i < seriesLen; i++) {
            var result = MapMatch.matchArea(seriesDataList[i], geoAreas, customResult)
            if (result) {
                resultMap[seriesDataList[i]] = result;
            }
        }
        return resultMap;
    }
};