Van.Utils.initFRVan();

Van.FRChartBridge.AbstractChart = function (options, dom) {
    if (options && dom) {
        this.options = options || {};
        this.dom = $(dom[0]);

        this.width = dom.width() || options.width;
        this.height = dom.height() || options.height;

        this.inits();

        return this;
    }
};

Van.FRChartBridge.AbstractChart.extend = function (props) {

    var NewClass = function () {
        if (this.initialize) {
            this.initialize.apply(this, arguments);
        }
    };

    //将父类的prototype取出并复制到NewClass的__super__ 静态变量中
    var parentProto = NewClass.__super__ = this.prototype;
    var proto = Object.create(parentProto); //复制parentProto到proto中
    //protos是一个新的prototype对象
    proto.constructor = NewClass;
    NewClass.prototype = proto;　//到这完成继承

    for (var i in this) {
        if (this.hasOwnProperty(i) && i !== 'prototype' && i !== '__super__') {
            NewClass[i] = this[i];
        }
    }

    _extend(proto, props); //将参数复制到NewClass的prototype中

    return NewClass;
};

function _extend(dest) {
    var i, j, len, src;

    for (j = 1, len = arguments.length; j < len; j++) {
        src = arguments[j];
        for (i in src) {
            dest[i] = src[i];
        }
    }
    return dest;
}

Van.FRChartBridge.AbstractChart.prototype = {
    initialize: Van.FRChartBridge.AbstractChart,

    inits: function () {

        this._store();

        this._createChart();

        this.autoRefresh();
    },

    _store: function () {
        //todo@shine 扩展图表发布之后整理下前端相关js
        this.chartID = this.options.chartID;

        this.sheetIndex = this.options.sheetIndex || 0;
        this.ecName = this.options.ecName || '';

        FR.Chart.WebUtils._storeChart(this, this.chartID);

        this.idInfo = this.options[Van.Cst.Common.CHART_ID_INFO];
        Van.Utils.storeChartBridge(this, this.idInfo);
    },

    _createChart: function () {

        this.clear();

        var op = this.options[Van.Cst.Common.CHART_ATTR];

        this.chart = this._init(this.dom[0], op);
    },

    /**
     * TODO 很经典的一个问题, FR的联动都是直接remove && reInit
     * 导致的之前的图表的自动刷新没有被暂停, Dom等全部释放不掉..........
     * 最合理的做法, 在联动的时候，把旧的ChartWrapper destroy掉
     * 以前VanCharts那边的做法是每次Init都去FR._chartAutoRefreshTimers里面清除不存在的Chart对应的Timer
     * 也算是一种解决方案吧
     * 这里先用这种containsNode的判断去处理, 后面研究下怎么实现第一种
     * @private
     */
    _checkDomExist: function () {
        var domNode = this.dom && this.dom[0];
        return domNode && document.body.contains(domNode);
    },


    // default empty function for handle callback in chartWidget.resize
    refresh: function () {
    },

    resize: function () {

        if (this.chart && this.chart.resize) {

            this.chart.resize();

        } else {

            // @CHART-3346  没有实现resize的图表，清空Dom重新实例化图表
            this.clear();

            this._createChart();
        }

    },

    clear: function () {
        this.chart && this.chart.clear && this.chart.clear();
        this.chart && this.chart.remove && this.chart.remove();
        this.chart && this.chart.dispose && this.chart.dispose();   // TODO 统一约定一下
        this.chart = null;

        // @CHART-4884
        // 这个问题的根本还是一些特殊的内存资源释放不掉
        // 这里比较明显的就是频繁的删除和新建webGLContext, 浏览器为了限制内存
        // 在页面内的WebGLContext多于一定数目的时候, 浏览器会自动释放掉旧的context
        // 使得render那边抛出` Too many active WebGL contexts. Oldest context will be lost `的警告
        // 这里在clearChart的时候, 统一做一下 loseContext的操作, 保证页面中正在运行的context不会被自动释放掉
        // 具体的图表在clear的时候, 还不确定是不是能够完全的将资源释放掉
        // 后面可能需要搞个专题测试一下
        // 此处改动主要参考自
        // https://github.com/pixijs/pixi.js/issues/2233#issuecomment-181650433
        var canvasArr = this.dom[0].getElementsByTagName('canvas');
        for (var i = 0; i < canvasArr.length; i++) {
            var webGLContext = canvasArr[i].getContext('webgl');
            webGLContext && webGLContext.getExtension('WEBGL_lose_context').loseContext()
        }

        this.dom.empty();
    },

    //动态参数刷新整个页面前会clearCharts
    clearAll: function () {
        this.clear();
        Van.Utils.clearCache(this);
    },

    getLinkFun: function () {
        return this._doLink.bind(this);
    },

    //图表超链
    //params:点击图形的数据信息的对象
    _doLink: function (params) {
        var hyperLink = params.hyperLink || this.options.chartAttr.hyperLink;

        this._doLinkWithFilter(params, hyperLink);
    },

    _doLinkWithFilter: function (params, hyperLink, filter) {

        if (hyperLink && hyperLink.parasMap) {

            var data = Van.Utils.createLinkData(params, hyperLink);

            $.extend(data, this.idInfo);

            Van.Utils.doLinkWithFilter(this, params, data, filter);
        }
    },

    _linkClickEvent: function (params) {
        return params.event;
    },

    //自动刷新
    autoRefresh: function () {
        var self = this, refreshTime = this.options.chartAttr.refreshTime || 0;

        if (refreshTime > 0) {
            this._autoRefreshTimer = setInterval(function () {
                if (self._checkDomExist()) {
                    self._refreshChart();
                } else {
                    clearInterval(self._autoRefreshTimer);
                    self.clear(); // 手动释放图表资源
                }
            }, refreshTime)
        }
    },

    //xxx联动图表（图表使用的参数para改变）
    dealChartAjax: function (para) {
        this._refreshChart(para);
    },

    _refreshChart: function (para, success) {
        para = para || {};

        var data = {
            'chartWidth': this.width || 0,
            'chartHeight': this.height || 0,
            '__time': new Date().getTime()
        };

        $.extend(data, para, this.idInfo);

        var self = this;

        if (!success) {
            success = function (chartRelateJS) {
                self._refreshOptions(chartRelateJS);
            }
        }

        FR.ajax({
            type: 'POST',
            url: FR.servletURL + '?op=chartlink&cmd=refresh_relate_data',
            data: data,
            headers: {
                sessionID: FR.SessionMgr.getSessionID()
            },
            dataType: 'json',
            beforeSend: function () {
                // 开始加载动画
                FR.HtmlLoader.loadingEffect({
                    el: self.dom,
                    show: true,
                    delay: 1000,
                    loadingType: 'local'  // 局部加载动画
                });
            },
            complete: function (result, status) {
                if (status === "success") {
                    result = FR.jsonDecode(result.responseText);
                    success(result);
                }

                // 结束加载动画
                FR.HtmlLoader.loadingEffect({
                    el: self.dom,
                    overflow: 'hidden'
                });
            }
        });
    },

    _refreshOptions: function (chartRelateJS) {
        var chartOp = chartRelateJS.chartAttr;

        if (this._refresh && this.chart) {
            //有刷新接口用刷新接口
            this._refresh(this.chart, chartOp);
            return;
        }

        //没有的话，先清空，再新建
        this.clear();

        this.options.chartAttr = chartOp;

        this._createChart();
    },

    _exportAsImage: function (index) {
        var data = {
            'sessionID': FR.SessionMgr.getSessionID(),
            'width': this.width || 0,
            'height': this.height || 0,
            'index': index,
            '__time': new Date().getTime()
        };

        $.extend(data, this.idInfo);

        var url = FR.servletURL + '?op=chart&cmd=export_image';

        for (var i in data) {
            url += ('&' + i + '=' + data[i]);
        }

        window.location = url;
    }
};

