var BackgroundType = {
    COLOR: "color",
    GRADIENT: "gradient",
    IMAGE: "image"
};

var GradientDirection = {
    TO_TOP: "toTop",
    TO_LEFT: "toLeft",
    TO_RIGHT: "toRight",
    TO_BOTTOM: "toBottom"
};

function linearGradientAngle(direction, startColor, endColor) {

    switch (direction) {
        case GradientDirection.TO_TOP:
            return 0;
        case GradientDirection.TO_BOTTOM:
            return 180;
        case GradientDirection.TO_LEFT:
            return 270;
        case GradientDirection.TO_RIGHT:
            return 90;
    }
}


ExtendedChart = Van.FRChartBridge.AbstractChart.extend({

    _store: function () {
        var options = this.options;

        this.chartID = options.chartID;

        this.sheetIndex = options.sheetIndex || 0;
        this.ecName = options.ecName || '';

        FR.Chart.WebUtils._storeChart(this, this.chartID);
    },

    _createChart: function () {

        this.clear();

        var op = this.options.chartAttr;

        if (this._checkEmpty(op)) {
            return;
        }

        this._dealChartBackground(op);

        this.chart = this._init(this.dom[0], op);

        this._checkEvent();
    },

    _checkEmpty: function(chartAttr) {
        this.emptyDataRender && this.emptyDataRender.remove();

        if (this._emptyData(chartAttr)) {

            this.emptyDataRender = VanCharts.showEmptyDataTip({
                dom: this.dom[0],
                emptyDataTip: chartAttr.emptyDataTip,
                chartWidth: this.width,
                chartHeight: this.height,
                language: chartAttr.language
            });

            this._backgroundEl && this._backgroundEl.remove();
            return true;
        }

        return false;
    },

    /**
     * TODO 很经典的一个问题, FR的联动都是直接remove && reInit
     * 导致的之前的图表的自动刷新没有被暂停, Dom等全部释放不掉..........
     * 最合理的做法, 在联动的时候，把旧的ChartWrapper destroy掉
     * 以前VanCharts那边的做法是每次Init都去FR._chartAutoRefreshTimers里面清除不存在的Chart对应的Timer
     * 也算是一种解决方案吧
     * 这里先用这种containsNode的判断去处理, 后面研究下怎么实现第一种
     * @private
     */
    _checkDomExist: function () {
        var domNode = this.dom && this.dom[0];
        return domNode && document.body.contains(domNode);
    },

    _checkEvent: function () {

        var self = this;

        if (!this.chart.on) {
            this.chart.on = function (eventType, fn) {
                self._events_ = self._events_ || {};

                var fnArray = self._events_[eventType] || [];

                fnArray.push(fn);

                self._events_[eventType] = fnArray;
            }
        }

        if (!this.chart.fire) {
            this.chart.fire = function (eventType, paras) {
                self._events_ = self._events_ || {};

                var fnArray = self._events_[eventType] || [];

                fnArray.forEach(function (fn) {
                    fn && fn.call(self, paras);
                })
            }
        }

        if (this.animateOverlap()) {
            this.chart.on('exitEnd', this._autoLink.bind(this));
        } else {
            this.chart.on('exitEnd', this._autoReloadLink.bind(this));
            this.chart.on('exitBegin', this._autoIncrementLink.bind(this));
        }
    },

    //return true。 动画重叠：指的是当前节点结束动画和下一个节点的初始动画是同时进行的。eg:目录齿轮、水球图、夜光仪表盘
    //默认 return false。动画不重叠：指的是当前节点结束动画完再进行下一个节点的初始动画。eg：轮播gis图、kpi、粒子计数器
    animateOverlap: function () {
        return false;
    },

    _emptyData: function (options) {
        return false;
    },

    hasBackground: function () {
        return false;
    },

    // 预处理图表背景设置
    _dealChartBackground: function (originOp) {

        var background = originOp.background;

        if (background && this.hasBackground() && this.dom[0].parentNode) {

            if (this._backgroundEl) {
                this._backgroundEl.prependTo(this.dom.parent());
                return;
            }

            if (!this._backgroundEl) {
                this._backgroundEl = $("<div></div>");
                this._backgroundEl.prependTo(this.dom.parent());
            }

            // fixME 父元素无z-index时cpt下background.style.zIndex为负数不生效
            this.dom[0].parentNode.style.zIndex = this.dom[0].parentNode.style.zIndex || 0;
            var opacity = originOp.opacity == null ? 1 : originOp.opacity;

            var backgroundValue = background.value,
                cssText =
                    "position:absolute;" +
                    "width:" + this.width + "px;" +
                    "height:" + this.height + "px;" +
                    "top:" + this.dom.css("top") + ";" +
                    "left:" + this.dom.css("left") + ";" +
                    "z-index: -1;";

            switch (background.type) {
                case BackgroundType.COLOR:
                    cssText += "background:" + backgroundValue + ";";
                    break;
                case BackgroundType.IMAGE:
                    cssText += "background:url(" + backgroundValue + ") no-repeat; background-size: cover;";
                    opacity = 1; // CHART-3192: 图片背景透明度不生效
                    break;
                case BackgroundType.GRADIENT:
                    cssText += [
                        "background: linear-gradient(",
                        linearGradientAngle(background.direction) + "deg,",
                        backgroundValue[0] + ",",
                        backgroundValue[1] + ");"
                    ].join("");
            }

            cssText += "opacity:" + opacity;

            this._backgroundEl[0].style.cssText = cssText;

            // remove origin background
            //没单独实现resize方法的图表 原先重绘时背景为空。此处保留originOp.background可fix此问题

        }

    },

    // default empty function for handle callback in chartWidget.resize
    refresh: function () {
    },

    resize: function () {

        if (this.chart && this.chart.resize) {

            this.chart.resize();

        } else {

            // @CHART-3346  没有实现resize的图表，清空Dom重新实例化图表
            this.clear();

            this._createChart();
        }

    },

    clear: function () {
        this.chart && this.chart.clear && this.chart.clear();
        this.chart && this.chart.remove && this.chart.remove();
        this.chart && this.chart.dispose && this.chart.dispose();   // TODO 统一约定一下
        this.chart = null;
        this._backgroundEl && this._backgroundEl.remove();
        this._backgroundEl = null;

        // @CHART-4884
        // 这个问题的根本还是一些特殊的内存资源释放不掉
        // 这里比较明显的就是频繁的删除和新建webGLContext, 浏览器为了限制内存
        // 在页面内的WebGLContext多于一定数目的时候, 浏览器会自动释放掉旧的context
        // 使得render那边抛出` Too many active WebGL contexts. Oldest context will be lost `的警告
        // 这里在clearChart的时候, 统一做一下 loseContext的操作, 保证页面中正在运行的context不会被自动释放掉
        // 具体的图表在clear的时候, 还不确定是不是能够完全的将资源释放掉
        // 后面可能需要搞个专题测试一下
        // 此处改动主要参考自
        // https://github.com/pixijs/pixi.js/issues/2233#issuecomment-181650433
        var canvasArr = this.dom[0].getElementsByTagName('canvas');
        for (var i = 0; i < canvasArr.length; i++) {
            var webGLContext = canvasArr[i].getContext('webgl');
            webGLContext && webGLContext.getExtension('WEBGL_lose_context').loseContext()
        }

        this.dom.empty();
    },

    //动态参数刷新整个页面前会clearCharts
    clearAll: function () {
        this.clear();
    },

    //调用前端库导出接口
    _exportChartImage: function () {
        if (!this._exportImage || !this.options.chartAttr.exportImage) {
            return;
        }
        var export_dom = document.createElement('div');
        export_dom.style.width = this.width + 'px';
        export_dom.style.height = this.height + 'px';

        var initFun = this._exportInit || this._init;
        var export_chart = initFun.bind(this)(export_dom, FR.clone(this.options.chartAttr));

        var self = this;
        setTimeout(function () {
            FR.ajax({
                type: 'POST',
                url: FR.servletURL + '?op=chart&cmd=save_export_image',
                data: {
                    chartID: self.chartID,
                    imageString: self._exportImage(export_chart)
                },
                headers: {
                    sessionID: FR.SessionMgr.getSessionID()
                },
                dataType: 'json'
            });
        }, 100);
    },

    getLinkFun: function () {
        return this._doLink.bind(this);
    },

    //图表超链
    //params:点击图形的数据信息的对象
    _doLink: function (params) {
        var hyperLink = params.hyperLink || this.options.chartAttr.hyperLink;

        this._doLinkWithFilter(params, hyperLink);
    },

    _getPopLinkID: function () {
        var chartPopLinkID = '';

        var chartWidget = FR.Chart.WebUtils.getChartWidget(this.chartID);
        if (chartWidget && chartWidget.options && chartWidget.options['ChartHyperlink_ID']) {
            //当前图表是： 超链-悬浮窗图表 对应的图表
            chartPopLinkID = chartWidget.options['ChartHyperlink_ID'];
        }

        return chartPopLinkID;
    },

    _doLinkWithFilter: function (params, hyperLink, filter) {

        if (hyperLink && hyperLink.parasMap) {

            var data = Van.Utils.createLinkData(params, hyperLink);

            $.extend(data, {
                'chartID': this.chartID,
                'sheetIndex': this.sheetIndex,
                'ecName': this.ecName,
                'ChartHyperlink_ID': this._getPopLinkID()
            }, this.idInfo);

            Van.Utils.doLinkWithFilter(this, params, data, filter);
        }
    },

    _linkClickEvent: function (params) {
        return params.event;
    },


    //执行所有超链 animateOverlap图表用
    _autoLink: function (params) {
        this._autoLinkWithFilter(params);
    },

    //执行 增量刷新动画超链 非animateOverlap图表用
    _autoIncrementLink: function (params) {
        this._autoLinkWithFilter(params, function (singleLink) {
            return singleLink && singleLink.data && singleLink.data.indexOf("animateType:\'increment\'") > -1;
        });
    },

    //执行 重新加载动画超链 非animateOverlap图表用
    _autoReloadLink: function (params) {
        this._autoLinkWithFilter(params, function (singleLink) {
            return singleLink && singleLink.data && singleLink.data.indexOf("animateType:\'reload\'") > -1;
        });
    },

    _autoLinkWithFilter: function (params, filter) {
        if (!this._checkDomExist()) {
            this.clear();
            return;
        }

        var autoLink = this.options.chartAttr.autoLink || {};

        params.linkKey = 'autoLink';
        this._doLinkWithFilter(params, autoLink, filter);
    },


    //xxx联动图表 图表动画为增量刷新
    dealIncrementChartAjax: function (para) {
        this._refreshChart(para, this._incrementRefresh.bind(this))
    },

    //平滑更新式的刷新 updateRefresh （大部分大屏图表为：不透明度由100%～0%，时间1s，函数ease）
    _incrementRefresh: function (relateChartList) {
        this.dom[0].className = 'bigscreen-fade-out-exit';
        var self = this;
        setTimeout(function () {
            self.clear();
            self.dom[0].className = ' ';

            self.options.chartAttr = relateChartList[0].chartAttr;

            self._createChart();
        }, 1000);
    },

    _reloadAnimate: function () {
        var animateProcessor = FR.Report.Plugin.AnimateProcessor;
        if (animateProcessor.item && FR.Plugin.validLevel(animateProcessor, animateProcessor.item)) {
            animateProcessor.item.action.call(this, this.dom.parent());
        }
    },

    //xxx联动图表 图表动画为重新加载
    dealReloadChartAjax: function (para) {
        this._reloadAnimate();
        this._refreshChart(para);
    },

    //自动刷新
    autoRefresh: function () {
        var self = this, refreshTime = this.options.chartAttr.refreshTime || 0;

        if (refreshTime > 0) {
            this._autoRefreshTimer = setInterval(function () {
                if (self._checkDomExist()) {
                    self._refreshChart();
                } else {
                    clearInterval(self._autoRefreshTimer);
                    self.clear(); // 手动释放图表资源
                }
            }, refreshTime)
        }
    },

    //xxx联动图表（图表使用的参数para改变）
    dealChartAjax: function (para) {
        this._refreshChart(para);
    },

    _refreshChart: function (para, success) {
        para = para || {};

        var data = {
            'chartID': this.chartID,
            'chartWidth': this.width || 0,
            'chartHeight': this.height || 0,
            'sheetIndex': this.sheetIndex,
            'ecName': this.ecName,
            '__parameters__': para,
            '__time': new Date().getTime()
        };

        $.extend(data, this.idInfo);

        var self = this;

        if (!success) {
            success = function (relateChartList) {
                self._refreshOptions(relateChartList[0].chartAttr);
            }
        }

        FR.ajax({
            type: 'POST',
            url: FR.servletURL + '?op=chartlink&cmd=refresh_relate_data',
            data: data,
            headers: {
                sessionID: FR.SessionMgr.getSessionID()
            },
            dataType: 'json',
            beforeSend: function () {
                // 开始加载动画
                FR.HtmlLoader.loadingEffect({
                    el: self.dom,
                    show: true,
                    delay: 1000,
                    loadingType: 'local'  // 局部加载动画
                });
            },
            complete: function (result, status) {
                if (status === "success") {
                    result =  FR.jsonDecode(result.responseText);
                    var attrList = result.relateChartList || result;

                    success(attrList);
                }

                // 结束加载动画
                FR.HtmlLoader.loadingEffect({
                    el: self.dom,
                    overflow: 'hidden'
                });
            }
        });
    },

    _refreshOptions: function (chartOp) {

        if (this._refresh && this.chart) {
            //有刷新接口用刷新接口
            this._refresh(this.chart, chartOp);
            return;
        }

        //没有的话，先清空，再新建
        this.clear();

        this.options.chartAttr = chartOp;

        this._createChart();
    }

});


