/**
 * Created by eason on 15/8/26.
 */
VanChartWrapper = ExtendedChart.extend({
    _init: function (dom, option) {

        this.autoRefreshTime = this.options.autoRefreshTime || 0;

        //地图的边框用dom的边框，预留下overflow的大小
        if (option.geo && option.borderWidth) {
            this.dom.css("width", this.width - option.borderWidth * 2)
                .css('height', this.height - option.borderWidth * 2);
        }

        option = this._bindOption(option);

        var chart = VanCharts.init(dom);

        chart.setOptions(option);

        chart.sheetIndex = this.sheetIndex;
        chart.ecName = this.ecName;

        //兼容之前开的接口
        this.vanCharts = chart;

        this._bindLinkFun(chart);

        return chart;
    },

    _bindLinkFun: function (chart) {
        chart.doHyperlink = this.getLinkFun();
    },

    _doLink: function (params) {
        Van.Utils.specialDealParams4VanChart(params);

        ExtendedChart.prototype._doLink.call(this, params);
    },

    _bindOption: function (option) {
        option = this._bindCursorOption(option);
        option = this._bindExportImageUrl(option);
        return option;
    },

    /**
     * 绑定drag和dragging的手型光标，用于IE下地图可以显示小手
     * @param  {[type]} options [description]
     * @return {[type]}         [description]
     */
    _bindCursorOption: function(options){
        var baseURL = window.FR && window.FR.servletURL;
        if(!baseURL){ return options; }

        var getCursorOp = function (){
            return {
                drag: baseURL + "?op=resource&resource=/com/fr/web/cursor/cursor_hand.cur",
                dragging: baseURL + "?op=resource&resource=/com/fr/web/cursor/cursor_drag_hand.cur"
            }
        };
        // 轮播图表
        if(options && FR.isArray(options.options)){
            for(var i = -1; ++i < options.options.length;){
                options.options[i].cursor = getCursorOp();
            }
        } else {   // 单个图表
            options.cursor = getCursorOp();
        }

        return options;
    },

    _bindExportImageUrl: function (options) {
        var baseURL = window.FR && window.FR.servletURL;
        if(!baseURL){ return options; }

        var self = this;
        var bindSingleOption = function (singleOption, i) {
            singleOption = singleOption || {};
            singleOption.tools = singleOption.tools || {};
            if (singleOption.tools.enabled) {
                singleOption.tools.toImage = singleOption.tools.toImage || {};
                singleOption.tools.toImage.imageUrl = baseURL + '?op=chart&cmd=export_image&sessionID='
                    + FR.SessionMgr.getSessionID() + '&chartID=' + encodeURIComponent(self.chartID)
                    + '&sheetIndex=' + self.sheetIndex + '&ecName=' + encodeURIComponent(self.ecName)
                    +'&index=' + i;
            }
        };

        if (options && FR.isArray(options.options)) {
            for(var i = 0, len = options.options.length; i < len; i++){
                bindSingleOption(options.options[i], i)
            }
        } else {
            bindSingleOption(options, 0);
        }

        return options;
    },

    clearAll: function () {
        this.chart && this.chart.clear();
        this._removeUseLessAutoRefreshTimer();
        this.chart = null;
    },

    _resetAttr: function (attr) {
        attr.startLoading = false;
        attr.endLoading = true;
        // @mango CHART-1786  moreLable置为null会导致merge option报错，监控刷新失效。
        // attr.moreLabel = null;
    },

    /*
     * 单个图表轮播切换接口
     * @param direction 轮播图表切换方向:1向右，-1向左
     */
    switchCarouselChart: function (direction) {
        this.chart.carousel.switchChartAndRestTimer(direction);
    },

    /*
    * 展示指定图表接口
    * @param index 指定图表的下标
    */
    showIndexChart: function (index) {
        this.chart.carousel.showIndexChartAndRestTimer(index);
    },

    getChartWithIndex: function (index) {
        return this.chart.charts[index];
    },

    refreshData: function (options) {

        if (options && Object.prototype.toString.apply(options.options) === '[object Array]') {
            for (var i = 0, len = options.options.length; i < len; i++) {
                this._resetAttr(options.options[i]);
            }
        } else {
            this._resetAttr(options)
        }

        options = this._bindOption(options);
        this.chart.setData(options);
    },

    //数据监控的刷新方式
    setData: function (options, index) {
        options = this._bindOption(options);
        this.chart.setData(options, index);
    },

    setAutoRefreshData: function (options, index) {
        // 这边第一次就已经把cursor传了过来，所以没有必要再更新了cursor属性了
        this.chart.setAutoRefreshData(options, index);
    },

    autoRefresh: function () {

        //去掉无用的刷新定时器
        this._removeUseLessAutoRefreshTimer();

        if (Array.isArray(this.autoRefreshTime)) {
            this._autoMultiRefresh(this.autoRefreshTime);
        } else {
            this._autoSingleRefresh(this.autoRefreshTime || 0);
        }

    },

    /**
     * 判断传入的索引值是否有效
     * @param s
     * @returns {boolean}
     * @private
     */
    _getValidIndex: function (s) {
        var charts = this.chart.charts, len = charts.length;
        return /^[0-9]+$/.test(s) && s < len;
    },

    /**
     * 数据刷新接口，
     * @param chartIndexList，可选参数，表示需要刷新的图表的索引值数组
     */
    dataRefresh: function (chartIndexList) {

        var self = this, chart = this.chart, charts = chart.charts;
        var len = charts.length;
        var getAllIndexList = function () {
            var i = len, indexList = [];
            while (i) {
                indexList.push(--i);
            }
            return indexList;
        };

        chartIndexList = FR.isArray(chartIndexList) ? chartIndexList : [chartIndexList];

        var validIndexList = chartIndexList.filter(function (indexValue) {
            return self._getValidIndex(indexValue);
        });

        if (validIndexList.length === 0) {
            validIndexList = getAllIndexList();
        }

        validIndexList.forEach(function (indexValue) {
            self._dataRefresh(indexValue);
        });

    },

    _removeUseLessAutoRefreshTimer: function () {
        //获取当前所有的定时器
        var timerArray = FR.autoRefreshTimers = FR.autoRefreshTimers || [];

        var i, j, timerEle, elementTimers, timerId, timerArrayLen = timerArray.length;

        for (i = timerArrayLen - 1; i >= 0; i--) {
            timerEle = timerArray[i];
            //删除无用的定时器
            if (!timerEle || !document.body.contains(timerEle)) {
                elementTimers = timerEle.__autoRefreshTimers || [];
                for (j = 0; j < elementTimers.length; j++) {
                    timerId = elementTimers[j];
                    clearInterval(timerId);
                }
                timerArray.splice(i, 1);
            }
        }

        var elementAutoRefreshTimers = this.dom[0].__autoRefreshTimers || [], len = elementAutoRefreshTimers.length;
        for (i = -1; ++i < len;) {
            clearInterval(elementAutoRefreshTimers.shift());
        }

    },

    _addNewAutoRefreshTimer: function (timerId) {
        var element = this.dom[0];
        //将定时器绑定在对应的dom上
        element.__autoRefreshTimers = element.__autoRefreshTimers || [];
        element.__autoRefreshTimers.push(timerId);

        FR.autoRefreshTimers.indexOf(element) < 0 && FR.autoRefreshTimers.push(element);
    },

    /**
     * 单个图表的监控刷新
     * @param time
     * @private
     */
    _autoSingleRefresh: function (time) {
        if (time < 1) {
            return;
        }

        var self = this;

        var timerId = setInterval(function () {

            self._dataRefresh();

        }, time * 1000);

        this._addNewAutoRefreshTimer(timerId);
    },

    /**
     * 更新图表数据，index表示图表在图表块内的索引值
     * @param index
     * @private
     */
    _dataRefresh: function (index) {
        if (!this.chart) {
            return;
        }
        var self = this,
            ID = self.chartID,
            width = self.width || 0,
            height = self.height || 0,
            sheetIndex = self.sheetIndex,
            ecName = self.ecName;

        var postData = {
            cmd: 'chart_auto_refresh',
            chartID: ID,
            chartWidth: width,
            chartHeight: height,
            sheetIndex: sheetIndex,
            ecName: ecName,
            __time: new Date().getTime()
        };

        // 图表切换必须指定index，非图表切换则不需要
        if (self.chart.charts.length > 1) {
            postData.index = index;
        }

        if (FR.servletURL) {
            FR.ajax({
                type: 'GET',
                url: FR.servletURL + '?op=chartauto',
                data: postData,
                headers: {
                    sessionID: FR.SessionMgr.getSessionID()
                },
                dataType: 'json',
                success: function (chartRelateJS) {
                    var attrList = chartRelateJS.relateChartList;
                    for (var i = 0, len = attrList.length; i < len; i++) {
                        var chartID = FR.Chart.WebUtils._getChartIDAndIndex(attrList[i].id);
                        var chartSet = FR.ChartManager[chartID[0]];
                        if (chartSet && chartSet[chartID[1]]) {
                            var chartAttr = attrList[i].chartAttr;
                            self._setAutoRefreshData(chartSet[chartID[1]], chartAttr, index);
                        }
                    }
                }
            });
        }
    },

    /**
     * 监控刷新，设置刷新数据
     * @param {[type]} chart     [description]
     * @param {[type]} chartAttr [description]
     */
    _setAutoRefreshData: function (chartSet, chartAttr, index) {
        if (chartAttr.add || chartAttr.update || chartAttr.remove) {
            chartSet.setAutoRefreshData(chartAttr, index)
        } else {
            chartSet.setData(chartAttr, index);
        }
    },

    _autoMultiRefresh: function (timeList) {
        var count = timeList.length;
        var self = this;
        var time;

        for (var index = 0; index < count; index++) {
            time = timeList[index];
            if (time < 1) {
                continue;
            }
            this._intervalRequest(self, time, index);
        }
    },

    /**
     * 图标切换中具体下标的单个图表监控刷新数据
     * @param self
     * @param time
     * @param index
     * @private
     */
    _intervalRequest: function (self, time, index) {
        var timerId = setInterval(function () {
            self._dataRefresh(index);
        }, time * 1000);

        this._addNewAutoRefreshTimer(timerId);
    },

    _chartRelated: function (attrList) {
        for (var i = 0; i < attrList.length; i++) {
            var chartID = FR.Chart.WebUtils._getChartIDAndIndex(attrList[i].id);
            var chartSet = FR.ChartManager[chartID[0]];
            if (chartSet && chartSet[chartID[1]]) {
                var chartAttr = attrList[i].chartAttr;
                chartSet[chartID[1]].refreshData(chartAttr);
            }
        }
    },

    dealChartAjax: function (para) {
        para = para || {};

        for (var w in para) {
            var value = para[w];
            if (w.startWith("$")) {
                w = w.substring(1)
            }
            if (value == undefined) {
                value = null;
            }
            para[w] = value;
        }

        var data = {
            'chartID': this.chartID,
            'chartWidth': this.width || 0,
            'chartHeight': this.height || 0,
            'sheetIndex': this.sheetIndex,
            'ecName': this.ecName,
            '__parameters__': para,
            '__time': new Date().getTime()
        };

        var self = this;

        FR.ajax({
            type: 'POST',  // @Chart-1185, 使用GET可能导致url过长抛414错误
            url: FR.servletURL + '?op=chartlink&cmd=refresh_relate_data',
            data: data,
            headers: {
                sessionID: FR.SessionMgr.getSessionID()
            },
            dataType: 'json',
            beforeSend: function () {
                // 开始加载动画
                FR.HtmlLoader.loadingEffect({
                    el: self.dom,
                    show: true,
                    delay: 1000,
                    loadingType: 'local'  // 局部加载动画
                });
            },
            complete: function (result, status) {
                if (status === "success") {
                    result =  FR.jsonDecode(result.responseText);
                    self._chartRelated(result.relateChartList);
                }

                // 结束加载动画
                FR.HtmlLoader.loadingEffect({
                    el: self.dom,
                    overflow: 'hidden'
                });
            }
        });

    },

    //xxx联动图表 图表动画为重新加载
    dealReloadChartAjax: function (para) {
        this._reloadAnimate();
        this.dealChartAjax(para);
    },

    //平滑更新式的刷新 updateRefresh （柱子涨落）
    _incrementRefresh: function (relateChartList) {
        var self = this;
        setTimeout(function () {
            self._chartRelated(relateChartList);
        }, 1000);
    }

});