/*
 * Copyright (c) 2001-2014,FineReport Inc, All Rights Reserved.
 */
/**
 * 坐标轴图形
 * @class FR.Chart.AxisGlyph
 * @abstract 抽象的坐标轴类
 *
 * @extends FR.Chart.ChartJsInit
 *
 * @param {JSON} js 属性配置.
 *
 * @param {Object} [js.titleGlyph = null] 默认标题文本.
 * @param {Boolean} [js.isZoom = false] 坐标轴缩放
 * @param {Boolean} [js.axisReversed = false] 坐标轴逆序
 * @param {Number} [js.tickMarkType = FR.Chart.Constants.TICK_MARK_OUTSIDE] 主要刻度
 * @param {Number} [js.secTickMarkType = FR.Chart.Constants.TICK_MARK_INSIDE] 次要刻度.
 * @param {Number} [js.position = FR.Chart.Constants.BOTTOM] 坐标轴位置, 默认下方, 绘图区底部.
 * @param {Number} [js.mainGridStyle = FR.Chart.Constants.LINE_NONE] 主要网格线线型.
 * @param {Color} [js.mainGridColor = null] 主要网格线颜色.
 * @param {Color} [js.lineColor = 'rgb(176, 176, 176)] 默认坐标轴颜色
 * @param {Number} [js.lineStyle = FR.Chart.Constants.LINE_THIN] 默认坐标轴线型
 * @param {Object} [js.textAttr = new FR.Chart.TextAttr()]; 坐标轴文本格式.
 * @param {Object} [js.format = null] 坐标轴数值格式.
 * @param {Number} [js.labelNumber = 0] 默认标签间隔数目.
 * @param {Boolean} [js.isShowAxisLabel = true] 默认展示坐标轴标签.
 * @param {Boolean} [js.isArrowShow = false] 默认不展示坐标轴箭头.
 * @param {Boolean} [js.isCustomMinValue = false]  自定义最小值.
 * @param {Number} [js.minValue] 最小值.
 * @param {Boolean} [js.isCustomMaxValue] 是否自定义最大值.
 * @param {Number} [js.maxValue] 最大值
 * @param {Boolean} [js.isCustomMainUnit] 是否自定义主要单位
 * @param {Number} [js.tickLength] 主要刻度.
 * @param {Boolean} [js.isCustomSecUnit = false] 是否自定义次要刻度单位.
 * @param {Number} [js.smallTickLength] 次要刻度单位
 * @param {Boolean} [js.drawBetweenTick] 是否画在刻度中间.
 *
 */
FR.Chart.AxisGlyph = FR.extend(FR.Chart.ChartJsInit, /**@class FR.Chart.AxisGlyph*/{
    initConstants: function (js) {
        js = js || {};

        this.titleGlyph = js.titleGlyph == undefined ? null : new FR.Chart.TextGlyph(js.titleGlyph);

        this.isZoom = FR.pick(js.isZoom, false);
        if (FR.Browser.isIE8Before()) {
            this.isZoom = false;
        }
        this.axisReversed = FR.pick(js.axisReversed, false);

        this.tickMarkType = FR.pick(js.tickMarkType, FR.Chart.Constants.TICK_MARK_OUTSIDE);
        this.secTickMarkType = FR.pick(js.secTickMarkType, FR.Chart.Constants.TICK_MARK_NONE);

        this.position = FR.pick(FR.Chart.ChartUtils.getPosition(js.position), FR.Chart.Constants.BOTTOM);

        this.mainGridStyle = FR.pick(js.mainGridStyle, FR.Chart.Constants.LINE_NONE);
        this.mainGridColor = js.mainGridColor;

        this.lineColor = FR.pick(js.lineColor, 'rgb(176,176,176)');
        this.lineStyle = FR.pick(js.lineStyle, FR.Chart.Constants.LINE_THIN);

        this.textAttr = new FR.Chart.TextAttr(js.textAttr);
        if(js.font){
            this.textAttr.frFont = new FR.Chart.FRFont(js.font);
            this.textAttr.rotation = FR.pick(js.rotation, 0);
            this.textAttr.alignText = FR.pick(js.alignText, 0);
        }
        this.format = js.format;

        this.labelNumber = FR.pick(js.labelNumber, js.gap, 0);
        this.isShowAxisLabel = FR.pick(js.isShowAxisLabel, true);
        this.isLabelWrap = FR.pick(js.isLabelWrap,false);
        this.isArrowShow = js.isArrowShow;

        this.isCustomMinValue = js.isCustomMinValue;
        this.minValue = FR.pick(js.minValue, 0);
        this.isCustomMaxValue = js.isCustomMaxValue;
        this.maxValue = FR.pick(js.maxValue, 0);

        this.isCustomMainUnit = js.isCustomMainUnit;
        this.tickLength = FR.pick(js.tickLength, 0);

        this.isCustomSecUnit = js.isCustomSecUnit;
        this.smallTickLength = FR.pick(js.smallTickLength, 0);
        this.drawBetweenTick = js.drawBetweenTick;

        // 以下纯粹初始化 不需要属性配置.
        this.titleDim = new FR.Chart.Dimension2D(0, 0);
        this.startLabelDim = new FR.Chart.Dimension2D(0, 0);
        this.endLabelDim = new FR.Chart.Dimension2D(0, 0);

        this.plotZeroBounds = null;// 初始PlotBounds和最终的plotBounds 记录绝对位置
        this.plotLastBounds = null;

        this.tooltipLabelBoundsList = [];
        this.tooltipLabelList = [];
        this.MAX_AXIS_LABEL_HEIGHT = 0.28;
        this.MAX_AXIS_LABEL_WIDTH = 0.28;
        this.TOP_AND_DOWN_GAP = 6;
        this.ELLIPSIS = "...";

        this.unitLengthNoScale = null;// 坐标轴放缩时 保持最初的刻度长度
        this.bounds = null;
        this.originPoint = null;

        this.zoomForward = 0;
        this.zoomBackward = 0;

        this.unitLength = 0;
        this.axisLength = 0;
        this.axisGridLength = 0;
        this.axisLabelWidth = 8;
    },

    configLabels:function(){

    },

    configAxisLabelsWithOptionData:function(){

    },

    isNeedTimeSwitch:function(){
        return false;
    },

    /**
     * kunsnat: 根据坐标轴的labelWidth, 调整plotBounds, 然后再去calculateAxisGlyph
     * @param {Object} plotBounds 绘图区边界.
     */
    dealPlotBoundsWithAxisLabel: function (plotBounds) {
        var cateLabelWidth = this.getCateLabelWidth(plotBounds);
        if (cateLabelWidth > 0) {
            cateLabelWidth += FR.Chart.AxisGlyph.LINE_LABEL_GAP;
        }

        cateLabelWidth += this.getTickLengthShow();

        this.axisLabelWidth = cateLabelWidth;

        this.calculateTitleDimesion();

        if (this.bounds == null) {
            this.bounds = plotBounds;
        }

        this.calculateBoundsWidthOrientationAndTitle(plotBounds, cateLabelWidth, this.titleDim);
        this.calBoundsWithMoreLabel4Cate(plotBounds);
    },


    dealWithPlotBounds:function(){
        return 0;
    },

    /**
     * 根据坐标轴标题 处理绘图区边界.
     * @param plotBounds 绘图区边界.
     */
    dealPlotBoundsWithAxisTitle: function (plotBounds) {
        this.calculateTitleDimesion();

        if (this.bounds == null) {
            this.bounds = plotBounds;
        }

        this.calculateBoundsWidthOrientationAndTitle(plotBounds, 0, this.titleDim);
        this.calBoundsWithMoreLabel4Cate(plotBounds);
    },

    /**
     * 根据坐坐标轴标签处理绘图区边界
     * @param plotBounds 绘图区边界.
     */
    dealPlotBoundsWithLabel: function (plotBounds) {
        var cateLabelWidth = this.getCateLabelWidth(plotBounds);
        if (cateLabelWidth > 0) {
            cateLabelWidth += FR.Chart.AxisGlyph.LINE_LABEL_GAP;
        }

        cateLabelWidth += this.getTickLengthShow();

        this.axisLabelWidth = cateLabelWidth;
        this.calculateBoundsWidthOrientationAndTitle(plotBounds, cateLabelWidth);
    },

    /**
     * 返回标签占用宽度
     * @param plotBounds 绘图区边界
     * @returns {number}  标签宽度.
     */
    getCateLabelWidth: function (plotBounds) {
        var cateLabelWidth = this.getMaxLabelWidthAndInitStartEndLabelDim();

        if (this.shouldBeHeight()) {
            return Math.min(plotBounds.height * this.MAX_AXIS_LABEL_HEIGHT, cateLabelWidth);
        } else {
            return Math.min(plotBounds.width * this.MAX_AXIS_LABEL_WIDTH, cateLabelWidth);
        }
    },

    /**
     *  处理水平方向 对应绘图区边界的影响
     * @param lastPlotBounds 之前的绘图区边界
     */
    dealPlotBoundsWithLabelInHorizontal: function (lastPlotBounds) {
        if (this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) {
            this._dealPlotBoundsLabelLeftRight(lastPlotBounds);
        } else {
            this._dealPlotBoundsLabelTopBottom(lastPlotBounds);
        }
    },

    dealPlotBounds4AreaWrapInHorizontal:function(lastPlotBounds){
        var tickNumber = this.getOriginalArrowValue() - this.getOriginalCrossValue();
        if(tickNumber <= 0){
            return ;
        }
        if (this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) {
            var height = lastPlotBounds.y - this.plotZeroBounds.y;
            var average = lastPlotBounds.height/tickNumber;
            if(average/2 > height){
                lastPlotBounds.y += average/2-height;
                lastPlotBounds.height -=average/2-height;
            }
            var start = this.plotZeroBounds.y + this.plotZeroBounds.height - lastPlotBounds.y - lastPlotBounds.height;
            if(average/2 > start && Math.round(start)>0){
                lastPlotBounds.height +=average/2- start;
            }
        } else {
            var width = this.plotZeroBounds.x + this.plotZeroBounds.width - lastPlotBounds.x - lastPlotBounds.width;
            var average = lastPlotBounds.width/tickNumber;
            if(average/2>width){
                lastPlotBounds.width -= average/2 - width;
            }
            var start = lastPlotBounds.x - this.plotZeroBounds.x;
            if (average/2 > start) {
                lastPlotBounds.x += average/2 - start;
                lastPlotBounds.width -= average/2 - start;
            }
        }
    },

    _dealPlotBoundsLabelLeftRight: function (lastPlotBounds) {
        var endHeight = this.endLabelDim.height / 2;
        var height = lastPlotBounds.y - this.plotZeroBounds.y;
        if (endHeight > height) {
            lastPlotBounds.y = lastPlotBounds.y + endHeight - height;
            lastPlotBounds.height = lastPlotBounds.height - endHeight + height;
        }

        var startHeight = this.startLabelDim.height / 2;
        var start = this.plotZeroBounds.y + this.plotZeroBounds.height - lastPlotBounds.y - lastPlotBounds.height;
        if (startHeight > start && Math.round(start) > 0) {// 下方全部不显示坐标轴标签 但是要和数据表对接
            lastPlotBounds.height = lastPlotBounds.height - startHeight + start;
        }
    },

    _dealPlotBoundsLabelTopBottom: function (lastPlotBounds) {
        var endWidth = this.endLabelDim.width / 2;
        var width = this.plotZeroBounds.x + this.plotZeroBounds.width - lastPlotBounds.x - lastPlotBounds.width;
        if (endWidth > width) {
            lastPlotBounds.width = lastPlotBounds.width - endWidth + width;
        }

        var startWidth = this.startLabelDim.width / 2;
        var start = lastPlotBounds.x - this.plotZeroBounds.x;
        if (startWidth > start) {
            lastPlotBounds.x = lastPlotBounds.x + startWidth - start;
            lastPlotBounds.width = lastPlotBounds.width - startWidth + start;
        }
    },

    /**
     * 返回刻度展示 用到的长度
     * @returns {number} 刻度长度
     */
    getTickLengthShow: function () {
        return Math.max(this._dealTickLengthInShow(this.tickMarkType, FR.Chart.AxisGlyph.MAIN_TICK_LENGTH),
            this._dealTickLengthInShow(this.secTickMarkType, FR.Chart.AxisGlyph.SEC_TICK_LENGTH));
    },

    _dealTickLengthInShow: function (tickType, tickLength) {
        var resulteShowTickLength = 0;

        switch (tickType) {
            case FR.Chart.Constants.TICK_MARK_OUTSIDE:
                resulteShowTickLength = tickLength;
                break;
            case FR.Chart.Constants.TICK_MARK_CROSS:
                resulteShowTickLength = tickLength / 2;
            default:
                break;
        }

        return resulteShowTickLength;
    },

    /**
     * 计算坐标轴Glyph中的边界, 标签, 位置, 单位等信息
     * @param bounds 初始化时用到的边界.
     */
    calculateAxisGlyph: function (bounds) {
        this.init(bounds);
        this.calculateAxisLengthUnit();
        // 保存最终的plotbounds 用于和原始的比较
        this.plotLastBounds = bounds;
    },

    /**
     * 计算标题占用的单位大小.
     */
    calculateTitleDimesion: function () {
        if (this.titleGlyph) {
            this.titleDim = this.titleGlyph.preferredDimension();
        }
    },

    /**
     * 计算多分类时 标签占用的bounds
     * @param plotBounds 绘图区边界.
     */
    calBoundsWithMoreLabel4Cate: function (plotBounds) {
        // donothing  categoryaxis中处理多分类.
    },

    /**
     * 根据标签的宽度, 标题的大小 调整绘图区的位置.
     * @param plotBounds 绘图区边界
     * @param cateLabelWidth  标签对应的宽度
     * @param titleDim 标题占用的大小.
     */
    calculateBoundsWidthOrientationAndTitle: function (plotBounds, cateLabelWidth, titleDim) {
        var preTitleWidthGap = 0;
        var preTitleHeightGap = 0;
        if (titleDim) {
            preTitleWidthGap = (titleDim.width > 0) ? titleDim.width + FR.Chart.AxisGlyph.LINE_LABEL_GAP : 0;
            preTitleHeightGap = (titleDim.height > 0) ? titleDim.height + FR.Chart.AxisGlyph.LINE_LABEL_GAP : 0;
        }
        switch (this.position) {
            case FR.Chart.Constants.LEFT:
                plotBounds.x = plotBounds.x + preTitleWidthGap + cateLabelWidth;
                plotBounds.width = plotBounds.width - cateLabelWidth - preTitleWidthGap;
                break;
            case FR.Chart.Constants.BOTTOM:
                plotBounds.height = plotBounds.height - cateLabelWidth - preTitleHeightGap;
                break;
            case FR.Chart.Constants.RIGHT:
                plotBounds.width = plotBounds.width - cateLabelWidth - preTitleWidthGap;
                break;
            default:
                plotBounds.height = plotBounds.height - cateLabelWidth - preTitleHeightGap;
                plotBounds.y = plotBounds.y + cateLabelWidth + preTitleHeightGap;
                break;
        }
    },

    /**
     * 坐标轴 根据绘图区的边界 初始化大小.
     * @param plotBounds 绘图区边界.
     */
    init: function (plotBounds) {
        var axisZero = null;
        var axisLength, axisGridLength;
        var bounds = null;

        switch (this.position) {
            case FR.Chart.Constants.LEFT:
                axisLength = plotBounds.height;
                axisGridLength = plotBounds.width;

                bounds = new FR.Chart.Bounds(plotBounds.x - this.axisLabelWidth, plotBounds.y, this.axisLabelWidth, plotBounds.height);
                if (!this.axisReversed) {
                    axisZero = new FR.Chart.Point2D(plotBounds.x, plotBounds.y + plotBounds.height);
                } else {
                    axisZero = new FR.Chart.Point2D(plotBounds.x, plotBounds.y);
                }

                break;
            case FR.Chart.Constants.BOTTOM:
                axisLength = plotBounds.width;
                axisGridLength = plotBounds.height;

                bounds = new FR.Chart.Bounds(plotBounds.x, plotBounds.y + plotBounds.height, plotBounds.width, this.axisLabelWidth);
                if (!this.axisReversed) {
                    axisZero = new FR.Chart.Point2D(plotBounds.x, plotBounds.y + plotBounds.height);
                } else {
                    axisZero = new FR.Chart.Point2D(plotBounds.x + plotBounds.width, plotBounds.y + plotBounds.height);
                }

                break;
            case FR.Chart.Constants.RIGHT:
                axisLength = plotBounds.height;
                axisGridLength = plotBounds.width;

                bounds = new FR.Chart.Bounds(plotBounds.x + plotBounds.width, plotBounds.y, this.axisLabelWidth, plotBounds.height);
                if (!this.axisReversed) {
                    axisZero = new FR.Chart.Point2D(plotBounds.x + plotBounds.width, plotBounds.y + plotBounds.height);
                } else {
                    axisZero = new FR.Chart.Point2D(plotBounds.x + plotBounds.width, plotBounds.y);
                }

                break;
            default:
                axisLength = plotBounds.width;
                axisGridLength = plotBounds.height;

                bounds = new FR.Chart.Bounds(plotBounds.x, plotBounds.y - this.axisLabelWidth, plotBounds.width, this.axisLabelWidth);
                if (!this.axisReversed) {
                    axisZero = new FR.Chart.Point2D(plotBounds.x, plotBounds.y);
                } else {
                    axisZero = new FR.Chart.Point2D(plotBounds.x + plotBounds.width, plotBounds.y);
                }

                break;
        }

        this.bounds = bounds;
        this.originPoint = axisZero;
        this.axisLength = axisLength;
        this.axisGridLength = axisGridLength;
    },

    /**
     * 坐标轴的值 根据格式 转为对应的字符串.
     * @param value 坐标轴对应的值
     * @returns {string} 转为对应的字符串.
     */
    value2String: function (value) {
        // 统一先在这里处理1.000000000001 = 1.0类似的小数误差
        var format = this.format;
        //有format也要处理1.000000000001 = 1.0类似的小数误差。bug:79253
        if (value > -1E-10 && value < 1E-10) {
            value = 0;
        }

        value = FR.Chart.ChartUtils.accNumber(value);

        var string;
        if (format != null) {
            // write.js中的format处理
            string = FR.contentFormat(value, format);
        } else {
            string = FR.Chart.ChartUtils.generalFormat(value);
        }
        return string;
    },

    /**
     * 返回坐标轴标签占用的宽度
     * @param text 坐标轴标签文本.
     * @returns 返回标签文本所占据的高度
     */
    getAxisLabelWidth: function (text) {
        var dim = this.getAxisLabelDim(text);

        return this.shouldBeHeight() ? dim.height : dim.width;
    },

    /**
     * 返回标签文本占用的空间大小.
     * @param text  标签文本.
     * @returns {dim} 返回标签占用的文本
     */
    getAxisLabelDim: function (text) {
        if (text == null) {
            return 0;
        }
        var textAttr = this.textAttr;
        if (textAttr == null) {
            textAttr = new FR.Chart.TextAttr();
        }

        return FR.Chart.TextUtils.calculateTextDimensionWithRotation(text, textAttr);
    },

    /**
     * 是否使用标签空间的高度  对应的上下位置时 使用高度, 左右位置时 使用宽度.
     * @returns {boolean} 返回标签位置.
     */
    shouldBeHeight: function () {
        return this.position == FR.Chart.Constants.TOP || this.position == FR.Chart.Constants.BOTTOM;
    },

    /**
     * 返回坐标轴占用的边界.
     * @returns {bounds} 坐标轴边界.
     */
    getBounds: function () {
        return this.bounds;
    },

    /**
     * 返回 坐标轴值 对应的 点.
     * @param value 坐标轴值
     * @returns {point} 值对应的位置点.
     */
    getPointInBounds: function (value) {
        if (value < this.getCrossValue()) {
            value = this.getCrossValue();
        }

        if (value > this.getArrowValue()) {
            value = this.getArrowValue();
        }

        return this.getPoint2D(value);
    },

    /**
     * 得到线上两点之间的距离.
     * @param start 起始值
     * @param end 结束值
     * @return 返回两个值之间的距离长度.
     */
    get2ValueLength: function (start, end) {
        var startPoint = this.getPointInBounds(start);
        var endPoint = this.getPointInBounds(end);

        if (this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) {
            return Math.abs(startPoint.y - endPoint.y);
        } else {
            return Math.abs(startPoint.x - endPoint.x);
        }
    },

    /**
     * 返回数值对应的 刻度序号.   只用到: 分类和日期的返回
     * @return 返回对应的值.
     */
    getTickIndex4Value: function (value) {
        return value;
    },

    /**
     * 根据值 返回坐标轴上对应的点
     * @param value 坐标轴值
     * @returns {point} 返回坐标轴上对应的点.
     */
    getPoint2D: function (value) {
        var point2D = null;
        value = value - (this.isDraw4Demo ? this.minValue : this.getCrossValue());

        if (this.originPoint == null) {
            point2D = new FR.Chart.Point2D(this.bounds.x, this.bounds.y);
        }

        var useLength = this.isDraw4Demo ? this.unitLengthNoScale : this.unitLength;
        if (this.position == FR.Chart.Constants.LEFT) {
            if (this.bounds.y == this.originPoint.y || this.axisReversed) {
                point2D = new FR.Chart.Point2D(this.originPoint.x, this.originPoint.y + useLength * value);
            } else {
                point2D = new FR.Chart.Point2D(this.originPoint.x, this.originPoint.y - useLength * value);
            }
        } else if (this.position == FR.Chart.Constants.RIGHT) {
            var detY = this.axisReversed ? useLength * value : -useLength * value;
            point2D = new FR.Chart.Point2D(this.originPoint.x, this.originPoint.y + detY);
        } else {
            var detX = this.axisReversed ? -useLength * value : useLength * value;
            point2D = new FR.Chart.Point2D(this.originPoint.x + detX, this.originPoint.y);
        }

        return point2D;
    },

    /**
     * 返回刻度对应的Line
     * @param value 刻度值
     * @param tickLength 刻度长度
     * @param tickMarkType 刻度标记类型.
     * @returns {line} 返回刻度线.
     */
    getTickLine: function (value, tickLength, tickMarkType) {
        var centerP = this.getPoint2D(value);
        return this.getTickLineByCenterPoint(centerP, tickLength, tickMarkType);
    },

    getTickLineByCenterPoint: function (centerP, tickLength, tickMarkType) {
        var leftP = new FR.Chart.Point2D(centerP.x - tickLength, centerP.y);
        var rightP = new FR.Chart.Point2D(centerP.x + tickLength, centerP.y);
        var topP = new FR.Chart.Point2D(centerP.x, centerP.y - tickLength);
        var bottomP = new FR.Chart.Point2D(centerP.x, centerP.y + tickLength);

        if (tickMarkType == FR.Chart.Constants.TICK_MARK_CROSS) {
            var leftCenterP = new FR.Chart.Point2D(centerP.x - tickLength / 2, centerP.y);
            var rightCenterP = new FR.Chart.Point2D(centerP.x + tickLength / 2, centerP.y);
            var topCenterP = new FR.Chart.Point2D(centerP.x, centerP.y - tickLength / 2);
            var bottomCenterP = new FR.Chart.Point2D(centerP.x, centerP.y + tickLength / 2);
            if (this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) {
                return new FR.Chart.Line2D(leftCenterP, rightCenterP);
            } else {
                return new FR.Chart.Line2D(topCenterP, bottomCenterP);
            }
        } else if (tickMarkType == FR.Chart.Constants.TICK_MARK_INSIDE) {
            if (this.position == FR.Chart.Constants.LEFT) {
                return new FR.Chart.Line2D(centerP, rightP);
            } else if (this.position == FR.Chart.Constants.TOP) {
                return new FR.Chart.Line2D(centerP, bottomP);
            } else if (this.position == FR.Chart.Constants.RIGHT) {
                return new FR.Chart.Line2D(centerP, leftP);
            } else {
                return new FR.Chart.Line2D(centerP, topP);
            }
        } else if (tickMarkType == FR.Chart.Constants.TICK_MARK_OUTSIDE) {
            if (this.position == FR.Chart.Constants.LEFT) {
                return new FR.Chart.Line2D(centerP, leftP);
            } else if (this.position == FR.Chart.Constants.TOP) {
                return new FR.Chart.Line2D(centerP, topP);
            } else if (this.position == FR.Chart.Constants.RIGHT) {
                return new FR.Chart.Line2D(centerP, rightP);
            } else {
                return new FR.Chart.Line2D(centerP, bottomP);
            }
        } else {
            return new FR.Chart.Line2D(centerP, centerP);
        }
    },

    /**
     * 鼠标移动 查找当前的Glyph
     * @param mousePos 鼠标位置
     * @param event 事件
     */
    findOnMoveGlyph: function (mousePos, event) {
        this.tooltip = null;
        if (this.tooltipLabelBoundsList.length === 0) {
            return;
        }

        for (var i = 0; i < this.tooltipLabelBoundsList.length; i++) {
            if (this.tooltipLabelBoundsList[i].containsPoint(mousePos.x, mousePos.y)) {
                event._hasfound = true;
                event._lastHover = this;
                this.tooltip = this.tooltipLabelList[i];
                return;
            }
        }
    },

    /**
     * 在鼠标移动时 响应的事件.
     * @param ctx 画布
     * @param mousePos 鼠标位置.
     */
    doOnMove: function (ctx, mousePos) {
        FR.Chart.Tooltip.dataPointTip(ctx, this.tooltip, mousePos, this.glyph && this.glyph.toolTipBackground);
    },

    /**
     * 返回, 旋转之后, 实际的Bounds, 占用最小矩形区域.
     */
    getLabelBounds4Point: function (value, offset, labelString, centerP) {
        var textAttr = this.textAttr;
        if (textAttr == null) {
            textAttr = new FR.Chart.TextAttr();
        }

        var dim = FR.Chart.TextUtils.calculateTextDimensionWithRotation(labelString, textAttr);
        var position = this.position;
        var margin = FR.Chart.AxisGlyph.LINE_LABEL_GAP + this.getTickLengthShow();
        var height = dim.height;
        var width = dim.width;

        if (this.shouldBeHeight()) {
            height = Math.min(height, this.axisLabelWidth - margin);
        } else {
            width = Math.min(width, this.axisLabelWidth - margin);
        }
        var labelBounds;
        if (position == FR.Chart.Constants.LEFT) {
            if (this.cubic && this instanceof FR.Chart.ValueAxisGlyph) {//3d的时候是布局在值轴区域中间
                labelBounds = new FR.Chart.Bounds(
                    centerP.x - this.bounds.width / 2.0 - width / 2.0, centerP.y - height / 2,
                    width, height
                );
            } else {
                labelBounds = new FR.Chart.Bounds(
                    centerP.x - width - margin, centerP.y - height / 2,
                    width, height
                );
            }
        } else if (position == FR.Chart.Constants.TOP) {
            labelBounds = new FR.Chart.Bounds(
                centerP.x - width / 2,
                centerP.y - height - margin,
                width, height);
        } else if (position == FR.Chart.Constants.RIGHT) {
            labelBounds = new FR.Chart.Bounds(
                centerP.x + margin,
                centerP.y - height / 2,
                width, height);
        } else {
            if (this.cubic && this instanceof FR.Chart.ValueAxisGlyph) {
                labelBounds = new FR.Chart.Bounds(
                    centerP.x - width / 2,
                    centerP.y + this.bounds.height / 2 - height / 2,
                    width, height);
            } else {
                labelBounds = new FR.Chart.Bounds(
                    centerP.x - width / 2,
                    centerP.y + margin,
                    width, height);
            }
        }


        return labelBounds;
    },

    /**
     * 返回 值 默认间距 标签的边界.
     * @param value 值
     * @param offset 默认间距
     * @param labelString 标签
     * @returns {bounds} 边界
     */
    getLabelBounds: function (value, offset, labelString) {
        var centerP = this.getPoint2D(value + offset);
        return this.getLabelBounds4Point(value, offset, labelString, centerP);
    },

    /**
     * AxisGlyph.drawLabel 坐标轴
     * @param ctx 画布
     * @param value 值
     * @param offset 间隔
     * @param labelString 标签
     */
    drawLabel: function (ctx, value, offset, labelString) {
        if (!this.isShowAxisLabel) {
            return;
        }
        var labelBounds = this.getLabelBounds(value, offset, labelString);
        this.drawLabelWithBounds(ctx, value, offset, labelString, labelBounds);
    },

    /**
     * 根据标签, 已有标签的路径, 间隔等画出标签字符串.
     * @param ctx 画布
     * @param value 值
     * @param offset 间隔
     * @param labelString 标签字符串.
     * @param labelPath 标签路径.
     */
    drawLabelWithPath: function (ctx, value, offset, labelString, labelPath) {
        if (!this.isShowAxisLabel) {
            return;
        }
        var labelBounds = this.getLabelBounds(value, offset, labelString);
        if (!labelPath.isInteract(labelBounds)) {
            this.drawLabelWithBounds(ctx, value, offset, labelString, labelBounds);
            labelPath.addBounds(labelBounds, false);
        }
    },

    /**
     * 省略画法. 首先 判断是否省略, 拿到带省略符号的最终画的字符串.
     */
    drawLabelWithBounds: function (ctx, value, offset, labelString, labelBounds) {
        ctx.useRight = this._isUseLabelStartRight();

        var isNeedTooltip = this._isUseTooltip(labelString, labelBounds, this.textAttr);

        if (isNeedTooltip) {
            var esString = this._getFinalStringWithEs(labelString, this.textAttr, labelBounds);
            FR.Chart.PaintUtils.paintContent(ctx, esString, this.textAttr, labelBounds.x, labelBounds.y, labelBounds.width, labelBounds.height);
            this.tooltipLabelBoundsList.push(labelBounds);
            this.tooltipLabelList.push(labelString);
        } else {
            FR.Chart.TextUtils.drawLabelWithRotation(ctx, labelString, this.textAttr, labelBounds);
        }

        ctx.useRight = false;
    },

    _getFinalStringWithEs: function (text, textAttr, bounds) {
        var shouldBeHeight = this.shouldBeHeight();// 根据textAttr Bounds计算得到最终的 字符结果 然后直接画. 暂时不考虑换行什么的.
        var tmp = '';
        var resultLabelString = '';
        for (var i = 0; i < text.length; i++) {
            tmp += text.charAt(i);

            var dim = FR.Chart.TextUtils.calculateTextDimensionWithRotation(tmp, textAttr);
            if (shouldBeHeight) {
                if (dim.height > bounds.height) {
                    if (resultLabelString.length > 1) {
                        return resultLabelString.substr(0, resultLabelString.length - 1) + FR.Chart.Constants.ESP;
                    } else {
                        return FR.Chart.Constants.ESP;
                    }
                }
            } else {
                if (dim.width > bounds.width) {
                    if (resultLabelString.length > 1) {
                        return resultLabelString.substr(0, resultLabelString.length - 1) + FR.Chart.Constants.ESP;
                    } else {
                        return FR.Chart.Constants.ESP;
                    }
                }
            }

            resultLabelString += text.charAt(i);
        }

        return resultLabelString;
    },

    _isUseTooltip: function (labelString, labelBounds, textAttr) {
        if (textAttr == null) {
            textAttr = new FR.Chart.TextAttr();
        }
        return FR.Chart.PaintUtils.isNeedToolTip4AxisLabel(labelString, textAttr, labelBounds, this.shouldBeHeight());
    },

    // 是否使用 画坐标轴标签时, 右侧对齐基准线.
    _isUseLabelStartRight: function () {
        if (this.position == FR.Chart.Constants.LEFT && FR.Browser.isIE8()) {
            if (this.textAttr && this.textAttr.rotation === 0) {
                return true;
            }
        }
        return false;
    },

    /**
     * 画刻度线
     * @param ctx 画布
     * @param line 线
     * @param basicStroke  线型
     */
    drawTickLine: function (ctx, line, basicStroke) {
        if (!line
            || !this.lineColor
            || this.lineStyle == FR.Chart.Constants.LINE_NONE
            || line.startPoint == line.endPoint) {
            return;
        }

        if(!ctx){     return; } ctx.save();
        ctx.lineWidth = basicStroke;
        ctx.strokeStyle = this.lineColor;

        FR.Chart.GraphHelper.dealLine4JSDraw(line);

        line.draw(ctx);

        ctx.restore();
    },

    /**
     * 返回缩略图demo的网格线
     * @param value 值
     * @returns 返回对应值的网格线.
     */
    getGridLine4Demo: function (value) {
        var centerP = this.getPoint2D(value);

        var line;

        if (this.position == FR.Chart.Constants.BOTTOM || this.position == FR.Chart.Constants.TOP) {
            var preY = this.position == FR.Chart.Constants.TOP ? 0 : FR.Chart.PreChartGlyph.MOVE_BACKGROUND_HEIGHT + 12;

            line = new FR.Chart.Line2D(new FR.Chart.Point2D(centerP.x, centerP.y - this.demoGridLength + preY),
                new FR.Chart.Point2D(centerP.x, centerP.y + preY));
        }

        return [line];
    },

    /**
     * 返回网格线
     * @param value 对应的值
     * @returns 返回网格线数组
     */
    getGridLine: function (value) {
        var centerP = this.getPoint2D(value);

        var line;

        if (this.position == FR.Chart.Constants.LEFT) {
            line = new FR.Chart.Line2D(new FR.Chart.Point2D(centerP.x, centerP.y), new FR.Chart.Point2D(centerP.x + this.axisGridLength, centerP.y));
        } else if (this.position == FR.Chart.Constants.RIGHT) {
            line = new FR.Chart.Line2D(new FR.Chart.Point2D(centerP.x - this.axisGridLength, centerP.y), new FR.Chart.Point2D(centerP.x, centerP.y));
        } else if (this.position == FR.Chart.Constants.TOP) {
            line = new FR.Chart.Line2D(new FR.Chart.Point2D(centerP.x, centerP.y + this.axisGridLength), new FR.Chart.Point2D(centerP.x, centerP.y));
        } else {
            line = new FR.Chart.Line2D(new FR.Chart.Point2D(centerP.x, centerP.y - this.axisGridLength), new FR.Chart.Point2D(centerP.x, centerP.y));
        }

        return [line];
    },

    /**
     * 画出坐标轴Glyph
     * @param ctx 画布.
     */
    draw: function (ctx) {// 改用Math.round, 因为处理线像素精确dealLine4JSDraw时 也为round
        this.drawAxisLine(ctx);

        this._drawAxisTitle(ctx);
    },

    /**
     * 画出缩略图Demo相关
     * @param ctx 画布
     * @param textAttr 文本格式.
     */
    draw4Demo: function (ctx, textAttr) {
        var test = this.textAttr;
        this.textAttr = textAttr;
        this.isDraw4Demo = true;

        ctx.notUseDivText = true;// 标签控制 如果是div 就无法控制重叠

        this.drawOnlyLabels4Demo(ctx);

        ctx.notUseDivText = false;
        this.isDraw4Demo = false;
        this.textAttr = test;
    },

    /**
     * 默认不画, 只有分类和日期轴会参与 缩略图的简略画法
     * 因为要画出所有的标签值, 所以直接取 最大最小 minValue maxValue 而不经过 getCrossValue和getArrowValue的过滤
     *
     */
    drawOnlyLabels4Demo: function (ctx) {

    },

    /**
     * 画出警戒线
     * @param ctx 画布
     */
    drawAlertValueLines: function (ctx) {

    },

    getShape: function () {
        return this.getBounds();
    },

    /**
     * 画坐标轴箭头
     * @param ctx 画布
     * @param p0 起点
     * @param p1 终点.
     */
    drawArrow: function (ctx, p0, p1) {
        if (this.isArrowShow) {
            if(!ctx){     return; } ctx.save();

            ctx.strokeStyle = this.lineColor;
            ctx.fillStyle = this.lineColor;

            var s = new FR.Chart.Point2D(p1.x - p0.x, p1.y - p0.y);
            var t;
            //d2-d5设定箭头的大小,p1-p2为坐标轴的延长线,p2-p5-p3-p6为箭头4个点的具体位置
            var d1 = Math.sqrt(s.x * s.x + s.y * s.y);
            var d2 = 9;
            var d3 = 15;
            var d4 = 7;
            var d5 = 3;
            t = new FR.Chart.Point2D(d2 * s.x / d1, d2 * s.y / d1);
            var p2 = new FR.Chart.Point2D(p1.x + t.x, p1.y + t.y);
            t = new FR.Chart.Point2D(d3 * s.x / d1, d3 * s.y / d1);
            var p3 = new FR.Chart.Point2D(p1.x + t.x, p1.y + t.y);
            t = new FR.Chart.Point2D(d4 * s.x / d1, d4 * s.y / d1);
            var p4 = new FR.Chart.Point2D(p1.x + t.x, p1.y + t.y);
            var p5 = new FR.Chart.Point2D(p4.x + s.y / d1 * d5, p4.y - s.x / d1 * d5);
            var p6 = new FR.Chart.Point2D(p4.x - s.y / d1 * d5, p4.y + s.x / d1 * d5);

            var arrow = new FR.Chart.GeneralPath();
            arrow.moveTo(p2.x, p2.y);
            arrow.lineTo(p5.x, p5.y);
            arrow.lineTo(p3.x, p3.y);
            arrow.lineTo(p6.x, p6.y);
            arrow.closePath();

            var line = new FR.Chart.Line2D(p1, p2);
            FR.Chart.GraphHelper.dealLine4JSDraw(line);
            line.draw(ctx);

            arrow.draw(ctx);
            arrow.paint(ctx);

            ctx.restore();
        }
    },

    _drawAxisTitle: function (ctx) {
        var titleGlyph = this.titleGlyph;
        if (titleGlyph) {
            // 计算下合适的边界
            var chartBounds = this._calculatorTitleBounds();
            titleGlyph.bounds = chartBounds;

            var rotation = this.titleGlyph.rotation;

            if(titleGlyph.textAttr.alignText == FR.Chart.Style.VERTICALTEXT){
                if(this.position == FR.Chart.Constants.LEFT){
                    titleGlyph.textAttr.rotation = 90;
                }else if(this.position == FR.Chart.Constants.RIGHT){
                    titleGlyph.textAttr.rotation = -90;
                }
            }

            titleGlyph.drawAxisTitle(ctx, FR.Chart.currentDivKey);

            if(titleGlyph.textAttr.alignText == FR.Chart.Style.VERTICALTEXT){
                titleGlyph.textAttr.rotation = 0;
            }

            this.titleGlyph.rotation = rotation;
        }
    },

    /**
     * 根据坐标点 返回对应的值
     * @param point 坐标点
     * @returns {number} 返回值.
     */
    getPointValue: function (point) {
        var value = 0;
        if (this.originPoint == null) {
            return value;
        }
        if (this.position == FR.Chart.Constants.LEFT
            || this.position == FR.Chart.Constants.RIGHT) {
            if (this.position == FR.Chart.Constants.LEFT && this.bounds.y == this.originPoint.y) {
                value = (point.y - this.originPoint.y) / this.unitLength;
            } else {
                if (!this.axisReversed) {
                    value = (this.originPoint.y - point.y) / this.unitLength;
                } else {
                    value = (point.y - this.originPoint.y) / this.unitLength;
                }
            }
        } else {
            if (!this.axisReversed) {
                value = (point.x - this.originPoint.x) / this.unitLength;
            } else {
                value = (this.originPoint.x - point.x) / this.unitLength;
            }
        }

        if (this.isLog && this.getCrossValue() > 0) {
            value = Math.pow(this.tickLength, value) * this.getCrossValue();
        } else {
            value = value + this.getCrossValue();
        }
        return value;
    },

    /**
     * 滚动条向指针移动, 右侧移动
     * @param {Number} number 移动数目.
     */
    onZoomBarIncrease: function (number) {
        if (FR.Chart.ChartUtils.accNumber(this.zoomForward) <= -this.zoomTickLength) {
            if (number) {
                number = Math.min(number, Math.abs(this.zoomForward));
                this.zoomBackward -= number;
                this.zoomForward += number;
            } else {
                var tmp = Math.min(this.zoomTickLength, Math.abs(this.zoomForward));
                this.zoomBackward -= tmp;
                this.zoomForward += tmp;
            }
            this.zoomForward = Math.min(0, this.zoomForward);

            return true;
        } else if (this.zoomForward < 0) {
            var number = Math.abs(this.zoomForward);
            this.zoomForward = 0;
            this.zoomBackward -= number;
            return true;
        }
        return false
    },

    /**
     * 滚动条向原点移动, 左侧
     * @param Number number 移动数目.
     * @returns {boolean}
     */
    onZoomBarDecrease: function (number) {
        if (FR.Chart.ChartUtils.accNumber(this.zoomBackward) <= -this.zoomTickLength) {
            if (number) {
                number = Math.min(number, Math.abs(this.zoomBackward));
                this.zoomBackward += number;
                this.zoomForward -= number;
            } else {
                var tmp = Math.min(this.zoomTickLength, Math.abs(this.zoomBackward));
                this.zoomBackward += tmp;
                this.zoomForward -= tmp;
            }
            this.zoomBackward = Math.min(this.zoomBackward, 0);

            return true;
        } else if (this.zoomBackward < 0) {
            var number = Math.abs(this.zoomBackward);
            this.zoomBackward = 0;
            this.zoomForward -= number;
            return true;
        }
        return false;
    },

    /**
     * 滚动条左侧或下侧放大
     * @param number 移动数目
     * @returns {boolean}
     */
    onZoomBarLeftUp: function (number) {
        if (FR.Chart.ChartUtils.accNumber(this.zoomBackward) <= -this.zoomTickLength) {
            if (number) {
                this.zoomBackward += number;
            } else {
                this.zoomBackward += this.zoomTickLength;
            }
            this.zoomBackward = Math.min(0, this.zoomBackward);
            return true;
        } else if (this.zoomBackward < 0) {
            this.zoomBackward = 0;
            return true;
        }
        return false;
    },

    /**
     * 检查缩小 forward 和 backward 中间应该有间隔, 不然无法正常显示图表.
     * Math.abs(forward + backward) < (this.maxValue - this.minValue) - this.zoomTickLength
     */
    _checZoomDown: function (number) {
        var maxWard = Math.abs(this.maxValue - this.minValue) - Math.abs(this.zoomTickLength);
        var nowWard = Math.abs(this.zoomForward + this.zoomBackward);

        if (nowWard + number >= maxWard) {
            number = maxWard - nowWard;
        }

        return number;
    },

    // 滚动条左侧或下侧缩小
    onZoomBarLeftDown: function (number) {
        var maxValue = this.getArrowValue() - this.getCrossValue();
        if (FR.Chart.ChartUtils.accNumber(maxValue) > this.zoomTickLength) {

            var tmpTick = number ? number : this.zoomTickLength;

            tmpTick = this._checZoomDown(tmpTick);
            this.zoomBackward -= tmpTick;

            this.zoomBackward = Math.min(this.zoomBackward, 0);

            return true;
        }
        return false;
    },

    /**
     * 滚动条右侧或上侧放大
     * @param number 移动数目
     * @returns {boolean}
     */
    onZoomBarRightUp: function (number) {
        if (FR.Chart.ChartUtils.accNumber(this.zoomForward) <= -this.zoomTickLength) {
            if (number) {
                this.zoomForward += number;
            } else {
                this.zoomForward += this.zoomTickLength;
            }
            this.zoomForward = Math.min(0, this.zoomForward);
            return true;
        } else if (this.zoomForward < 0) {
            this.zoomForward = 0;
            return true;
        }
        return false;
    },

    /**
     * 滚动条右侧或上侧缩小
     * @param number 移动数目
     * @returns {boolean}
     */
    onZoomBarRightDown: function (number,isTimeChange) {
        var maxValue = this.getArrowValue() - this.getCrossValue();
        if (FR.Chart.ChartUtils.accNumber(maxValue) > this.zoomTickLength) {

            var tmpTick = number ? number : this.zoomTickLength;
            tmpTick =isTimeChange ? tmpTick : this._checZoomDown(tmpTick);
            this.zoomForward -= tmpTick;

            this.zoomForward = Math.min(this.zoomForward, 0);
            return true;
        }
        return false;
    },

    _calculatorTitleBounds: function () {
        var titleBounds = null;

        var dim = this.titleGlyph.preferredDimension();
        var titlePosition = this.titleGlyph.position;

        if (this.position == FR.Chart.Constants.TOP) {
            titleBounds = this._getTopTitleBounds(dim, titlePosition);
        }
        else if (this.position == FR.Chart.Constants.LEFT) {
            titleBounds = this._getLeftTitleBounds(dim, titlePosition);
        }
        else if (this.position == FR.Chart.Constants.BOTTOM) {
            titleBounds = this._getBottomTitleBounds(dim, titlePosition);
        }
        else if (this.position == FR.Chart.Constants.RIGHT) {
            titleBounds = this._getRightTitleBounds(dim, titlePosition);
        }

        titleBounds.x += this.bounds.x;
        titleBounds.y += this.bounds.y;

        return titleBounds;
    },

    _getTopTitleBounds: function (dim, titlePosition) {
        return new FR.Chart.Bounds(this._getTopBottomX(titlePosition),
            -FR.Chart.AxisGlyph.LINE_LABEL_GAP - this.titleDim.height,
            dim.width, dim.height);
    },

    _getTopBottomX: function (titlePosition) {
        var startX = (this.axisLength - this.titleDim.width) / 2;
        if (titlePosition == FR.Chart.Constants.LEFT) {
            startX = -12;
        } else if (titlePosition == FR.Chart.Constants.RIGHT) {
            startX = this.axisLength - this.titleDim.width;
        }

        return startX;
    },

    /**
     * 返回底部 标签的位置.  被继承 . 考虑到分类轴标签的多分类 以及 数据表.
     * @param dim
     * @param titlePosition
     * @returns {FR.Chart.Bounds}
     * @private
     */
    _getBottomTitleBounds: function (dim, titlePosition) {
        return new FR.Chart.Bounds(
            this._getTopBottomX(titlePosition),
            this.axisLabelWidth + FR.Chart.AxisGlyph.LINE_LABEL_GAP,
            dim.width, dim.height);
    },

    _getLeftTitleBounds: function (dim, titlePosition) {
        return  new FR.Chart.Bounds(
            -this.titleDim.width - FR.Chart.AxisGlyph.LINE_LABEL_GAP,
            this._getLeftRightY(titlePosition),
            dim.width, dim.height);
    },

    _getLeftRightY: function (titlePosition) {
        var startY = (this.axisLength - this.titleDim.height) / 2.0;
        if (titlePosition == FR.Chart.Constants.LEFT) {
            startY = -12;
        } else if (titlePosition == FR.Chart.Constants.RIGHT) {
            startY = this.axisLength - this.titleDim.height;
        }

        return startY;

    },

    _getRightTitleBounds: function (dim, titlePosition) {
        return  new FR.Chart.Bounds(
            this.axisLabelWidth + FR.Chart.AxisGlyph.LINE_LABEL_GAP,
            this._getLeftRightY(titlePosition),
            dim.width, dim.height);
    }
});
$.extend(FR.Chart.AxisGlyph, /**@class FR.Chart.AxisGlyph*/{
    /**
     * @static
     * @property 线的标签间隔
     */
    LINE_LABEL_GAP: 4,
    /**
     * @static
     * @property 标签和标题间隔
     */
    LABEL_TITLE_GAP: 1,

    /**
     * @static
     * @property 主要刻度长度
     */
    MAIN_TICK_LENGTH: 5,
    /**
     * @static
     * @property 次要刻度长度
     */
    SEC_TICK_LENGTH: 3,

    /**
     * 根据最大最小值 计算合适的刻度间隔大小
     * @static
     * @param minValue 最小值
     * @param maxValue 最大值
     * @returns {number} 返回刻度间隔大小
     */
    calculateIncrement: function (minValue, maxValue) {
        if (minValue >= maxValue) {
            return 1;
        }

        var increValue = maxValue - minValue;
        if (increValue === 1) {
            return 0.2;
        }

        var tmp = 0;
        if (increValue > 0 && increValue < 1) {
            while (increValue < 1) {
                increValue = increValue * 10;
                tmp--;
            }
        } else if (increValue >= 10) {
            while (increValue >= 10) {
                increValue = increValue / 10;
                tmp++;
            }
        }
        var base = Math.pow(10, tmp);
        var creBase = parseInt((maxValue - minValue) / base);

        if (creBase === 2 || creBase === 1) {
            return base / 2;
        }
        if (creBase >= 3 && creBase <= 5) {
            return base;
        }
        if (creBase >= 6 && creBase <= 7) {
            return base * 2;
        }
        if (creBase >= 8) {
            return base * 3;
        }

        return increValue;
    }
});

/**
 * 分类坐标轴
 *
 *      @example
 *      var axisGlyph = new FR.Chart.CategoryAxisGlyph();
 *
 * @class FR.Chart.CategoryAxisGlyph
 * @extends FR.Chart.AxisGlyph
 *
 * @param {JSON} js 配置属性
 * @param {Array} [js.categoryLabelList = []] 标签数组
 *
 */
FR.Chart.CategoryAxisGlyph = FR.extend(FR.Chart.AxisGlyph, {

    /**
     * @property 标签偏移量
     */
    labelOffset: 0.5,

    initConstants: function (js) {
        js = js || {};

        this.categoryLabelList = [];
        if (js.categoryLabelList && js.categoryLabelList.length > 0) {
            for (var i = 0; i < js.categoryLabelList.length; i++) {
                this.categoryLabelList.push(js.categoryLabelList[i]);
            }
        }

        this.secondLabels = [];
        if (js.secondLabels && js.secondLabels.length > 0) {
            for (var i = 0; i < js.secondLabels.length; i++) {
                this.secondLabels.push(js.secondLabels[i]);
            }
        }

        this.thirdLabels = [];
        if (js.thirdLabels && js.thirdLabels.length > 0) {
            for (var i = 0; i < js.thirdLabels.length; i++) {
                this.thirdLabels.push(js.thirdLabels[i]);
            }
        }

        this.timeSwitchArray = [];
        if(js.timeSwitchArray && js.timeSwitchArray.length >0){
            for (var i = 0; i < js.timeSwitchArray.length; i++) {
                this.timeSwitchArray.push(js.timeSwitchArray[i]);
            }
        }

        this.zoomTickLength = 1;
        this.PRE_LABEL_HEIGHT = 20;

        FR.Chart.CategoryAxisGlyph.superclass.initConstants.apply(this, [js]);
    },



    isNeedTimeSwitch:function(){
        return this.timeSwitchArray && this.timeSwitchArray.length >0;
    },


    configLabels:function(chartData){
        this.categoryLabelList = chartData.category_array || [];
        this.secondLabels = chartData.secondCates || [];
        this.thirdLabels = chartData.thirdLabels || [];

        if (this.categoryLabelList) {
            this.maxValue = this.categoryLabelList.length;
            this.minValue = 0;
            this.tickLength = this.isCustomMainUnit ? this.tickLength : 1;
            this.smallTickLength = this.isCustomSecUnit ? this.smallTickLength : this.tickLength/5;
        }
    },

    configAxisLabelsWithOptionData:function(data){
        this.categoryLabelList = data.category || [];
        if (this.categoryLabelList) {
            this.maxValue = this.categoryLabelList.length;
            this.minValue = 0;
            this.tickLength = this.isCustomMainUnit ? this.tickLength : 1;
            this.smallTickLength = this.isCustomSecUnit ? this.smallTickLength : this.tickLength/5;
        }
    },

    /**
     *    根据配置属性, 初始化一些必须的数值.
     *    @param config 配置属性.
     */
    initConfig: function (config) {

        if (this.categoryLabelList) {
            this.maxValue = this.categoryLabelList.length;
            this.minValue = 0;
            this.tickLength = config.tickLength || 1;
            this.smallTickLength = this.tickLength / 5;
            this.drawBetweenTick = true;// 分类轴 非面积图时 默认在中间.
        }
    },

    /**
     * 计算坐标轴的长度单位.
     */
    calculateAxisLengthUnit: function () {
        var tickNumber = this.getArrowValue() - this.getCrossValue();

        var tickNoScalNumber = this.maxValue - this.minValue;

        if (tickNumber <= 0) {
            this.unitLength = 1;
            return;
        }

        if (this.drawBetweenTick) {
            this.unitLength = this.axisLength / tickNumber;
            this.unitLengthNoScale = this.axisLength / ((tickNoScalNumber) === 0 ? 1 : tickNoScalNumber);
        } else {
            this.unitLength = this.axisLength / Math.max(1, tickNumber - 1);
            this.unitLengthNoScale = this.axisLength / Math.max(1, tickNoScalNumber - 1);
        }
        if (tickNoScalNumber <= 0) {
            this.unitLengthNoScale = 1;
        }
    },

    /**
     * 根据标签的 分类 计算新的PlotBounds
     * @param plotBounds 绘图区边界
     */
    calBoundsWithMoreLabel4Cate: function (plotBounds) {
        if (this.position == FR.Chart.Constants.BOTTOM && this.isShowAxisLabel) {
            var height = this.secondLabels.length === 0 ? 0 : this.PRE_LABEL_HEIGHT;
            height += this.thirdLabels.length === 0 ? 0 : this.PRE_LABEL_HEIGHT;

            plotBounds.height = plotBounds.height - height;
        }
    },


    dealWithPlotBounds:function(){
        var maxlength = this.getMaxLength();
        //因为有换行，所有重新调整绘图区边界
        if(maxlength){
            maxlength += FR.Chart.AxisGlyph.LINE_LABEL_GAP;
        }
        maxlength += this.getTickLengthShow();
        if(this.shouldBeHeight()){
            maxlength = Math.min(this.plotZeroBounds.height * this.MAX_AXIS_LABEL_HEIGHT+
                 FR.Chart.AxisGlyph.LINE_LABEL_GAP+this.getTickLengthShow(),maxlength);
        }else{
            maxlength = Math.min(this.plotZeroBounds.width * this.MAX_AXIS_LABEL_WIDTH +
            FR.Chart.AxisGlyph.LINE_LABEL_GAP+this.getTickLengthShow(),maxlength);
        }
        var list = this.categoryLabelList;
        if(list.length>0){
            if(this.shouldBeHeight()){
                var firstString = ((this.format == null) ? (list[0].toString()) : (FR.contentFormat(list[0].toString(), this.format)));
                this.startLabelDim = new FR.Chart.Dimension2D(this.dealWithStartAndEndDim(firstString),this.startLabelDim.height);
                var lastString =  ((this.format == null) ? (list[list.length-1].toString()) : (FR.contentFormat(list[list.length-1].toString(), this.format)));
                this.endLabelDim = new FR.Chart.Dimension2D(this.dealWithStartAndEndDim(lastString),this.endLabelDim.height);
            }else{
                var firstString = ((this.format == null) ? (list[0].toString()) : (FR.contentFormat(list[0].toString(), this.format)));
                this.startLabelDim = new FR.Chart.Dimension2D(this.startLabelDim.width,this.dealWithStartAndEndDim(firstString));
                var lastString =  ((this.format == null) ? (list[list.length-1].toString()) : (FR.contentFormat(list[list.length-1].toString(), this.format)));
                this.endLabelDim = new FR.Chart.Dimension2D(this.endLabelDim.width,this.dealWithStartAndEndDim(lastString));
            }

        }
        var diff = maxlength -this.axisLabelWidth;
        this.axisLabelWidth = maxlength;
        return diff;
    },

    dealWithStartAndEndDim:function(label){
        var normalPoint = FR.Chart.TextUtils.calculateTextDimensionWithRotation("a", this.textAttr);
        var dim = this.getAxisLabelDim(label);
        if(this.shouldBeHeight()){
            return Math.min(dim.width,this.unitLength - normalPoint);
        }else{
            return Math.min(this.unitLength-6,dim.height);
        }
    },

    initMinMaxValue: function (autoMin, autoMax){
        this.minValue = autoMin;
        this.maxValue = autoMax;
    },

    getMaxLength:function(){
        if(!this.isShowAxisLabel){
            return 0;
        }

        var labelWidth = 0;
        var list = this.categoryLabelList;

        for (var i = 0; i < list.length; i += 1) {
            var labelString = ((this.format == null) ? (list[i].toString()) : (FR.contentFormat(list[i].toString(), this.format)));
            var tmpWidth =this.getAxisLabelMoreWidth(labelString);
            labelWidth = (tmpWidth < labelWidth ? labelWidth : tmpWidth );
        }
        return labelWidth;
    },

    //占用的最大的高度 或者宽度
    getAxisLabelMoreWidth:function(label,i){
        var dim = this.getAxisLabelDim(label);
        var normalPoint = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation("a", this.textAttr);
        if(this.textAttr.alignText == FR.Chart.Style.HORIZONTALTEXT){
            if(this.shouldBeHeight()){
                var maxWidth = this.unitLength - normalPoint.width;
                if(this.textAttr.rotation === 0){
                    return this.calculateAllNeedHeight4Horizontal(label,maxWidth);
                }else{
                    var maxHeight = this.plotZeroBounds.height * this.MAX_AXIS_LABEL_HEIGHT;
                    var rotation = this.textAttr.rotation;
                    var angle = rotation * Math.PI / 180;
                    var showWidth = 0.5 * maxWidth /Math.abs(Math.cos(angle));
                    var allRotaitonHeight =  this.calculateAllNeedHeight4Horizontal(label,showWidth);
                    var heightUsed = FR.Chart.TextUtils._calDimWithRotation(new FR.Chart.Dimension2D(showWidth,allRotaitonHeight),this.textAttr).height;
                    if(heightUsed > maxHeight){
                        var drawStrings = [];
                        var isNotFullShow = [];
                        heightUsed = FR.Chart.TextUtils._calDimWithRotation(this.calculateDimUsed4Rotation(label,maxHeight,maxWidth,drawStrings,isNotFullShow),this.textAttr).height;
                    }
                    return heightUsed;

                }
            }else{
                if(this.textAttr.rotation === 0){
                    return dim.width
                }else{
                    var maxWidth =this.plotZeroBounds.width * this.MAX_AXIS_LABEL_WIDTH;
                    var maxHeight = this.unitLength-this.TOP_AND_DOWN_GAP;
                    var rotation = this.textAttr.rotation;
                    var angle = rotation * Math.PI / 180;
                    var showWidth  = 0.5 * maxHeight / Math.abs(Math.sin(angle));
                    var allRotaitonHeight =  this.calculateAllNeedHeight4Horizontal(label,showWidth);
                    var widthUsed = FR.Chart.TextUtils._calDimWithRotation(new FR.Chart.Dimension2D(showWidth,allRotaitonHeight),this.textAttr).width;
                    if(widthUsed > maxWidth){
                        var drawStrings = [];
                        var isNotFullShow = [];
                        widthUsed = FR.Chart.TextUtils._calDimWithRotation(this.calculateDimUsed4Rotation(label,maxHeight,maxWidth,drawStrings,isNotFullShow),this.textAttr).width;
                    }
                    return widthUsed;
                }
            }
        }else{
            if(this.shouldBeHeight()){
                return dim.height;
            }else{
                return this.calculateAllNeedWidth4Vertival(label,this.unitLength-this.TOP_AND_DOWN_GAP);
            }
        }
    },

    calculateAllNeedHeight4Horizontal:function(label,maxWidth){
        var allHieght = 0;
        var subString = label;
        var margin = FR.Chart.AxisGlyph.LINE_LABEL_GAP + this.getTickLengthShow();
        do{
            for(var i =1 ;i<=subString.length;i++){
                var size = FR.Chart.TextUtils.calculateTextDimensionWithRotation(subString.substring(0,i),this.textAttr);
                var over = size.width > maxWidth || size.height >this.axisLabelWidth - margin;
                if(i === 1 && over){
                    subString = "";
                    size = FR.Chart.TextUtils.calculateTextDimensionWithRotation(this.ELLIPSIS,this.textAttr);
                    allHieght += size.height;
                    break;
                }
                var width = size.width;
                if(width > maxWidth && i!==1){
                    allHieght += size.height;
                    subString = subString.substring(i-1);
                    break;
                }else if(i == subString.length){
                    subString = "";
                    allHieght +=size.height;
                    break;
                }
            }
        }while(subString !="");
        return allHieght;
    },

    calculateAllNeedWidth4Vertival:function(label,maxHeight){
        var allWidth = -this.TOP_AND_DOWN_GAP/2;
        var subString = label;
        var margin = FR.Chart.AxisGlyph.LINE_LABEL_GAP + this.getTickLengthShow();
        do{
            allWidth += this.TOP_AND_DOWN_GAP/2;
            for(var i =1 ;i<=subString.length;i++){
                var size = FR.Chart.TextUtils.calculateTextDimensionWithRotation(subString.substring(0,i),this.textAttr);
                var over = size.width > this.axisLabelWidth - margin || size.height >maxHeight;
                if(i === 1 && over){
                    subString = "";
                    size = FR.Chart.TextUtils.calculateTextDimensionWithRotation(this.ELLIPSIS,this.textAttr);
                    allWidth += size.width;
                    break;
                }
                var height = size.height;
                if(height > maxHeight && i!== 1){
                    allWidth += size.width;
                    subString = subString.substring(i-1);
                    break;
                }else if(i == subString.length){
                    subString = "";
                    allWidth +=size.width;
                    break;
                }
            }
        }while(subString !="");
        return allWidth;
    },

    /**
     * 返回坐标轴标签占据的大小宽度, 并且初始化 结束开始的标签大小.
     * @returns {number} 标签大小.
     */
    getMaxLabelWidthAndInitStartEndLabelDim: function () {
        if (!this.isShowAxisLabel) {
            return 0;
        }

        var labelWidth = 8;
        var list = this.categoryLabelList;

        var incre = 1;  // getTickSamplingTime();
        for (var i = 0; i < list.length; i += incre) {
            var labelString = ((this.format == null) ? (list[i].toString()) : (FR.contentFormat(list[i].toString(), this.format)));
            var dim = this.getAxisLabelDim(labelString);
            if (i === 0) {
                this.startLabelDim = dim;
            } else {
                this.endLabelDim = dim;
            }

            var tmpWidth = this.shouldBeHeight() ? dim.height : dim.width;
            labelWidth = (tmpWidth < labelWidth ? labelWidth : tmpWidth );
        }
        return labelWidth;
    },

    /**
     * 返回数据表 对左侧 坐标轴的影响
     * @returns {number} 位置大小.
     */
    getPreLeftWidth4DataSheet: function () {
        this.calculateTitleDimesion();

        var titlePreWidth = 0;
        if (this.titleDim.width > 0) {
            titlePreWidth = this.titleDim.width + FR.Chart.AxisGlyph.LINE_LABEL_GAP;
        }
        var labelWidth = this.getMaxLabelWidthAndInitStartEndLabelDim();
        return (this.position == FR.Chart.Constants.LEFT) ? labelWidth + titlePreWidth : 0;
    },

    /**
     * 添加 分类标签
     * @param label 标签.
     */
    addCategoryLabel: function (label) {
        this.categoryLabelList.push(label);
    },

    /**
     * 返回最大的刻度值.
     * @returns 刻度值.
     */
    getMaxTickValue: function () {
        var tickValue = this.getArrowValue();
        if (!this.drawBetweenTick) {// 1个标签的时候, max = 1
            if (this.categoryLabelList && this.categoryLabelList.length > 1) {
                tickValue = tickValue - this.tickLength;
            }
        }

        return tickValue;
    },

    /**
     * @return 刻度抽样次数，因为在这个抽样前, axisLength还没有进行初始化 所以只是大概的计算下 标签的跳跃个数.
     */
    getTickNumber4CalDim: function () {
        var smallTickLength = 15;
        var tickNumber = this.getMaxTickValue() - this.getCrossValue();

        var count = 1;
        var useTick = Math.abs(tickNumber);
        while (smallTickLength < Math.ceil(useTick / (count * this.getDateTypeNumber()))) {
            count++;
        }
        return count;
    },

    /**
     * @return 刻度抽样次数，当坐标轴宽度大于15px且坐标刻度之间的间隔<15px的时候，刻度间隔显示
     */
    getTickSamplingTime: function () {
        return this.getTickIntervalCount(this.getMaxTickValue() - this.getCrossValue(), 15);
    },

    /**
     * smallTickLength : 允许最小的刻度间隔像素 一般是15, 对于缩略图则是40
     */
    getTickIntervalCount: function (tickNumber, smallTickLength) {
        var count = 1;
        if (this.axisLength
            && this.tickLength
            && tickNumber && Math.abs(tickNumber) > 0
            && smallTickLength
            && this.axisLength > smallTickLength) {
            var maxCount = this.axisLength / smallTickLength;
            var useTick = Math.abs(tickNumber);
            while (maxCount < Math.ceil(useTick / ( this.tickLength * count * this.getDateTypeNumber()))) {
                count++;
            }
        }
        return count;
    },

    /**
     * 返回日期类型
     * @returns {number}
     */
    getDateTypeNumber: function () {
        return 1;
    },

    /**
     * 返回最大的刻度值
     * @returns {Number|*} 最大刻度值.
     */
    getMaxTickValueDemo: function () {
        var tickValue = this.maxValue;
        if (!this.drawBetweenTick) {// 1个标签的时候, max = 1
            if (this.categoryLabelList && this.categoryLabelList.length > 1) {
                tickValue = tickValue - this.tickLength;
            }
        }

        return tickValue;
    },

    /**
     * 为缩略图准备的间隔计算, 默认最少间隔为40px. 并且最大最小值 是不变化的
     */
    getTickSamplingTime4Demo: function () {
        return this.getTickIntervalCount(this.getMaxTickValueDemo() - this.minValue, 40);
    },

    /**
     * 获取标签样式 最终的间隔个数
     * @param tickSamplingTime 刻度间隔样式个数
     * @param isToNumber 是否能转为numer
     * @param labelSamplingTime  自定义标签间隔样式个数
     * @param offset 间隔
     * @returns {*} 返回实际的标签间隔个数.
     */
    getLabelSamplingTime: function (tickSamplingTime, isToNumber, labelSamplingTime, offset) {
        var tmp = tickSamplingTime * labelSamplingTime;
        var labelBoundsManager = new FR.Chart.BoundsManager();
        for (var value = 0; value < this.categoryLabelList.length; value += tmp) {
            var labelString = this.getLabelString(value, isToNumber);
            var labelBounds = this.getLabelBounds(value, offset, labelString);
            if (labelBoundsManager.isInteract(labelBounds)) {
                labelSamplingTime++;
                return this.getLabelSamplingTime(tickSamplingTime, isToNumber, labelSamplingTime, offset);
            } else {
                labelBoundsManager.addBounds(labelBounds);
            }
        }
        return labelSamplingTime;
    },

    /**
     * 画坐标轴间隔线
     * @param ctx 画布.
     */
    drawAxisGrid: function (ctx) {

        var maxTickValue = this.getMaxTickValue();
        var tickSamplingTime = this.getTickSamplingTime();

        if(!ctx){     return; } ctx.save();
        if (this.mainGridStyle != FR.Chart.Constants.LINE_NONE && this.tickLength > 0
            && this.mainGridColor != null) {
            ctx.lineWidth = FR.Chart.GraphHelper.getLineStyleSize(this.mainGridStyle);
            ctx.strokeStyle = this.mainGridColor;

            for (var value = (this.tickLength + this.getCrossValue()) * tickSamplingTime; value <= maxTickValue; value += this.tickLength * tickSamplingTime) {
                var lines = this.getGridLine(value);
                for (var i = 0; i < lines.length; i++) {
                    FR.Chart.GraphHelper.dealLine4JSDraw(lines[i]);
                    lines[i].draw(ctx);
                }
            }
        }
        ctx.restore();
    },

    /**
     * CategoryAxisGlyph.drawAxisLine.start
     * 分类轴 画坐标轴线
     * @param ctx 画布
     */
    drawAxisLine: function (ctx) {
        if(!ctx){     return; } ctx.save();

        if (this.lineColor && this.lineStyle != FR.Chart.Constants.LINE_NONE) {
            ctx.lineWidth = FR.Chart.GraphHelper.getLineStyleSize(this.lineStyle);
            ctx.strokeStyle = this.lineColor;

            var line = new FR.Chart.Line2D(this.getPoint2D(this.getCrossValue()), this.getPoint2D(this.getMaxTickValue()));
            FR.Chart.GraphHelper.dealLine4JSDraw(line);
            line.draw(ctx);
            this.drawArrow(ctx, line.startPoint, line.endPoint);

        }
        this.drawTicks(ctx);

        this.drawMoreTickLabel(ctx);

        ctx.restore();
    },

    /**
     * 画多分类刻度标签.
     * @param ctx 画布.
     */
    drawMoreTickLabel: function (ctx) {
        if (!this.isShowAxisLabel) {
            return;
        }

        this._drawMore(ctx, this.secondLabels, 0);
        this._drawMore(ctx, this.thirdLabels, 1);

        this._drawMoreTicks(ctx, 0);
        this._drawMoreTicks(ctx, 1);
    },

    _drawMore: function (ctx, labels, index) {
        if (labels == null || labels.length === 0) {
            return;
        }

        var labelBoundsManager = new FR.Chart.BoundsManager();
        var secondAttr = new FR.Chart.TextAttr();
        secondAttr.frFont = new FR.Chart.FRFont();

        secondAttr.frFont.fontName = "Microsoft YaHei";
        secondAttr.frFont.size = '12';

        for (var labelIndex = 0, size = labels.length; labelIndex < size; labelIndex++) {
            var labelString = labels[labelIndex];
            var dim = FR.Chart.TextUtils.calculateTextDimensionWithRotation(labelString, secondAttr);

            var x = this.bounds.x + this._getSecondX(labelIndex, size);

            var labelx = x - dim.width / 2.0;
            var labely = this.bounds.y + this.axisLabelWidth + index * this.PRE_LABEL_HEIGHT + 7;

            var labelBounds = new FR.Chart.Bounds(labelx, labely, dim.width, dim.height);

            if (!labelBoundsManager.isInteract(labelBounds)) {
                labelBoundsManager.addBounds(labelBounds);

                var isNeedTooltip = this._isUseTooltip(labelString, labelBounds, secondAttr);
                if (isNeedTooltip) {
                    labelString = this._getFinalStringWithEs(labelString, secondAttr, labelBounds);
                }

                FR.Chart.PaintUtils.paintContent(ctx, labelString, secondAttr, labelBounds.x, labelBounds.y, labelBounds.width, labelBounds.height);
            }
        }
    },

    _drawMoreTicks: function (ctx, index) {
        var labels = index === 0 ? this.secondLabels : this.thirdLabels;

        if (!labels || labels.length <= 0) {
            return;
        }

        var tickLength = (index + 1) * this.PRE_LABEL_HEIGHT + this.axisLabelWidth;
        var firstCount = parseInt(this.categoryLabelList.length / labels.length);

        for (var value = 0; value <= labels.length; value++) {
            var line = this.getTickLine(value * firstCount, tickLength, FR.Chart.Constants.TICK_MARK_OUTSIDE);
            this.drawTickLine(ctx, line, FR.Chart.Constants.AxisTickWidth);
        }
    },

    _getSecondX: function (index, size) {
        var width = this.bounds.width;

        return width * (index + 1.0 / 2.0) / size;
    },

    _getBottomTitleBounds: function (dim, titlePosition) {
        var titleY = this.axisLabelWidth + FR.Chart.AxisGlyph.LINE_LABEL_GAP;
        if (this.secondLabels && this.secondLabels.length > 0) {
            titleY += this.PRE_LABEL_HEIGHT;
        }
        if (this.thirdLabels && this.thirdLabels.length > 0) {
            titleY += this.PRE_LABEL_HEIGHT;
        }

        return new FR.Chart.Bounds(this._getTopBottomX(titlePosition), titleY, dim.width, dim.height);
    },

    /**
     * 判断 分类标签 能否全部转为数据
     * @returns {boolean}
     */
    isToNumber: function () {
        // 判断是否能转化为number 对所有的
        for (var i = 0; i < this.categoryLabelList.length; i++) {
            // 在部分分类是数字 部分不是的时候   保证用同样的格式 所以循环整个分类
            for (var p = 0; p < this.categoryLabelList[i].toString().length; p++) {
                var ch = this.categoryLabelList[i].toString().charAt(p);
                if (isNaN(ch) && ch != '.') {
                    return false;
                }
            }
        }
        return true;
    },

    getLabelString: function (value) {
        var labelString = this.categoryLabelList[value].toString();
        // 数据格式
        var format = this.format;
        if (format != null) {
            labelString = FR.contentFormat(labelString, format);
        }
        return labelString;
    },

    /**
     *只画标签 (在缩略图中)
     * @param ctx 画布.
     */
    drawOnlyLabels4Demo: function (ctx) {
        if (this.axisLength <= 15) {
            return;
        }

        if (this.tickLength > 0 && this.axisLength >= 40) {// 就是画网格线 间隔最少40
            var tickSamplingTime4Demo = this.getTickSamplingTime4Demo();
            for (var value = (this.tickLength + this.minValue) * tickSamplingTime4Demo; value < this.maxValue;
                 value += this.tickLength * tickSamplingTime4Demo) {
                var lines = this.getGridLine4Demo(value);
                for (var i = 0; i < lines.length; i++) {
                    if (lines[i]) {
                        FR.Chart.GraphHelper.dealLine4JSDraw(lines[i]);
                        lines[i].draw(ctx);
                    }
                }
            }
        }

        var tickSamplingTime = this.getTickIntervalCount(this.getMaxTickValueDemo() - this.minValue, 15);
        var isToNumber = this.isToNumber();
        var offset = (this.drawBetweenTick ? 0.5 : 0);
        // 标签跳跃个数
        var labelCrement = ((this.labelNumber > 0) ?
            this.labelNumber : tickSamplingTime * this.getLabelSamplingTime(tickSamplingTime, isToNumber, 1, offset));
        for (var value = this.minValue < 0 ? 0 : this.minValue; // 这边是小于maxValue 因为要画最初的标签
             value < this.categoryLabelList.length && value < this.maxValue; value += labelCrement) {
            var labelString = this.getLabelString(value, isToNumber);
            FR.Chart.CategoryAxisGlyph.superclass.drawLabel.apply(this, [ctx, value, offset, labelString]);
        }
    },

    /**
     * 画刻度  CategoryAxis.drawTicks
     * @param ctx 画布.
     */
    drawTicks: function (ctx) {
        if (this.axisLength <= 15) {
            return;
        }
        var tickValue = this.getMaxTickValue();
        var tickSamplingTime = this.getTickSamplingTime();

        //主要刻度
        var tickLength = this.secondLabels && this.secondLabels.length === 0 ? FR.Chart.AxisGlyph.MAIN_TICK_LENGTH : this.axisLabelWidth;
        for (var value = this.getCrossValue() + tickSamplingTime; value <= tickValue; value += tickSamplingTime) {
            // 主刻度线
            var line = this.getTickLine(value, tickLength, this.tickMarkType);
            this.drawTickLine(ctx, line, FR.Chart.Constants.AxisTickWidth);
        }

        if (tickSamplingTime <= 1 && Math.abs(this.smallTickLength) > 0) {
            for (var value = this.getCrossValue() + this.smallTickLength; value <= tickValue; value += this.smallTickLength) {
                // 次要刻度线
                var line = this.getTickLine(value, FR.Chart.AxisGlyph.SEC_TICK_LENGTH, this.secTickMarkType);
                this.drawTickLine(ctx, line, FR.Chart.Constants.AxisTickWidth);
            }
        }
        this.drawTickLabel(ctx);
    },

    /**
     * 画刻度标签
     * @param ctx 画布.
     */
    drawTickLabel: function (ctx) {
        if (this.axisLength <= 15) {
            return;
        }
        var tickSamplingTime = this.getTickSamplingTime();
        var isToNumber = this.isToNumber();
        var offset = (this.drawBetweenTick ? 0.5 : 0);
        // 标签跳跃个数
        var labelCrement = ((this.labelNumber > 0) ?
            this.labelNumber : tickSamplingTime * this.getLabelSamplingTime(tickSamplingTime, isToNumber, 1, offset));
        if(this.isLabelWrap && labelCrement ===1){  //显示方式换行显示
            for (var value = this.getCrossValue() < 0 ? 0 : this.getCrossValue();
                 value < this.categoryLabelList.length && value < this.getArrowValue(); value += labelCrement) {
                var labelString = this.getLabelString(value, isToNumber);
                this.drawLabelWrapWhenNeed(ctx,value,offset,labelString);
            }
        }else{
            for (var value = this.getCrossValue() < 0 ? 0 : this.getCrossValue();
                 value < this.categoryLabelList.length && value < this.getArrowValue(); value += labelCrement) {
                var labelString = this.getLabelString(value, isToNumber);
                FR.Chart.CategoryAxisGlyph.superclass.drawLabel.apply(this, [ctx, value, offset, labelString]);
            }
        }
    },

    //画换行的坐标轴标签
    drawLabelWrapWhenNeed:function(ctx, value, offset, labelString) {
        if (!this.isShowAxisLabel) {
           return;
        }
        if(this.textAttr.alignText == FR.Chart.Style.HORIZONTALTEXT && this.textAttr.rotation ===0){
           this.drawTextHorizontalWrap(ctx,value,offset,labelString);
        }else if(this.textAttr.alignText == FR.Chart.Style.HORIZONTALTEXT && this.textAttr.rotation !==0){
           this.drawTextWithRotation4Wrap(ctx,value,offset,labelString);
        }else{
            //竖排文字
           this.drawTextVerticalWrap(ctx,value,offset,labelString);
        }
    },


    //画倾斜的文字
    drawTextWithRotation4Wrap:function(ctx, value, offset, labelString){
        var margin = FR.Chart.AxisGlyph.LINE_LABEL_GAP + this.getTickLengthShow();
        var normalPoint = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation("a", this.textAttr);
        var dim = FR.Chart.TextUtils.calculateTextDimensionWithRotation(labelString,this.textAttr);
        var maxWidth = this.shouldBeHeight() ? this.unitLength-normalPoint.width:this.axisLabelWidth-margin;
        if(dim.width <= maxWidth){
            var labelBounds = this.getLabelBounds(value, offset, labelString);
            this.drawLabelWithBounds(ctx, value, offset, labelString, labelBounds);
            return ;
        }
        var maxHeight = this.shouldBeHeight() ?this.axisLabelWidth : this.unitLength -this.TOP_AND_DOWN_GAP;
        var drawStrings = [];
        var isNotFullShow = [];
        var realDim = this.calculateDimUsed4Rotation(labelString,maxHeight,maxWidth,drawStrings,isNotFullShow);

        var centerPoint = this.getPoint2D(value+offset);
        var rotaAngle = this.textAttr.rotation;
        var toAngle = Math.PI / 180;
        var offY = FR.Chart.PaintUtils.isOffSetY4SomeFont(ctx, this.textAttr.frFont);
        var roValue = -rotaAngle * toAngle;
        if(this.position == FR.Chart.Constants.BOTTOM){
            ctx.translate(centerPoint.x, centerPoint.y+margin);
            if(rotaAngle<0){
                ctx.translate(-2,-3);
            }
        }else if(this.position == FR.Chart.Constants.TOP){
            ctx.translate(centerPoint.x, centerPoint.y- realDim.height)
            var angle = this.textAttr.rotation * Math.PI / 180;
            var heightRotation = realDim.width*  Math.abs(Math.sin(angle));
            ctx.translate(0, -heightRotation);
        }else if(this.position == FR.Chart.Constants.LEFT){
            ctx.translate(centerPoint.x-margin, centerPoint.y);
        } else{
            ctx.translate(centerPoint.x+margin+realDim.width, centerPoint.y);
        }
        ctx.rotate(roValue);

        if(this.shouldBeHeight()){
            if(this.textAttr.rotation > 0){
                for(var i = 0;i< drawStrings.length ;i++){
                    var label = drawStrings[i];
                    var size = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(label,this.textAttr);
                    ctx.fillText(label, -realDim.width, offY+margin);// 旋转字体的时候 画两遍. 能清楚一些
                    ctx.fillText(label, -realDim.width, offY+margin);
                    offY += size.height;
                }
            }else{
                for(var i = 0;i< drawStrings.length ;i++){
                    var label = drawStrings[i];
                    var size = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(label,this.textAttr);
                    ctx.fillText(label, margin, offY+margin);// 旋转字体的时候 画两遍. 能清楚一些
                    ctx.fillText(label, margin, offY+margin);
                    offY += size.height;
                }
            }
        }else{
            if(this.textAttr.rotation>0){
                for(var i = 0;i< drawStrings.length ;i++){
                    var label = drawStrings[i];
                    var size = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(label,this.textAttr);
                    ctx.fillText(label, - realDim.width, offY-realDim.height/2);// 旋转字体的时候 画两遍. 能清楚一些
                    ctx.fillText(label, - realDim.width,offY-realDim.height/2);
                    offY += size.height;
                }

            }else{
                for(var i = 0;i< drawStrings.length ;i++){
                    var label = drawStrings[i];
                    var size = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(label,this.textAttr);
                    ctx.fillText(label, - realDim.width, offY);// 旋转字体的时候 画两遍. 能清楚一些
                    ctx.fillText(label,- realDim.width, offY);
                    offY += size.height;
                }

            }
        }
        ctx.rotate(-roValue);
        if(this.position == FR.Chart.Constants.BOTTOM){
            ctx.translate(-centerPoint.x, -centerPoint.y-margin);
            if(rotaAngle<0){
                ctx.translate(2,3);
            }
        }else if(this.position == FR.Chart.Constants.TOP){
            ctx.translate(-centerPoint.x,-centerPoint.y+realDim.height);
            var angle = this.textAttr.rotation * Math.PI / 180;
            var heightRotation = realDim.width*  Math.abs(Math.sin(angle));
            ctx.translate(0, heightRotation);
        }else if(this.position == FR.Chart.Constants.LEFT){
            ctx.translate(-centerPoint.x+margin, -centerPoint.y);
        } else{
            ctx.translate(-centerPoint.x-margin-realDim.width, -centerPoint.y);
        }

        var startX =0;
        var startY = 0;
        var boundW = 0;
        var boundh =0;
        if(this.shouldBeHeight()){
            if(this.textAttr.rotation>0){
                startX = centerPoint.x - maxWidth/2;
                startY =this.position == FR.Chart.Constants.BOTTOM? centerPoint.y+margin:centerPoint.y-margin*2-realDim.height;
            }else{
                startX = centerPoint.x;
                startY =this.position == FR.Chart.Constants.BOTTOM?centerPoint.y+margin:centerPoint.y-margin*2-realDim.height;
            }
            boundW = maxWidth/2;
            boundh = maxHeight;
        }else{
            if(this.textAttr.rotation>0){
                startX =this.position == FR.Chart.Constants.LEFT? centerPoint.x - margin - maxWidth:centerPoint.x + margin;
                startY = centerPoint.y;
            }else{
                startX =this.position == FR.Chart.Constants.LEFT? centerPoint.x - margin - maxWidth:centerPoint.x + margin;
                startY = centerPoint.y-maxHeight/2;
            }
            boundW = maxWidth;
            boundh = maxHeight/2;
        }

        if(isNotFullShow.length>0){
            this.tooltipLabelBoundsList.push(new FR.Chart.Bounds(startX,startY,boundW,boundh));
            this.tooltipLabelList.push(labelString);
        }
    },


    //画正常文字
    drawTextHorizontalWrap:function(ctx, value, offset, labelString){
        var margin = FR.Chart.AxisGlyph.LINE_LABEL_GAP + this.getTickLengthShow();
        var normalPoint = FR.Chart.TextUtils.calculateTextDimensionWithRotation("a", this.textAttr);
        var dim = FR.Chart.TextUtils.calculateTextDimensionWithRotation(labelString,this.textAttr);
        var maxWidth = this.shouldBeHeight() ? this.unitLength-normalPoint.width:this.axisLabelWidth-margin;
        if(dim.width <= maxWidth){
            var labelBounds = this.getLabelBounds(value, offset, labelString);
            this.drawLabelWithBounds(ctx, value, offset, labelString, labelBounds);
            return ;
        }
        var maxHeight = this.shouldBeHeight() ?this.axisLabelWidth - margin : this.unitLength -this.TOP_AND_DOWN_GAP;
        var drawStrings = [];
        var isNotFullShow = [];
        var realHeightUsed = this.calculateHeightUsed4Horizontal(labelString,maxHeight,maxWidth,drawStrings,isNotFullShow);
        var startY;
        var centerPoint = this.getPoint2D(value+offset);
        if(!this.shouldBeHeight()){
            startY = centerPoint.y-realHeightUsed/2;
        } else{
            startY = this.position == FR.Chart.Constants.BOTTOM ? centerPoint.y+margin: centerPoint.y- margin - realHeightUsed;
        }

        for(var i = 0;i< drawStrings.length ;i++){
            var label = drawStrings[i];
            var size = FR.Chart.TextUtils.calculateTextDimensionWithRotation(label,this.textAttr);
            var x = this.shouldBeHeight() ? centerPoint.x - size.width/2 :
                (this.position == FR.Chart.Constants.LEFT?centerPoint.x -margin - size.width:centerPoint.x + margin);
            var tmpBounds = new FR.Chart.Bounds(x,startY,size.width,size.height);
            this.drawLabelWithBounds(ctx, value, offset, label, tmpBounds);
            if(isNotFullShow.length>0){
                this.tooltipLabelBoundsList.push(tmpBounds);
                this.tooltipLabelList.push(labelString);
            }
            startY +=size.height;
        }
    },

    //画竖排文字
    drawTextVerticalWrap:function(ctx, value, offset, labelString){
        var margin = FR.Chart.AxisGlyph.LINE_LABEL_GAP + this.getTickLengthShow();
        var dim = FR.Chart.TextUtils.calculateTextDimensionWithRotation(labelString,this.textAttr);
        var maxHeight = this.shouldBeHeight() ? this.axisLabelWidth - margin :this.unitLength-this.TOP_AND_DOWN_GAP;
        if(dim.height <= maxHeight){
            var labelBounds = this.getLabelBounds(value, offset, labelString);
            this.drawLabelWithBounds(ctx, value, offset, labelString, labelBounds);
            return ;
        }
        var normalPoint = FR.Chart.TextUtils.calculateTextDimensionWithRotation("a", this.textAttr);
        var maxWidth = this.shouldBeHeight() ?this.unitLength -normalPoint.width:this.axisLabelWidth - margin;
        var drawStrings = [];
        var isNotFullShow = [];
        var realWidthUsed = this.calculateWidthUsed4Vertical(labelString,maxHeight,maxWidth,drawStrings,isNotFullShow);
        var startPoint;
        var centerPoint = this.getPoint2D(value+offset);
        if(!this.shouldBeHeight()){
            startPoint = new FR.Chart.Point2D((this.position == FR.Chart.Constants.LEFT ?centerPoint.x-margin : centerPoint.x +margin + realWidthUsed),centerPoint.y-maxHeight/2);
        } else{
            startPoint = new FR.Chart.Point2D(centerPoint.x+realWidthUsed/2,this.position == FR.Chart.Constants.BOTTOM ?centerPoint.y+margin:centerPoint.y-margin-maxHeight);
        }

        var startX = startPoint.x;

        for(var i = 0;i< drawStrings.length ;i++){
            var label = drawStrings[i];
            var size = FR.Chart.TextUtils.calculateTextDimensionWithRotation(label,this.textAttr);
            var tmpBounds = new FR.Chart.Bounds(startX - size.width,startPoint.y,size.width,size.height);
            this.drawLabelWithBounds(ctx, value, offset, label, tmpBounds);
            if(isNotFullShow.length>0){
                this.tooltipLabelBoundsList.push(tmpBounds);
                this.tooltipLabelList.push(labelString);
            }
            startX -=size.width+this.TOP_AND_DOWN_GAP/2;
        }

    },

    calculateDimUsed4Rotation:function(subString, maxHeight, maxWidth ,drawStrings,isNotFullShow){
        var showWidth = 0;
        var showHeight = 0;
        var rotation = this.textAttr.rotation;
        var angle = rotation * Math.PI / 180;
        if(this.shouldBeHeight()){ //
            showWidth = 0.5 * maxWidth /Math.abs(Math.cos(angle));
            var tmpHeight = showWidth * Math.abs(Math.sin(angle));
            tmpHeight  = maxHeight - tmpHeight;
            showHeight = tmpHeight /Math.abs(Math.cos(angle));
        }else{
            showWidth  = 0.5 * maxHeight / Math.abs(Math.sin(angle));
            var way1 = showWidth * Math.abs(Math.cos(angle));
            way1 = maxWidth - way1;
            way1 = way1 / Math.abs(Math.sin(angle));
            way2 =0.5 * maxHeight /Math.abs(Math.cos(angle));
            showHeight = Math.min(way1,way2);
        }
        var heightUsed = this.calculateHeightUsed4Horizontal(subString,showHeight,showWidth,drawStrings,isNotFullShow);
        if(drawStrings.length === 1 && drawStrings[0] ===  this.ELLIPSIS){
            return FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(this.ELLIPSIS,this.textAttr);
        }
        return new FR.Chart.Dimension2D(showWidth,heightUsed);
    },

    calculateHeightUsed4Horizontal:function(subString, maxHeight, maxWidth ,drawStrings,isNotFullShow){
        var heightUsed = 0;
        var lastHeight = 0;
        do{
             for (var i = 1; i <= subString.length; i++) {
                 var size = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(subString.substring(0, i), this.textAttr);
                 var over = size.width>maxWidth || size.height>maxHeight;
                 if (i === 1 && over) {
                     subString = "";
                     isNotFullShow.push(true);
                     drawStrings.push(this.ELLIPSIS);
                     size = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(this.ELLIPSIS, this.textAttr);
                     heightUsed += size.height;
                     break;
                 }
                 if (size.width > maxWidth && i!==1) {
                     drawStrings.push(subString.substring(0,i-1));
                     subString = subString.substring(i - 1);
                     heightUsed += size.height;
                     lastHeight = size.height;
                     break;
                 }else if( i == subString.length){
                     drawStrings.push(subString);
                     subString = "";
                     heightUsed += size.height;
                     lastHeight = size.height;
                     break;
                 }
             }

        }while (heightUsed < maxHeight && !(subString == ""));

        if (heightUsed<=maxHeight) {
            return heightUsed;
        } else {
            drawStrings.pop();
            if(drawStrings.length === 0){
                drawStrings.push(this.ELLIPSIS);
            }else {
                var lastShowString = drawStrings[drawStrings.length-1];
                drawStrings.pop();
                drawStrings.push(this.calculateStrWithEllipse(maxWidth,lastShowString,true));
            }
            isNotFullShow.push(true);
            return heightUsed - lastHeight;
        }
    },


    calculateWidthUsed4Vertical:function(subString, maxHeight, maxWidth ,drawStrings,isNotFullShow){
        var widthUsed = -this.TOP_AND_DOWN_GAP / 2;
        var lastWidth = 0;
        do{
            widthUsed += this.TOP_AND_DOWN_GAP / 2;
             for (var i = 1; i <= subString.length; i++) {
                 var size = FR.Chart.TextUtils.calculateTextDimensionWithRotation(subString.substring(0, i), this.textAttr);
                 var over = size.width>maxWidth || size.height>maxHeight;
                 if (i === 1 && over) {
                     subString = "";
                     isNotFullShow.push(true);
                     drawStrings.push(this.ELLIPSIS);
                     size = FR.Chart.TextUtils.calculateTextDimensionWithRotation(this.ELLIPSIS, this.textAttr);
                     widthUsed += size.width;
                     break;
                 }
                 var height = size.height;
                 if (height > maxHeight && i!==1) {
                     drawStrings.push(subString.substring(0,i-1));
                     subString = subString.substring(i - 1);
                     widthUsed += size.width;
                     lastWidth = size.width;
                     break;
                 }else if( i == subString.length){
                     drawStrings.push(subString);
                     subString = "";
                     widthUsed += size.width;
                     lastWidth = size.width;
                     break;
                 }
             }

        }while (widthUsed < maxWidth && !(subString == ""));

        if (widthUsed <= maxWidth) {
            return widthUsed;
        } else {
            drawStrings.pop();
            if(drawStrings.length === 0){
                drawStrings.push(this.ELLIPSIS);
            }else {
                var lastShowString = drawStrings[drawStrings.length-1];
                drawStrings.pop();
                drawStrings.push(this.calculateStrWithEllipse(maxHeight,lastShowString,false));
            }
            isNotFullShow.push(true);
            return widthUsed - lastWidth - this.TOP_AND_DOWN_GAP / 2;
        }

    },


    calculateStrWithEllipse:function(maxLength,label,isHorizontal){
        for(var i =1;i<=label.length;i++){
            var tmpStr =label.substring(0,i)+this.ELLIPSIS;
            var tmpDim = isHorizontal ? FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(tmpStr,this.textAttr):
                FR.Chart.TextUtils.calculateTextDimensionWithRotation(tmpStr,this.textAttr);
            var length = isHorizontal? tmpDim.width:tmpDim.height;
            if(length > maxLength){
                if(i === 0){
                    return this.ELLIPSIS;
                }else{
                    return label.substring(0,i-1)+this.ELLIPSIS;
                }
            }
        }
        return label+this.ELLIPSIS;
    },


    /**
     * 返回实际的最小值.
     * @returns {number}
     */
    getOriginalCrossValue: function () {
        return this.minValue;
    },

    /**
     * 返回实际的最大值.
     * @returns {Number|*}
     */
    getOriginalArrowValue: function () {
        return this.maxValue;
    },

    /**
     * 返回 包括缩放在内的 最小值.
     * @returns {number}
     */
    getCrossValue: function () {
        return this.minValue - this.zoomBackward;
    },

    /**
     * 返回 包括缩放在内的 最大值.
     * @returns {*}
     */
    getArrowValue: function () {
        return this.maxValue + this.zoomForward;
    },

    //3D的分类轴不需要画tick
    draw4ThreeD: function (ctx) {
        this.drawTickLabel(ctx);
        this._drawAxisTitle(ctx);
    }
});

/**
 * 数值坐标轴 抽象类
 * @class FR.Chart.NumberAxisGlyph
 * @extends FR.Chart.AxisGlyph
 * @abstract 数值抽象类
 *
 * @param {JSON} [js] 配置属性
 * @param {String} [js.showUnit] 显示单位
 * @param {Boolean} [js.isPercentage = false] 是否为百分比坐标轴
 * @param {Boolean} [js.isLog = false] 是否为对数坐标轴
 * @param {Number} [js.baseLog = 10] 对数的基数
 * @param {Array} [js.alertValueList = []] 警戒线数组
 */
FR.Chart.NumberAxisGlyph = FR.extend(FR.Chart.AxisGlyph, /**@class FR.Chart.NumberAxisGlyph*/{
    initConstants: function (js) {
        js = js || {};

        this.showUnit = js.showUnit;
        this.unitPadding = js.unitPadding || "";
        this.isPercentage = js.isPercentage;

        this.isLog = js.isLog;
        this.baseLog = FR.pick(js.baseLog, 10);

        this.alertValueList = [];
        if (js.alertList) {
            for (var i = 0; i < js.alertList.length; i++) {
                this.alertValueList[i] = new FR.Chart.ChartAlertValue(js.alertList[i]);
            }
        }

        this.zoomTickLength = FR.pick(js.smallTickLength, 1);

        FR.Chart.NumberAxisGlyph.superclass.initConstants.apply(this, [js]);

        this.oldMin = null;
        this.oldMax = null;
        this.oldTick = null;
    },

    /**
     *  系列消失或出现变动, NumberAxisGlyph 值轴重新布局, 重新根据最大最小值计算 刻度间隔.
     * @param autoMin
     * @param autoMax
     */
    initMinMaxValue: function (autoMin, autoMax) {
        if (this.isCustomMinValue) {
            autoMin = this.minValue;
        } else {
            this.minValue = autoMin;
        }

        if (this.isCustomMaxValue) {
            autoMax = this.maxValue;
        } else {
            this.maxValue = autoMax;
        }

        // 计算大小, 刻度 取得合适的最大值
        if (this.isLog) {
            if (this.isCustomMainUnit) {
                // 自定义标签增量
                this.tickLength = this.tickLength;
            } else {
                this.tickLength = this.baseLog; //刻度
            }
            this.smallTickLength = this.tickLength / 5;  //小刻度
        } else if (autoMax - autoMin > 0) {
            if (this.isCustomMainUnit) {
                if (this.isPercentage) {
                    this.tickLength = this.tickLength;
                }
            } else {
                this.tickLength = FR.Chart.AxisGlyph.calculateIncrement(this.minValue, this.maxValue); //刻度
            }

            if (!this.isCustomSecUnit) {
                this.smallTickLength = this.tickLength / 5;
            }
        }

        if (this.isLog) {
            if (!this.isCustomMinValue) {
                if (this.minValue <= 1e-10) {
                    this.minValue = 1;
                } else if (this.minValue < 1) {
                    this.minValue = Math.pow(this.tickLength, Math.round(Math.log(this.minValue) / Math.log(this.tickLength) - 1));
                } else if (this.minValue >= 1) {
                    this.minValue = 1;
                }
            }

            if (!this.isCustomMaxValue) {
                if (this.maxValue <= 0) {
                    this.maxValue = 10;
                }
            }
        } else {
            var minT = parseInt(Math.abs(this.minValue) / this.tickLength);

            if (this.isCustomMinValue) {

            } else if (this.minValue < 0) {
                this.minValue = -(minT + 1) * this.tickLength;
            } else if (this.minValue > 0) {
                this.minValue = minT * this.tickLength;
            }

            var sizeT = parseInt((this.maxValue - this.minValue) / this.tickLength);
            var countValue = (this.maxValue - this.minValue) / this.tickLength;
            if (countValue == sizeT) {
                if (!this.isPercentage) {
                    sizeT += 1;
                }
            } else {
                sizeT += 1;
            }

            if (!this.isCustomMaxValue) {
                this.maxValue = this.minValue + sizeT * this.tickLength;
            }
        }

        // 这边处理 自定义的最小值 大于 从数据集中自动取出来的值autoMax情况.
        if (this.minValue >= this.maxValue) {
            if (this.tickLength > 0) {
                this.maxValue = this.minValue + this.tickLength * 3;
            } else {
                this.maxValue = this.minValue + 10;
                this.tickLength = 2;
                this.smallTickLength = this.tickLength / 5;
            }
        }
        this.zoomTickLength = this.smallTickLength;
    }
});
$.extend(FR.Chart.NumberAxisGlyph, /**@class FR.Chart.NumberAxisGlyph*/{
    /**
     * @static
     * @property 单位长度
     */
    UNIT_HEIGHT: 16
});

/**
 * 值轴坐标轴
 * @example
 * var valueaxis = new FR.Chart.ValueAxisGlyph();
 *
 * @class FR.Chart.ValueAxisGlyph
 * @extends FR.Chart.NumberAxisGlyph
 *
 * @param {JSON} js 配置属性
 * @param {Boolean} [js.alignZeroValue=false] 是否根据0值对齐
 */
FR.Chart.ValueAxisGlyph = FR.extend(FR.Chart.NumberAxisGlyph, /**@class FR.Chart.ValueAxisGlyph*/{

    /**
     * 根据js初始化所以属性
     * @param js 配置属性
     */
    initConstants: function (js) {
        js = js || {};

        this.alignZeroValue = js.alignZeroValue;

        this.markAxisType = "valueaxis";// 标记下类型, instanceof竟然没用.
        this.AUTO_WRAP_WIDTH = 0.12;
        FR.Chart.ValueAxisGlyph.superclass.initConstants.apply(this, [js]);
    },

    /**
     * 根据config初始化指定属性
     * @param config 指定属性.
     */
    initConfig: function (config) {
        this.minValue = config.minValue || 0;
        this.maxValue = config.maxValue || 0;
        this.tickLength = config.tickLength || 0;
        this.smallTickLength = this.tickLength / 5;
    },

    /**
     * 返回标签的最大宽度 并且初始化开始和结束标签的dim
     * @returns {Number} 返回标签对应的最大宽度.
     */
    getMaxLabelWidthAndInitStartEndLabelDim: function () {
        if (!this.isShowAxisLabel) {
            return 0;
        }

        var labelWidth = 8;
        var minValue = this.minValue;
        var maxValue = this.maxValue;
        var increment = 1;
        if (this.isCustomMainUnit) {
            increment = this.tickLength;
        } else {
            if (this.isLog) {
                increment = this.baseLog;
            } else {
                increment = FR.Chart.AxisGlyph.calculateIncrement(minValue, maxValue);
            }
        }

        if (increment <= 0) {
            return labelWidth;
        }

        // 考虑设置单位情况.
        var divide_unit = 1.0;
        var showUnit = this.showUnit;
        if (showUnit != null) {
            divide_unit = FR.Chart.ValueAxisGlyph.getDivideUnit(showUnit);
        }

        increment = this.checkIncrementNotTooSmall(increment, maxValue - minValue);

        for (var min = minValue; min <= maxValue; min = this.isLog ? min * increment : min + increment) {

            var dim = this.getAxisLabelDim(this.value2String(min / divide_unit));
            if (min == minValue) {
                this.startLabelDim = dim;
            } else {
                this.endLabelDim = dim;
            }

            var tmpWidth = this.shouldBeHeight() ? dim.height : dim.width;
            labelWidth = (tmpWidth < labelWidth ? labelWidth : tmpWidth );
        }
        var tickDim = this.getAxisLabelDim(this.value2String(this.tickLength));
        var tickWidth = this.shouldBeHeight() ? tickDim.height : tickDim.width;
        return Math.max(tickWidth, labelWidth);
    },

    /**
     * 检查增量 是否太小, 如果太小, 坐标轴显示不全. 占用性能 浪费 . 就增加增量.
     * @param increment 增量
     * @param allValue 总差值
     * @returns {*} 返回合适的增量
     */
    checkIncrementNotTooSmall: function (increment, allValue) {
        if (!this.isLog) {// 对数时 暂不做处理.
            var inValue = increment * 1000;
            if (inValue < allValue) {
                return this.getTickSamplingTime();
            }
        }

        return increment;
    },

    /**
     * 返回最大的标签宽度.
     * @returns {Number}  标签宽度.
     */
    getMaxLabelWidth: function () {// TODO 待删除
        if (!this.isShowAxisLabel) {
            return 0;
        }

        var labelWidth = 8;
        var minValue = this.minValue;
        var maxValue = this.maxValue;
        var increment = 1;
        if (this.isCustomMainUnit) {
            increment = this.tickLength;
        } else {
            if (this.isLog) {
                increment = this.baseLog;
            } else {
                increment = FR.Chart.AxisGlyph.calculateIncrement(minValue, maxValue);
            }
        }

        if (increment <= 0) {
            return labelWidth;
        }

        increment = this.checkIncrementNotTooSmall(increment, maxValue - minValue);

        for (var min = minValue; min <= maxValue; min = this.isLog ? min * increment : min + increment) {
            var dim = this.getAxisLabelDim(this.value2String(min));
            var tmpWidth = this.shouldBeHeight() ? dim.width : dim.height;
            labelWidth = (tmpWidth < labelWidth ? labelWidth : tmpWidth );
        }
        return labelWidth;
    },

    /**
     * 根据坐标轴标签 处理合适绘图区的边界.
     * @param plotBounds 绘图区的边界.
     */
    dealPlotBoundsWithAxisLabel: function (plotBounds) {
        FR.Chart.ValueAxisGlyph.superclass.dealPlotBoundsWithAxisLabel.apply(this, [plotBounds]);
        this.dealPlotBoundsWithAlert(plotBounds);
    },

    /**
     * 根据警戒线 处理合适绘图区的边界
     * @param plotBounds 绘图区的边界.
     */
    dealPlotBoundsWithAlert: function (plotBounds) {
        this.dealAlertLabelWidth();

        var x = plotBounds.x, y = plotBounds.y;

        var preLeftAlert = ((this.maxLeftAlert > 0) ? this.maxLeftAlert + FR.Chart.ValueAxisGlyph.ALERT_LABEL_LINE_GAP : 0);
        var preRightAlert = ((this.maxRightAlert > 0) ? this.maxRightAlert + FR.Chart.ValueAxisGlyph.ALERT_LABEL_LINE_GAP : 0);
        var preUnitLabel = ((this.showUnit || this.unitPadding) && this.isShowAxisLabel) ? this.getMaxLabelWidth() : 0;
        var preUnitPadding = this.getPreUnitPadding();
        preUnitLabel += preUnitPadding;

        var labelAttr = this.textAttr;
        var labelSize = 0;
        if(labelAttr && labelAttr.frFont){
            labelSize = labelAttr.frFont.size
        }

        if (this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) {
            plotBounds.x = x + preLeftAlert;
            plotBounds.width = plotBounds.width - preLeftAlert - preRightAlert;
            if(this.axisReversed){
                plotBounds.height = plotBounds.height - preUnitLabel - labelSize/2 - FR.Chart.ValueAxisGlyph.UNIT_GAP_LABEL;
            }
        } else {
            plotBounds.y = y + preRightAlert;
            plotBounds.height = plotBounds.height - preRightAlert - preLeftAlert;
            plotBounds.width -= preUnitLabel;
            if(this.axisReversed){
                plotBounds.x = x + preUnitLabel;
            }
        }
    },

    getPreUnitPadding:function(){

        var unitGlyph = new FR.Chart.TextGlyph();
        unitGlyph.text = this.unitPadding;
        var dim = unitGlyph.preferredDimension();

        return this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT ?
            dim.height : dim.width;
    },

    /**
     * 根据数据表  处理绘图区的边界
     * @returns {*} 返回影响绘图区的 数据表宽度.
     */
    getPreLeftWidth4DataSheet: function () {
        this.calculateTitleDimesion();
        this.dealAlertLabelWidth();

        if (this.position == FR.Chart.Constants.LEFT) {
            return this.getMaxLabelWidthAndInitStartEndLabelDim() + this.maxLeftAlert + this.titleDim.width + this.getTickLengthShow();
        } else if (this.position == FR.Chart.Constants.RIGHT) {
            return this.getTickLengthShow() + this.maxLeftAlert;
        } else {
            return 0;
        }
    },

    /**
     * 继续处理警戒线标签  分类两种: 左侧标签 右侧标签.
     *
     */
    dealAlertLabelWidth: function () {
        // 继续处理PlotBounds 考虑警戒线的宽度高度  还要区分左右中..
        this.maxLeftAlert = 0;
        this.maxRightAlert = 0;
        for (var i = 0; i < this.alertValueList.length; i++) {
            var chartAlertValue = this.alertValueList[i];
            var alertTextAttr = new FR.Chart.TextAttr();
            alertTextAttr.frFont = chartAlertValue.alertFont;

            var alertString = chartAlertValue.alertContent;

            var alertValue = chartAlertValue.formula;
            var valueString = "";
            if(alertValue != null){
                if (alertValue instanceof Array) {
                    for (var i = 0; i < alertValue.length; i++) {
                        if (alertValue[i] <= this.getArrowValue() && alertValue[i] >= this.getCrossValue()) {
                            valueString = "(" + this.value2String(alertValue[i]) + ")";
                        }
                    }
                } else {
                    if (alertValue <= this.getArrowValue() && alertValue >= this.getCrossValue()) {
                        valueString = "(" + this.value2String(alertValue) + ")";
                    }
                }
            }

            if ((this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) && this.plotZeroBounds != null) {
                alertString = FR.Chart.ChartUtils.dealLabelStringAutoWrap(alertString + valueString, alertTextAttr, this.plotZeroBounds.width * this.AUTO_WRAP_WIDTH);
            }

            var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(alertString, alertTextAttr);

            var alertLength = (this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) ?
                dim.width : dim.height;

            if (chartAlertValue.alertPosition == FR.Chart.Constants.LEFT) {
                this.maxLeftAlert = Math.max(alertLength, this.maxLeftAlert);
            } else {
                this.maxRightAlert = Math.max(alertLength, this.maxRightAlert);
            }
        }
    },

    /**
     * 计算坐标轴长度  等单位.
     */
    calculateAxisLengthUnit: function () {
        if (this.isLog) {
            this.unitLength = this.axisLength / (Math.log(this.maxValue) / Math.log(this.tickLength) -
                Math.log(this.minValue) / Math.log(this.tickLength));
        } else if (this.maxValue - this.minValue > 0) {
            this.unitLength = this.axisLength / (this.getArrowValue() - this.getCrossValue());
        }
    },

    /**
     * 返回坐标轴简化的次数. 避免太多的时候 诸葛探查.
     * @returns {Number} 标签简洁的 间隔次数..
     */
    getTickSamplingTime: function () {
        var count = 1;
        var tickNumber = (this.maxValue - this.minValue) / this.tickLength;

        if (this.isLog) {// 对数时 另外计算
            var mainValue = Math.abs(this.maxValue - this.minValue);
            var x = 0;
            while (Math.pow(this.tickLength, x) < mainValue) {
                if (Math.pow(this.tickLength, x + 1) > mainValue) {
                    tickNumber = x;
                    break;
                }
                x++;
            }
        }

        var maxCount = this.axisLength / 15;
        var useTick = Math.abs(tickNumber);
        while (maxCount < Math.ceil(useTick / count)) {
            count++;
            if (count > 1000) {
                count = parseInt(tickNumber / 20);
                break;
            }
        }
        return count;
    },

    /**
     * 得到坐标轴交叉点的值
     * @returns {Number} 值.
     */
    getCrossValue: function () {
        return this.minValue - this.zoomBackward;
    },

    /**
     * 返回坐标轴最大值.
     * @returns {*}
     */
    getArrowValue: function () {
        return this.maxValue + this.zoomForward;
    },

    /**
     * 返回实际的最小值
     * @returns {Number|*|number}
     */
    getOriginalCrossValue: function () {
        return this.minValue;
    },

    /**
     * 返回实际的最大值
     * @returns {Number}
     */
    getOriginalArrowValue: function () {
        return this.maxValue;
    },

    /**
     * 根据值 得到坐标轴上对应的坐标轴点.
     * @param value 坐标轴值
     * @returns {FR.Chart.Point2D} 返回对应的坐标轴点
     */
    getPointInBounds: function (value) {
        if (!(this.isLog && this.getCrossValue() > 0)) {
            if (value < this.getCrossValue()) {
                value = this.getCrossValue();
            }

            if (value > this.getArrowValue()) {
                value = this.getArrowValue();
            }
        }
        return this.getPoint2D(value);
    },

    /**
     * 这边是趋势线的用到的. 通过像素位置得到对应值轴值
     * @param plotValueY y像素点位置
     * @param plotValueX x像素点位置
     * @returns {string} 返回对应的值转为的格式字符串.
     */
    getPointValue4TrendLine: function (plotValueY, plotValueX) {
        var value = this.getCrossValue();
        if (this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) {
            if (plotValueY > this.bounds.height) {
                value = this.getCrossValue();
            }
            if (plotValueY < 0) {
                value = this.maxValue;
            }

            value = (this.originPoint.y - plotValueY) / this.unitLength;
            if (this.isLog && this.getCrossValue() > 0) {
                value = Math.pow(Math.E, value * Math.log(this.tickLength) + Math.log(this.getCrossValue()));
            }
        } else {
            if (plotValueX > this.bounds.width) {
                value = this.maxValue;
            }
            if (plotValueX < 0) {
                value = this.getCrossValue();
            }

            value = (plotValueX - this.originPoint.y) / this.unitLength;
            if (this.isLog && this.getCrossValue() > 0) {
                value = Math.pow(Math.E, value * Math.log(this.tickLength) + Math.log(this.getCrossValue()));
            }
        }
        var string = this.value2String(value);
        if (this.format == null) {// 默认保留2位
            string = FR.contentFormat(value, '#.##');
        }

        return string;
    },

    /**
     * 根据值返回坐标点
     * @param value 轴值
     * @returns {FR.Chart.Point2D}
     */
    getPoint2D: function (value) {
        // 指数型坐标
        if (this.isLog && this.getCrossValue() > 0) {
            if (value <= 1e-10) {
                value = this.getCrossValue();
            }
            value = (Math.log(value) - Math.log((this.isDraw4Demo ? this.minValue : this.getCrossValue()))) / Math.log(this.tickLength);
        }
        // default
        else {
            value = value - (this.isDraw4Demo ? this.minValue : this.getCrossValue());
        }

        if (this.originPoint == null) {
            return new FR.Chart.Point2D(0, 0);
        }

        var detX = this.axisReversed ? -this.unitLength * value : this.unitLength * value;
        var detY = this.axisReversed ? this.unitLength * value : -this.unitLength * value;
        if (this.position == FR.Chart.Constants.LEFT
            || this.position == FR.Chart.Constants.RIGHT) {
            return new FR.Chart.Point2D(this.originPoint.x, this.originPoint.y + detY);
        } else {
            return new FR.Chart.Point2D(this.originPoint.x + detX, this.originPoint.y);
        }
    },

    /**
     * 画出网格线
     * @param ctx 画布
     */
    drawAxisGrid: function (ctx) {
        if (this.axisLength <= 15) {
            return;
        }
        var tickSamplingTime = this.getTickSamplingTime();

        if(!ctx){     return; } ctx.save();
        // 主网格线.
        if (this.mainGridStyle != FR.Chart.Constants.LINE_NONE && this.tickLength > 0
            && this.mainGridColor != null) {
            ctx.lineWidth = FR.Chart.GraphHelper.getLineStyleSize(this.mainGridStyle);
            ctx.strokeStyle = this.mainGridColor;

            if (this.isLog) {
                if (this.tickLength > 1.0) {
                    for (var value = this.getCrossValue(), i = Math.log(value) / Math.log(this.tickLength);
                         value <= this.getArrowValue(); value = Math.exp((++i) * Math.log(this.tickLength))) {

                        var lines = this.getGridLine(value);
                        for (var k = 0; k < lines.length; k++) {
                            FR.Chart.GraphHelper.dealLine4JSDraw(lines[k]);
                            lines[k].draw(ctx);
                        }
                    }
                }
            } else {
                for (var value = this.getCrossValue() * tickSamplingTime;
                     value <= this.getArrowValue(); value += this.tickLength * tickSamplingTime) {

                    var lines = this.getGridLine(value);
                    for (var i = 0; i < lines.length; i++) {
                        FR.Chart.GraphHelper.dealLine4JSDraw(lines[i]);
                        lines[i].draw(ctx);
                    }
                }
            }
        }
        ctx.restore();

    },
    /**
     * 在Plot的上层 画警戒线
     * @param  ctx 画布
     */
    drawAlertValueLines: function (ctx) {
        if (this.alertValueList) {
            if(!ctx){     return; } ctx.save();
            // 传过来的都是最终的值, 没有公式了.
            for (var alertIndex = 0; alertIndex < this.alertValueList.length; alertIndex++) {
                var chartAlert = this.alertValueList[alertIndex];

                ctx.globalAlpha = chartAlert.alertLineAlpha;
                ctx.lineWidth = FR.Chart.GraphHelper.getLineStyleSize(chartAlert.attrLineStyle.lineStyle);

                if (chartAlert.attrLineColor.seriesColor == null) {
                    continue;
                }
                ctx.strokeStyle = chartAlert.attrLineColor.seriesColor;

                var alertValue = chartAlert.formula;
                if(alertValue != null){
                    if (alertValue instanceof Array) {
                        for (var i = 0; i < alertValue.length; i++) {
                            if (alertValue[i] <= this.getArrowValue() && alertValue[i] >= this.getCrossValue()) {

                                var lines = this.getGridLine(alertValue[i]);
                                for (var j = 0; j < lines.length; j++) {
                                    FR.Chart.GraphHelper.dealLine4JSDraw(lines[j]);
                                    lines[j].draw(ctx);
                                }
                            }
                        }
                    } else {
                        if (alertValue <= this.getArrowValue() && alertValue >= this.getCrossValue()) {
                            var lines = this.getGridLine(alertValue);
                            for (var i = 0; i < lines.length; i++) {
                                FR.Chart.GraphHelper.dealLine4JSDraw(lines[i]);
                                lines[i].draw(ctx);
                            }
                        }
                    }
                }
            }
            ctx.restore();
        }
    },

    /**
     * 画坐标轴本身 和 刻度.. value
     */
    drawAxisLine: function (ctx) {
        if(!ctx){     return; } ctx.save();

        if (this.minValue == this.maxValue) {
            return;
        }

        if (this.lineColor && this.lineStyle != FR.Chart.Constants.LINE_NONE) {
            ctx.lineWidth = FR.Chart.GraphHelper.getLineStyleSize(this.lineStyle);
            ctx.strokeStyle = this.lineColor;
            var line = new FR.Chart.Line2D(this.getPoint2D(this.minValue), this.getPoint2D(this.maxValue));
            FR.Chart.GraphHelper.dealLine4JSDraw(line);

            line.draw(ctx);
            this.drawArrow(ctx, line.startPoint, line.endPoint);
        }
        this.drawTicks(ctx);
        ctx.restore();
    },

    /**
     * 画出刻度. ValueAxisGlyph.drawTicks
     * @param ctx
     */
    drawTicks: function (ctx) {
        if (this.axisLength <= 15) {
            return;
        }
        var tickSamplingTime = this.getTickSamplingTime();
        if (this.isLog) {
            if (this.tickLength > 1.0) {
                for (var value = this.getCrossValue(), i = Math.log(value) / Math.log(this.tickLength);
                     value <= this.getArrowValue(); value = Math.exp((++i) * Math.log(this.tickLength))) {
                    // 主要刻度线
                    var line = this.getTickLine(value, FR.Chart.AxisGlyph.MAIN_TICK_LENGTH, this.tickMarkType);
                    this.drawTickLine(ctx, line, FR.Chart.Constants.AxisTickWidth);
                }
            }
            // 对数 : 如果刻度小于1, 将会是负数的话  有可能会溢出导致死机.
            if (this.smallTickLength > 1.0) {
                var value = this.getCrossValue(), i = 0;
                for (var k = 0; value <= this.getArrowValue(); value += i, k++) {
                    // 跳过和主刻度相同的值
                    if (k % 5 === 0) {
                        i = (Math.pow(this.tickLength, Math.log(value) / Math.log(this.tickLength) + 1) - value) / 5;
                        continue;
                    }
                    // 次要刻度线
                    var line = this.getTickLine(value, FR.Chart.AxisGlyph.SEC_TICK_LENGTH, this.secTickMarkType);
                    this.drawTickLine(ctx, line, FR.Chart.Constants.AxisTickWidth);
                }
            }

        } else if (this.getArrowValue() > this.getCrossValue() && this.tickLength > 0 && this.smallTickLength > 0) {
            var tmp = this.tickLength * tickSamplingTime;
            for (var value = this.getCrossValue() + tmp; value <= this.getArrowValue(); value += tmp) {
                // 主要刻度线
                var line = this.getTickLine(value, FR.Chart.AxisGlyph.MAIN_TICK_LENGTH, this.tickMarkType);
                this.drawTickLine(ctx, line, FR.Chart.Constants.AxisTickWidth);
            }
            if (tickSamplingTime === 1) {
                for (var value = this.getCrossValue() + this.smallTickLength; value <= this.getArrowValue(); value += this.smallTickLength) {
                    // 次要刻度线
                    var line = this.getTickLine(value, FR.Chart.AxisGlyph.SEC_TICK_LENGTH, this.secTickMarkType);
                    this.drawTickLine(ctx, line, FR.Chart.Constants.AxisTickWidth);
                }
            }
        }

        this.drawTickLabel(ctx);
        this.drawAlertLinesLabel(ctx);
    },

    /**
     * 画刻度标签
     * @param ctx 刻度.
     */
    drawTickLabel: function (ctx) {
        if (this.axisLength <= 15) {
            return;
        }

        var divide_unit = 1.0;
        var tickSamplingTime = this.getTickSamplingTime();
        // showUnit处理
        var showUnit = this.showUnit;
        if (showUnit != null) {
            divide_unit = FR.Chart.ValueAxisGlyph.getDivideUnit(showUnit);
        }

        this.drawUnitGlyph(ctx, showUnit);

        var labelCrement = (this.labelNumber > 0 ? this.labelNumber : tickSamplingTime);
        var labelBoundsManager = new FR.Chart.BoundsManager();
        if (this.isLog) {
            if (this.tickLength > 1) {
                for (var value = this.getCrossValue(); FR.Chart.ChartUtils.compareDouble(value, this.getArrowValue(),FR.Chart.Compare.LESS_THAN_OR_EQUAL); value *= Math.pow(this.tickLength, labelCrement)) {
                    var unitValue = value / divide_unit;
                    this.drawLabel(ctx, value, 0, this.value2String(unitValue));
                }
            }
        } else if (this.getArrowValue() > this.getCrossValue() && this.tickLength > 0 && this.smallTickLength > 0) {
            for (var value = this.getCrossValue(); FR.Chart.ChartUtils.compareDouble(value, this.getArrowValue(),FR.Chart.Compare.LESS_THAN_OR_EQUAL); value += (this.tickLength * labelCrement)) {
                var unitValue = value / divide_unit;
                this.drawLabelWithPath(ctx, value, 0, this.value2String(unitValue), labelBoundsManager);
            }
        }
    },

    /**
     * 画单位对应的glyph
     * @param ctx 画布
     * @param showUnit 显示的单位
     */
    drawUnitGlyph: function (ctx, showUnit) {
        showUnit = showUnit ? FR.Chart.ValueAxisGlyph.getUnitKye2Value(showUnit) : '';

        var unitGlyph = new FR.Chart.TextGlyph();
        unitGlyph.text = showUnit + this.unitPadding;

        var dim = unitGlyph.preferredDimension();
        var labelWidth = this.getMaxLabelWidth();

        var labelAttr = this.textAttr;
        var labelSize = 0;
        if(labelAttr && labelAttr.frFont){
            labelSize = labelAttr.frFont.size;
        }

        var chartBounds = null;
        if (this.position == FR.Chart.Constants.TOP) {
            if(this.axisReversed){
                chartBounds = new FR.Chart.Bounds(-dim.width-FR.Chart.ValueAxisGlyph.UNIT_GAP_LABEL-labelWidth/2, dim.height, dim.width,dim.height);
            }else{
                chartBounds = new FR.Chart.Bounds(this.bounds.width + labelWidth + FR.Chart.ValueAxisGlyph.UNIT_GAP_LABEL,
                    +dim.height,
                    dim.width, dim.height);
            }

        } else if (this.position == FR.Chart.Constants.LEFT) {
            if(this.axisReversed){
                chartBounds = new FR.Chart.Bounds(0, this.bounds.height + FR.Chart.ValueAxisGlyph.UNIT_GAP_LABEL + labelSize / 2, dim.width, dim.height);
            }else{
                chartBounds = new FR.Chart.Bounds(0, -dim.height - FR.Chart.ValueAxisGlyph.UNIT_GAP_LABEL - labelSize / 2, dim.width, dim.height);
            }
        } else if (this.position == FR.Chart.Constants.BOTTOM) {
            if(this.axisReversed){
                chartBounds = new FR.Chart.Bounds(-dim.width-FR.Chart.ValueAxisGlyph.UNIT_GAP_LABEL - labelWidth/2, -dim.height /2, dim.width, dim.height);
            }else{
                chartBounds = new FR.Chart.Bounds(this.bounds.width + labelWidth / 2 + FR.Chart.ValueAxisGlyph.UNIT_GAP_LABEL,
                        -dim.height/2, dim.width, dim.height);
            }
        } else if (this.position == FR.Chart.Constants.RIGHT) {
            if(this.axisReversed){
                chartBounds = new FR.Chart.Bounds(labelWidth - dim.width,  this.bounds.height + FR.Chart.ValueAxisGlyph.UNIT_GAP_LABEL + labelSize / 2,
                    dim.width, dim.height);
            }else{
                chartBounds = new FR.Chart.Bounds(labelWidth - dim.width, -dim.height - FR.Chart.ValueAxisGlyph.UNIT_GAP_LABEL - labelSize / 2,
                    dim.width, dim.height);
            }
        }

        var axisBounds = this.getBounds();
        chartBounds.x += axisBounds.x;
        chartBounds.y += axisBounds.y;

        unitGlyph.bounds = chartBounds;
        unitGlyph.draw(ctx);
    },

    /**
     * 画警戒线线的标签
     * @param ctx 画布
     */
    drawAlertLinesLabel: function (ctx) {
        if (this.alertValueList == null) {
            return;
        }

        var alertOrderList = [];

        // 传过来的都是最终的值, 没有公式了.
        for (var alertIndex = 0; alertIndex < this.alertValueList.length; alertIndex++) {
            var chartAlert = this.alertValueList[alertIndex];

            var alertValue = chartAlert.formula;
            if (alertValue != null) {
                if (alertValue instanceof Array) {
                    for (var i = 0; i < alertValue.length; i++) {
                        if (alertValue[i] >= this.minValue && alertValue[i] <= this.maxValue) {
                            this.addAlertValueInOrder(alertOrderList, alertValue[i], chartAlert);
                        }
                    }
                } else {
                    if (alertValue >= this.minValue && alertValue <= this.maxValue) {
                        this.addAlertValueInOrder(alertOrderList, alertValue, chartAlert);
                    }
                }
            }
        }

        var rightLabelPath = new FR.Chart.BoundsManager();
        var leftLabelPath = new FR.Chart.BoundsManager();
        for (var alertIndex = 0; alertIndex < alertOrderList.length; alertIndex++) {
            var alertIndexList = alertOrderList[alertIndex];
            var alertIndexValue = alertIndexList[0];
            var chartAlertValue = alertIndexList[1];

            var alertValuePath = chartAlert.alertPosition == FR.Chart.Constants.LEFT ? leftLabelPath : rightLabelPath;
            this.drawAlertValueLabel(ctx, alertIndexValue, chartAlertValue, alertValuePath);
        }
    },

    /**
     * 按照顺序添加警戒线
     * @param alertOrderList 警戒线的已有顺序
     * @param alertValue 警戒值
     * @param chartAlertValue 图表警戒值.
     */
    addAlertValueInOrder: function (alertOrderList, alertValue, chartAlertValue) {
        var valueList = [];
        valueList[0] = alertValue;
        valueList[1] = chartAlertValue;

        for (var alertIndex = 0; alertIndex < alertOrderList.length; alertIndex++) {
            var alertIndexList = alertOrderList[alertIndex];
            var alertIndexValue = alertIndexList[0];
            if (alertIndexValue > alertValue) {
                for (var index = alertOrderList.length; index > alertIndex; index--) {
                    alertOrderList[index] = alertOrderList[index - 1];
                }
                alertOrderList[alertIndex] = valueList;
                return;
            }
        }

        alertOrderList[alertOrderList.length] = valueList;
    },

    /**
     *  画出警戒值的标签
     * @param ctx 画布
     * @param value 警戒值
     * @param chartAlert 图表警戒值
     * @param alertValuePath 警戒值已占用路径
     */
    drawAlertValueLabel: function (ctx, value, chartAlert, alertValuePath) {
        var centerP = this.getPoint2D(value);

        var oldColor = chartAlert.alertFont.color;
        var textAttr = new FR.Chart.TextAttr();

        if (chartAlert.attrLineColor.seriesColor != null) {
            chartAlert.alertFont.color = chartAlert.attrLineColor.seriesColor;
        }
        textAttr.frFont = chartAlert.alertFont;

        var alertString = chartAlert.alertContent;

        if(alertString == ""){
            //如果警戒内容设置成空""，则不显示value。如果设置成" ",则显示value。
            return;
        }

        alertString = alertString + "(" + this.value2String(value) + ")";
        if ((this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) && this.plotZeroBounds != null) {
            alertString = FR.Chart.ChartUtils.dealLabelStringAutoWrap(alertString, textAttr, this.plotZeroBounds.width * this.AUTO_WRAP_WIDTH);
        }
        var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(alertString, textAttr);
        FR.Chart.TextUtils.drawLabelWithRotation(ctx, alertString, textAttr, this.getAlertLabelBounds(centerP, dim, alertValuePath, chartAlert.alertPosition));

        chartAlert.alertFont.color = oldColor;
    },

    /**
     * 根据已有位置 等信息 返回合适的警戒值 位置
     * @param centerP 中心点位置
     * @param dim 警戒值占用的大小
     * @param alertValuePath 警戒值已占用的路径
     * @param alertPosition 警戒值的位置
     * @returns {Bounds} 返回边界.
     */
    getAlertLabelBounds: function (centerP, dim, alertValuePath, alertPosition) {
        var x = 0, y = 0;

        var alertBounds = this._getAlertBounds(alertPosition);

        if (this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) {
            x = alertBounds.x;
            y = centerP.y - dim.height / 2;
        } else {
            x = centerP.x - dim.width / 2;
            y = alertBounds.y;
        }

        var labelBounds = new FR.Chart.Bounds(x, y, dim.width, dim.height);
        return this.dealIntersectsBounds(alertBounds, alertValuePath, labelBounds);
    },

    /**
     * 处理相关 重叠的标签问题.
     * @param alertBounds 警戒值边界
     * @param alertValuePath 警戒值已占用的路径
     * @param resultLabelBounds 预先结果边界
     * @returns {*} 返回最终结果 resultLabelBounds
     */
    dealIntersectsBounds: function (alertBounds, alertValuePath, resultLabelBounds) {

        //默认从左往右移动,从下往上移动
        var moveInOrder = true;
        if(this.position == FR.Chart.Constants.TOP || this.position == FR.Chart.Constants.BOTTOM){
            if(resultLabelBounds.x + resultLabelBounds.width > alertBounds.x + alertBounds.width){
                moveInOrder = false;
            }
        }

        var reachEnd = false;

        if(!alertValuePath.isEmpty()){
            while(alertValuePath.isInteract(resultLabelBounds) && !reachEnd){
                reachEnd = this._moveResultBounds(resultLabelBounds, moveInOrder);
            }
        }else{
            while(!alertBounds.contains(resultLabelBounds.x, resultLabelBounds.y, resultLabelBounds.width, resultLabelBounds.height) && !reachEnd){
                reachEnd = this._moveResultBounds(resultLabelBounds, moveInOrder);
            }
        }

        if (!alertBounds.contains(resultLabelBounds.x, resultLabelBounds.y, resultLabelBounds.width, resultLabelBounds.height)) {
            return null;
        } else {
            alertValuePath.addBounds(resultLabelBounds);
        }

        return resultLabelBounds;
    },

    _moveResultBounds: function (resultLabelBounds, moveInOrder) {
        if (this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) { // 4像素逐步移动
            resultLabelBounds.y = resultLabelBounds.y - 4;
            if(this.bounds.y > resultLabelBounds.y){
                return true;
            }
        } else {
            if(moveInOrder){
                resultLabelBounds.x += 4;
                if(this.bounds.x + this.bounds.width < resultLabelBounds.x){
                    return true;
                }
            }else{
                resultLabelBounds.x -= 4;
                if(this.bounds.x > resultLabelBounds.x){
                    return true;
                }
            }
        }

        return false;
    },

    _getAlertBounds: function (alertPosition) {
        var PRE_GAP_ALERT = 20;

        var preTitleWidth = (this.titleDim.width === 0) ? 0 : this.titleDim.width + FR.Chart.AxisGlyph.LINE_LABEL_GAP;
        var preTitleHeight = (this.titleDim.height === 0) ? 0 : this.titleDim.height + FR.Chart.AxisGlyph.LINE_LABEL_GAP;

        var axisLabelTitleWidth = preTitleWidth + this.axisLabelWidth;
        var axisLabelTitleHeight = preTitleHeight + this.axisLabelWidth;

        if (this.position == FR.Chart.Constants.LEFT) {
            return this._getAlertBoundsWhenAtLeft(alertPosition, axisLabelTitleWidth, PRE_GAP_ALERT);
        } else if (this.position == FR.Chart.Constants.TOP) {
            return this._getAlertBoundsWhenAtTop(alertPosition, axisLabelTitleHeight, PRE_GAP_ALERT);
        } else if (this.position == FR.Chart.Constants.RIGHT) {
            return this._getAlertBoundsWhenAtRight(alertPosition, axisLabelTitleWidth, PRE_GAP_ALERT);
        } else {
            return this._getAlertBoundsWhenOther(alertPosition, axisLabelTitleHeight, PRE_GAP_ALERT);
        }
    },

    _getAlertBoundsWhenAtLeft: function (alertPosition, axisLabelTitleWidth, PRE_GAP_ALERT) {
        var x;
        if (alertPosition == FR.Chart.Constants.LEFT) {
            x = this.originPoint.x - this.maxLeftAlert - FR.Chart.ValueAxisGlyph.ALERT_LABEL_LINE_GAP - axisLabelTitleWidth;
        } else {
            x = this.originPoint.x + this.plotZeroBounds.width - this.plotLastBounds.x + this.plotZeroBounds.x
                - this.maxRightAlert;
        }
        return new FR.Chart.Bounds(x, this.bounds.y-PRE_GAP_ALERT,
            alertPosition == FR.Chart.Constants.LEFT ? this.maxLeftAlert : this.maxRightAlert, this.bounds.height + PRE_GAP_ALERT);
    },

    _getAlertBoundsWhenAtTop: function (alertPosition, axisLabelTitleHeight, PRE_GAP_ALERT) {
        var y;
        if (alertPosition == FR.Chart.Constants.LEFT) {
            y = this.originPoint.y;
        } else {
            y = this.originPoint.y - this.maxRightAlert - FR.Chart.ValueAxisGlyph.ALERT_LABEL_LINE_GAP - axisLabelTitleHeight;
        }
        return new FR.Chart.Bounds(this.bounds.x-this.maxLeftAlert, y, this.bounds.width + this.maxLeftAlert + this.maxRightAlert,
            alertPosition == FR.Chart.Constants.LEFT ? this.maxLeftAlert : this.maxRightAlert);
    },

    _getAlertBoundsWhenAtRight: function (alertPosition, axisLabelTitleWidth, PRE_GAP_ALERT) {
        var x;
        if (alertPosition == FR.Chart.Constants.LEFT) {
            x = this.plotZeroBounds.x;
        } else {
            x = this.originPoint.x + axisLabelTitleWidth + FR.Chart.ValueAxisGlyph.ALERT_LABEL_LINE_GAP;
        }
        return new FR.Chart.Bounds(x, this.bounds.y-PRE_GAP_ALERT, alertPosition == FR.Chart.Constants.LEFT ? this.maxLeftAlert : this.maxRightAlert,
            this.bounds.height + PRE_GAP_ALERT);
    },

    _getAlertBoundsWhenOther: function (alertPosition, axisLabelTitleHeight, PRE_GAP_ALERT) {
        var y;
        if (alertPosition == FR.Chart.Constants.LEFT) {
            y = this.originPoint.y + FR.Chart.ValueAxisGlyph.ALERT_LABEL_LINE_GAP + axisLabelTitleHeight;
        } else {
            y = this.originPoint.y - this.axisGridLength - this.maxRightAlert - FR.Chart.ValueAxisGlyph.ALERT_LABEL_LINE_GAP;
        }
        return new FR.Chart.Bounds(this.bounds.x-this.maxLeftAlert, y, this.bounds.width + this.maxLeftAlert + this.maxRightAlert,
            alertPosition == FR.Chart.Constants.LEFT ? this.maxLeftAlert : this.maxRightAlert);
    },

    _drawValueAxisBackground: function (ctx) {
        if(!ctx){     return; } ctx.save();

        ctx.globalAlpha = 0.18;
        if (this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) {
            var grd = ctx.createLinearGradient(this.bounds.x, this.bounds.y + this.bounds.height,
                this.bounds.x, this.bounds.y);
            grd.addColorStop(0, 'rgb(145,151,156)');
            grd.addColorStop(1, 'rgb(198,205,210)');
            ctx.fillStyle = grd;
            this.bounds.paint(ctx);
        } else if (this.position == FR.Chart.Constants.BOTTOM) {
            var grd = ctx.createLinearGradient(this.bounds.x, this.bounds.y,
                    this.bounds.x + this.bounds.width, this.bounds.y);
            grd.addColorStop(0, 'rgb(145,151,156)');
            grd.addColorStop(1, 'rgb(198,205,210)');
            ctx.fillStyle = grd;
            this.bounds.paint(ctx);
        }

        ctx.restore();
    },

    /**
     * 3D的值轴需要一个渐变的背景
     * @param ctx 画布.
     */
    draw4ThreeD: function (ctx) {
        this._drawValueAxisBackground(ctx);

        this.drawTickLabel(ctx);

        this._drawAxisTitle(ctx);
    },

    /**
     * 重写一下这个方法，3D的值轴底座大小不一样
     * @param plotBounds 绘图区
     * @returns {*} 返回标签宽度.
     */
    getCateLabelWidth: function (plotBounds) {
        if (this.cubic) {
            var cateLabelWidth = this.getMaxLabelWidthAndInitStartEndLabelDim();
            cateLabelWidth += 2 * FR.Chart.ValueAxisGlyph.LABEL_GAP;
            cateLabelWidth = Math.max(FR.Chart.ValueAxisGlyph.DEFAULT_WIDTH, cateLabelWidth);
            if (this.shouldBeHeight()) {
                return Math.min(plotBounds.height * FR.Chart.ValueAxisGlyph.MAX_LABEL_PERCENT, cateLabelWidth);
            } else {
                return Math.min(plotBounds.width * FR.Chart.ValueAxisGlyph.MAX_LABEL_PERCENT, cateLabelWidth);
            }
        } else {
            return FR.Chart.ValueAxisGlyph.superclass.getCateLabelWidth.apply(this, [plotBounds]);
        }
    }


});
$.extend(FR.Chart.ValueAxisGlyph, /**@class FR.Chart.ValueAxisGlyph*/{
    UNIT_STRING: [
        'DataFunction-None',
        'Unit_Hundred',
        'Unit_Thousand',
        'Unit_Ten_Thousand',
        'Unit_Hundred_Thousand',
        'Unit_Million',
        'Unit_Ten_Million',
        'Unit_Hundred_Million',
        'Unit_Billion'
    ],

    UNIT_STRING_VALUES: [
        FR.i18nText('Fine-Dec_Chart_DataFunction_None'),
        FR.i18nText('Fine-Dec_Chart_Unit_Hundred'),
        FR.i18nText('Fine-Dec_Chart_Unit_Thousand'),
        FR.i18nText('Fine-Dec_Chart_Unit_Ten_Thousand'),
        FR.i18nText('Fine-Dec_Chart_Unit_Hundred_Thousand'),
        FR.i18nText('Fine-Dec_Chart_Unit_Million'),
        FR.i18nText('Fine-Dec_Chart_Unit_Ten_Million'),
        FR.i18nText('Fine-Dec_Chart_Unit_Hundred_Million'),
        FR.i18nText('Fine-Dec_Chart_Unit_Billion')
    ],

    unit: null,

    UNIT_GAP_LABEL: 3,
    ALERT_LABEL_LINE_GAP: 2,
    DEFAULT_WIDTH: 30,
    MAX_LABEL_PERCENT: 0.28,
    LABEL_GAP: 8,

    getUnitKye2Value: function (unitKey) {
        if (!FR.Chart.ValueAxisGlyph.unitKey2Value) {
            FR.Chart.ValueAxisGlyph.unitKey2Value = new FR.Chart.HashMap();

            for (var i = 1; i < FR.Chart.ValueAxisGlyph.UNIT_STRING.length; i++) {
                FR.Chart.ValueAxisGlyph.unitKey2Value.put(
                    FR.Chart.ValueAxisGlyph.UNIT_STRING[i], FR.Chart.ValueAxisGlyph.UNIT_STRING_VALUES[i]
                );
            }
        }

        if (FR.Chart.ValueAxisGlyph.unit.containsKey(unitKey)) {
            return FR.Chart.ValueAxisGlyph.unitKey2Value.get(unitKey);
        }

        return FR.i18nText('Fine-Dec_Chart_DataFunction_None');
    },

    getDivideUnit: function (str) {
        if (!FR.Chart.ValueAxisGlyph.unit) {
            FR.Chart.ValueAxisGlyph.unit = new FR.Chart.HashMap();
            FR.Chart.ValueAxisGlyph.unit.put(FR.Chart.ValueAxisGlyph.UNIT_STRING[0], 1);
            var divide = 100;
            for (var i = 1; i < FR.Chart.ValueAxisGlyph.UNIT_STRING.length; i++) {
                FR.Chart.ValueAxisGlyph.unit.put(FR.Chart.ValueAxisGlyph.UNIT_STRING[i], divide);
                divide *= 10;
            }

            FR.Chart.ValueAxisGlyph.unit.put("10000", 10000);
            FR.Chart.ValueAxisGlyph.unit.put("100000", 100000);
            FR.Chart.ValueAxisGlyph.unit.put("10000000", 10000000);
            FR.Chart.ValueAxisGlyph.unit.put("100000000", 100000000);
        }

        if (FR.Chart.ValueAxisGlyph.unit.containsKey(str)) {
            return FR.Chart.ValueAxisGlyph.unit.get(str);
        }

        return 1;
    }
});

/**
 * 雷达图
 *
 *      @example
 *      var axisGlyph = new FR.Chart.RadarAxisGlyph();
 *
 * @class FR.Chart.RadarAxisGlyph 雷达图
 * @extends FR.Chart.NumberAxisGlyph
 * @param {JSON} js 配置属性
 * @param {Number} [js.categoryCount = 0] 分类个数.
 * @param {Array} [js.labelList = []] 分类标签数组
 * @param {Array} [js.radarMin = []] 自定义最小值数组
 * @param {Array} [js.radarMax = []] 自定义最大值数组
 * @param {Boolean} [js.isAllMaxMin = true] 默认统一计算大小
 */
FR.Chart.RadarAxisGlyph = FR.extend(FR.Chart.NumberAxisGlyph, /**@class FR.Chart.RadarAxisGlyph*/{
    initConstants: function (js) {
        js = js || {};

        this.categoryCount = FR.pick(js.categoryCount, 0);

        this.labelList = [];
        if (js.labelList && js.labelList.length > 0) {
            for (var i = 0; i < js.labelList.length; i++) {
                this.labelList.push(js.labelList[i]);
            }
        }

        this.radarMin = [];
        if (js.radarMin) {
            for (var i = 0; i < js.radarMin.length; i++) {
                this.radarMin[i] = js.radarMin[i];
            }
        }

        this.radarMax = [];
        if (js.radarMax) {
            for (var i = 0; i < js.radarMax.length; i++) {
                this.radarMax[i] = js.radarMax[i];
            }
        }

        this.isAllMaxMin = FR.pick(js.isAllMaxMin, true);

        FR.Chart.RadarAxisGlyph.superclass.initConstants.apply(this, [js]);

        this.D = 0;
        this.radian = 0;
    },

    configLabels:function(cateLabels){
        this.labelList = cateLabels || [];
        this.categoryCount = cateLabels.length;
    },

    /**
     * 根据边界大小初始化 相关单位长度等.
     * @param plotBounds 绘图区边界.
     */
    init: function (plotBounds) {
        this.bounds = plotBounds;
        // 原点
        this.originPoint = new FR.Chart.Point2D(plotBounds.x + plotBounds.width / 2, plotBounds.y + plotBounds.height / 2);
        // 直径
        this.D = Math.min(plotBounds.width, plotBounds.height);
        // 角度
        this.radian = Math.PI * 2 / this.categoryCount;

        this.axisLength = this.D / 2;
    },

    /**
     * 返回标签宽度 并且初始化开始和结束的标签占用大小
     * @returns {Number} 返回标签宽度.
     */
    getMaxLabelWidthAndInitStartEndLabelDim: function () {
        if (!this.isShowAxisLabel) {
            return 0;
        }

        var labelWidth = 8;
        for (var i = 0; i < this.categoryCount; i++) {
            var cateMin = this._getCateMin(i);
            var cateMax = this._getCateMax(i);

            var cateTickLength = FR.Chart.AxisGlyph.calculateIncrement(cateMin, cateMax);

            if (cateTickLength <= 0) {
                continue;
            }

            for (var value = cateMin; value <= cateMax; value += cateTickLength) {
                var tmpWidth = this.getAxisLabelWidth(this.value2String(value));

                labelWidth = (tmpWidth < labelWidth ? labelWidth : tmpWidth );
            }
        }

        return labelWidth;
    },

    /**
     * radar 根据标签的宽度 调整绘图区的宽度
     * @param plotBounds 绘图区边界
     */
    dealPlotBoundsWithAxisLabel: function (plotBounds) {
        var maxLeft = 8;
        var maxRight = 8;
        var labelLength = this.labelList.length;
        for (var i = 0; i < labelLength; i++) {
            var label = this.labelList[i];
            var labelWidth = FR.Chart.TextUtils.calculateTextDimensionWithRotation(label, new FR.Chart.TextAttr()).width;
            if (i < labelLength / 2) {
                maxLeft = Math.max(labelWidth, maxLeft);
            } else {
                maxRight = Math.max(labelWidth, maxRight);
            }
        }

        var tickLabel = this.getMaxLabelWidthAndInitStartEndLabelDim();
        var height = 12;
        maxLeft = Math.max(maxLeft, 12);
        maxRight = Math.max(maxRight, 12);

        if (this.textAttr.alignText == FR.Chart.Style.VERTICALTEXT) {
            height = Math.max(12, tickLabel);
        }

        plotBounds.x = plotBounds.x + maxLeft;
        plotBounds.y = plotBounds.y + height;
        plotBounds.width = plotBounds.width - maxLeft - maxRight;
        plotBounds.height = plotBounds.height - height*3;
    },

    /**
     * 为数据表返回预定义的 左侧宽度.  没有数据表.
     * @returns {number} 左侧宽度   0
     */
    getPreLeftWidth4DataSheet: function () {
        return 0
    },

    /**
     * 计算坐标轴单位长度.
     */
    calculateAxisLengthUnit: function () {
        this.unitLength = this.axisLength / (this.getArrowValue() - this.getCrossValue());
    },

    /**
     * 返回最小的分类值.
     * @param categoryIndex 分类间隔
     * @returns {*}
     */
    _getCateMin: function (categoryIndex) {
        if (categoryIndex >= 0 && categoryIndex < this.radarMin.length) {
            return this.radarMin[categoryIndex];
        }
        return this.getCrossValue();
    },

    /**
     * 返回最大的分类值.
     * @param categoryIndex 分类间隔.
     * @returns {*}
     */
    _getCateMax: function (categoryIndex) {
        if (categoryIndex >= 0 && categoryIndex < this.radarMax.length) {
            return this.radarMax[categoryIndex];
        }
        return this.maxValue;
    },

    /**
     * 根绝值 分类序号 返回对应轴上的位置
     * @param categoryIndex 分类序号
     * @param value 值
     * @returns {point2d} 返回坐标轴上的点
     */
    getPointInBounds: function (categoryIndex, value) {
        var cateMin = this._getCateMin(categoryIndex);
        var cateMax = this._getCateMax(categoryIndex);

        if (value < cateMin) {
            value = cateMin;
        }
        if (value > cateMax) {
            value = cateMax;
        }
        return this.getPoint2D(categoryIndex, value);
    },

    getPoint2D: function (categoryIndex, value) {
        var cateMin = this._getCateMin(categoryIndex);
        var cateMax = this._getCateMax(categoryIndex);

        var cateValue = value - cateMin;
        var cateUnitLength = this.axisLength / (cateMax - cateMin);

        return new FR.Chart.Point2D(
            this.originPoint.x + Math.cos(this.radian * categoryIndex + Math.PI / 2) * cateValue * cateUnitLength,
            this.originPoint.y - Math.sin(this.radian * categoryIndex + Math.PI / 2) * cateValue * cateUnitLength
        );
    },

    /**
     * 网格线
     */
    drawAxisGrid: function (ctx) {
        if (!this.isAllMaxMin) {
            return;
        }
        if(!ctx){     return; } ctx.save();
        // 主网格线
        if (this.mainGridStyle != FR.Chart.Constants.LINE_NONE && this.tickLength > 0
            && this.mainGridColor != null) {

            ctx.lineWidth = FR.Chart.GraphHelper.getLineStyleSize(this.mainGridStyle);
            ctx.strokeStyle = this.mainGridColor;

            for (var value = this.getCrossValue(); value <= this.maxValue; value += this.tickLength) {
                var line = this.getGridLine(value);
                for (var i = 0; i < line.length; i++) {
                    FR.Chart.GraphHelper.dealLine4JSDraw(line[i]);
                    line[i].draw(ctx);
                }
            }
        }
        ctx.restore();
    },

    /**
     * 返回网格线
     * @param value 值
     * @returns {Array} 网格线数组.
     */
    getGridLine: function (value) {
        var axisLines = [];
        for (var i = 0; i < this.categoryCount; i++) {
            axisLines[i] = new FR.Chart.Line2D(this.getPoint2D(i, value), this.getPoint2D(i + 1, value));
        }
        return axisLines;
    },

    /**
     * 雷达轴 先不做js精确处理 画出坐标轴线.
     * @param ctx 画布
     */
    drawAxisLine: function (ctx) {
        if(!ctx){     return; } ctx.save();

        var labelBoundsPath = new FR.Chart.BoundsManager();
        if (this.lineColor && this.lineStyle != FR.Chart.Constants.LINE_NONE) {
            ctx.lineWidth = FR.Chart.GraphHelper.getLineStyleSize(this.lineStyle);
            ctx.strokeStyle = this.lineColor;

            // 一个Category对应一条线
            var lines = this._getAxisLines();
            for (var i = 0; i < lines.length; i++) {
                var cline = lines[i];
                FR.Chart.GraphHelper.dealLine4JSDraw(cline);

                cline.draw(ctx);

                this.drawArrow(ctx, cline.startPoint, cline.endPoint);

                if (this.isArrowShow) {
                    this.drawCateLabel(ctx, i, cline.endPoint, 20, labelBoundsPath);
                } else {
                    this.drawCateLabel(ctx, i, cline.endPoint, 0, labelBoundsPath);
                }
            }
        }

        // 画刻度
        this.drawTicks(ctx, labelBoundsPath);
        ctx.restore();
    },

    /**
     * 画出雷达背景
     * @param ctx 画布
     * @param color 背景颜色
     */
    paintRadarBackground: function (ctx, color) {
        if (this.isAllMaxMin) {
            if(!ctx){     return; } ctx.save();

            if (this.tickLength > 0 && color != null) {

                ctx.fillStyle = color;

                for (var value = this.getCrossValue(); value <= this.maxValue - this.tickLength; value += this.tickLength * 2) {
                    // canvas的环形路径  直接到内部 填充了.
                    this.getRadarPath(value, this.tickLength, ctx);
                }
            }

            ctx.restore();
        }
    },

    /**
     * 获取背景的对应路径
     * @param value 值
     * @param length 长度
     * @param ctx 画布
     */
    getRadarPath: function (value, length, ctx) {
        for (var i = 0; i < this.categoryCount; i++) {
            if (i == this.categoryCount - 1) {
                ctx.beginPath();
                ctx.moveTo(this.getPoint2D(i, value).x, this.getPoint2D(i, value).y);
                ctx.lineTo(this.getPoint2D(0, value).x, this.getPoint2D(0, value).y);
                ctx.lineTo(this.getPoint2D(0, value + length).x, this.getPoint2D(0, value + length).y);
                ctx.lineTo(this.getPoint2D(i, value + length).x, this.getPoint2D(i, value + length).y);
                ctx.fill();
            } else {
                ctx.beginPath();
                ctx.moveTo(this.getPoint2D(i, value).x, this.getPoint2D(i, value).y);
                ctx.lineTo(this.getPoint2D(i + 1, value).x, this.getPoint2D(i + 1, value).y);
                ctx.lineTo(this.getPoint2D(i + 1, value + length).x, this.getPoint2D(i + 1, value + length).y);
                ctx.lineTo(this.getPoint2D(i, value + length).x, this.getPoint2D(i, value + length).y);
                ctx.fill();
            }
        }
    },

    /**
     * 画出刻度  radarAxisGlyph.drawticks
     * @param ctx 画布
     * @param labelBoundsPath 标签的占用路径.
     */
    drawTicks: function (ctx, labelBoundsPath) {
        for (var i = 0; i < this.categoryCount; i++) {
            // 标签跳跃个数
            var labelCrement = 0;
            if (this.labelNumber === -1 || this.labelNumber === 0) {
                labelCrement = 1;
            } else {
                labelCrement = this.labelNumber;
            }

            var cateMin = this._getCateMin(i);
            var cateMax = this._getCateMax(i);

            var cateTickLength = this.isCustomMainUnit === true ? this.tickLength : FR.Chart.AxisGlyph.calculateIncrement(cateMin, cateMax);
            var smallCateTickLength = this.isCustomSecUnit === true ? this.smallTickLength : cateTickLength / 5;

            var divide_unit = 1.0;

            var showUnit = this.showUnit;
            if (showUnit || this.unitPadding) {
                divide_unit = FR.Chart.ValueAxisGlyph.getDivideUnit(showUnit);
                var unitGlyph = new FR.Chart.TextGlyph();

                var showText = showUnit ? FR.i18nText('Fine-Dec_Chart_Use_Unit') + ":" + FR.Chart.ValueAxisGlyph.getUnitKye2Value(showUnit) : '';
                showText += this.unitPadding;
                unitGlyph.text = showText;

                var dim = unitGlyph.preferredDimension();
                var chartBounds = new FR.Chart.Bounds(this.bounds.x, this.bounds.y, dim.width, dim.height);

                unitGlyph.bounds = chartBounds;
                unitGlyph.draw(ctx);
            }

            for (var value = cateMin; value <= cateMax; value += cateTickLength * labelCrement) {
                if (i === 0 || !this.isAllMaxMin) {
                    this.drawTickLine(ctx, this.getTickLine(i, value, FR.Chart.AxisGlyph.MAIN_TICK_LENGTH, this.tickMarkType),
                        FR.Chart.Constants.AxisTickWidth);

                    this.drawFirstAxisLabel(ctx, i, value, divide_unit, labelBoundsPath);
                }
            }

            // 次刻度线
            for (var value = cateMin; value <= cateMax; value += smallCateTickLength) {
                if ((i === 0 || !this.isAllMaxMin)) {
                    this.drawTickLine(ctx, this.getTickLine(i, value, FR.Chart.AxisGlyph.SEC_TICK_LENGTH, this.secTickMarkType),
                        FR.Chart.Constants.AxisTickWidth);
                }
            }
        }
    },

    /**
     * 获取刻度线
     * @param categoryIndex 分类序号
     * @param value 值
     * @param tickLength 刻度长度
     * @param tickMarkType 刻度类型
     * @returns {FR.Chart.Line2D}
     */
    getTickLine: function (categoryIndex, value, tickLength, tickMarkType) {
        var centerP = this.getPoint2D(categoryIndex, value);

        var leftP = new FR.Chart.Point2D(
            centerP.x - tickLength / 2.0 * Math.cos(categoryIndex * this.radian),
            centerP.y + tickLength / 2.0 * Math.sin(categoryIndex * this.radian));

        var rightP = new FR.Chart.Point2D(centerP.x + tickLength / 2.0 * Math.cos(categoryIndex * this.radian),
            centerP.y - tickLength / 2.0 * Math.sin(categoryIndex * this.radian));

        if (tickMarkType == FR.Chart.Constants.TICK_MARK_CROSS) {
            return new FR.Chart.Line2D(leftP, rightP);
        } else if (tickMarkType == FR.Chart.Constants.TICK_MARK_INSIDE) {
            return new FR.Chart.Line2D(leftP, centerP);
        } else if (tickMarkType == FR.Chart.Constants.TICK_MARK_OUTSIDE) {
            return new FR.Chart.Line2D(centerP, rightP);
        } else {
            return new FR.Chart.Line2D(centerP, centerP);
        }
    },

    /**
     * 画出第一个分类的坐标轴标签
     * @param ctx 画布
     * @param categoryIndex 分类序号
     * @param value 值
     * @param divide_unit 单位间隔
     * @param labelBoundsPath 标签占用路径
     */
    drawFirstAxisLabel: function (ctx, categoryIndex, value, divide_unit, labelBoundsPath) {
        if (!this.isShowAxisLabel) {
            return;
        }
        // 取点按照 0---刻度长
        var centerP = this.getPoint2D(categoryIndex, value);
        // label 按照 最小到最大..
        var text = this.value2String(value / divide_unit);

        var textAttr = this.textAttr;
        if (textAttr == null) {
            textAttr = new FR.Chart.TextAttr();
        }

        var dim = FR.Chart.TextUtils.calculateTextDimensionWithRotation(text, textAttr);
        var margin = FR.Chart.AxisGlyph.LINE_LABEL_GAP;

        var labelBounds = new FR.Chart.Bounds(
            centerP.x - dim.width - margin,
            centerP.y - dim.height / 2,
            dim.width, dim.height);
        if (this.labelNumber !== 0) {
            FR.Chart.TextUtils.drawLabelWithRotation(ctx, text, textAttr, labelBounds);
        } else if (!labelBoundsPath.isInteract(labelBounds)) {
            labelBoundsPath.addBounds(labelBounds);
            FR.Chart.TextUtils.drawLabelWithRotation(ctx, text, textAttr, labelBounds);
        }
    },

    /**
     * 画雷达图的 分类标签
     * @param p1 坐标轴箭头的起始点
     * @param length 从坐标轴箭头的起点越过箭头的长度 20
     */
    drawCateLabel: function (ctx, cateNumber, p1, leng, labelBoundsPath) {
        var text = this.labelList[cateNumber].toString();

        var newPoint = new FR.Chart.Point2D(
            p1.x + Math.cos(this.radian * cateNumber + Math.PI / 2) * leng,
            p1.y - Math.sin(this.radian * cateNumber + Math.PI / 2) * leng
        );

        var leftOffset = 2;
        if (!this.isAllMaxMin) {
            leftOffset = 4 + FR.Chart.TextUtils.calculateTextDimensionWithRotation(this.getRealMaxValueText(cateNumber), new FR.Chart.TextAttr()).width;
        }

        var dim = FR.Chart.TextUtils.calculateTextDimensionWithRotation(text, new FR.Chart.TextAttr());

        var valueAngle = (cateNumber * 360 / this.categoryCount + 90) % 360;

        var x = newPoint.x;
        var y = newPoint.y;
        if (valueAngle > 90 && valueAngle < 270) {
            x = x - leftOffset - dim.width;
        }
        else if ((valueAngle > 0 && valueAngle < 90) || (valueAngle > 270 && valueAngle < 360)) {
            y = y - dim.height / 2;
        } else if (valueAngle === 90) {
            x = x - dim.width / 2;
            y = y - dim.height * 1.5;
        } else if (valueAngle === 270) {
            x = x - dim.width / 2;
        }

        var labelBounds = new FR.Chart.Bounds(x, y, dim.width, dim.height);
        if (!labelBoundsPath.isInteract(labelBounds)) {
            labelBoundsPath.addBounds(labelBounds);
            FR.Chart.TextUtils.drawLabelWithRotation(ctx, text, new FR.Chart.TextAttr(), labelBounds);
        }
    },

    getRealMaxValueText: function(cateNumber) {
        // 标签跳跃个数
        var labelCrement = this.labelNumber > 0 ? this.labelNumber : 1;

        var cateMin = this._getCateMin(cateNumber);
        var cateMax = this._getCateMax(cateNumber);

        var cateTickLength = this.isCustomMainUnit === true ? this.tickLength : FR.Chart.AxisGlyph.calculateIncrement(cateMin, cateMax);
        var divide_unit = 1;
        var showUnit = this.showUnit;
        if (showUnit != null) {
            divide_unit = FR.Chart.ValueAxisGlyph.getDivideUnit(showUnit);
        }

        var realValue = cateMin;
        while(realValue <= cateMax){
            realValue += cateTickLength * labelCrement
        }

        return this.value2String(realValue / divide_unit);
    },

    _getAxisLines: function () {
        var axis_lines = [];

        for (var i = 0; i < this.categoryCount; i++) {
            axis_lines[i] = new FR.Chart.Line2D(
                this.getPoint2D(i, this._getCateMin(i)), this.getPoint2D(i, this._getCateMax(i))
            );
        }

        return axis_lines;
    },

    /**
     * 返回坐标轴对应的Shape
     * @returns {FR.Chart.GeneralPath}
     */
    getShape: function () {
        var path = new FR.Chart.GeneralPath();
        var lines = this._getAxisLines();
        for (var i = 0; i < lines.length; i++) {
            path.append(lines[i], false);
        }

        return path;
    },

    /**
     * 返回最小值
     * @returns {*}
     */
    getCrossValue: function () {
        return this.minValue;
    },

    /**
     * 返回最大值
     * @returns {*}
     */
    getArrowValue: function () {
        return this.maxValue;
    },

    /**
     * 文本是否使用高度. 根据文本格式判断.
     * @returns {boolean}
     */
    shouldBeHeight: function () {
        if (this.textAttr == null) {
            this.textAttr = new FR.Chart.TextAttr();
        }
        return this.textAttr.alignText == FR.Chart.Style.VERTICALTEXT;
    },

    /**
     * 没有标题.
     * @param ctx 画布.
     */
    drawAxisTitle: function (ctx) {

    }
});

/**
 * 全距图坐标轴
 *    @example
 *    var range = new FR.Chart.RangeAxisGlyph();
 *
 * @class FR.Chart.RangeAxisGlyph
 * @extends FR.Chart.ValueAxisGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {Color} [js.leftColor = 'red] 默认左侧 红色
 * @param {Color} [js.rightColor = 'green'] 默认右侧 绿色
 *
 */
FR.Chart.RangeAxisGlyph = FR.extend(FR.Chart.ValueAxisGlyph, /**@class FR.Chart.RangeAxisGlyph*/{
    initConstants: function (js) {
        js = js || {};

        this.leftColor = FR.pick(js.leftColor, 'rgb(215, 214, 214)');
        this.rightColor = FR.pick(js.rightColor, 'rgb(240, 239, 239)');

        FR.Chart.RangeAxisGlyph.superclass.initConstants.apply(this, [js]);
    },

    /**
     * 根据坐标轴 调整绘图区的大小
     * @param plotBounds 绘图区边界大小.
     */
    dealPlotBoundsWithAxisLabel: function (plotBounds) {
        FR.Chart.RangeAxisGlyph.superclass.dealAlertLabelWidth.apply(this);
    },

    /**
     * 初始化原点 坐标轴长度 等
     * @param axisBounds 坐标轴边界
     * @param axisGridLength 坐标轴长度.
     */
    init: function (axisBounds, axisGridLength) {
        this.bounds = axisBounds;

        this.originPoint = new FR.Chart.Point2D(axisBounds.x, axisBounds.y + axisBounds.height / 2);
        this.axisLength = axisBounds.width;
        this.axisGridLength = axisGridLength;
    },

    /**
     * 画坐标轴网格线
     * @param ctx 画布
     */
    drawAxisGrid: function (ctx) {
        ctx.translate(this.getBounds().x, this.getBounds().y);

        FR.Chart.RangeAxisGlyph.superclass.drawAxisGrid.apply(this, [ctx]);

        this._drawAxisGround(ctx);

        ctx.translate(-this.getBounds().x, -this.getBounds().y)
    },

    _drawAxisGround: function (ctx) {
        if(!ctx){     return; } ctx.save();

        var linegrad = ctx.createLinearGradient(0, 0, this.axisLength, 0);
        linegrad.addColorStop(0, this.leftColor);
        linegrad.addColorStop(1, this.rightColor);

        ctx.fillStyle = linegrad;
        ctx.fillRect(0, 0, this.axisLength, this.bounds.height);

        ctx.restore();
    },

    /**
     * 返回网格线
     * @param value 轴值
     * @returns {*[]} 返回网格线数组
     */
    getGridLine: function (value) {
        var centerP = this.getPoint2D(value);
        // 全距图 坐标轴只留一种情况.
        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(centerP.x, 0 - this.bounds.y),
            new FR.Chart.Point2D(centerP.x, this.plotZeroBounds.height - this.bounds.y));
        return [line];
    },

    /**
     * 画全距图坐标轴的标签, RangeAxisGlyph.drawLabel
     **/
    drawLabel: function (ctx, value, offset, str) {
        if (!this.isShowAxisLabel) {
            return;
        }

        var centerP = this.getPoint2D(value + offset);
        var textAttr = this.textAttr;
        if (textAttr == null) {
            textAttr = new FR.Chart.TextAttr();
        }
        var dim = FR.Chart.TextUtils.calculateTextDimensionWithRotation(str, textAttr);
        // 这边处理了显示标签的位置
        var bounds = new FR.Chart.Bounds(centerP.x - dim.width / 2, centerP.y - this.bounds.height * 0.5 - dim.height - 16, dim.width, dim.height);

        var noRoDim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(str, textAttr);
        bounds.width = noRoDim.width;
        bounds.height = noRoDim.height;
        FR.Chart.TextUtils.drawLabelWithRotation(ctx, str, textAttr, bounds);
    },

    /**
     * 返回 系列点值 对应的标签边界.
     * @param value
     * @param offset
     * @param labelString
     * @param centerP
     * @returns {FR.Chart.Bounds}
     */
    getLabelBounds4Point: function (value, offset, labelString, centerP) {
        var textAttr = this.textAttr;
        if (textAttr == null) {
            textAttr = new FR.Chart.TextAttr();
        }

        var dim = FR.Chart.TextUtils.calculateTextDimensionWithRotation(labelString, textAttr);
        var labelBounds = new FR.Chart.Bounds(centerP.x - dim.width / 2,
                centerP.y - this.bounds.height * 0.5 - dim.height - 12, dim.width, dim.height);


        return labelBounds;
    },

    /**
     * 返回警戒标签的边界
     * @param centerP 中心的点
     * @param dim 占用大小
     * @param alertValuePath 警戒线的占用路径
     * @param alertPosition 警戒位置.
     * @returns {*} 返回标签边界.
     */
    getAlertLabelBounds: function (centerP, dim, alertValuePath, alertPosition) {
        var bounds = null;

        var alertBounds = null;

        if (alertPosition == FR.Chart.Constants.LEFT) {
            alertBounds = new FR.Chart.Bounds(0, this.plotZeroBounds.height - this.bounds.y - this.maxLeftAlert,
                this.bounds.width, this.maxLeftAlert);
            bounds = new FR.Chart.Bounds(centerP.x, this.plotZeroBounds.height - this.bounds.y - this.maxLeftAlert, dim.width, dim.height);
        } else {
            alertBounds = new FR.Chart.Bounds(0, 0 - this.bounds.y, this.bounds.width, this.maxRightAlert);
            bounds = new FR.Chart.Bounds(centerP.x, 0 - this.bounds.y, dim.width, dim.height);
        }

        return FR.Chart.RangeAxisGlyph.superclass.dealIntersectsBounds.apply(this, [alertBounds, alertValuePath, bounds]);
    }

});

/**
 * 日期坐标轴
 *
 *      @example
 *      var dateaxis = new FR.Chart.DateAxisGlyph();
 *
 * @class FR.Chart.DateAxisGlyph 日期坐标轴
 * @extends FR.Chart.CategoryAxisGlyph
 *
 * @param {JSON} js 配置属性
 * @param {Number} [js.mainType = FR.Chart.Constants.DAY] 默认主要日期刻度类型 天.
 * @param {Number} [js.secondType = FR.Chart.Constants.DAY] 默认次要日期刻度类型 天.
 */
FR.Chart.DateAxisGlyph = FR.extend(FR.Chart.CategoryAxisGlyph, /**@class FR.Chart.DateAxisGlyph*/{
    initConstants: function (js) {
        js = js || {};

        this.mainType = FR.pick(js.mainType, FR.Chart.Constants.DAY);
        this.secondType = FR.pick(js.secondType, FR.Chart.Constants.DAY);

        FR.Chart.DateAxisGlyph.superclass.initConstants.apply(this, [js]);
        this.zoomTickLength = this.tickLength;

        this.jsType = 'DateAxis';
    },

    initMinMaxValue:function(autoMin, autoMax){
        this.minValue = FR.Chart.ChartUtils.int2Date(autoMin, FR.Chart.Constants.Second);
        this.maxValue = FR.Chart.ChartUtils.int2Date(autoMax, FR.Chart.Constants.Second);
        this.minValue = FR.Chart.ChartUtils.date2Int(this.minValue, this.mainType);
        this.maxValue = FR.Chart.ChartUtils.date2Int(this.maxValue, this.mainType);

        if(!this.isCustomMainUnit){
            this.tickLength = 5;
        }

        if(!this.isCustomSecUnit){
            this.smallTickLength = 1;
        }
    },

    /**
     * 返回 最大的刻度标签长度, 并且初始化开始和结束的标签大小.
     * @returns {Number} 返回最大标签长度.
     */
    getMaxLabelWidthAndInitStartEndLabelDim: function () {
        if (this.isShowAxisLabel) {
            var labelWidth = 8;
            var mainDate = FR.Chart.ChartUtils.int2Date(this.minValue, this.mainType);

            var tickSamplingTime = this.getTickNumber4CalDim();
            var labelCrement = ((this.labelNumber > 0) ? this.labelNumber : tickSamplingTime);

            var first = 0;
            while (FR.Chart.ChartUtils.date2Int(mainDate, this.mainType) <= this.maxValue && parseInt(this.tickLength * labelCrement) > 0) {
                var valueString = "";
                if (this.format != null) {
                    valueString = FR.contentFormat(mainDate, this.format);
                } else {// 注意下格式. 和java是不同的.
                    valueString = FR.contentFormat(mainDate, "Dyyyy-MM-dd");
                }

                var dim = this.getAxisLabelDim(valueString);
                if (first === 0) {
                    this.startLabelDim = dim;
                    first++;
                } else {
                    this.endLabelDim = dim;
                }

                var tmpWidth = this.shouldBeHeight() ? dim.height : dim.width;
                labelWidth = (tmpWidth < labelWidth ? labelWidth : tmpWidth );

                FR.Chart.DateAxisGlyph.nextDate(mainDate, this.mainType, parseInt(this.tickLength * labelCrement));
            }

            return labelWidth;
        }

        return 0;
    },

    /**
     * 返回 日期的计算 数目 只是大致计算. 所以不是很精确.
     * @returns {number} 返回日期 计算个数.
     */
    getDateTypeNumber: function () {
        var dateTypeNumber = 1;
        if (this.mainType == FR.Chart.Constants.MONTH) {
            dateTypeNumber = 30;
        } else if (this.mainType == FR.Chart.Constants.YEAR) {
            dateTypeNumber = 365;
        }
        return dateTypeNumber;
    },

    /**
     * 计算坐标轴 单位长度.
     */
    calculateAxisLengthUnit: function () {
        this.unitLength = this.axisLength / (this.getArrowValue() - this.getCrossValue() + this.tickNumberChangeUponLabelLocation());
        this.unitLengthNoScale = this.axisLength / (this.maxValue - this.minValue + this.tickNumberChangeUponLabelLocation());
    },

    tickNumberChangeUponLabelLocation : function() {
        return this.drawBetweenTick ? 0 : -1;
    },

    /**
     * 返回值 对应的刻度.
     * @param value 目标值
     * @returns {Number} 返回目标值对应的日期 转化的值.
     */
    getTickIndex4Value: function (value) {
        var date = FR.Chart.ChartUtils.object2Date(this.categoryLabelList[value]);
        return FR.Chart.ChartUtils.date2Int(date, this.mainType);
    },


    /**
     * 得到标签的日期值
     */
    getDate4Value:function(value){
        return FR.Chart.ChartUtils.object2Date(this.categoryLabelList[value]);
    },


    /**
     * 画坐标轴网格线.
     * @param ctx 画布
     */
    drawAxisGrid: function (ctx) {
        // 主网格线.
        if (this.mainGridStyle != FR.Chart.Constants.LINE_NONE && parseInt(this.tickLength) > 0
            && this.mainGridColor != null) {
            this.drawGridLine(ctx, this.mainGridStyle, this.mainGridColor,
                parseInt(this.tickLength), this.mainType);
        }
    },

    /**
     * 画网格线的线.
     * @param ctx 画布
     * @param lineStyle 线型
     * @param lineColor 线的颜色
     * @param unitValue 单位值
     * @param type 刻度类型
     */
    drawGridLine: function (ctx, lineStyle, lineColor, unitValue, type) {
        if (this.axisLength <= 15) {
            return;
        }
        if(!ctx){     return; } ctx.save();

        ctx.lineWidth = FR.Chart.GraphHelper.getLineStyleSize(lineStyle);
        ctx.strokeStyle = lineColor;
        var tickSampingTime = this.getTickSamplingTime();
        var valueDate = FR.Chart.ChartUtils.int2Date(this.getCrossValue(), type);
        while (FR.Chart.ChartUtils.date2Int(valueDate, type) <= this.getArrowValue() && unitValue > 0) {

            var tmpValue = FR.Chart.ChartUtils.date2Int(valueDate, type);

            var lines = this.getGridLine(tmpValue);
            for (var i = 0; i < lines.length; i++) {
                FR.Chart.GraphHelper.dealLine4JSDraw(lines[i]);
                lines[i].draw(ctx);
            }

            FR.Chart.DateAxisGlyph.nextDate(valueDate, type, unitValue * tickSampingTime);
        }

        ctx.restore();
    },

    /**
     * 画坐标轴线
     * @param ctx 画布
     */
    drawAxisLine: function (ctx) {
        if(!ctx){     return; } ctx.save();

        if (this.minValue == this.maxValue) {
            return;
        }

        if (this.lineColor && this.lineStyle != FR.Chart.Constants.LINE_NONE) {
            ctx.strokeStyle = this.lineColor;
            ctx.linwWidth = FR.Chart.GraphHelper.getLineStyleSize(this.lineStyle);
            var line = new FR.Chart.Line2D(this.getPoint2D(this.getCrossValue()),
                this.getPoint2D(this.getArrowValue() + this.tickNumberChangeUponLabelLocation()));

            FR.Chart.GraphHelper.dealLine4JSDraw(line);

            line.draw(ctx);
            this.drawArrow(ctx, line.startPoint, line.endPoint);

        }
        this.drawTicks(ctx);

        ctx.restore();
    },

    /**
     * 画刻度  DateAxisGlyph.drawTicks
     * @param ctx 画布
     */
    drawTicks: function (ctx) {
        if (this.axisLength <= 15 || this.maxValue <= this.minValue) {
            return;
        }
        // 主要刻度
        var mainDate = FR.Chart.ChartUtils.int2Date(this.getCrossValue(), this.mainType);
        //标签偏移
        var offset = (this.drawBetweenTick ? 0.5 : 0);
        //标签间隔
        var tickSamplingTime = this.getTickSamplingTime();
        var labelCrement = (this.labelNumber > 0 ? this.labelNumber : tickSamplingTime);

        var oldMainType = this.mainType;
        if(this.stockChartMainType){
            this.mainType = this.stockChartMainType;
        }

        //画主要刻度线
        while (FR.Chart.ChartUtils.date2Int(mainDate, this.mainType) < this.getArrowValue() && parseInt(this.tickLength) > 0) {
            var tmpValue = FR.Chart.ChartUtils.date2Int(mainDate, this.mainType);
            var line = this.getTickLine(tmpValue, FR.Chart.AxisGlyph.MAIN_TICK_LENGTH, this.tickMarkType);
            this.drawTickLine(ctx, line, FR.Chart.Constants.AxisTickWidth);
            FR.Chart.DateAxisGlyph.nextDate(mainDate, this.mainType, this.stockChartMainType ? 1 : parseInt(this.tickLength * labelCrement));
        }

        //画次要刻独线
        if(tickSamplingTime <= 1 && Math.abs(this.smallTickLength) > 0){
            mainDate = FR.Chart.ChartUtils.int2Date(this.getCrossValue(), this.secondType);
            while (FR.Chart.ChartUtils.date2Int(mainDate, this.secondType) < this.getArrowValue() && parseInt(this.smallTickLength) > 0) {
                var tmpValue = FR.Chart.ChartUtils.date2Int(mainDate, this.secondType);
                var line = this.getTickLine(tmpValue, FR.Chart.AxisGlyph.SEC_TICK_LENGTH, this.secTickMarkType);
                this.drawTickLine(ctx, line, FR.Chart.Constants.AxisTickWidth);
                FR.Chart.DateAxisGlyph.nextDate(mainDate, this.secondType, parseInt(this.smallTickLength * labelCrement));
            }

        }

        //画标签
        var labelPath = new FR.Chart.BoundsManager();
        mainDate = FR.Chart.ChartUtils.int2Date(this.stockChartMinDateInt ? this.stockChartMinDateInt :this.getCrossValue(), this.mainType);// 不能<=, 在缩略显示时超出范围了!
        while (FR.Chart.ChartUtils.date2Int(mainDate, this.mainType) < this.getArrowValue() && parseInt(this.tickLength) > 0) {
            var tmpValue = FR.Chart.ChartUtils.date2Int(mainDate, this.mainType);
            var valueString = "";
            valueString = this.getDateString4Type(mainDate,this.mainType);
            var labelBounds = this.getLabelBounds(tmpValue, offset, valueString);
            if (!labelPath.isInteract(labelBounds)) {
                this.drawLabel(ctx, tmpValue, offset, valueString);
                labelPath.addBounds(labelBounds);
            }
            FR.Chart.DateAxisGlyph.nextDate(mainDate, this.mainType,this.stockChartMainType ? 1 : parseInt(this.tickLength * labelCrement));
        }

        this.mainType = oldMainType;
    },

    //根据坐标轴类型返回string
    getDateString4Type:function(date,mainType){
        if(!this.stockChartMinDateInt){ //没有stockChartMinDateInt说明是正常的没有stockchart大数据调整过的坐标轴
            return  (this.format == null ? FR.contentFormat(date, "Dyyyy-MM-dd") : FR.contentFormat(date, this.format));
        }

        //stockchart大数据调整过的坐标轴显示标签
        return FR.Chart.ChartUtils.calculateAdjustDateRange(date,mainType);
    } ,


    /**
     * 在PreDemo画时, 只画标签部分.
     * @param ctx 画布
     */
    drawOnlyLabels4Demo: function (ctx) {
        if (this.axisLength <= 15 || this.maxValue <= this.minValue) {
            return;
        }

        var tickSampingTime4Demo = this.getTickSamplingTime4Demo();
        var valueDate = FR.Chart.ChartUtils.int2Date(this.minValue, this.mainType);
        while (FR.Chart.ChartUtils.date2Int(valueDate, this.mainType) < this.maxValue && this.tickLength > 0) {
            var tmpValue = FR.Chart.ChartUtils.date2Int(valueDate, this.mainType);
            var lines = this.getGridLine4Demo(tmpValue);
            for (var i = 0; i < lines.length; i++) {
                FR.Chart.GraphHelper.dealLine4JSDraw(lines[i]);
                lines[i].draw(ctx);
            }

            FR.Chart.DateAxisGlyph.nextDate(valueDate, this.mainType, this.tickLength * tickSampingTime4Demo);
        }

        var tickSamplingTime = this.getTickIntervalCount(this.getMaxTickValueDemo() - this.minValue, 15);
        var labelCrement = (this.labelNumber > 0 ? this.labelNumber : tickSamplingTime);
        var offset = (this.drawBetweenTick ? 0.5 : 0);
        var labelPath = new FR.Chart.BoundsManager();
        var mainDate = FR.Chart.ChartUtils.int2Date(this.minValue, this.mainType);
        while (FR.Chart.ChartUtils.date2Int(mainDate, this.mainType) < this.maxValue && parseInt(this.tickLength) > 0) {
            var tmpValue = FR.Chart.ChartUtils.date2Int(mainDate, this.mainType);
            var valueString = "";
            valueString = (this.format == null ? FR.contentFormat(mainDate, "DMM-dd") : FR.contentFormat(mainDate, this.format));
            var labelBounds = this.getLabelBounds(tmpValue, offset, valueString);
            if (!labelPath.isInteract(labelBounds)) {
                this.drawLabel(ctx, tmpValue, offset, valueString);
                labelPath.addBounds(labelBounds, false);
            }
            FR.Chart.DateAxisGlyph.nextDate(mainDate, this.mainType, parseInt(this.tickLength * labelCrement));
        }
    },

    /**
     * 根据值获取对应的坐标轴点
     * @param value 值
     * @returns {FR.Chart.Point2D} 值对应的坐标轴点
     */
    getPoint2D: function (value) {
        var date = FR.Chart.ChartUtils.int2Date(this.minValue, this.mainType);
        value = value - (this.isDraw4Demo ? FR.Chart.ChartUtils.date2Int(date, this.mainType) : this.getCrossValue());

        var useLength = this.isDraw4Demo ? this.unitLengthNoScale : this.unitLength;

        if (this.originPoint == null) {
            return new FR.Chart.Point2D(0, 0);
        }

        var det = useLength * value;
        var detY = this.axisReversed ? det : -det;
        var detX = this.axisReversed ? -det : det;

        if (this.position == FR.Chart.Constants.LEFT || this.position == FR.Chart.Constants.RIGHT) {
            return new FR.Chart.Point2D(this.originPoint.x, this.originPoint.y + detY);
        } else {
            return new FR.Chart.Point2D(this.originPoint.x + detX, this.originPoint.y);
        }
    },

    /**
     * 返回 原始的 (未缩放)实际的最小值
     * @returns {Number} 返回最小值日期对应的数值.
     */
    getOriginalCrossValue: function () {
        var date = FR.Chart.ChartUtils.int2Date(this.minValue, this.mainType);
        FR.Chart.DateAxisGlyph.nextDate(date, this.mainType, 0);
        return FR.Chart.ChartUtils.date2Int(date, this.mainType);
    },

    /**
     * 返回原始的 (未缩放)实际的最大值
     * @returns {Number} 返回最大值日期对应的数值.
     */
    getOriginalArrowValue: function () {
        var date = FR.Chart.ChartUtils.int2Date(this.maxValue, this.mainType);
        //这时候应该是取最后一天的数值,如果主类型超过一天，则可能会超过最大数值好多
        var type = this.mainType;
        if(type === FR.Chart.Constants.MONTH || FR.Chart.Constants.YEAR ){
            type = FR.Chart.Constants.DAY;
        }
        FR.Chart.DateAxisGlyph.nextDate(date, type, 1);
        return FR.Chart.ChartUtils.date2Int(date, this.mainType);
    },

    /**
     * 返回 最小值 (实际的 包括缩放)
     * @returns {Number} 日期最大值.
     */
    getCrossValue: function () {
        var date = FR.Chart.ChartUtils.int2Date(this.minValue, this.mainType);
        FR.Chart.DateAxisGlyph.nextDate(date, this._getMoreThanDayType(), -this.zoomBackward);
        return FR.Chart.ChartUtils.date2Int(date, this.mainType);
    },

    /**
     * 返回最大值 (实际的 包括缩放)
     * @returns {Number} 日期最小值
     */
    getArrowValue: function () {
        var date = FR.Chart.ChartUtils.int2Date(this.maxValue, this.mainType);
        FR.Chart.DateAxisGlyph.nextDate(date, this._getMoreThanDayType(), this.zoomForward + 1);
        return FR.Chart.ChartUtils.date2Int(date, this.mainType);
    },


    getEndValue4UnitBasedMinDate:function(tickType, unitValue){
        var date = FR.Chart.ChartUtils.int2Date(this.minValue, this.mainType);
        FR.Chart.DateAxisGlyph.nextDate(date, this._getMoreThanDayType(), -this.zoomBackward);
          //此时date是实际的坐标轴最小值
        FR.Chart.DateAxisGlyph.nextDate(date,tickType,unitValue);
        return FR.Chart.ChartUtils.date2Int(date, this.mainType);
    },

    getStartValue4UnitBasedOriginalMaxDate:function( tickType, unitValue){
        var date = FR.Chart.ChartUtils.int2Date(this.maxValue, this.mainType);
        //这时候应该是取最后一天的数值,如果主类型超过一天，则可能会超过最大数值好多
        var type = this.mainType;
        if(type === FR.Chart.Constants.MONTH || FR.Chart.Constants.YEAR ){
            type = FR.Chart.Constants.DAY;
        }
        FR.Chart.DateAxisGlyph.nextDate(date, type, 1);
          //此时date是原始的坐标轴最大值
        FR.Chart.DateAxisGlyph.preDate(date,tickType,unitValue);
        return FR.Chart.ChartUtils.date2Int(date, this.mainType);
    },

    /**
     * 至少按照 日期天 计算.
     * @returns {Number} 精确类型.
     */
    _getMoreThanDayType: function () {// 主日期类型中, 应该是不超过日的. 因为计算的具体时间的时候 最高是以天数为准日期.
        if (this.mainType == FR.Chart.Constants.YEAR || this.mainType == FR.Chart.Constants.MONTH) {
            return FR.Chart.Constants.DAY;
        }
        return this.mainType;
    }
});
$.extend(FR.Chart.DateAxisGlyph, /**@class FR.Chart.DateAxisGlyph*/{
    /**
     * 静态方法, 根据日期起始值, 刻度类型. 单位值, 返回下一个日期刻度.
     * @static
     * @param valueDate 日期值
     * @param tickType 刻度类型
     * @param unitValue 刻度单位.
     */
    nextDate: function (valueDate, tickType, unitValue) {
        if (tickType == FR.Chart.Constants.DAY) {

            valueDate.setDate(valueDate.getDate() + unitValue);
        } else if (tickType == FR.Chart.Constants.MONTH) {

            valueDate.setMonth(valueDate.getMonth() + unitValue);
        } else if (tickType == FR.Chart.Constants.YEAR) {

            valueDate.setYear(valueDate.getFullYear() + unitValue);
        } else if (tickType == FR.Chart.Constants.Minute) {

            valueDate.setMinutes(valueDate.getMinutes() + unitValue);
        } else if (tickType == FR.Chart.Constants.Hour) {

            valueDate.setHours(valueDate.getHours() + unitValue);
        } else if (tickType == FR.Chart.Constants.Second) {

            valueDate.setSeconds(valueDate.getSeconds() + unitValue);
        }
    },

    /**
     * 静态方法, 根据日期起始值, 刻度类型. 单位值, 返回上一个日期刻度.
     * @static
     * @param valueDate 日期值
     * @param tickType 刻度类型
     * @param unitValue 刻度单位.
     */
    preDate: function (valueDate, tickType, unitValue) {
        if (tickType == FR.Chart.Constants.DAY) {
            valueDate.setDate(valueDate.getDate() - unitValue);
        } else if (tickType == FR.Chart.Constants.MONTH) {

            valueDate.setMonth(valueDate.getMonth() - unitValue);
        } else if (tickType == FR.Chart.Constants.YEAR) {

            valueDate.setYear(valueDate.getFullYear() - unitValue);
        } else if (tickType == FR.Chart.Constants.Minute) {

            valueDate.setMinutes(valueDate.getMinutes() - unitValue);
        } else if (tickType == FR.Chart.Constants.Hour) {

            valueDate.setHours(valueDate.getHours() - unitValue);
        } else if (tickType == FR.Chart.Constants.Second) {

            valueDate.setSeconds(valueDate.getSeconds() - unitValue);
        }
    }
});