/**
 * Created by eason on 14-10-23.
 */

/**
 *各种图表的数据类型的定义
 * @para tableData 拿到的原始的二维表的数据对象
 * {
 *  columnName1:[...],
 *  columnName2:[...],
 *  columnName3:[...]
 *  ....
 * }tableData是类似的数据库表的形式
 *
 */
FR.Chart.DataDefinition = FR.extend(FR.Chart.ChartJsInit,{
    initConstants:function(js){
        this.dataSource = FR.pick(js.dataSource, 'local')
        if(this.dataSource === 'local'){
            this.tableData = FR.pick(js.tableData, {});
        }else{
            this.jsonURL = FR.pick(js.tableData);
            this.loadRemoteJSONData();
        }
    },

    loadRemoteJSONData:function(){
        var self = this;
        FR.ajax({
            url:self.jsonURL,
            dataType:'json',
            type:'POST',
            async:false,
            success:function(data){
                self.tableData = data;
            }
        });
    },

    createChartData:function(){

    },

    //数组标签去重,利用属性不能重复，遍历一次
    distinctLabels:function(labels){
        if(!labels){
            return [];
        }

        var newArr=[],obj={};
        for(var i=0,len = labels.length; i<len; i++){
            if(!obj[labels[i]]){
                newArr.push(labels[i]);
                obj[labels[i]] = "new";
            }
        }
        return newArr;
    },

    //返回指定列的第rowIndex行数据
    getValueAt:function(rowIndex, columnName){
        var column =  this.tableData[columnName];
        if(column && rowIndex < column.length){
            return column[rowIndex];
        }
        return 0;
    }

});

FR.Chart.NormalDataDefinition = FR.extend(FR.Chart.DataDefinition,{

    initConstants:function(js){
        FR.Chart.NormalDataDefinition.superclass.initConstants.apply(this, [js]);

        this.categoryName = FR.pick(js.categoryName, "");
        this.moreCateLabels = FR.pick(js.moreCateLabels, []);

        this.categoryLabels = [];
        this.secondLabels = [];
        this.thirdLabels = [];
    },

    check4IndexInObjects:function(label, objects){
        for(var i = 0; i < objects.length; i++){
            if(label === objects[i]){
                return i;
            }
        }
        return 0;
    },

    //初始化所有的分类轴标签
    initAllCateLabels:function(){
        var firstLabels = this.distinctLabels(this.tableData[this.categoryName]);

        if(this.moreCateLabels.length > 0){
            var secondName = this.moreCateLabels[0];
            this.secondLabels = this.distinctLabels(this.tableData[secondName]);
        }

        if(this.moreCateLabels.length > 1){
            var thirdName = this.moreCateLabels[1];
            this.thirdLabels = this.distinctLabels(this.tableData[thirdName]);
        }

        var firstSize = firstLabels.length;
        var secondSize = this.secondLabels.length !== 0 ? secondLabels.length : 1;
        if(secondSize > 0) {
            firstSize = firstSize * secondSize;
        }

        var thirdSize = this.thirdLabels.length !== 0 ? this.thirdLabels.length : 1;
        if(thirdSize > 0) {
            firstSize = firstSize * thirdSize;
        }

        for(var i = 0; i < firstSize; i++) {
            this.categoryLabels[i] = firstLabels[i % firstLabels.length];
        }
    },

    //返回某一行数据的分类的index
    getCategoryIndex:function(rowIndex){
        var categoryOb = this.getValueAt(rowIndex, this.categoryName);

        var firstIndex = this.check4IndexInObjects(categoryOb, this.categoryLabels);
        var secondIndex = 0;
        var thirdIndex = 0;

        if(this.moreCateLabels.length > 0){
            var secondOb = this.getValueAt(rowIndex, this.moreCateLabels[0]);
            secondIndex = this.check4IndexInObjects(secondOb, this.secondLabels);
        }

        if(this.moreCateLabels.length > 1){
            var thirdOb = this.getValueAt(rowIndex, this.moreCateLabels[1]);
            thirdIndex = this.check4IndexInObjects(thirdOb, this.thirdLabels);
        }

        var firstCount = this.categoryLabels.length === 0 ? 1 : this.categoryLabels.length;
        var secondCount = this.secondLabels.length === 0 ? 1 : this.secondLabels.length;
        var thirdCount = this.thirdLabels.length === 0 ? 1 : this.thirdLabels.length;

        if(thirdCount > 0) {
            return firstIndex + thirdIndex * firstCount / thirdCount + secondIndex * secondCount;
        } else {
            return firstIndex + secondIndex * secondCount;
        }
    }
});

//系列是取的单个字段
FR.Chart.SingleColumnDefinition = FR.extend(FR.Chart.NormalDataDefinition,{
    initConstants:function(js){
        FR.Chart.SingleColumnDefinition.superclass.initConstants.apply(this, [js]);
        this.seriesColumnName = FR.pick(js.seriesName, "");
        this.valueColumnName = FR.pick(js.value, "");
    },

    createChartData:function(){
        this.initAllCateLabels();

        var seriesNames = this.distinctLabels(this.tableData[this.seriesColumnName]);
        var v_2D_list = [];//所有系列分组以后的数据

        if(FR.isEmpty(this.categoryName)){
            var rowCount = this.tableData[this.seriesColumnName].length;
            for (var series_index = 0, seriesLength = seriesNames.length; series_index < seriesLength; series_index++) {
                var currentName = seriesNames[series_index];
                for(var i = 0; i < rowCount; i ++) {
                    var current_series_name = this.getValueAt(i, this.seriesColumnName);
                    if (FR.equals(current_series_name, currentName)) {
                        v_2D_list.push([this.getValueAt(i, this.valueColumnName)])
                        break;
                    }
                }
            }
            this.categoryLabels.push("");
        }else{
            var map = {};
            this.addValuesInValueList(map);
            for(var seriesIndex = 0, seriesCount = seriesNames.length; seriesIndex < seriesCount; seriesIndex++){
                var currentList = [];
                var seriesName = seriesNames[seriesIndex];
                var categoryMap = map[seriesName];
                for(var index = 0; index < this.categoryLabels.length; index++){
                    var value_list = categoryMap[index+""];
                    var result = 0;
                    if(value_list){
                        for(var i = 0; i < value_list.length; i++){
                            result += parseFloat(value_list[i]);
                        }
                    }
                    currentList.push(result);
                }

                v_2D_list.push(currentList);
            }
        }

        var normal = new FR.Chart.NormalChartData(this.categoryLabels, seriesNames, v_2D_list);
        normal.secondCates = this.secondLabels;
        normal.thirdCates = this.thirdLabels;
        return normal;
    },

    //把所有的值都映射到分类和系列的map里
    addValuesInValueList:function(map){
        var values = this.tableData[this.valueColumnName];
        for(var rowIndex = 0, rowCount = values.length; rowIndex < rowCount; rowIndex++){
            var seriesName = this.getValueAt(rowIndex, this.seriesColumnName);

            var categoty_map = {};
            if(map[seriesName]){
                categoty_map = map[seriesName];
            }else{
                map[seriesName] = categoty_map;
            }

            var categoryIndex = this.getCategoryIndex(rowIndex);
            categoryIndex += "";
            var value_list = [];
            if(categoty_map[categoryIndex]){
                value_list = categoty_map[categoryIndex];
            }else{
                categoty_map[categoryIndex] = value_list;
            }

            value_list.push(values[rowIndex]);
        }
    }
});

//多个数据列作为系列的时候的配置
FR.Chart.ChartSummaryColumn = function(seriesName, columnName){
    this.seriesName = FR.pick(seriesName, "");//需要显示的系列的名字
    this.columnName = FR.pick(columnName, "");//tableData中的系列值数组的key
};

FR.Chart.MutiColumnDefinition = FR.extend(FR.Chart.NormalDataDefinition,{
    initConstants:function(js){
        FR.Chart.MutiColumnDefinition.superclass.initConstants.apply(this, [js]);

        this.chartSummaryColumn = [];
        var columns = FR.pick(js.summaryColumns, []);
        for(var i = 0; i < columns.length; i++){
            var column = columns[i];
            this.chartSummaryColumn.push(new FR.Chart.ChartSummaryColumn(column.seriesName, column.columnName));
        }

    },

    createChartData:function(){
        this.initAllCateLabels();
        var v_2D_list = [];
        var seriesNameArray = this.createSeriesNamesArray();
        var columnNameArray = this.createColumnNamesArray();
        if(FR.isEmpty(this.categoryName)){
            for (var seriesIndex = 0, seriesLen = seriesNameArray.length; seriesIndex < seriesLen; seriesIndex++) {
                v_2D_list.push([this.getValueAt(0, columnNameArray[seriesIndex])]);
            }
            this.categoryLabels.push("");
        }else{
            this.add2Value2DList(seriesNameArray, columnNameArray, v_2D_list);
        }
        var normal = new FR.Chart.NormalChartData(this.categoryLabels, seriesNameArray, v_2D_list);

        normal.secondCates = this.secondLabels;
        normal.thirdCates = this.thirdLabels;
        return normal;
    },

    //把所有的数值传到二维表里
    add2Value2DList:function(seriesNameArray, columnNameArray, v_2D_list){
        var map = {};
        this.addValuesInValueList(seriesNameArray, columnNameArray, map);
        for (var seriesIndex = 0, seLength = seriesNameArray.length; seriesIndex < seLength; seriesIndex++) {
            var currentList = [];
            var category_map = map[seriesNameArray[seriesIndex]];
            if (category_map) {
                for (var cateIndex = 0, cate_len = this.categoryLabels.length; cateIndex < cate_len; cateIndex ++) {
                    var value_list = category_map[cateIndex+""];
                    var value = 0;
                    if (value_list) {
                        for (var i = 0, size = value_list.length; i < size; i++) {
                            value += parseFloat(value_list[i]);
                        }
                    }
                    currentList.push(value);	// function 处理结果.
                }
            }
            v_2D_list.push(currentList);	// 一个系列的所有分类list 加入二维数组.
        }
    },

    addValuesInValueList:function(seriesNameArray, columnNameArray, map){
        for (var seriesIndex = 0; seriesIndex < this.chartSummaryColumn.length; seriesIndex++) {
            var columnName = columnNameArray[seriesIndex];
            var columnValues = this.tableData[columnName];
            for(var rowIndex = 0, tableRowCount = columnValues.length; rowIndex < tableRowCount; rowIndex++) {// 循环 每行, 找到 对应的系列的值
                var seriesName = seriesNameArray[seriesIndex];

                var category_map = map[seriesName];
                if (category_map == null) {
                    category_map = {};
                    map[seriesName] = category_map;
                }

                var categoryIndex = this.getCategoryIndex(rowIndex);
                var value_list = category_map[categoryIndex+""];// 根据对应的序号
                if (value_list == null) {
                    value_list = [];
                    category_map[categoryIndex+""] = value_list;
                }

                value_list.push(this.getValueAt(rowIndex, columnName));
            }
        }
    },

    createColumnNamesArray:function(){
        var columnNameArray = [];
        for(var index =0; index < this.chartSummaryColumn.length; index++){
            var column = this.chartSummaryColumn[index];
            columnNameArray.push(column.columnName);
        }
        return columnNameArray;
    },

    //创建所有要显示的系列的名字
    createSeriesNamesArray:function(){
        var seriesNameArray = [];
        for(var index =0; index < this.chartSummaryColumn.length; index++){
            var summaryColumn = this.chartSummaryColumn[index];
            if(FR.isEmpty(summaryColumn.seriesName)){
                seriesNameArray.push(summaryColumn.columnName);
            }else{
                seriesNameArray.push(summaryColumn.seriesName);
            }
        }

        return seriesNameArray;
    }

});

FR.Chart.ScatterDefinition = FR.extend(FR.Chart.DataDefinition,{
    initConstants:function(js){
        FR.Chart.ScatterDefinition.superclass.initConstants.apply(this, [js]);

        this.seriesName = FR.pick(js.seriesName, "");
        this.scatterX = FR.pick(js.scatterX, "");
        this.scatterY = FR.pick(js.scatterY, "");
    },

    createChartData:function(){
        var x2DList = [];
        var y2DList = [];

        // 处理图例选项个数.
        var itemNameList = this.distinctLabels(this.tableData[this.seriesName]);

        //处理x值
        this.addValueInColumnWithName(itemNameList, x2DList, this.scatterX);

        //处理y值
        this.addValueInColumnWithName(itemNameList, y2DList, this.scatterY);

        return new FR.Chart.ScatterChartData(itemNameList, x2DList, y2DList);
    },

    addValueInColumnWithName:function(nameList, valueList, valueName){
        for(var nameIndex = 0; nameIndex < nameList.length; nameIndex++) {
            var nameObject = nameList[nameIndex];
            var currentList = [];
            var values = this.tableData[valueName];
            for(var rowIndex = 0; rowIndex < values.length; rowIndex++) {
                if(FR.equals(nameObject, this.getValueAt(rowIndex, this.seriesName))){
                    currentList.push(parseFloat(this.getValueAt(rowIndex, valueName)));
                }
            }
            valueList.push(currentList);
        }
    }
});

FR.Chart.BubbleDefinition = FR.extend(FR.Chart.DataDefinition,{
    initConstants:function(js){
        FR.Chart.BubbleDefinition.superclass.initConstants.apply(this, [js]);

        this.bubbleX = FR.pick(js.bubbleX, "");
        this.bubbleY = FR.pick(js.bubbleY, "");
        this.bubbleSize = FR.pick(js.bubbleSize, "");
        this.seriesName = FR.pick(js.seriesName, "");
    },

    createChartData:function(){

        var bubbleX_2DList = [];
        var bubbleY_2DList = [];
        var bubbleSize_2DList = [];

        var seriesNameList = this.distinctLabels(this.tableData[this.seriesName]);

        // 开始处理x值
        this.addValueInColumnWithName(seriesNameList, bubbleX_2DList, this.bubbleX);

        // 开始处理y值
        this.addValueInColumnWithName(seriesNameList, bubbleY_2DList, this.bubbleY);

        // 开始处理size
        this.addValueInColumnWithName(seriesNameList, bubbleSize_2DList, this.bubbleSize);

        return new FR.Chart.BubbleChartData(seriesNameList, bubbleX_2DList, bubbleY_2DList, bubbleSize_2DList);
    },

    addValueInColumnWithName:function(nameList, valueList, valueName){
        for(var nameIndex = 0; nameIndex < nameList.length; nameIndex++) {
            var nameObject = nameList[nameIndex];
            var currentList = [];
            var values = this.tableData[valueName];
            for(var rowIndex = 0; rowIndex < values.length; rowIndex++) {
                if(FR.equals(nameObject, this.getValueAt(rowIndex, this.seriesName))){
                    currentList.push(parseFloat(this.getValueAt(rowIndex, valueName)));
                }
            }
            valueList.push(currentList);
        }
    }
});

FR.Chart.MeterDefinition = FR.extend(FR.Chart.DataDefinition,{
    initConstants:function(js){
        FR.Chart.MeterDefinition.superclass.initConstants.apply(this, [js]);

        this.name = FR.pick(js.itemName, "");
        this.value = FR.pick(js.value, "");
    },

    createChartData:function(){
        var nameList = this.distinctLabels(this.tableData[this.name]);
        var valueList = [];

        for(var i = 0; i < nameList.length; i++) {
            var cateName = nameList[i];
            var values = this.tableData[this.value];
            for(var rowIndex = 0, rouCount = values.length; rowIndex < rouCount; rowIndex++) {
                var rowName = this.getValueAt(rowIndex, this.name);
                if(FR.equals(cateName, rowName)) {
                    valueList.push(parseFloat(this.getValueAt(rowIndex, this.value)));
                    break;
                }
            }
        }
        return new FR.Chart.MeterChartData(nameList, valueList);
    }
});

FR.Chart.StockDefinition = FR.extend(FR.Chart.DataDefinition,{
    initConstants:function(js){
        FR.Chart.StockDefinition.superclass.initConstants.apply(this, [js]);

        this.cateTime = FR.pick(js.timeLine, "");
        this.volume = FR.pick(js.volume, "");
        this.open = FR.pick(js.open, "");
        this.close = FR.pick(js.close, "");
        this.low = FR.pick(js.low, "");
        this.high = FR.pick(js.high, "");
    },

    createChartData:function(){
        var cateList = this.distinctLabels(this.tableData[this.cateTime]);
        var openList = [];
        var highList = [];
        var lowList = [];
        var closeList = [];
        var volumeList = [];

        this.addValueInColunmWithName(cateList, openList, this.open);

        this.addValueInColunmWithName(cateList, highList, this.high);

        this.addValueInColunmWithName(cateList, lowList, this.low);

        this.addValueInColunmWithName(cateList, closeList, this.close);

        this.addValueInColunmWithName(cateList, volumeList, this.volume);

        return new FR.Chart.StockChartData(cateList, volumeList, openList, closeList, lowList, highList);
    },

    addValueInColunmWithName:function(nameList, valueList, valueName){
        for(var nameIndex = 0; nameIndex < nameList.length; nameIndex++) {
            var nameObject = nameList[nameIndex];
            var values = this.tableData[valueName];
            if(nameObject) {
                for(var rowIndex = 0; rowIndex < values.length; rowIndex++) {
                    if(FR.equals(nameObject, this.getValueAt(rowIndex, this.cateTime))) {
                        valueList.push(parseFloat(this.getValueAt(rowIndex, valueName)));
                        break;
                    }
                }
            }
        }
    }
});

FR.Chart.GanttDefinition = FR.extend(FR.Chart.DataDefinition,{
    initConstants:function(js){
        FR.Chart.GanttDefinition.superclass.initConstants.apply(this, [js]);

        this.stepString = FR.pick(js.step, "");
        this.planStart = FR.pick(js.planStart, "");
        this.planEnd = FR.pick(js.planEnd, "");
        this.realStart = FR.pick(js.realStart, "");
        this.realEnd = FR.pick(js.realEnd, "");
        this.progress = FR.pick(js.progress, "");
        this.item = FR.pick(js.items, "");
    },

    createChartData:function(){
        var stepList = this.distinctLabels(this.tableData[this.stepString]);
        var itemList = this.distinctLabels(this.tableData[this.item]);
        var planStart2DList = [];
        var planEnd2DList = [];
        var realStart2DList = [];
        var realEnd2DList = [];
        var progress2DList = [];

        this.addValueInColumnWithName(stepList, itemList, planStart2DList, this.planStart, true);

        this.addValueInColumnWithName(stepList, itemList, planEnd2DList, this.planEnd, true);

        this.addValueInColumnWithName(stepList, itemList, realStart2DList, this.realStart, true);

        this.addValueInColumnWithName(stepList, itemList, realEnd2DList, this.realEnd, true);

        this.addValueInColumnWithName(stepList, itemList, progress2DList, this.progress, false);

        return new FR.Chart.GanttChartData(stepList, planStart2DList, planEnd2DList,
            realStart2DList, realEnd2DList, progress2DList, itemList);
    },

    addValueInColumnWithName:function(nameList, itemIDList, valueList, valueName, isDate){
        if(itemIDList.length !== 0) {// 根据步骤确定
            for(var itemIndex = 0; itemIndex < itemIDList.length; itemIndex++) {
                var currentList = [];
                var itemObject = itemIDList[itemIndex];
                for(var nameIndex = 0; nameIndex < nameList.length; nameIndex++) {
                    var stepName = nameList[nameIndex];
                    var values = this.tableData[valueName];
                    for(var rowIndex = 0; rowIndex < values.length; rowIndex++) {
                        if(FR.equals(stepName, this.getValueAt(rowIndex, this.stepString))){
                            if(FR.equals(itemObject, this.getValueAt(rowIndex, this.item))){
                                var value = this.getValueAt(rowIndex, valueName);
                                if(isDate){
                                    value = FR.Chart.ChartUtils.string2Date(value);
                                    value = FR.Chart.ChartUtils.date2Int(value, FR.Chart.Constants.Second);
                                }else{
                                    value = parseFloat(value);
                                }
                                currentList.push(value);
                            }
                        }
                    }
                }
                valueList.push(currentList);
            }
        } else {// 直接对应步骤name添加了.
            var currentList = [];
            for(var nameIndex = 0; nameIndex < nameList.length; nameIndex++) {
                var nameObject = nameList[nameIndex];
                var values = this.tableData[valueName];
                for(var rowIndex = 0; rowIndex < values.length; rowIndex++) {
                    if(FR.equals(nameObject, this.getValueAt(rowIndex, this.stepString))){
                        var value = this.getValueAt(rowIndex, valueName);
                        if(isDate){
                            value = FR.Chart.ChartUtils.string2Date(value);
                            value = FR.Chart.ChartUtils.date2Int(value, FR.Chart.Constants.Second);
                        }else{
                            value = parseFloat(value);
                        }
                        currentList.push(value);
                    }
                }
            }
            valueList.push(currentList);
        }
    }
});

FR.Chart.SeriesDefinition = function(title, value){
    this.title = title;
    this.value = value;
}

FR.Chart.MapSingleLayerDefinition = FR.extend(FR.Chart.DataDefinition,{
    initConstants:function(js){
        FR.Chart.MapSingleLayerDefinition.superclass.initConstants.apply(this,[js]);
        this.areaName = FR.pick(js.areaName, "");
        this.list = [];
        if(js.titleValues){
            for(var i = 0; i < js.titleValues.length; i++){
                var definition = js.titleValues[i];
                this.list.push(new FR.Chart.SeriesDefinition(definition.title, definition.value));
            }
        }
    },

    createChartData:function(){
        var mapChartData = new FR.Chart.MapChartData();
        this.createMapDataWithLayerIndex(mapChartData);
        return mapChartData;
    },

    createMapDataWithLayerIndex:function(mapChartData){
        var listSize = this.list.length;
        if(!FR.isEmpty(this.areaName) && listSize > 0) {
            var areaNames = this.tableData[this.areaName];
            var rowCount = areaNames.length;
            for(var rowIndex = 0; rowIndex < rowCount; rowIndex++) {
                var areaName = areaNames[rowIndex];
                if(!FR.isEmpty(areaName)) {
                    var mapArea = new FR.Chart.MapAreaValue();
                    mapChartData.addMapData(areaName, mapArea);

                    for(var index = 0; index < listSize; index++) {
                        var singleDefinition = this.list[index];
                        if(singleDefinition && singleDefinition.value) {
                            var title = singleDefinition.title;
                            var value = singleDefinition.value;
                            var titleValue = this.getValueAt(rowIndex, value);
                            mapArea.titleValues.push(new FR.Chart.MapTitleValue({title:title, value:titleValue}))
                        }
                    }
                }
            }
        }
    }
});

FR.Chart.MapMoreLayerDefinition = FR.extend(FR.Chart.DataDefinition,{
    initConstants:function(js){
        FR.Chart.MapMoreLayerDefinition.superclass.initConstants.apply(this, [js]);
        this.nameValueList = [];
        if(js.layers){
            for(var i = 0; i < js.layers.length; i++){
                var config = js.layers[i];
                this.nameValueList.push(new FR.Chart.MapSingleLayerDefinition(config));
            }
        }
    },

    createChartData:function(){
        var mapChartData = new FR.Chart.MapChartData();
        for(var i = 0; i < this.nameValueList.length; i++) {
            var singleDefinition = this.nameValueList[i];
            if(singleDefinition) {
                singleDefinition.createMapDataWithLayerIndex(mapChartData);
            }
        }
        mapChartData.hasLayerTo = true;
        return mapChartData;
    }
});

FR.Chart.GisMapDefinition = FR.extend(FR.Chart.DataDefinition,{
    initConstants:function(js){
        this.address = FR.pick(js.address, "");
        this.addressName = FR.pick(js.addressName, "");
        this.isAddress = FR.pick(js.isAddress, true);
        this.isLngFirst = FR.pick(js.isLngFirst, true);
        this.list = [];
        if(js.titleValues){
            for(var i = 0; i < js.titleValues.length; i++){
                var definition = js.titleValues[i];
                this.list.push(new FR.Chart.SeriesDefinition(definition.title, definition.value));
            }
        }
        FR.Chart.GisMapDefinition.superclass.initConstants.apply(this, [js]);
    },

    createChartData:function(){
        var chartData = new FR.Chart.GisMapChartData();
        chartData.isAddress = this.isAddress;
        chartData.isLngFirst = this.isLngFirst;
        this.createGisMapChartData(chartData);
        return chartData;
    },

    createGisMapChartData:function(chartData){
        if(FR.isEmpty(this.address)){
            return;
        }
        var listSize = this.list.length;
        var addressTittleValue = {};
        var validAddresses = [];
        var addressNameInfo = {};
        var addresses = this.tableData[this.address];
        var rowCount = addresses.length;
        for(var rowIndex = 0; rowIndex < rowCount; rowIndex++) {
            var address = addresses[rowIndex];
            if(!FR.isEmpty(address)){
                var mapArea = new FR.Chart.MapAreaValue();
                addressTittleValue[address] = mapArea;
                for(var index = 0; index < listSize; index++) {
                    var singleDefinition = this.list[index];
                    if(singleDefinition && singleDefinition.value) {
                        var title = singleDefinition.title;
                        var value = singleDefinition.value;
                        var titleValue = this.getValueAt(rowIndex, value);
                        mapArea.titleValues.push(new FR.Chart.MapTitleValue({title:title, value:titleValue}));
                    }
                }
                validAddresses.push(address);
            }
        }

        this.dealWithAddressName(addressNameInfo);
        chartData.addressName = addressNameInfo;
        chartData.addressTitleValue = addressTittleValue;
        chartData.addresses = validAddresses;
    },

    //处理地址和地址名之间对应
    dealWithAddressName:function(addressNameInfo){
        if(FR.isEmpty(this.addressName)) {
            return;
        }
        var addresses = this.tableData[this.address];
        var rowCount = addresses.length;
        for(var rowIndex = 0; rowIndex < rowCount; rowIndex++) {
            var address = addresses[rowIndex];
            if(!FR.isEmpty(address)) {
                var addressName = this.getValueAt(rowIndex, this.addressName);
                addressNameInfo[address] = addressName;
            }
        }
    }
});

FR.Chart.GisMapChartData = function(isAddress, isLngFirst, addressName, addressTitleValue){
    this.isAddress = FR.pick(isAddress, true);
    this.isLngFirst = FR.pick(isLngFirst, true);
    this.addressName = FR.pick(addressName, {});
    this.addressTitleValue = FR.pick(addressTitleValue, {});
    this.addresses = [];//有效的地址信息
};

FR.Chart.MapChartData = function(dataLayers, hasLayerTo){
    this.dataLayers = FR.pick(dataLayers, {});
    this.hasLayerTo = FR.pick(hasLayerTo, false);
};

$.extend(FR.Chart.MapChartData.prototype,{
    addMapData:function(areaName, areaValue){
        this.dataLayers[areaName] = areaValue;
    },

    getMapData:function(dataUseName){
        return this.dataLayers[dataUseName] ? this.dataLayers[dataUseName] : new FR.Chart.MapAreaValue();
    }

});

FR.Chart.GanttChartData = function(stepName, planStart, planEnd, realStart, realEnd, progress, itemID){
    this.stepName = FR.pick(stepName, []);
    this.planStart = FR.pick(planStart, []);
    this.planEnd = FR.pick(planEnd, []);
    this.realStart = FR.pick(realStart, []);
    this.realEnd = FR.pick(realEnd, []);
    this.progress = FR.pick(progress, []);
    this.itemID = FR.pick(itemID, []);
};

$.extend(FR.Chart.GanttChartData.prototype,{
    getIDCount:function(){
        return this.itemID ? Math.max(this.itemID.length, 1) : 1;
    },

    getIDName:function(index){
        return this.itemID[index];
    },

    getCategoryLabelCount:function(){
        return this.stepName ? this.stepName.length : 0;
    },

    getCategoryLabel:function(index){
        return this.stepName[index];
    },

    //itemIndex行，cateIndex列
    getDataValue:function(itemIndex, cateIndex, value_2d){
        if(!value_2d || value_2d.length <= itemIndex || itemIndex < 0) {
            return null;
        }
        var value_1d = value_2d[itemIndex];
        if (value_1d.length <= cateIndex) {
            return null;
        }
        return value_1d[cateIndex];
    },

    getPlanStart:function(idIndex, categoryIndex){
        return this.getDataValue(idIndex, categoryIndex, this.planStart);
    },

    getPlanEnd:function(idIndex, categoryIndex){
        return this.getDataValue(idIndex, categoryIndex, this.planEnd);
    },

    getRealStart:function(idIndex, categoryIndex){
        return this.getDataValue(idIndex, categoryIndex, this.realStart);
    },

    getRealEnd:function(idIndex, categoryIndex){
        return this.getDataValue(idIndex, categoryIndex, this.realEnd);
    },

    getProgress:function(idIndex, categoryIndex){
        return this.getDataValue(idIndex, categoryIndex, this.progress);
    }

});

FR.Chart.StockChartData = function(cateTime, volume, open, close, low, high){
    this.cateTime = FR.pick(cateTime, []);
    this.volume = FR.pick(volume, []);
    this.open = FR.pick(open, []);
    this.close = FR.pick(close, []);
    this.low = FR.pick(low, []);
    this.high = FR.pick(high, []);
};
$.extend(FR.Chart.StockChartData.prototype,{
    getCategoryLabelCount:function(){
        return this.cateTime ? this.cateTime.length : 0;
    },

    getCategoryLabel:function(index){
        return this.cateTime[index];
    },

    getCloseValue:function(categoryIndex){
        if(categoryIndex < this.close.length){
            return this.close[categoryIndex];
        }

        return Number.NaN;
    },

    getLowValue:function(categoryIndex){
        if(categoryIndex < this.low.length){
            return this.low[categoryIndex];
        }

        return Number.NaN;

    },

    getHighValue:function(categoryIndex){
        if(categoryIndex < this.high.length){
            return this.high[categoryIndex];
        }

        return Number.NaN;
    },

    getVolume:function(categoryIndex){
        if(categoryIndex < this.volume.length){
            return this.volume[categoryIndex];
        }

        return Number.NaN;
    },

    getOpenValue:function(categoryIndex){
        if(categoryIndex < this.open.length){
            return this.open[categoryIndex];
        }

        return Number.NaN;
    }

});


FR.Chart.MeterChartData = function(name, value){
    this.name = FR.pick(name, []);
    this.value = FR.pick(value, []);
};

$.extend(FR.Chart.MeterChartData.prototype, {
    getCategoryLabelCount:function(){
        return this.name ? this.name.length : 0;
    },

    getValue:function(valueIndex){
        if(this.value.length <= valueIndex){
            return null;
        }
        return this.value[valueIndex];
    },

    getCategoryLabel:function(index){
        if(this.name.length <= index) {
            return "";
        }

        return this.name[index];
    }

});

FR.Chart.BubbleChartData = function(items, bubbleX, bubbleY, bubbleSize){
    this.items = FR.pick(items, []);
    this.bubbleX = FR.pick(bubbleX, []);
    this.bubbleY = FR.pick(bubbleY, []);
    this.bubbleSize = FR.pick(bubbleSize, []);
};
$.extend(FR.Chart.BubbleChartData.prototype,{
    getItemCount:function(){
        return this.items ? this.items.length : 0;
    },

    getOriginalItem:function(itemIndex){
        if(this.items && itemIndex < this.items.length) {
            return this.items[itemIndex];
        }
        return "";
    },

    getCategoryLabelCount:function(){
        return this.bubbleX && this.bubbleX[0] ? this.bubbleX[0].length : 0;
    },

    //itemIndex行，cateIndex列
    getDataValue:function(itemIndex, cateIndex, value_2d){
        if(!value_2d || value_2d.length <= itemIndex || itemIndex < 0) {
            return null;
        }
        var value_1d = value_2d[itemIndex];
        if (value_1d.length <= cateIndex) {
            return null;
        }
        return value_1d[cateIndex];
    },

    getX:function(itemIndex, cateIndex){
        return this.getDataValue(itemIndex, cateIndex, this.bubbleX);
    },

    getY:function(itemIndex, cateIndex){
        return this.getDataValue(itemIndex, cateIndex, this.bubbleY);
    },

    getSize:function(itemIndex, cateIndex){
        return this.getDataValue(itemIndex, cateIndex, this.bubbleSize);
    }

});

FR.Chart.ScatterChartData = function(items, scatterX, scatterY){
    this.items = FR.pick(items, []);
    this.scatterX = FR.pick(scatterX, []);
    this.scatterY = FR.pick(scatterY, []);
};
$.extend(FR.Chart.ScatterChartData.prototype,{
    getItemCount:function(){
        return this.items ? this.items.length : 0;
    },

    getCategoryLabelCount:function(){
        return this.scatterX && this.scatterX[0] ? this.scatterX[0].length : 0;
    },

    //itemIndex行，cateIndex列
    getDataValue:function(itemIndex, cateIndex, value_2d){
        if(!value_2d || value_2d.length <= itemIndex || itemIndex < 0) {
            return null;
        }
        var value_1d = value_2d[itemIndex];
        if (value_1d.length <= cateIndex) {
            return null;
        }
        return value_1d[cateIndex];
    },

    getX:function(itemIndex, cateIndex){
        return this.getDataValue(itemIndex, cateIndex, this.scatterX);
    },

    getY:function(itemIndex, cateIndex){
        return this.getDataValue(itemIndex, cateIndex, this.scatterY);
    },

    getOriginalItem:function(itemIndex){
        if(this.items && itemIndex < this.items.length) {
            return this.items[itemIndex];
        }
        return "";
    }

});

FR.Chart.NormalChartData = function(category_array, series_array, series_value_2D){
    this.category_array = category_array;
    this.series_array = series_array;
    this.series_value_2D = series_value_2D;

    this.secondCates = [];
    this.thirdCates = [];
};

$.extend(FR.Chart.NormalChartData.prototype,{
    getSeriesCount:function(){
        return this.series_array.length ? this.series_array.length : 0;
    },

    getCategoryLabelCount:function(){
        return this.category_array ? this.category_array.length : 0;
    },

    getSeriesLabel:function(seriesIndex){
        if(seriesIndex < this.getSeriesCount()){
            return this.series_array[seriesIndex];
        }
        return "";
    },

    getCategoryLabel:function(categoryIndex){
        if(categoryIndex < this.getCategoryLabelCount()){
            return this.category_array[categoryIndex];
        }
        return "";
    },

    getValueAt:function(seriesIndex, categoryIndex){
        if(!this.series_value_2D || this.series_value_2D.length <= seriesIndex || seriesIndex < 0) {
            return null;
        }
        var value_1D = this.series_value_2D[seriesIndex];

        if (value_1D.length <= categoryIndex) {
            return null;
        }

        return value_1D[categoryIndex];
    }
});