/**
 * 图表系列点 图形  画出每个绘图的系列点  如柱形柱子  折线图的标记点 饼图的扇形 等。
 *     @example
 *     var dataPoint = new FR.Chart.DataPoint();
 *
 * @class FR.Chart.DataPoint
 * @extends FR.Chart.ChartJsInit
 *
 * @param {JSON} js 配置属性
 * @param {Boolean} [js.valueIsNull = false] 默认 是否为空值.
 *
 * @param {Number} [js.seriesIndex = 0] 系列序号
 * @param {Number} [js.categoryIndex = 0] 分类序号
 * @param {Number} [js.value = 0] 系列值
 * @param {Number} [js.percentValue = 0] 百分比
 * @param {String} [js.categoryName = ''] 分类名
 * @param {String} [js.secondCate =''] 第二分类
 * @param {String} [js.thirdCate = ''] 第三分类
 * @param {String} [js.seriesName = ''] 系列名
 * @param {String}[js.seriesLabel = ''] 系列标签
 * @param {String}[js.categoryOriginalName = ''] 分类标签的实际值  (分类名是显示值)
 * @param {String}[js.dataLabel = ''] 数据标签
 * @param {String}[js.hyperlink = ''] 超级链接
 * @param {String}[js.tooltip = ''] 热点提示
 * @param {Object}[js.glyph] 数据点的图形
 * @param {Object} [js.dataPointStyle] 数据点风格 (高亮 透明等)
 * @param {Object} [js.lastGlyph] 动态变化的上一次 记录
 * @param {Boolean}[js.surpportFocus = true] 是否支持焦点 效果
 * @param {Boolean} [js.clickable] 是否显示 可点击效果
 */
FR.Chart.DataPoint = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (js) {
        this.js = js || {};
        this.valueIsNull = this.js.valueIsNull || false;

        this.seriesIndex = FR.pick(this.js.seriesIndex, 0);
        this.categoryIndex = FR.pick(this.js.categoryIndex, 0);
        this.value = FR.pick(this.js.value, 0);
        this.valueByUnit = FR.pick(this.js.valueByUnit, 0);
        this.percentValue = FR.pick(this.js.percentValue, 0);
        this.categoryName = this.js.categoryName;
        this.secondCate = this.js.secondCate;
        this.thirdCate = this.js.thirdCate;
        this.seriesName = this.js.seriesName;
        this.seriesLabel = this.js.seriesLabel;
        this.categoryOriginalName = this.js.categoryOriginalName;
        this.dataLabel = this.js.dataLabel == undefined ? null : new FR.Chart.TextGlyph(this.js.dataLabel);

        this.hyperlink = this.js.hyperlink;

        this.relateHyerlink = [];

        this.tooltip = null;// 只要在画结束时保证 已经创建完毕.
        this.glyph = null;
        this.isOutBounds = false; //是不是初始布局的时候就在边界外，然后不显示（用于在缩放后判断）
        this.dataPointStyle = null;// layout之后 实例化
        this.lastGlyph = null;

        this.jsType = "DataPoint";
        this.surpportFocus = true;
        this.isCustomTooltip = false;//是否通过接口自定义数据点提示的内容
        this.isCustomLabel = false; //是否通过接口自定义的标签内容

        this.clickable = this.hyperlink != null;
        this.stockDataPointCount = 1;
    },


    revert2Original:function(){
        this.valueIsNull = this.js.valueIsNull || false;
        this.value = FR.pick(this.js.value, 0);
        this.percentValue = FR.pick(this.js.percentValue, 0);
        this.categoryName = this.js.categoryName;
        this.seriesName = this.js.seriesName;
        this.dataLabel = this.js.dataLabel == undefined ? null : new FR.Chart.TextGlyph(this.js.dataLabel);

        this.tooltip = null;// 只要在画结束时保证 已经创建完毕.
        this.glyph = null;
        this.lastGlyph = null;
        this.surpportFocus = true;
        this.clickable = this.hyperlink != null;
        this.stockDataPointCount = 1;
    } ,

    /**
     * 合并两个数据点(用于大数据量调整时同一系列不同分类的合同)
     * @param dataPoint 数据点
     */
    addDataPoint4StockCategory:function(dataPoint){
        this.addBaseDataPoint(dataPoint)
    },

    addBaseDataPoint:function(dataPoint){
        this.valueIsNull = this.valueIsNull && dataPoint.valueIsNull;
        this.lastGlyph = null;
        this.value = FR.pick(this.value, 0) + FR.pick(dataPoint.value);
        this.percentValue = FR.pick(this.percentValue, 0) + FR.pick(dataPoint.percentValue,0);
        this.tooltip = null;
        this.surpportFocus = true;
        this.clickable = this.hyperlink != null;
    },


    /**
     * 合并两个数据点
     * @param dataPoint 提供的用于合并的数据点
     */
    addDataPoint: function(dataPoint){
        this.addBaseDataPoint(dataPoint);
        this.seriesName = this.seriesName + "&" + dataPoint.seriesName;
    },

    setCustomLabel:function(label){
        if(this.dataLabel){
            this.dataLabel.text = label;
            this.isCustomLabel = true;

            //pie funnel
            if(this.isShowLabel){
                this.showLabel = label;
            }
        }
    },

    //根据具体的数据点替换参数
    resultParameter:function(parameters){
        if(parameters){
            var result = {};
            for(var para in parameters){
                var value = this.getValueByPara(parameters[para]);
                result[para] = value;
            }
            return result;
        }
    },

    getValueByPara:function(para){
        if(!this.parameters){
            this.parameters = {
                CATEGORY:this.categoryName,
                SERIES:this.seriesName,
                VALUE:this.value
            }
        }

        if(para.charAt(0) == '='){
            para = para.substr(1);
        }

        return '"' + this.parameters[para] + '"';
    },

    toResult:function(columnName){
        if (FR.equals(columnName, 'CategoryIndex')){
            return this.categoryIndex + 1;
        } else if (FR.equals(columnName, 'SeriesIndex')){
            return this.seriesIndex + 1;
        } else if (FR.equals(columnName, 'Value')){
            return this.value;
        } else if (FR.equals(columnName, 'CategoryName')){
            return this.categoryName;
        } else if (FR.equals(columnName, 'SeriesName')){
            return this.seriesName;
        } else if (FR.equals(columnName, 'ProjectID')){
            return this.seriesName;
        } else if (FR.equals(columnName, 'StepIndex')){
            return this.categoryIndex;
        }
        return "";
    },

    equals:function(obj){
        if(!obj){
            return false;
        }

        return obj.jsType == this.jsType
            && obj.seriesIndex == this.seriesIndex
            && obj.categoryIndex == this.categoryIndex;
    },

    /**
     * 为每个DataPoint生成热点提示文字. 每个提示的字体 内容
     * @param hotTooltipStyle 热点提示内容
     * @param outerBounds plot
     * @param style 样式
     */
    createHotTooltip: function (hotTooltipStyle, outerBounds, style) {
        if (hotTooltipStyle && hotTooltipStyle.seriesLabel) {
            var resultHTML = '';
            this.outerBounds = outerBounds;
            this.isTooltip4Series = false;
            this.tooltipStyle = style;
            var labelString = hotTooltipStyle.seriesLabel;
            var newLine = '<br />';
            resultHTML += this._createCateTooltipString() + newLine;
            if (this.seriesName) {
                resultHTML += this.seriesName + ': ';
            }
            if (labelString.indexOf('{VALUE}') != -1) {
                // 处理数字格式. style进行处理
                resultHTML += this._createValueTooltipString(this.value, hotTooltipStyle);
                if(labelString.indexOf('{PERCENT}') != -1){
                    resultHTML += ',';
                    resultHTML += this._createPercentTooltipString(this.percentValue, hotTooltipStyle);
                }
            } else if (labelString.indexOf('{PERCENT}') != -1) {// 只显示百分比
                resultHTML += this._createPercentTooltipString(this.percentValue, hotTooltipStyle)
            }
            this.tooltip = resultHTML;
        }
    },

    _createCateTooltipString:function(){
        var cateName = '';
        if (this.categoryName || this.secondCate || this.thirdCate) {
            if(this.thirdCate) {
                cateName += this.thirdCate + "  ";
            }
            if(this.secondCate) {
                cateName += this.secondCate + "  ";
            }
            if(this.categoryName) {
                cateName += this.categoryName;
            }
        }
        return cateName;
    },

    _createPercentTooltipString:function(percentValue, hotTooltipStyle){
        var percentString = '';
        if (hotTooltipStyle.percentFormat) {
            percentString = FR.contentFormat(percentValue, hotTooltipStyle.percentFormat);
        } else {
            percentString = FR.contentFormat(percentValue, '#.##%');
        }
        return percentString;
    },

    _createValueTooltipString:function(value, hotTooltipStyle){
        var valueTipString = "";
        if (hotTooltipStyle.format) {
            valueTipString = FR.contentFormat(value, hotTooltipStyle.format);
        } else {
            valueTipString = value + '';
        }
        return valueTipString;
    },

    createHotTooltip4Map:function(dataPoint4Map, isHeatMap, heatIndex, hotTooltipStyle){
        if(hotTooltipStyle && hotTooltipStyle.seriesLabel){

            var resultHTML = '';
            var newLine = '<br />';

            //区域名
            resultHTML += dataPoint4Map.categoryOriginalName + newLine;

            if(hotTooltipStyle.seriesLabel.indexOf('{VALUE}') != -1){
                resultHTML += this.seriesName + ': ' + this._createValueTooltipString(this.value, hotTooltipStyle) + newLine;
            }

            if(hotTooltipStyle.seriesLabel.indexOf('{PERCENT}') != -1){
                resultHTML += this.seriesName + FR.i18nText('Fine-Dec_Chart_DataFunction_Percent') + ': ' + this._createPercentTooltipString(this.percentValue, hotTooltipStyle) + newLine;
            }

            if(isHeatMap && dataPoint4Map.areaValue){
                var titleValue = dataPoint4Map.areaValue.getTitleValueByIndex(heatIndex);
                if(titleValue){
                    var title = titleValue.title;
                    if(hotTooltipStyle.seriesLabel.indexOf('{VALUE}') != -1){
                        resultHTML += title + ': ' + this._createValueTooltipString(dataPoint4Map.value, hotTooltipStyle) + newLine;
                    }

                    if(hotTooltipStyle.seriesLabel.indexOf('{PERCENT}') != -1){
                        resultHTML += title + FR.i18nText('Fine-Dec_Chart_DataFunction_Percent') + ': ' + this._createPercentTooltipString(dataPoint4Map.percentValue, hotTooltipStyle) + newLine;
                    }
                }
            }

            this.tooltip = resultHTML;
        }
    },

    /**
     * 为每个DataPoint生成热点提示文字. 每个提示的字体 内容
     * @param hotTooltipStyle 热点提示内容
     * @param outerBounds plot
     * @param style 样式
     */
    createHotTooltip4Series: function (hotTooltipStyle, outerBounds, style, colors) {
        if (hotTooltipStyle && hotTooltipStyle.seriesLabel) {
            this.outerBounds = outerBounds;
            this.isTooltip4Series = true;
            this.tooltipStyle = style;

            var series = outerBounds.dataSeries;
            var round = [];
            for(var i = 0; i < series.length; i++){
                round[i] = '<span style="font-size:12px; color: ' + colors[i] + '">'+'●  '+'</span>';
            }

            var resultHTML = '';
            var newLine = '<br />';
            var labelString = hotTooltipStyle.seriesLabel;
            resultHTML += this._createCateTooltipString() + newLine;

            for (var i = 0, seriesCount = series.length; i < seriesCount; i++) {

                if(series[i].paintState < FR.Chart.ChartPaintConstants.STATE_ENABLE){
                    continue;
                }

                var point = series[i].points[this.categoryIndex];

                if (series[i].seriesName) {
                    resultHTML += round[i] + series[i].seriesName + ': ';
                }

                if (labelString.indexOf('{VALUE}') != -1) {

                    resultHTML += this._createValueTooltipString(point.value, hotTooltipStyle);

                    if (labelString.indexOf('{PERCENT}') != -1) {
                        resultHTML += ',' + this._createPercentTooltipString(point.percentValue, hotTooltipStyle);
                    }
                } else if (labelString.indexOf('{PERCENT}') != -1) {// 只显示百分比
                    resultHTML += this._createPercentTooltipString(point.percentValue, hotTooltipStyle);
                }
                resultHTML += newLine;
            }

            this.tooltip = resultHTML;
        }
    },

    getValueParaString:function(attrContents, value, isPercent){
        if(attrContents && (attrContents.format || attrContents.percentFormat)){
            if(isPercent){
                return FR.contentFormat(value, attrContents.percentFormat);
            }else{
                return FR.contentFormat(value, attrContents.format);
            }
        }else{
            if(isPercent){
                return FR.contentFormat(value, '#.##%')
            }else{
                return FR.contentFormat(value, '#.##')
            }
        }
    },

    /**
     * 在前端创建标签字符串
     * @param attrContents
     */
    getDataPointLabelString:function(attrContents, separator){

        separator = separator || '';

        var sepSymbol = (separator.indexOf('{BR}') != -1) ? '\n' : separator;

        if(attrContents && attrContents.seriesLabel){
            var seriesLabel = attrContents.seriesLabel;
            var resultString = "";
            var dotCount = attrContents.seriesLabel.split(separator).length - 1;

            if(seriesLabel.indexOf('{CATEGORY}') != -1){
                resultString += this.categoryName;
                if(dotCount-- > 0){
                    resultString += sepSymbol;
                }
            }

            if(seriesLabel.indexOf('{SERIES}') != -1){
                resultString += this.seriesName;
                if(dotCount-- > 0){
                    resultString += sepSymbol;
                }
            }

            if(seriesLabel.indexOf('{VALUE}') != -1){
                resultString += this.getValueParaString(attrContents, this.value);
                if(dotCount > 0){
                    resultString += sepSymbol;
                }
            }

            if(seriesLabel.indexOf('{PERCENT}') != -1){
                resultString += this.getValueParaString(attrContents, this.percentValue, true);
            }

            return resultString;
        }
    },

    /**
     * 创建 冒号 热点格式
     * @returns {FR.Chart.TooltipString}
     */
    createMaoHao: function () {
        var maoHao = new FR.Chart.TooltipString();
        maoHao.value = " : ";
        maoHao.font.fontName = FR.i18nText('Fine-Dec_Chart_Song_TypeFace');
        maoHao.font.size = '12';

        return maoHao;
    },

    /**
     * 创建逗号 热点格式
     * @returns {FR.Chart.TooltipString}
     */
    createDouHao: function () {
        var douHao = new FR.Chart.TooltipString();
        douHao.value = "  ";
        douHao.font.fontName = FR.i18nText('Fine-Dec_Chart_Song_TypeFace');
        douHao.font.size = '12';

        return douHao;
    },

    /**
     * 设置数据点的 图形
     * @param shape 图形
     */
    setShape: function (shape) {
        var shapeGlyph = new FR.Chart.ShapeGlyph();
        shapeGlyph.shape = shape;
        this.glyph = shapeGlyph;
    },

    /**
     * 返回数据点的图形
     * @returns {Object} 图形
     */
    getShape: function () {
        return this.glyph == null ? null : this.glyph.getShape();
    },

    /**
     * 数据点响应点击
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    onClick: function (mousePos, event) {
        event.hyperlink = this.hyperlink;
    },

    /**
     * 在悬浮元素上 查找
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    findOnMoveGlyph: function (mousePos, event) {
        if (this.glyph && !this.valueIsNull
            && this.glyph.getShape()
            && this.containsPointJudge4Moved(mousePos.x, mousePos.y)) {

            if (this.surpportFocus) {
                event._lastHover = this;
                event._hasfound = true;
            }

            if (this.hyperlink != null) {
                event.cursor = ChartHandler.POINTER;
            }
        }
    },

    containsPointJudge4Moved: function (mousePos_x, mousePos_y) {
        var bounds = this.glyph.getShape();
        if(bounds.width <= FR.Chart.Constants.MarkerDefaultSize && bounds.height <= FR.Chart.Constants.MarkerDefaultSize) {
            //折线图超链不好点，扩大范围
            bounds = new FR.Chart.Bounds(bounds.x - 5, bounds.y - 5, bounds.width + 10, bounds.height + 10);
        }
       return bounds.containsPoint(mousePos_x, mousePos_y);
    },

    containsMousePos:function(mousePos){
        return this.glyph && !this.valueIsNull
                    && this.glyph.getShape()
                    && this.glyph.getShape().containsPoint(mousePos.x, mousePos.y) && this.surpportFocus ;
    } ,

    /**
     * 用于面积图和条形图，只要在绘图区，就选中最近的数据点进行提示，但是超链什么的一定要在
     * @param mousePos
     * @param event
     */
    dealTooltip:function(mousePos, event) {
        if (this.glyph && !this.valueIsNull) {
            if (this.surpportFocus) {
                event._lastHover = this;
                event._hasfound = true;
                event._isPlotAreaHover  = true;
            }
        }
    },

    /**
     * 在鼠标悬浮时 响应.
     * @param ctx4OnMove 画布
     * @param mousePos 鼠标位置
     */
    doOnMove:function(ctx4OnMove, mousePos, plotGlyph, mapOnMoveCtx) {
        //截取绘图区原因：放大后悬浮在某区域，超出绘图区范围到图表区了。bug：74495
        mapOnMoveCtx.save();// 整个动态开始前 保存, 以便于clip截取绘图区. 然后在整个动态(包括after 最后的标记结束后) 再复位.
        plotGlyph._saveClip4SeriesDrawThrowOutOfPlot(mapOnMoveCtx, plotGlyph);

        this.doOnMoveWithoutTooltip(ctx4OnMove, plotGlyph, mapOnMoveCtx)
        this.doTooltip(ctx4OnMove, mousePos, plotGlyph);

        mapOnMoveCtx.restore();// 整个动态结束 包括最后的标记类型, 复位.
    },

    doOnMoveWithoutTooltip:function(ctx4OnMove, plotGlyph, mapOnMoveCtx){
        if(this.glyph && this.glyph.shapeType && this.glyph.shapeType == "meter"){
            return;
        }

        if(plotGlyph && plotGlyph.dataSeries[this.seriesIndex].paintState < FR.Chart.ChartPaintConstants.STATE_ENABLE){
            return ;
        }

        if(this.glyph && this.glyph.marker && this.glyph.marker.getMarkerType){//kunsnat: 热点提示线型标记的单独画法
            this.glyph.drawOnMove = true;
            this.glyph.draw(ctx4OnMove);
            this.glyph.drawOnMove = false;
        } else {
            FR.Chart.HighUtils.doFocus(this.glyph, ctx4OnMove, mapOnMoveCtx);

            if(this.isDataPointStyle() && this.dataLabel && plotGlyph && plotGlyph.seriesLabelsInSide(this)){
                ctx4OnMove.notUseDivText = true;
                var originalBounds = this.dataLabel.bounds || {};
                //doFocus的时候往上扩了一个像素，所以label的边框画的时候降低一个像素，不然会和之前的错开，产生重影.加这边代码是柱形图设置风格渐变高光等的时候，标签没有显示出来
                this.dataLabel.bounds = new FR.Chart.Bounds(originalBounds.x,originalBounds.y+1,originalBounds.width,originalBounds.height);
                this.drawLabel(ctx4OnMove);
                this.dataLabel.bounds = originalBounds;
                ctx4OnMove.notUseDivText = false;
            }else if(this.glyph & this.glyph.isFade && mapOnMoveCtx){
                mapOnMoveCtx.notUseDivText = true;
                this.drawLabel(mapOnMoveCtx);
                mapOnMoveCtx.notUseDivText = false;
            }
        }
    },


    /**
     * 是否设置了风格
     */
    isDataPointStyle:function(){
       return this.glyph && this.glyph.dataPointStyle;
    },

    //画数据点提示
    doTooltip:function(ctx4OnMove, mousePos, plotGlyph){
        if(plotGlyph instanceof FR.Chart.MeterPlotGlyph){
            return ;
        }
        //没有画出的系列，不应弹出数据点提示
        if (this.outerBounds && this.outerBounds.dataSeries && plotGlyph && plotGlyph.isTooltipEnable
            && this.outerBounds.dataSeries[this.seriesIndex].paintState == FR.Chart.ChartPaintConstants.STATE_ENABLE) {
            this.outerBounds.currentChosenSeriesIndex = this.seriesIndex;
            if (this.isTooltip4Series) {
                var colors = [];
                var seriesIndex = 0;
                var left = this.outerBounds.bounds.x;
                var top = this.outerBounds.bounds.y + this.outerBounds.bounds.height;
                for (var i = 0; i < this.outerBounds.dataSeries.length; i++) {
                    var dataSeries = this.outerBounds.dataSeries[i];
                    if (dataSeries && dataSeries.points && dataSeries.paintState > FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                        var dataPoint = dataSeries.points[this.categoryIndex];
                        if (dataPoint && dataPoint.glyph) {
                            var glyph = dataPoint.glyph;
                            this.drawChosenPoint(dataPoint, ctx4OnMove);
                            var x = 0;
                            var y = 0;
                            if (glyph.shape) {
                                x = glyph.shape.x;
                                y = glyph.shape.y;
                            } else if (glyph.bounds) {
                                x = glyph.bounds.x;
                                y = glyph.bounds.y;
                            } else {
                                x = glyph.marker.x;
                                y = glyph.marker.y;
                            }
                            if (y < top || y == top && x > left) {
                                top = y;
                                left = x;
                                seriesIndex = i;
                            }
                            if (glyph && glyph.toolTipBackground) {
                                colors[i] = glyph.toolTipBackground.color;
                            } else {
                                glyph = this.outerBounds.dataSeries[i].glyph;
                                colors[i] = glyph.toolTipBackground.color;
                            }
                        }
                    }
                }
                if (this.outerBounds.dataSeries[seriesIndex] && this.outerBounds.dataSeries[seriesIndex].points[this.categoryIndex]) {
                    glyph = this.outerBounds.dataSeries[seriesIndex].points[this.categoryIndex].glyph;
                }
                if (glyph) {
                    FR.Chart.Tooltip.dataPointTip4Series(ctx4OnMove, this.tooltip, mousePos, glyph,
                        glyph.toolTipBackground, this.outerBounds, this.tooltipStyle, colors);
                }
            } else {
                if (this.glyph && this.glyph.toolTipBackground) {
                    if (this.glyph.colorInfo && this.glyph.colorInfo.seriesAttrColor) {
                        this.glyph.toolTipBackground = FR.Chart.ColorBackground.getInstance(this.glyph.colorInfo.seriesAttrColor.seriesColor);
                    }else if(this.glyph.marker && this.glyph.marker.background && this.glyph.marker.background.color){
                        this.glyph.toolTipBackground = FR.Chart.ColorBackground.getInstance(this.glyph.marker.background.color);
                    }
                }
                FR.Chart.Tooltip.dataPointTip4Point(ctx4OnMove, this.tooltip, mousePos, this.glyph,
                    this.glyph && this.glyph.toolTipBackground, this.outerBounds, this.tooltipStyle);
            }
        }
    },

    drawChosenPoint:function (point, ctx) {
        if(point.glyph && point.glyph.marker && point.glyph.marker.getMarkerType){//kunsnat: 热点提示线型标记的单独画法
            point.glyph.drawOnMove = true;
            point.glyph.draw(ctx);
            point.glyph.drawOnMove = false;
        } else {
            FR.Chart.HighUtils.doFocus(point.glyph, ctx);
        }
    },

    /**
     * 画出系列点 DataPoint
     * @param ctx 画布
     * @param time 动态时间
     */
    draw: function (ctx, time) {
        if (this.glyph && (!this.dataPointStyle || !this.dataPointStyle.avoidOriginDraw)) {
            this.glyph.draw(ctx, time);
        }

        if (this.dataPointStyle) {
            this.dataPointStyle.paintStyle(ctx);
        }
    },

    /**
     * 为缩略图画 datapoint.draw4Demo
     * @param ctx 画布
     */
    draw4Demo: function (ctx) {
        if (this.glyph && this.glyph.draw4Demo) {
            this.glyph.draw4Demo(ctx);
        }
    },

    /**
     * 返回动态步骤
     * @param step 动态步骤
     * @param actionModel 动态方式
     * @param paintState 动态状态
     */
    getAnimateGlyph: function (step, actionModel, paintState) {
        if (this.glyph || this.lastGlyph) {
            this.animateGlyph = (this.glyph || this.lastGlyph).getAnimateGlyph(step, actionModel, this.lastGlyph, paintState);
        }else{
            this.animateGlyph = [];
        }
    },

    /**
     * 画出动态Glyph
     * @param ctx 画布
     * @param count
     */
    drawAnimateGlyph: function (ctx, count) {
        if (this.animateGlyph && this.animateGlyph.length > 0 && count < this.animateGlyph.length) {
            this.animateGlyph[count].draw(ctx);
        }
    },

    /**
     * 边框是否是默认的
     * @param ctx 画布
     * @param count
     */
    isBorderDefault:function (count){
        return true;
    },

    //只画边框
    drawOnlyBorder:function(ctx,count){

    },

    /**
     * 画出数据标签
     * @param ctx 画布
     * @param currentDivKey 当前div的key
     */
    drawLabel: function (ctx, currentDivKey) {
        if (this.valueIsNull) {
            return;
        }

        if (this.dataLabel && this.dataLabel.visible) {
            this.dataLabel.draw(ctx, currentDivKey);
        }
    }
});

/**
 * 气泡图系列点
 *      @example
 *      var bubble = new FR.Chart.DataPoint4Bubble();
 *
 * @class FR.Chart.DataPoint4Bubble
 * @extends FR.Chart.DataPoint
 *
 * @param {JSON} js 配置属性
 * @param {Number} [js.x = 0] 气泡图x值
 * @param {Number} [js.sizeValue = 0] 气泡图大小值
 *
 */
FR.Chart.DataPoint4Bubble = FR.extend(FR.Chart.DataPoint, {
    initConstants: function (js) {
        js = js || {};

        this.x = FR.pick(js.x, 0);
        this.sizeValue = FR.pick(js.sizeValue, 0);
        FR.Chart.DataPoint4Bubble.superclass.initConstants.apply(this, [js]);
    },

    /**
     * 为每个DataPoint生成热点提示文字. 每个提示的字体 内容
     * @param hotTooltipStyle 热点内容
     * @param outerBounds plot
     * @param style 类型
     */
    createHotTooltip: function (hotTooltipStyle, outerBounds, style) {
        if (hotTooltipStyle && hotTooltipStyle.seriesLabel) {
            this.outerBounds = outerBounds;
            this.tooltipStyle = style;
            var labelString = hotTooltipStyle.seriesLabel;
            var resultHTML = '';
            if (this.seriesName) {
                resultHTML += this.seriesName;
                resultHTML += '<br />';
            }
            if (labelString.indexOf('{VALUE}') != -1) {
                resultHTML += "(" + this._createValueTooltipString(this.x, hotTooltipStyle)
                                        + ',' + this._createValueTooltipString(this.value, hotTooltipStyle) + ')' + this.sizeValue;
            }

            this.tooltip = resultHTML;
        }
    },

    /**
     * 在前端创建标签字符串
     * @param attrContents
     */
    getDataPointLabelString:function(attrContents, separator){
        if(attrContents && attrContents.seriesLabel){
            var seriesLabel = attrContents.seriesLabel;
            var resultString = "";

            var dotCount = attrContents.seriesLabel.split(separator).length - 1;

            if(seriesLabel.indexOf('{SERIES}') != -1){
                resultString += this.seriesName;
                if(dotCount > 0){
                    resultString += separator;
                }
            }

            if(seriesLabel.indexOf('{VALUE}') != -1){
                var valueString = "(" + this.getValueParaString(attrContents, this.x);
                valueString += ",";
                valueString += this.getValueParaString(attrContents, this.value) + ")";
                valueString += " size" + this.sizeValue;

                resultString += valueString;
            }

            return resultString;
        }
    }
});

/**
 * 散点图系列点
 *          @example
 *          var xy = new FR.Chart.DataPoint4XY();
 *
 * @class FR.Chart.DataPoint4XY
 * @extends FR.Chart.DataPoint
 *
 * @param {JSON} js 配置属性
 * @param {Number} [js.x = 0] 散点x值
 *
 */
FR.Chart.DataPoint4XY = FR.extend(FR.Chart.DataPoint, {
    initConstants: function (js) {
        js = js || {};

        this.x = FR.pick(js.x, 0);
        FR.Chart.DataPoint4XY.superclass.initConstants.apply(this, [js]);
    },

    /**
     * 为每个DataPoint生成热点提示文字. 每个提示的字体 内容
     * @param hotTooltipStyle 热点提示内容
     */
    createHotTooltip: function (hotTooltipStyle, outerBounds, style) {
        if (hotTooltipStyle && hotTooltipStyle.seriesLabel) {
            this.outerBounds = outerBounds;
            this.tooltipStyle = style;
            var labelString = hotTooltipStyle.seriesLabel;
            var newLine = '<br />';
            var resultHTML = '';
            if (this.seriesName) {
                resultHTML += this.seriesName + newLine;
            }

            if (labelString.indexOf('{VALUE}') != -1) {
                resultHTML += '(' + this._createValueTooltipString(this.x, hotTooltipStyle)
                                + ',' + this._createValueTooltipString(this.value, hotTooltipStyle) + ')';

            }

            this.tooltip = resultHTML;
        }
    },

    /**
     * 在前端创建标签字符串
     * @param attrContents
     */
    getDataPointLabelString:function(attrContents, separator){
        separator = separator || '';

        if(attrContents && attrContents.seriesLabel){
            var seriesLabel = attrContents.seriesLabel;
            var resultString = "";
            var dotCount = attrContents.seriesLabel.split(separator).length - 1;

            var sepSymbol = (separator.indexOf('{BR}') != -1) ? '\n' : separator;

            if(seriesLabel.indexOf('{SERIES}') != -1){
                resultString += this.seriesName;
                if(dotCount > 0){
                    resultString += sepSymbol;
                }
            }

            if(seriesLabel.indexOf('{VALUE}') != -1){
                var valueString = this.getValueParaString(attrContents, this.x);
                valueString += " " + this.getValueParaString(attrContents, this.value);

                resultString += valueString;
            }

            return resultString;
        }
    }
});

/**
 * 饼图系列点
 *          @example
 *          var pie = new FR.Chart.DataPoint4Pie();
 *
 * @class FR.Chart.DataPoint4Pie
 * @extends FR.Chart.DataPoint
 *
 * @param {JSON} js 配置属性
 * @param {Object} [js.leadLine = null] 牵引线
 * @param {Boolean} [js.isSeparateOut = false] 是否弹出
 *
 * @param {Boolean} [js.clickable = true] 默认支持可点击 (可弹出 + 超链)
 */
FR.Chart.DataPoint4Pie = FR.extend(FR.Chart.DataPoint, {
    initConstants: function (js) {
        js = js || {};

        this.leadLine = null;
        this.isSeparateOut = false;
        FR.Chart.DataPoint4Pie.superclass.initConstants.apply(this, [js]);
        this.isShowLabel = true;
        this.clickable = true;
        this.showLabel =this.dataLabel? this.dataLabel.text:null;
    },

    /**
     * 画出动态Glyph
     * @param ctx 画布
     * @param count
     */
    drawAnimateGlyph: function (ctx, count) {
        if (this.animateGlyph && this.animateGlyph.length > 0 && count < this.animateGlyph.length) {
            this.animateGlyph[count].drawWithOutBorder(ctx);
        }
    },


    //只画边框
    drawOnlyBorder:function(ctx,count){
        if (this.animateGlyph && this.animateGlyph.length > 0 && count < this.animateGlyph.length) {
            this.animateGlyph[count].drawOnlyBorder(ctx);
        }
    },

    /**
     * 边框是否是默认的
     */
    isBorderDefault:function (count){
        if (!this.animateGlyph || count >= this.animateGlyph.length) {
            return true;
        }
        return this.animateGlyph[count].isBorderDefault();
    },

    /**
     * 饼图显示值: 系列值, 值所占比率
     * @param hotTooltipStyle 热点提示内容
     */
    createHotTooltip: function (hotTooltipStyle, outerBounds, style) {
        if (hotTooltipStyle && hotTooltipStyle.seriesLabel) {
            this.outerBounds = outerBounds;
            var labelString = hotTooltipStyle.seriesLabel;
            this.tooltipStyle = style;
            var resultHTML = '';
            if (this.seriesName) {
                resultHTML += this.seriesName + ': ';
            }

            if (labelString.indexOf('{VALUE}') != -1) {
                resultHTML += this._createValueTooltipString(this.value, hotTooltipStyle);
                if (labelString.indexOf('{PERCENT}') != -1) {
                    resultHTML += ',' + this._createPercentTooltipString(this.percentValue, hotTooltipStyle);
                }
            } else if (labelString.indexOf('{PERCENT}') != -1) {// 只显示百分比
                resultHTML += this._createPercentTooltipString(this.percentValue, hotTooltipStyle);
            }

            this.tooltip = resultHTML;
        }
    },

    /**
     * 响应饼图 扇形 点击事件
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    onClick: function (mousePos, event) {
        event.hyperlink = this.hyperlink;
        this.isSeparateOut = !this.isSeparateOut;
        event.needRefresh = true;
    },

    /**
     * 查找 悬浮所在元素
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    findOnMoveGlyph: function (mousePos, event) {
        if (this.glyph && this.glyph.getShape() && this.glyph.getShape().containsPoint(mousePos.x, mousePos.y)) {
            event.cursor = ChartHandler.POINTER;

            event._lastHover = this;
            event._hasfound = true;
        }
    },




    /**
     * 画标签
     * @param ctx 画布
     * @param currentDivKey 当前div的key
     */
    drawLabel: function (ctx, currentDivKey) {
        if (this.valueIsNull) {
            return;
        }

        if (this.isShowLabel && this.dataLabel) {
            var originalText = this.dataLabel.text;
            this.dataLabel.text = this.showLabel;
            this.dataLabel.draw(ctx, currentDivKey);
            this.dataLabel.text = originalText;

            if (this.leadLine) {
                if(this.glyph.info.seriesAttrBackground && this.glyph.info.seriesAttrBackground.seriesBackground.color) {
                    var lineColor = new FR.Chart.AttrColor();
                    lineColor.seriesColor = this.glyph.info.seriesAttrBackground.seriesBackground.color;
                    this.leadLine.lineStyleInfo.attrLineColor = lineColor;
                    this.leadLine.draw(ctx);
                }
            }
        }
    }
});

/**
 * 漏斗图系列点
 *          @example
 *          var funnel = new FR.Chart.DataPoint4Funnel();
 *
 * @class FR.Chart.DataPoint4Funnel
 * @extends FR.Chart.DataPoint
 *
 * @param {JSON} js 配置属性
 * @param {Object} [js.leadLine = null] 牵引线
 */
FR.Chart.DataPoint4Funnel = FR.extend(FR.Chart.DataPoint, {
    initConstants: function (js) {
        js = js || {};

        FR.Chart.DataPoint4Funnel.superclass.initConstants.apply(this, [js]);
        this.leadLine = null;
        this.isShowLabel = true;
        this.showLabel =this.dataLabel? this.dataLabel.text:null;
        this.seriesString = js.seriesString;
        this.valueString = js.valueString;
        this.valueWidth = 0;
        this.seriesText = new FR.Chart.TextGlyph();
        this.valueText = new FR.Chart.TextGlyph();
        this.topColor = 'rgb(115,115,115)';
        this.lineColor = 'rgb(204,204,204)';
        this.seriesColor = 'rgb(0,0,0)';
    },

    /**
     * 漏斗图显示值: 系列值, 值所占比率
     * @param hotTooltipStyle 热点提示内容
     */
    createHotTooltip: function (hotTooltipStyle, outerBounds, style) {
        if (hotTooltipStyle && hotTooltipStyle.seriesLabel) {
            this.outerBounds = outerBounds;
            var labelString = hotTooltipStyle.seriesLabel;
            this.tooltipStyle = style;
            var resultHTML = '';
            var newLine = '<br />';

            if (this.seriesName) {
                resultHTML += this.seriesName + newLine;
            }

            if (labelString.indexOf('{VALUE}') != -1) {
                resultHTML += this._createValueTooltipString(this.value, hotTooltipStyle);
                if (labelString.indexOf('{PERCENT}') != -1) {
                    resultHTML += ',' + this._createPercentTooltipString(this.percentValue, hotTooltipStyle);
                }
            } else if (labelString.indexOf('{PERCENT}') != -1) {// 只显示百分比
                resultHTML += this._createPercentTooltipString(this.percentValue, hotTooltipStyle);
            }

            this.tooltip = resultHTML;
        }
    },

    /**
     * 响应漏斗图点击事件
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    onClick: function (mousePos, event) {
        event.hyperlink = this.hyperlink;
    },

    /**
     * 查找 悬浮所在元素
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    findOnMoveGlyph: function (mousePos, event) {
        if (this.glyph && this.glyph.getShape() && this.glyph.getShape().containsPoint(mousePos.x, mousePos.y)) {
            event.cursor = ChartHandler.POINTER;

            event._lastHover = this;
            event._hasfound = true;
        }
    },



    /**
     * 画标签
     * @param ctx 画布
     * @param currentDivKey 当前div的key
     */
    drawLabel: function (ctx, currentDivKey) {
        if (this.valueIsNull) {
            return;
        }

        if (this.isShowLabel && this.dataLabel) {
            if (this.valueText && this.valueText.text) {
                if(this.glyph.info.seriesAttrBackground && this.glyph.info.seriesAttrBackground.seriesBackground.color) {
                    var color = this.glyph.info.seriesAttrBackground.seriesBackground.color;
                    this.valueText.textAttr.frFont.color = color;
                    if (this.seriesIndex === 0) {
                        this.valueText.textAttr.frFont.color = this.topColor;
                    }
                }
                var text = this.valueText.text;
                var textAttr = this.valueText.textAttr;
                var bounds = this.valueText.bounds;
                FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, text, textAttr, bounds, currentDivKey);
            }
            if (this.seriesText && this.seriesText.text) {
                this.seriesText.textAttr = (this.dataLabel.textAttr);
                this.seriesText.textAttr.frFont.color = (this.seriesColor);
                if (this.seriesIndex === 0) {
                    this.seriesText.textAttr.frFont.color = this.topColor;
                }
                var text = this.seriesText.text;
                var textAttr = this.seriesText.textAttr;
                var bounds = this.seriesText.bounds;
                FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, text, textAttr, bounds, currentDivKey);
            }
            if (this.leadLine) {
                var lineColor = new FR.Chart.AttrColor();
                lineColor.seriesColor = this.lineColor;
                this.leadLine.lineStyleInfo.attrLineColor = lineColor;
                this.leadLine.draw(ctx);
            }
        }
    },

    /**
     * 获取标签系列&值
     * @return 系列&值
     */
    getSeriesValueString: function() {
        if(this.valueIsNull || !this.isShowLabel) {
            return null;
        }
        if (this.dataLabel == null || this.showLabel == null) {
            return null;
        }

        return [this.seriesString, this.valueString];
    },

    initLabelBounds: function() {
        if(this.valueIsNull || !this.isShowLabel) {
            return null;
        }
        if (this.dataLabel == null || this.dataLabel.bounds == null) {
            return null;
        }
        if (this.showLabel == null) {
            return null;
        }
        var rect = this.dataLabel.getBounds();
        if (this.valueString) {
            var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(this.valueString, this.dataLabel.textAttr);
            var x = rect.x + rect.width - this.valueWidth;
            var valueBounds = new FR.Chart.Bounds(x, rect.y, this.valueWidth + 5, dim.height);
            this.valueText.text = this.valueString;
            this.valueText.textAttr = this.dataLabel.textAttr;
            this.dealTextWithEllipsis(this.valueText, rect.width);
            this.valueText.bounds = (valueBounds);
        }
        if (this.seriesString) {
            var width = rect.width;
            if (this.valueText != null) {
                width -= this.valueWidth;
            }
            if (width > 0) {
                var seriesBounds = new FR.Chart.Bounds(rect.x, rect.y, width, rect.height);
                this.seriesText.text = this.seriesString;
                this.seriesText.textAttr = this.dataLabel.textAttr;
                this.dealTextWithEllipsis(this.seriesText, width);
                this.seriesText.bounds = (seriesBounds);
            }
        }
    },

    dealTextWithEllipsis: function(text, width){
        var string = text.text;
        var textWidth = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(string, text.textAttr).width;
        if (textWidth <= width) {
            return;
        }
        var index;
        for(index = string.length; index >= 0; index--){
            var tmp  = string.substring(0,index)+ FR.Chart.Constants.ESP;
            textWidth = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(tmp, text.textAttr).width;
            if(textWidth <= width){
                break;
            }
        }
        if(index === 0){
            text.text = (FR.Chart.Constants.ESP);
        }else{
            text.text = (string.substring(0, index) + FR.Chart.Constants.ESP);
        }
    }
});

/**
 * 甘特图系列点
 *
 *              @example
 *              var gantt = new FR.Chart.DataPoint4Gantt();
 *
 * @class FR.Chart.DataPoint4Gantt
 * @extends FR.Chart.DataPoint
 *
 * @param {JSON} js 配置属性
 *
 * @param {Number} [js.planStart] 计划开始日期值
 * @param {Number} [js.planEnd] 计划结束日期值
 * @param {Number}[js.realStart] 实际开始日期值
 * @param {Number}[js.realEnd] 实际结束日期值
 * @param {Object} [js.realStartImpl] 实际开始的标记
 * @param {Object} [js.realEndImpl] 实际结束的标记
 * @param {Object}[js.progressImpl] 进度的glyph
 */
FR.Chart.DataPoint4Gantt = FR.extend(FR.Chart.DataPoint, {
    initConstants: function (js) {
        js = js || {};

        this.planStart = FR.pick(js.planStart, -1);
        this.planEnd = FR.pick(js.planEnd, -1);
        this.realStart = FR.pick(js.realStart, -1);
        this.realEnd = FR.pick(js.realEnd, -1);

        this.realStartImpl = null;
        this.realEndImpl = null;
        this.progressImpl = null;
        FR.Chart.DataPoint4Pie.superclass.initConstants.apply(this, [js]);
    },

    /**
     * 创建热点提示内容,  甘特图无热点提示.
     * @param hotTooltipStyle 热点提示内容.
     */
    createHotTooltip: function (hotTooltipStyle) {
        // do nothing
    },

    /**
     * 画出甘特图系列点
     * @param ctx 画布
     * @param time 动态时间
     */
    draw: function (ctx, time) {
        if (this.glyph) {
            this.glyph.draw(ctx, time);
        }

        if (this.progressImpl) {
            this.progressImpl.draw(ctx, time);
        }
    },

    /**
     * 画出甘特图系列点的实际 时间 (在动态结束之后)
     * @param ctx 画布
     */
    drawReal: function (ctx) {
        if (this.realStartImpl) {
            this.realStartImpl.draw(ctx);
        }

        if (this.realEndImpl) {
            this.realEndImpl.draw(ctx);
        }
    },

    /**
     * 响应点击事件
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    onClick: function (mousePos, event) {
        event.hyperlink = this.hyperlink;
    },

    /**
     * 查找当前鼠标所在的Glyph
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    findOnMoveGlyph: function (mousePos, event) {
        if ((this.glyph && this.glyph.getShape() && this.glyph.getShape().containsPoint(mousePos.x, mousePos.y))
            || (this.realStartImpl && this.realStartImpl.getShape().containsPoint(mousePos.x, mousePos.y))
            || (this.realEndImpl && this.realEndImpl.getShape().containsPoint(mousePos.x, mousePos.y))
            || (this.progressImpl && this.progressImpl.getShape().containsPoint(mousePos.x, mousePos.y))) {
            if (this.hyperlink != null) {
                event.cursor = ChartHandler.POINTER;
                event._hasfound = true;
                event._lastHover = this;
            }
        }
    },

    /**
     * 返回动态的Glyph数组
     * @param step 步骤个数
     * @param actionModel 动态方式
     */
    getAnimateGlyph: function (step, actionModel) {
        if (this.glyph) {
            this.animateGlyph4Plan = this.glyph.getAnimateGlyph(step, actionModel);
        }

        if (this.progressImpl) {
            this.animateGlyph4Progress = this.progressImpl.getAnimateGlyph(step, actionModel);
        }

        this.animateGlyph = this.animateGlyph4Progress || this.animateGlyph4Plan;
    },

    /**
     * 画出动态方式.
     * @param ctx 画布
     * @param count 动态序号
     */
    drawAnimateGlyph: function (ctx, count) {
        if (this.animateGlyph4Plan && this.animateGlyph4Plan.length > 0 && count < this.animateGlyph4Plan.length) {
            this.animateGlyph4Plan[count].draw(ctx);
        }
        if (this.animateGlyph4Progress && this.animateGlyph4Progress.length > 0 && count < this.animateGlyph4Progress.length) {
            this.animateGlyph4Progress[count].draw(ctx);
        }
    }
});

/**
 * gis地图系列点
 *              @example
 *              var gismap = new FR.Chart.DataPoint4GisMap();
 *
 * @class FR.Chart.DataPoint4GisMap
 * @extends FR.Chart.DataPoint
 *
 * @param {JSON} js 配置属性
 * @param {String} [js.mapAreaValue] 地图区域值
 * @param {String} [js.addressName] 地图区域名
 */
FR.Chart.DataPoint4GisMap = FR.extend(FR.Chart.DataPoint, {
    initConstants: function (js){
        js = js || {};
        this.mapAreaValue = new FR.Chart.MapAreaValue(js.mapAreaValue);
        this.addressName = FR.pick(js.addressName, "");
        this.location = null;
        this.lastOverlay = null;
        this.map = null;
        this.ComplexCustomOverlay = null;
        this.attrContents = null;
        FR.Chart.DataPoint4GisMap.superclass.initConstants.apply(this, [js]);
    },

    setCustomLabel:function(label){
        if(this.dataLabel){
            this.dataLabel.text = label;
            this.isCustomLabel = true;
        }

        //删除之前的overlay
        if(this.location && this.map && this.attrContents){
            var Overlay = this.ComplexCustomOverlay;
            this.map.removeOverlay(this.lastOverlay);
            var newOverlay = new Overlay(this.location, this, this.attrContents);
            this.map.addOverlay(newOverlay);
            this.lastOverlay = newOverlay;
        }

    }
});


/**
 * 地图系列点
 *          @example
 *          var map = new FR.Chart.DataPoint4Map();
 *
 * @class FR.Chart.DataPoint4Map
 * @extends FR.Chart.DataPoint
 *
 * @param {JSON} js 配置属性
 * @param {Boolean} [js.hasNextTo] 地图区域系列点 是否有钻取下一层
 * @param {String} [js.nameLayerTo] 地图区域系列点 钻取下一层的名字
 * @param {String} [js.nameFrom] 地图区域系列点 来源钻取的名字
 * @param {Number} [js.layerIndex] 地图的钻取层级
 * @param {Number} [js.areaValue] 地图的区域值
 * @param {Object} [js.leadLine] 必要时的 标签牵引线
 * @param {Boolean} [js.clickable] 地图在是否钻取 是否有超链时, 支持可点击
 */
FR.Chart.DataPoint4Map = FR.extend(FR.Chart.DataPoint, {
    initConstants: function (js) {
        js = js || {};

        this.hasNextTo = FR.pick(js.hasNextTo, false);
        this.nameLayerTo = FR.pick(js.nameLayerTo, "");
        this.nameFrom = FR.pick(js.nameFrom, "");
        this.layerIndex = js.layerIndex || 0;

        this.areaValue = new FR.Chart.MapAreaValue(js.areaValue);

        FR.Chart.DataPoint4Map.superclass.initConstants.apply(this, [js]);

        this.leadLine = null;
        this.clickable = this.hasNextTo || this.hyperlink;

        if(js.combineGlyph){
            this.combineGlyph = new FR.Chart.ChartGlyph(js.combineGlyph);
        }
    },

    /**
     * 为每个DataPoint生成热点提示文字. 每个提示的字体 内容
     * 区域值 域 百分比 或者 区域值, 百分比   (其中都包括多哎区域标题和多区域值.
     * 数据格式 百分比数据格式 只第一个 区域之是用.
     * @param hotTooltipStyle 热点提示内容
     */
    createHotTooltip: function (hotTooltipStyle, outerBounds, style) {
        if (hotTooltipStyle && hotTooltipStyle.seriesLabel) {
            this.outerBounds = outerBounds;
            this.tooltipStyle = style;
            var labelString = hotTooltipStyle.seriesLabel;
            var newLine = '<br />';
            var resultHTML = this.categoryOriginalName + newLine;
            var heatIndex = outerBounds.heatIndex;
            if(this.areaValue.getTitleValueByIndex(heatIndex)){
                var titleValue = this.areaValue.getTitleValueByIndex(heatIndex);
                if (labelString.indexOf('{VALUE}') != -1) {
                    resultHTML += titleValue.title + ':' + this._createValueTooltipString(this.value, hotTooltipStyle);
                    if(labelString.indexOf('{PERCENT}') != -1) {
                        resultHTML += ',' + this._createPercentTooltipString(this.percentValue, hotTooltipStyle);
                    }
                } else if (labelString.indexOf('{PERCENT}') != -1) {
                    resultHTML += titleValue.title + ':' + this._createPercentTooltipString(this.percentValue, hotTooltipStyle);
                }
            }

            if(this.combineGlyph == null){
                var titleValues = this.areaValue.titleValues;
                for(var i = 0; i < titleValues.length; i++){
                    if(i === heatIndex){
                        continue;
                    }
                    var titleValue = titleValues[i];
                    resultHTML += newLine;
                    if (labelString.indexOf('{VALUE}') != -1) {
                        resultHTML += titleValue.title + ':' + this._createValueTooltipString(titleValue.value, hotTooltipStyle);
                        if(labelString.indexOf('{PERCENT}') != -1) {
                            resultHTML += ',' + this._createPercentTooltipString(titleValue.percentValue, hotTooltipStyle);
                        }
                    } else if (labelString.indexOf('{PERCENT}') != -1) {
                        resultHTML += titleValue.title + ':' + this._createPercentTooltipString(titleValue.percentValue, hotTooltipStyle);
                    }
                }
            }

            this.tooltip = resultHTML;
        }
    },

    /**
     * 查找当前移动所在的Glyph
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     * @param ctx4OnMove 画布
     */
    findOnMoveGlyph: function (mousePos, event, ctx4OnMove) {
        if(this.combineGlyph){
            this.combineGlyph.findOnMoveGlyph(mousePos, event);
        }

        if(!event._hasfound) {
            FR.Chart.DataPoint4Map.superclass.findOnMoveGlyph.apply(this, [mousePos, event, ctx4OnMove]);
        }
    },

    //地图不用containsPoint上下减5像素,用contains。
    containsPointJudge4Moved: function (mousePos_x, mousePos_y) {
        return this.glyph.getShape().contains(mousePos_x, mousePos_y, 1, 1);
    },

    /**
     * 响应点击事件
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    onClick: function (mousePos, event) {// 点击之后 刷新ChartGlyph  当超链和钻取共存时  添加列表
        event.toNext = true;
        event.hyperlink = this.hyperlink;
    },

    /**
     * 地图系列标签
     * @param ctx 画布
     * @param currentDivKey 当前div的key
     */
    drawLabel: function (ctx, currentDivKey) {
        if (this.valueIsNull) {
            return;
        }

        if (this.leadLine) {
            this.leadLine.draw(ctx);
        }
        if (this.dataLabel) {
            this.dataLabel.draw(ctx, currentDivKey);
        }
    }
});

/**
 * 股价图系列点
 *              @example
 *              var stock = new FR.Chart.DataPoint4Stock();
 * @class FR.Chart.DataPoint4Stock
 * @extends FR.Chart.DataPoint
 *
 * @param {JSON} js 配置属性
 * @param {Array} [js.values] 成交量 开盘 盘高 盘低 收盘
 * @param {Object} [js.highLowLine] 盘高盘低线
 * @param {Object}[js.volumeGlyph] 成交量
 */
FR.Chart.DataPoint4Stock = FR.extend(FR.Chart.DataPoint, {
    initConstants: function (js) {

        this.js = js || {};
         // 成交量 开盘 盘高 盘底 收盘  对应存储
        this.values = [0, 0, 0, 0, 0];
        if (this.js.values) {
            for (var i = 0; i < this.js.values.length; i++) {
                this.values[i] = this.js.values[i];
            }
        }

        this.stockLabels = [FR.i18nText('Fine-Dec_Chart_Stock_Volume'),FR.i18nText('Fine-Dec_Chart_Stock_Open'),
            FR.i18nText('Fine-Dec_Chart_Stock_High'),FR.i18nText('Fine-Dec_Chart_Stock_Low'),FR.i18nText('Fine-Dec_Chart_Stock_Close')];

        if(this.js.stockLabels){
            for(var i = 0;i<this.js.stockLabels.length;i++){
                this.stockLabels[i] = this.js.stockLabels[i];
            }
        }

        this.highLowLine = null;// 盘高盘底线
        this.volumeGlyph = null;// 成交量
        this.choosedBounds = null;
        FR.Chart.DataPoint4Stock.superclass.initConstants.apply(this, [js]);
    },

    revert2Original:function(){
        // 成交量 开盘 盘高 盘底 收盘  对应存储
        if (this.js.values) {
            for (var i = 0; i < this.js.values.length; i++) {
                this.values[i] = this.js.values[i];
            }
        }

        this.stockLabels = [FR.i18nText('Fine-Dec_Chart_Stock_Volume'),FR.i18nText('Fine-Dec_Chart_Stock_Open'),
            FR.i18nText('Fine-Dec_Chart_Stock_High'),FR.i18nText('Fine-Dec_Chart_Stock_Low'),FR.i18nText('Fine-Dec_Chart_Stock_Close')];

        if(this.js.stockLabels){
            for(var i = 0;i<this.js.stockLabels.length;i++){
                this.stockLabels[i] = this.js.stockLabels[i];
            }
        }

        this.highLowLine = null;// 盘高盘底线
        this.volumeGlyph = null;// 成交量
        this.choosedBounds = null;
        FR.Chart.DataPoint4Stock.superclass.revert2Original.apply(this);
    },

    addDataPoint4StockCategory:function(dataPoint){
        this.addBaseDataPoint(dataPoint)
        //成交量求和,开盘取 "月初"或者 "年初"第一天的数据,收盘取"月末"或者"年末"最后一天的数据。
        //盘高取当月或者当年 最高值。 盘低取当月或者当年最低值。
        this.values[0] += dataPoint.values[0];
        this.values[2] = Math.max(this.values[2],dataPoint.values[2]);
        this.values[3] = Math.min(this.values[3],dataPoint.values[3]);
        this.values[4] = dataPoint.values[4];
    },

    _createStockInfo:function(infoIndex, hotTooltipStyle){
        var newLine = '<br />'
        var resultHTML = this.stockLabels[infoIndex] + ':';
        if (!isNaN(this.values[infoIndex])) {
            resultHTML += this._createValueTooltipString(this.values[infoIndex], hotTooltipStyle);
        }
        resultHTML += newLine;
        return resultHTML;
    },

    /**
     * 创建热点提示
     * @param hotTooltipStyle 热点提示内容
     */
    createHotTooltip: function (hotTooltipStyle, outerBounds, style) {
        if (hotTooltipStyle && hotTooltipStyle.seriesLabel) {
            this.outerBounds = outerBounds;
            this.tooltipStyle = style;
            var newLine = '<br />';
            var resultHTML = '';
            resultHTML += this.categoryName + newLine;
            for(var i = 1; i < 5; i++){
                resultHTML += this._createStockInfo(i, hotTooltipStyle);
            }
            resultHTML += this._createStockInfo(0, hotTooltipStyle);
            this.tooltip = resultHTML;
        }
    },

    /**
     * 画出股价图系列点
     * @param ctx 画布
     * @param time 动态时间
     */
    draw: function (ctx, time) {
        if (this.highLowLine) {
            this.highLowLine.draw(ctx, time);
        }

        if (this.volumeGlyph) {
            this.volumeGlyph.draw(ctx, time);
        }

        FR.Chart.DataPoint4Stock.superclass.draw.apply(this, [ctx, time]);
    },

    /**
     * 为缩略图画  datapoint.draw4demo stock
     * @param ctx 画布
     */
    draw4Demo: function (ctx) {
        if (this.highLowLine) {
            this.highLowLine.draw4Demo(ctx);
        }

        if (this.volumeGlyph) {
            this.volumeGlyph.draw4Demo(ctx);
        }

        FR.Chart.DataPoint4Stock.superclass.draw4Demo.apply(this, [ctx]);
    },

    /**
     * 响应点击事件
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    onClick: function (mousePos, event) {
        event.hyperlink = this.hyperlink;
    },

    /**
     * 查找当前移动的Glyph
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    findOnMoveGlyph: function (mousePos, event) {
        if(this.glyph && this.choosedBounds.containsPoint(mousePos.x,mousePos.y)){
            if (this.hyperlink != null) {
                event.cursor = ChartHandler.POINTER;
            }
            event._hasfound = true;
            event._lastHover = this;
        }
    },

    /**
     * 返回动态数组
     * @param step 动态步骤
     * @param actionModel 动态方式
     */
    getAnimateGlyph: function (step, actionModel) {
        if (this.glyph) {
            this.animateGlyph = this.glyph.getAnimateGlyph(step, actionModel);
        }else{
            this.animateGlyph = null;
        }

        if (this.highLowLine) {
            this.animateGlyph4HighLowLine = this.highLowLine.getAnimateGlyph(step, actionModel);
        }else{
            this.animateGlyph4HighLowLine = null;
        }

        if (this.volumeGlyph) {
            this.animateGlyph4Volume = this.volumeGlyph.getAnimateGlyph(step, actionModel);
        }else{
            this.animateGlyph4Volume = null;
        }
    },

    /**
     * 画出动态步骤
     * @param ctx 画布
     * @param count 动态序号
     */
    drawAnimateGlyph: function (ctx, count) {
        if (this.animateGlyph4HighLowLine && this.animateGlyph4HighLowLine.length > 0 && count < this.animateGlyph4HighLowLine.length) {
            this.animateGlyph4HighLowLine[count].draw(ctx);
        }
        if (this.animateGlyph4Volume && this.animateGlyph4Volume.length > 0 && count < this.animateGlyph4Volume.length) {
            this.animateGlyph4Volume[count].draw(ctx);
        }
        if (this.animateGlyph && this.animateGlyph.length > 0 && count < this.animateGlyph.length) {
            this.animateGlyph[count].draw(ctx);
        }
    }
});