;(function($){
    /**
     * 图表 表单初始化控件, 用于在Form指定位置 展示图表
     *
     *     @example
     *     var $root = $('<div>').appendTo('body');
     *     var chart = new FR.ChartWidget({}, $root);
     *
     * @class FR.ChartWidget
     */
    FR.ChartWidget = FR.extend(FR.Widget, /**@class FR.ChartWidget*/{
        _init: function () {
            FR.ChartWidget.superclass._init.apply(this, arguments);
            this.loadData();
            this._init4Style();
        },

        /**
         * 初始化布局样式，包括背景，边框，圆角
         * @private
         */
        _init4Style: function () {
        	this._initBackGround();
            var border = this.options.border;
            if (border) {
                this.element.css('border-style', border.type);
                this.element.css('border-color', border.color);
                this.element.css('border-width', border.width);
                this.element.css('border-radius', border.borderRadius);
            }
            //CHART-2764 老图表z-index设置为0，与控件保持一致
                        this.element.css('z-index', 0);
        },
        _initBackGround: function () {
        	//设置控件背景
            if (!this.options.widgetBackground) {
            	return;
            }

            //因为有透明度, 如果直接设置opacity属性, 会被子层div继承, 因此平级放一个背景div
            // 无论是否有透明度，都平级放一个背景div。
            // 因为ie兼容模式下FR.setBackground用的一个单独的img标签，addChartComp时element.empty()会把背景给去掉
            this.$background = $("<div class='widgetBackground'></div>");
            this.$background.prependTo(this.element);
            this.$background.css('position', 'absolute');
            FR.setBackground(this.$background,  this.options.widgetBackground, this.element.height());

            var alpha = this.options.widgetOpacity;
            if(alpha === undefined){
            	return;
            }

        	//IE
            this.$background.css('filter', 'alpha(opacity=' + alpha * 100  +')');
            //Chrome ff
            this.$background.css('opacity', alpha);
        },
        /**
         * 加载数据
         */
        loadData: function () {
            var o = this.options;
            if (o.width && o.height) {
                this.element.width(o.width);
                this.element.height(o.height);
                this.reload();
            }
        },

        /**
         * 设置依赖参数 重新加载
         * @param source 依赖参数
         */
        setSource: function (source) {
            this.options.dependPara = source;
            this.reload();
        },

        /**
         * 返回依赖参数 宽度 高度
         * @returns {Json} 依赖参数
         */
        getDependence: function () {
            if (this.options.dependPara) {
                // 说明
                var depO = this.options.dependPara;
                depO.width = this.options.width;
                depO.height = this.options.height;
                delete this.options.dependPara;
                return depO;
            }

            var depO = {};
            if (FR.isArray(this.options.widgetExeDependence)) {
                for (var idx = 0; idx < this.options.widgetExeDependence.length; idx++) {
                    var dep = this.options.widgetExeDependence[idx];
                    var val = this.options.form.resolveVariable(dep);
                    if (val != "fr_primitive" && val != null) {
                        depO[dep.toUpperCase()] = val;
                    }
                }
            }
            return {
                para: depO,
                width: this.options.width,
                height: this.options.height
            };
        },

        //表单计算联动的图表, 表单其他控件联动图表一定会发请求，忽略oldDependence
        executeChart: function () {
            this.chartWidget && this.chartWidget.destroy && this.chartWidget.destroy();
            this.reload(true);
        },

        /**
         * 重新加载
         */
        reload: function (ignoreOldDependence) {

            //图表组件的宽度存在，并且大于0，然后reload
            var width = this.options.width, height = this.options.height;
            if (!width || !height || width < 0 || height < 0) {
                return;
            }

            var resizeTime = new Date();
            this.lastResizeTime = resizeTime;
            var self = this;
            setTimeout(function () {
                if (resizeTime == self.lastResizeTime) {
                	delete self.lastResizeTime;
    	            var dep = self.getDependence();
			        if (FR.equals(self.oldDependence, dep) && !ignoreOldDependence) {
			            return;
			        }

			        self.oldDependence = dep;

		            var para = {};
                    //对表单中图表para的处理参照报表块
                    //form.js collectionValue
		            for (var w in dep.para) {
		                var value = dep.para[w];
		                if (w.startWith("$")) {
		                    w = w.substring(1)
		                }
                        if (value == undefined) {
                            value = null;
                        }
		                para[w] = value;
		            }
		            var sourceUrl = FR.buildServletUrl({
		                op: 'fr_form',
		                cmd: 'form_getsource',
		                __chartsourcename__: self.options.widgetName,
		                __chartsize__: {
		                    width: self.options.width,
		                    height: self.options.height
		                }
		            });
		            //不同步，firefox下加载时，背景会闪动
		            FR.ajax({
		                url: sourceUrl,
		                type: 'POST',
		                data: {
		                    __parameters__: FR.jsonEncode(para)
		                },
                        headers: {
                            sessionID: self.options.form.sessionID
                        },
		                // 数据量很大的时候，这里的Ajax请求时间很长
		                // 这里加一个fr的loading,缓解一下请求过程中页面空白的尴尬
		                beforeSend: function() {
                            // 开始加载动画
                            FR.HtmlLoader.loadingEffect({
                                el: self.element,
                                show: true,
                                delay: 1000,
                                loadingType: 'local'  // 局部加载动画
                            });
		                },
		                complete: function (res, status) {
		                    if (!res.responseText) {
		                        return;
		                    }
		                    if (status == 'success') {
		                        self.chart = FR.jsonDecode(res.responseText);
		                        if(self.chart.length === 0){
		                        	self.element.html(res.responseText);
		                        	return;
		                        }
                                self.chart.AnimateAttr = self.options.AnimateAttr;

                                self.addChartComp();

                                var animateProcessor = FR.Report.Plugin.AnimateProcessor;
                                if (animateProcessor.item && FR.Plugin.validLevel(animateProcessor, animateProcessor.item)) {
                                    animateProcessor.item.action.call(self, self.element);
                                }

                                FR.HtmlLoader.loadingEffect({
                                    el: self.element,
                                    overflow:'hidden'
                                });
		                    }
		                }
		            });
                }
            }, 100);
        },

        /**
         * 添加图表控件
         */
        addChartComp: function () {
            if (!this.chart) {
                return;
            }
            try {
                this.chartWidget = FR.createWidget(this.chart, !!FR.SimpleChart);
            } catch(e) {
                this.chartWidget = new FR.SimpleChart(this.chart);
            }
            this.element.empty();
            if(this.$background){
                this.$background.css({
                    //@Mango CHART-1776，修改下获取宽高方式，之前获取方式有问题，会影响搜索框样式。。
                    width: this.chart.chartWidth,
                    height: this.chart.chartHeight
                });
                this.$background.appendTo(this.element);
            }
            this.chartWidget.element.appendTo(this.element);
        },

        //图表超链表单对象 表单对象为图表块
        formHyperlink: function () {
            var params = arguments[0];
            params.idInfo = {
                type: "CHART_EDITOR",
                name: params.widgetName
            };
            FR.Chart.WebUtils.changeParameter(params);
        },

        /**
         * 重置
         */
        reset: function () {
            this.reload();
        },

        /**
         * 重新设定大小
         * @param size
         */
        doResize: function (size) {
	        if (this.options.width == size.width
	            && this.options.height == size.height) {
	            return;
	        }
            var resizeTime = new Date();
            this.lastResizeTime = resizeTime;
            var self = this;
            setTimeout(function () {
                if (resizeTime == self.lastResizeTime) {
                	delete self.lastResizeTime;

                	// 表单图表控件的边框阴影是画在框架外部，即上层div中画，有标题的话不单独画
			    	if (!(FR.Browser.isIE && FR.Browser.isIE8Before()) && self.options.border) {
			    		//初始化时候父元素为空，resize时加上
			        	self.element.parent().css('box-shadow', self.options.border.borderStyle);
			        }

                    //REPORT-27528 这边需要更新下图表块隐藏时宽高为0的情形
                    if (size.width >= 0) {
                        self.options.width = size.width;
                        self.element.width(size.width);
                    }
                    if (size.height >= 0) {
                        self.options.height = size.height;
                        self.element.height(size.height);
                    }
                    
                    if (self.$background) {
                        self.$background.css('width', size.width);
                        self.$background.css('height', size.height);
                    }

                    if(self.chartWidget){
                        self.chartWidget.resize(size.width, size.height);
                    }else{
                        self.reload();
                    }

                }
            }, 100);
        }
    });
    $.shortcut("chartwidget", FR.ChartWidget);
})(jQuery);
