// ChartHandler.start
/**
 * 图表 事件封装
 * @class ChartHandler
 * @param {jQuery} dom 图表所在的jQuery节点
 * @param chartGlyph  当前dom中展示的图表
 * @returns {ChartHandler}  返回自身
 * @constructor
 */
function ChartHandler(dom, chartGlyph, backgroundGlyph) {

    var root = dom[0];
    var self = this;
    this._chartGlyph = chartGlyph;
    this._chartGlyph.handler = this;
    this._backgroundGlyph = backgroundGlyph;
    this.getTooltipDiv = function(){
        return _tooltipDiv;
    };

    var _tooltipDiv = null;
    var _tooltipHideTick = null;
    var _tooltipDisplayTick = null;

    var _lastHover = null;  // 上次 悬浮over中的元素.
    var _isPlotAreaHover = false; //是否是在绘图区就判断的选中 ，用于折线图和面积图,dealToolTip

    var _isMouseDown = false;// 鼠标是否按压
    var _downPosition = null;// 鼠标按压时的初始位置
    var _hasfound = false; // 是否找到底层的Glyph 用于拖动 点击等.

    var _draggingTarget = null; // 拖拽目标
    var _isDragging = false; // 是否在拖拽状态.
    var _isMouseMoving = false; //判断鼠标是否在移动
    var _zoomMoving = false;


    // 选中 --->  选中悬浮响应 (hasFoundOver)---> 选中拖拽响应 (hasFoundDrag)  TODO

    // 鼠标变化
    // 超级链接  && 饼图可点击
    // 按压可拖动

    // 为了响应 拖动事件, 在某Glyph范围内, 可以拖拽. 滚动.

    var DRAG = "url(\"" + FR.servletURL + "?op=resource&resource=/com/fr/web/cursor/cursor_drag_hand.cur\"),auto";
    var HAND = "url(\"" + FR.servletURL + "?op=resource&resource=/com/fr/web/cursor/cursor_hand.cur\"),auto";

    /**
     * 初始化，事件绑定，支持的所有事件都由如下原生事件计算得来
     *
     * 1.不论鼠标指针离开被选元素还是任何子元素，都会触发 mouseout 事件。2.只有在鼠标指针离开被选元素时，才会触发 mouseleave 事件。
     * 所以这边应该用mouseleave，而不是mouseout,不然拖拽的时候才离开绘图区且没有离开图表区的时候就会被判定拖拽移除。
     *
     */
    function _init() {
        if (root.addEventListener) {
            root.addEventListener('click', _clickHandler);
            root.addEventListener('mousemove', _mouseMoveHandler);
            root.addEventListener('mousedown', _mouseDownHandler);
            root.addEventListener('mouseup', _mouseUpHandler);
            root.addEventListener('mouseleave', _mouseLeaveHandler);
            root.addEventListener('DOMMouseScroll',_mouseWheelHandler);
        }
        else {
            root.attachEvent('onclick', _clickHandler);
            root.attachEvent('onmousemove', _mouseMoveHandler);
            root.attachEvent('onmouseleave', _mouseLeaveHandler);
            root.attachEvent('onmousedown', _mouseDownHandler);
            root.attachEvent('onmouseup', _mouseUpHandler);
            root.attachEvent('onmousewheel',_mouseWheelHandler);
        }

        root.onmousewheel=_mouseWheelHandler;

        _tooltipDiv = document.createElement('div');
        _tooltipDiv.onselectstart = function(){
            return false;
        }
        root.appendChild(_tooltipDiv);
    }


    /**
     * 点击事件
     * @param {event} event dom事件对象
     */
    function _clickHandler(event) {
        _isMouseMoving = false;
        _tooltipDiv.style.display = 'none';
        clearTimeout(_tooltipHideTick);
        clearTimeout(_tooltipDisplayTick);

        //单独的背景的事件处理
        if(self._backgroundGlyph){
            var mousePos = _fixedPosition(event);
            self._backgroundGlyph.doOnClick(mousePos);
        }

        self.needRefresh = false;
        self.needAnimateRefresh = true;
        self.needRefreshLabel = false;
        self.hyperlink = null;
        self.toNext = false;
        self.returnBack = false;
        if (!_lastHover) {
            return;
        }

        // 直接从当前的悬浮over元素中,  响应点击事件.
        if (_lastHover && _lastHover.onClick && !_isPlotAreaHover) {
            var mousePos = _fixedPosition(event);
            _lastHover.onClick(mousePos, self);

            //组合地图
            if(_lastHover.dataPoint4Map){
                self.hyperlink = _lastHover.dataPoint4Map.hyperlink;
                self.toNext = true;
            }
        }

        _doHyperlink(event);

        if (self.toNext) {
            layerToDataPointNext(_lastHover);
        } else if (self.returnBack) {
            returnBackToLayer();
        }

        if(self.needRefresh){
            if(self.needRefreshLabel && self._chartGlyph.plotGlyph){
                self._chartGlyph.plotGlyph.refreshDataPointLabel();
            }
            if(self.needAnimateRefresh){
                _refresh();
            }else{
                refreshWithoutAnimate();
            }
        }

    }


    function layerToDataPointNext(dataPoint) {
        dataPoint = FR.pick(dataPoint.dataPoint4Map ,dataPoint);
        self._chartGlyph.layerToDataPointNext(dataPoint, self);

        if (!FR.Browser.isIE()) {
            doRefreshIfNeed();
        }
    }

    function returnBackToLayer() {
        self._chartGlyph.returnBackToLayer();
        doRefreshIfNeed();
    }

    function _doHyperlink(event) {
        if (self.hyperlink) {
            FR.doHyperlink(event, (new Function("return " + self.hyperlink))(), true);
        }
    }

    /**
     * 鼠标 移动事件
     *
     * 鼠标连续快速拖拽，会出现一个禁止操作的小提示，不能再拖动，导致mouseup不能释放,
     * 这是浏览器的默认行为，因此只要阻止浏览器默认行为就可以了.event.preventDefault();
     *
     * @param event
     * @private
     */
    function _mouseMoveHandler(event) {

        //sb ie678
        if(event.preventDefault){
            event.preventDefault();
        }

        _isMouseMoving = true;
        var mousePos = _fixedPosition(event);

        if(self._backgroundGlyph){
            var onMoveText = self._backgroundGlyph.doOnMove(mousePos);
            if(onMoveText){
                clearTimeout(_tooltipHideTick);
                clearTimeout(_tooltipDisplayTick);
                var ctx4OnMove = self._chartGlyph.getOnMoveCtx();
                ctx4OnMove._tooltipDiv = _tooltipDiv;
                FR.Chart.Tooltip.dataPointTip(ctx4OnMove, onMoveText, {x:mousePos.x, y:mousePos.y + 30});
            }else{
                _tooltipHideDelay();
            }
        }

        //安卓和ie678不能连续拽
        var stupidBrowser = FR.Browser.isIE8Before();

        self._chartGlyph.clearOnMoveCtx();

        if (self._mouseStyle == "hand") {
            root.style.cursor = HAND;
        } else if (self._mouseStyle == "drag") {
            root.style.cursor =DRAG;
        }else if(self._mouseStyle == "arrow"){
            root.style.cursor = 'e-resize';
        } else {
            if (_draggingTarget && _draggingTarget.isScroll) {
                root.style.cursor = 'move';
            } else if (_hasfound && _lastHover && _lastHover.clickable && !_isPlotAreaHover) {
                root.style.cursor = 'pointer';
            } else {
                root.style.cursor = 'default';
            }
        }
        event._mouseStyle = self._mouseStyle;


        var ctx4OnMove = self._chartGlyph.getOnMoveCtx();
        var plotGlyph = self._chartGlyph.plotGlyph;

        if (_draggingTarget && _isDragging) {// 拖拽事件处理
            if (_draggingTarget.jsType){
                if(_draggingTarget.jsType == "DataSeries"){
                    plotGlyph.onSeriesDrag(_downPosition, mousePos, ctx4OnMove, _draggingTarget);
                }
            } else if (_draggingTarget.onDrag) {
                _draggingTarget.onDrag(_downPosition, mousePos, ctx4OnMove, event, stupidBrowser);
                if (!!event.zoomRefresh) {//防止未定义
                    refreshWithoutAnimate();
                }
            }
        } else { //鼠标移动
            _findOnMoveGlyph(mousePos);
            //兼容原来的画法
            ctx4OnMove._tooltipDiv = _tooltipDiv;
            if (_lastHover) {
                clearTimeout(_tooltipHideTick);
                clearTimeout(_tooltipDisplayTick);
                if (_lastHover.doOnMove) {
                    if(FR.equals(_lastHover.jsType, 'controlBar')){
                        _tooltipDiv.style.display = 'none';
                    }
                    _lastHover.doOnMove(ctx4OnMove, mousePos, plotGlyph, self._chartGlyph.getMapOnMoveCtx());
                }
            } else {
                _tooltipHideDelay();
            }

            //是否找到形状都要响应
            self._chartGlyph.doOnMove(mousePos);
        }
        _zoomMove(mousePos);
    }

    function _tooltipHideDelay(){
        clearTimeout(_tooltipHideTick);
        clearTimeout(_tooltipDisplayTick);
        _tooltipHideTick = setTimeout(function(){
            _tooltipDiv.style.opacity = 0;
        }, 400);

        _tooltipDisplayTick = setTimeout(function(){
            _tooltipDiv.style.display = 'none';
        }, 800);
    }

    function _zoomMove(mousePosition) {
        if (!_isMouseDown) {
            return;
        }
        if (self._chartGlyph == undefined || self._chartGlyph.plotGlyph == undefined) {
            return;
        }
        var plot = self._chartGlyph.plotGlyph;
        if (plot.isSupportZoom() && plot.bounds.containsPoint(_downPosition.x, _downPosition.y)) {
            var fresh = plot.setCurrentOffset(mousePosition.x - _downPosition.x, mousePosition.y - _downPosition.y);
            if (fresh) {
                self._chartGlyph.clearAll();
                self._chartGlyph.clearOnMoveCtx();
                self._chartGlyph.draw(FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL);
                _zoomMoving = true;
            }
        }
    }

    function _zoomUp() {
        if (_zoomMoving) {
            _zoomMoving = false;
            if (self._chartGlyph == undefined || self._chartGlyph.plotGlyph == undefined) {
                return;
            }
            var plot = self._chartGlyph.plotGlyph;
            if (plot.isSupportZoom()) {
                plot.setTotalOffset();
                _lastHover = null;
            }
        }
    }

    /**
     * 鼠标移除
     * @private
     */
    function _mouseLeaveHandler(event) {
        event = event || window.event;
        if (_isDragging && _draggingTarget) {
            if (_draggingTarget.jsType && _draggingTarget.jsType == "DataSeries") {
                self._chartGlyph.plotGlyph.onDragMouseOut(_draggingTarget);
                self._chartGlyph.plotGlyph.refreshDataPointLabel();
                self._chartGlyph.refreshLegendLabel();
                self._chartGlyph.clearOnMoveCtx();
                _refresh();
            }
        }else{
            self._chartGlyph.clearOnMoveCtx();
        }

        _tooltipHideDelay();

        root.style.cursor = 'default';
        _isMouseDown = false;
        _isDragging = false;
        _isMouseMoving = false;
        _draggingTarget = null;
        _zoomUp();
    }

    /**
     * 鼠标滚轮
     */
    function _mouseWheelHandler(event){
        _isMouseMoving = false;
        event = event || window.event;
        if (!self._chartGlyph || !self._chartGlyph.plotGlyph) {
            return;
        }
        var mousePos = _fixedPosition(event);
        var plot = self._chartGlyph.plotGlyph;
        if (plot.isSupportZoom() && plot.bounds.containsPoint(mousePos.x, mousePos.y)) {
            var fresh = true;

            if(event.wheelDelta){// IE/Opera/Chrome
                if (event.wheelDelta < 0) { //上滚
                    fresh = plot.setZoom(0.1);
                } else { //下滚
                    fresh = plot.setZoom(-0.1);
                }
            }else if(event.detail){// Firefox
                if (event.detail < 0) { //上滚
                    fresh = plot.setZoom(0.1);
                } else { //下滚
                    fresh = plot.setZoom(-0.1);
                }
            }

            //IE9以下禁用缩放
            if (fresh && !FR.Browser.isIE8Before()) {
                self._chartGlyph.clearAll();
                self._chartGlyph.clearOnMoveCtx();
                self._chartGlyph.draw(FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL);
                event.returnValue = false;
            }
        } else {
            event.returnValue = true;
        }
    }

    /**
     * 查找当前onmove的Glyph
     * @param mousePos
     * @private
     */
    function _findOnMoveGlyph(mousePos) {
        self._hasfound = false;
        self._lastHover = null;
        self._isPlotAreaHover  = false;
        self._draggingTarget = null;
        self._isMouseDown = _isMouseDown;
        self._isMouseMoving = _isMouseMoving;
        self._mouseStyle = "";
        self.needRefresh = false;
        self._chartGlyph.findOnMoveGlyph(mousePos, self);
        _lastHover = self._lastHover;
        _hasfound = self._hasfound;
        _isPlotAreaHover = self._isPlotAreaHover;

        //动画画完才做拖拽
        if(FR.Chart.GraphHelper.isDrawingEnd){
            _draggingTarget = self._draggingTarget;
            if (self.needRefresh) {
                refreshWithoutAnimate();
            }
        }

    }

    function _mouseDownHandler(event) {

        if(event.preventDefault){
            event.preventDefault();
        }

        _downPosition = _fixedPosition(event);
        _isMouseDown = true;
        _isMouseMoving = false;

        _findOnMoveGlyph(_downPosition);//down了之后需要判断一下鼠标悬浮在哪个对象上

        _dragStartHandler(_downPosition);
    }

    /**
     * 鼠标（手指）抬起响应函数
     * @param {event} event dom事件对象
     */
    function _mouseUpHandler(event) {
        root.style.cursor = 'default';

        _dragEndHandler(event);
        _isMouseMoving = false;
        _isMouseDown = false;

        self._chartGlyph.clearOnMoveCtx();
        _zoomUp();

        var mousePos = _fixedPosition(event);
        _findOnMoveGlyph(mousePos);
    }

    /**
     * 拖拽开始
     * 有这几个地方可以拖拽， prechartglyph, plotglyph, dataPoint.glyph, legendGlyph
     */
    function _dragStartHandler(mousePos) {
        if (_isMouseDown && _lastHover && _draggingTarget) {
            _isDragging = true;

            if(_draggingTarget.makeSureDragFocus){
               _draggingTarget.makeSureDragFocus(mousePos)
            }

            if (_draggingTarget && _draggingTarget.jsType) {
                if(_draggingTarget.jsType == "DataSeries"){
                    self._chartGlyph.reDrawPlotGlyphNoAnimate();
                    self._chartGlyph.plotGlyph.onDragStart(self._chartGlyph.getOnMoveCtx(), _draggingTarget);
                }
            } else {
                refreshWithoutAnimate();
            }
        }
    }

    /**
     * 拖拽结束
     */
    function _dragEndHandler(event) {
        var pos = _fixedPosition(event);

        //确实鼠标有移动
        if (_downPosition && (Math.abs(pos.x - _downPosition.x) > 1 || Math.abs(pos.y - _downPosition.y) > 1)) {
            if (_draggingTarget && _draggingTarget.onDragEnd) {
                _draggingTarget.onDragEnd(_downPosition, pos);
                refreshWithoutAnimate();
                _lastHover = null;
            } else if (_draggingTarget && _draggingTarget.jsType && _draggingTarget.jsType == "DataSeries") {
                self.needRefresh = false;
                self._chartGlyph.plotGlyph.onSeriesDragEnd(_draggingTarget, _downPosition, pos, self);
                self._chartGlyph.refreshLegendLabel();
                if (self.needRefresh) {
                    doRefreshIfNeed();
                } else {//无效操作的时候不需要animate刷新
                    refreshWithoutAnimate();
                }
                _lastHover = null;
            }
        } else if (_draggingTarget && _draggingTarget.paintState == FR.Chart.ChartPaintConstants.STATE_TRANSPARENT) {//清除状态
            _draggingTarget.paintState = FR.Chart.ChartPaintConstants.STATE_ENABLE;
            refreshWithoutAnimate();
        }

        _isDragging = false;
        _draggingTarget = null;
    }

    function doRefreshIfNeed() {
        if (self.needRefresh) {
            _refresh();
        }
    }

    function refreshWithoutAnimate() {
        self._chartGlyph.clearAll();
        self._chartGlyph.plotGlyph.refreshAxisGlyph();
        self._chartGlyph.draw(FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL);
    }

    function _refresh() {
        self._chartGlyph.clearAll();
        self._chartGlyph.plotGlyph.refreshAxisGlyph();
        self._chartGlyph.draw(FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE);
    }

    function _fixedPosition(event) {
        var ev = event;
        if (event.targetTouches && event.targetTouches.length > 0) {
            ev = event.targetTouches[0];
        } else {
            if (event && event.touches) {
                if (event.touches.length > 0) {
                    ev = event.touches[0];
                } else if (event.changedTouches && event.changedTouches.length > 0) {
                    ev = event.changedTouches[0];
                }
            }
        }

        var pos = {x: 0, y: 0};

        var scales = 1;

        pos.x += ev.clientX;	// 加上鼠标在视窗中的位置
        pos.y += ev.clientY;

        var offset = dom.offset();
        pos.x -= offset.left;
        pos.y -= offset.top;

        var contentPane = contentPane;
        var pagePane = undefined;
        if (!FR.BIDezi) {
            if (contentPane) {
                pagePane = contentPane;
            } else if (ev.view && ev.view.contentPane) {
                pagePane = ev.view.contentPane;
            } else {
                if (FR.Browser.isIE()) {
                    try {
                        pagePane = window.parent.contentPane;// frame 权限
                    } catch (e) {
                        pagePane = null;
                    }
                }
            }

            if (pagePane && pagePane.zoom) {
                scales = pagePane.zoom;
            }
        }
        pos.x = pos.x / scales;
        pos.y = pos.y / scales;

        return pos;
    }

    _init();

    return self;
}

$.extend(ChartHandler, {
    POINTER: "pointer",
    AUTO: "auto",
    DRAG: "url(\"" + FR.servletURL + "?op=resource&resource=/com/fr/web/cursor/cursor_drag_hand.cur\"),auto",
    HAND: "url(\"" + FR.servletURL + "?op=resource&resource=/com/fr/web/cursor/cursor_hand.cur\"),auto"
});

$.extend(ChartHandler.prototype, {
    reStart: function (chartGlyph) {
        this._chartGlyph = chartGlyph;
        this._chartGlyph.handler = this;
    },

    doRefresh: function () {
        this._chartGlyph.clearAll();
        this._chartGlyph.plotGlyph.refreshAxisGlyph();
        this._chartGlyph.draw(FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE);
    }
});

// ChartHandler.end