/**
 *
 * 各种静态的方法. Utils && Factory  Color HashmMap
 *
 */

FR.Chart = FR.Chart || {};

FR.Chart.chartDivCache = FR.Chart.chartDivCache || {};

/**
 * 图表的初始化类, 主要用于继承初始化方法
 * @class FR.Chart.ChartJsInit
 * @abstract
 */
FR.Chart.ChartJsInit = function (js) {
    this.initConstants(js);
};
$.extend(FR.Chart.ChartJsInit.prototype, /**@class FR.Chart.ChartJsInit*/{
    initConstants: function (js) {
        // 暂时donothing. 被继承了
    }
});

/**
 * 图表文本Div管理.
 * @type {{addDiv: addDiv, clearDiv: clearDiv}}
 */
FR.Chart.DivCon = {

    /**
     * 添加图表文本div.
     * @static
     * @param key 文本div的key
     * @param newDiv 新的文本div
     */
    addDiv: function (key, newDiv) {
        if (!key) {
            key = FR.Chart.currentDivKey;
        }

        var currentChart = FR.Chart.chartDivCache[key.id];
        if (currentChart) {
            var div = currentChart.div;
            if (div) {
                div.appendChild(newDiv);
            }
            var list = currentChart.labelList;
            if (list) {
                list[list.length] = newDiv;
            }
        }
    },

    /**
     * 清空当前div对应的文本
     * @static
     * @param parentDiv 父类对应的div
     */
    clearDiv: function (parentDiv) {
        if (parentDiv) {
            var key = parentDiv.id;

            var currentChart = FR.Chart.chartDivCache[key];
            if (currentChart) {
                var div = currentChart.div;
                var list = currentChart.labelList;
                if (div && list) {
                    for (var i = 0; i < list.length; i++) {
                        if (div.contains(list[i])) {
                            div.removeChild(list[i]);
                        }
                    }
                    currentChart.labelList = [];
                }
            } else {
                div = parentDiv;
                list = [];

                FR.Chart.chartDivCache[key] = {'div': div, 'labelList': list};
            }
        }

        FR.Chart.currentDivKey = parentDiv;
    }
};

/**
 * 地图各个区域的属性封装
 * @param markType 标记点类型 0是标记点展示的地图， 1是区域展示的地图
 * @param name 区域的名字
 * @param shapePath 区域的边界
 * @constructor
 */
FR.Chart.MapShapeValue = function(areaName, shapePath, markType){
    this.markType = FR.pick(markType, 1);//默认都是区域类型的地图
    this.areaName = FR.pick(areaName, "");
    this.shapePath = FR.pick(shapePath, new FR.Chart.GeneralPath());
};

FR.Chart.MapAttr = function(pathMap, imageBounds, imageSrc){
    this.imageBounds = FR.pick(imageBounds, new FR.Chart.Bounds());
    this.imageSrc = FR.pick(imageSrc, {});
    this.shapeValues = {};
    this.cacheImage = null;//缓存的图片
    if(pathMap){
        for(var index = 0; index < pathMap.length; index++){
            var entry = pathMap[index];
            var generalPath = new FR.Chart.GeneralPath(entry.path);
            if (generalPath.x && generalPath.x.length > 1) {
                generalPath.closePath();
            }
            var markType = entry.markType;
            var areaName = entry.key;
            var shapeValue = new FR.Chart.MapShapeValue(markType, generalPath, areaName);
            this.shapeValues[areaName] = shapeValue;
        }
    }
};

$.extend(FR.Chart.MapAttr.prototype,{
    getPathByName:function(shapeName){
        if(this.shapeValues[shapeName]){
            return this.shapeValues[shapeName].shapePath;
        }
        return new FR.Chart.GeneralPath();
    },

    drawBackgroundImage:function(ctx, transFrom, plotBounds){
        if(!ctx){     return; } ctx.save();

        if (!this.imageBounds.width || !this.imageBounds.height) {
            return;
        }

        var imageX = plotBounds.x;
        var imageY = plotBounds.y;

        var imageWidth = this.imageBounds.width;
        var imageHeight = this.imageBounds.height;

        var width = plotBounds.width;
        var height = plotBounds.height;

        var scaleWidth = width / imageWidth;
        var scaleHeight = height / imageHeight;

        var paintWidth = width;
        var paintHeight = height;

        if (scaleWidth > scaleHeight) {
            paintWidth = (width * scaleHeight / scaleWidth);
            imageX += (width - paintWidth) / 2;
        } else {
            paintHeight = (height * scaleWidth / scaleHeight);
            imageY += (height - paintHeight) / 2;
        }

        if (this.cacheImage) {
            ctx.drawImage(this.cacheImage,imageX,imageY,paintWidth, paintHeight);
        } else {
            this.cacheImage = new Image();
            var self = this;
            this.cacheImage.onload = function () {
                // kunsnat: 预防IE9的概率性加载失败
                setTimeout(function () {
                        ctx.drawImage(self.cacheImage,imageX,imageY,paintWidth, paintHeight);
                    },
                    100);
            };

            if (FR.Browser.isIE()) {
                this.cacheImage.src = FR.servletURL + "?op=fr_attach&cmd=ah_image&id=" + this.imageSrc.onie;
            } else {
                this.cacheImage.src = this.imageSrc.onsrc;
            }
        }

        ctx.restore();
    }

});

FR.Chart.MapSvgAttr = function(svgRoot){
    this.svgRoot = svgRoot;
    this.layerTo = {};//钻取对应
    this.nameTo = {};//别名对应
    this.namedNodes = [];//有名字的节点
    this.names = [];
    this.imageBounds = FR.Chart.SVGExtension.getImageBounds(svgRoot);
    this.nodeList = FR.Chart.SVGExtension.svgResolve(svgRoot, this.namedNodes, this.nameTo, this.layerTo, this.names);
};

$.extend(FR.Chart.MapSvgAttr.prototype,{
    getPathByName:function(shapeName){
        var resultPath = new FR.Chart.GeneralPath();
        for(var i = 0; i < this.namedNodes.length; i++){
            var name = this.namedNodes[i].name;
            var path = this.namedNodes[i].path;
            if(FR.equals(name, shapeName)){
                resultPath.append(FR.Chart.SVGExtension.getPath4Shape(path));
            }
        }
        return resultPath;
    },

    drawBackgroundImage:function(ctx, transFrom){
        if(!this.nodeList){
            return;
        }

        for(var pathIndex = 0; pathIndex < this.nodeList.length; pathIndex++){
            var nodeShape = this.nodeList[pathIndex];
            FR.Chart.SVGExtension.drawNodeShape(ctx, nodeShape, transFrom);
        }
    },

    _findFirstNodeByName:function(nodeName){
        for(var i = 0; i < this.namedNodes.length; i++){
            var name = this.namedNodes[i].name;
            var path = this.namedNodes[i].path;
            if(FR.equals(name, nodeName)){
                return path;
            }
        }
    },

    drawBorder:function(ctx, name, path){
        var node = this._findFirstNodeByName(name);
        if(node && path){
            ctx.globalAlpha = node.opacity ? node.opacity : 1;
            ctx.lineWidth = node.lineWidth ? node.lineWidth : 0;
            ctx.strokeStyle = node.strokeColor ? node.strokeColor : 'rgba(255,255,255,0)';
            path.draw(ctx);
        }
    },

    getLayerTo:function(name){
        return this.layerTo[name];
    },

    getNameToValue:function(name){
        return this.nameTo[name] ? this.nameTo[name] : name;
    }

});

//地图的帮助类
FR.Chart.MapHelper = {
    bitMapAttr:{},

    svgMapAttr:{},

    containsBitMap:function(mapName){
        return !FR.isEmpty(this.bitMapAttr[mapName]);
    },

    containsSvgMap:function(mapName){
        return !FR.isEmpty(this.svgMapAttr[mapName])
    },

    addBitMapAttr:function(mapName, bitMapAttr){
        this.bitMapAttr[mapName] = bitMapAttr;
    },

    addSvgMapAttr:function(mapName, svgMapAttr){
        this.svgMapAttr[mapName] = svgMapAttr;
    },

    getBitMapAttr:function(mapName){
        return this.bitMapAttr[mapName];
    },

    getSvgMapAttr:function(mapName){
        return this.svgMapAttr[mapName];
    },

    getMapAttr:function(mapName, isSvg){
        return isSvg ? this.getSvgMapAttr(mapName) : this.getBitMapAttr(mapName);
    }
};

/**
 * 标签边界管理, 用于收集标签的边界, 判断是否重叠.
 * @example
 * var boundsManager = new FR.Chart.BoundsManager();
 *
 * @class FR.Chart.BoundsManager
 *
 * @property {Array} allBounds 用于储存所有相关的边界.
 */
FR.Chart.BoundsManager = function () {
    this.allBounds = [];
};
$.extend(FR.Chart.BoundsManager.prototype, /**@class FR.Chart.BoundsManager*/{

    /**
     * 添加边界
     * @param bounds 边界
     */
    addBounds: function (bounds) {
        this.allBounds.push(bounds);
    },

    /**
     * 判断边界 是否和之前的标签边界重叠.
     * @param {*} bounds 当前要判断的边界
     * @returns {Boolean} true 表示和之前的重叠, false 表示不重叠.
     */
    isInteract: function (bounds) {
        var length = this.allBounds.length;
        for (var i = 0; i < length; i++) {
            if (FR.Chart.ChartUtils.rectangleOverlapped(bounds, this.allBounds[i])) {
                return true;
            }
        }
        return false;
    },

    /**
     * 判断是否为空
     * @returns {boolean} 返回是否为空
     */
    isEmpty : function() {
        return this.allBounds.length === 0;
    }

});
// BoundsManager.end

/*
 * Chart工具类集合
 */
$.extend(FR.Chart, /**FR.Chart*/{

    /**
     * 图表工具类集合
     * @class FR.Chart.ChartUtils
     */
    ChartUtils: {

        //比较num1和num2大小，num1 <= num2 返回true
        compareDouble:function(num1, num2, compareType){
            switch (compareType){
                case FR.Chart.Compare.EQUALS:
                    return num1 === num2;

                case FR.Chart.Compare.NOT_EQUAL:
                    return num1 !== num2;

                case FR.Chart.Compare.GREATER_THAN:
                    return num1 > num2 - 1E-15;

                case FR.Chart.Compare.GREATER_THAN_OR_EQUAL:
                    return num1 >= num2 - 1E-15;

                case FR.Chart.Compare.LESS_THAN:
                    return num1 < num2 + 1E-15;

                case FR.Chart.Compare.LESS_THAN_OR_EQUAL:
                    return num1 <= num2 + 1E-15;

                default:
                    return false;
            }
        },

        retinaOptimization:function(canvas, ctx, width, height){
            var devicePixelRatio = window.devicePixelRatio || 1;
            var backingStorePixelRatio = ctx.webkitBackingStorePixelRatio ||
                ctx.mozBackingStorePixelRatio ||
                ctx.msBackingStorePixelRatio ||
                ctx.oBackingStorePixelRatio ||
                ctx.backingStorePixelRatio || 1;

            var ratio = devicePixelRatio / backingStorePixelRatio;

            canvas.style.width = width + 'px';
            canvas.style.height = height + 'px';
            canvas.setAttribute('width', width * ratio);
            canvas.setAttribute('height', height * ratio);
            if (ratio !== 1) {
                ctx.scale(ratio, ratio);
            }
            ctx.clearRect(0, 0, width * ratio, height * ratio);
        },

        inArray:function(array, value){
            if(!array || (value == null || value == undefined)){
                return false;
            }

            for(var i = 0; i < array.length; i++){
                if(array[i] == value){
                    return true;
                }
            }

            return false;
        },

        /**
         * 判断是否为IE11的版本, 因为IE11. msie中不再提供.
         * @returns {*|boolean}
         */
        isIE11 : function() {
            return FR.Browser.isIE11();// mozilla	true  version 11.0 判断是否为IE11.
        },

        isIE6Before:function(){
            return FR.Browser.isIE6Before();
        },

        //判断是否是IE9以及以后版本的IE浏览器
        isIE9Later:function(){
            return FR.Browser.isIE9Later() || FR.Chart.ChartUtils.isIE11();
        },

        isExcanvas:function(){
            return FR.Browser.isIE8Before();
        },

        /**
         * @static
         * @param bounds 边界
         * @param isToRound 是否为圆角边界
         * @returns {FR.Chart.RoundBounds} 圆角边界
         */
        rectangle2RoundRectangle: function (bounds, isToRound) {
            var roundBorder = 10;
            var round = new FR.Chart.RoundBounds(bounds.x, bounds.y, bounds.width, bounds.height, roundBorder, roundBorder);
            return isToRound ? round : bounds;
        },

        /**
         * @static
         * @param date 日期
         * @param dayType 日期类型
         * @returns {Number} 返回日期到指定日期的间隔数目(根据 日期类型计算)
         */
        date2Int: function (date, dayType) {
            if (date == null) {
                date = new Date();
            }
            var baseDate = new Date("1970/01/01");

            return FR.Chart.ChartUtils.getTwoDay(date, baseDate, dayType);
        },

        /**
         * 将数据转为对应的日期.
         * @static
         * @param number 数据
         * @param dayType  日期类型
         * @returns {Date} 返回对应的日期
         */
        int2Date: function (number, dayType) {
            var baseDate = new Date("1970/01/01");

            var dateNumber = 24 * 60 * 60 * 1000;
            if (dayType == FR.Chart.Constants.Second) {
                dateNumber = 1000;
            } else if (dayType == FR.Chart.Constants.Minute) {
                dateNumber = 60 * 1000;
            } else if (dayType == FR.Chart.Constants.Hour) {
                dateNumber = 60 * 60 * 1000;
            }

            var dateTime = number * dateNumber + baseDate.getTime();

            return new Date(dateTime);
        },

        /**
         * 得到两个时间的天数差
         * @static
         * @param date1 日期1
         * @param date2 日期2
         * @param dayType 日期类型
         * @returns {number} 返回间隔数 (在指定日期类型下)
         */
        getTwoDay: function (date1, date2, dayType) {
            var dateNumber = 24 * 60 * 60 * 1000;
            if (dayType == FR.Chart.Constants.Second) {
                dateNumber = 1000;
            } else if (dayType == FR.Chart.Constants.Minute) {
                dateNumber = 60 * 1000;
            } else if (dayType == FR.Chart.Constants.Hour) {
                dateNumber = 60 * 60 * 1000;
            }

            var day = (date1.getTime() - date2.getTime()) / dateNumber;
            return Math.floor(day);
        },

        isBelongToSameYear:function(date1,date2){
            return date1.getFullYear() === date2.getFullYear();
        },

        isBelongToSameMonth:function(date1,date2){
            return  FR.Chart.ChartUtils.isBelongToSameYear(date1,date2) &&
                (date1.getMonth() === date2.getMonth());
        },

        isBelongToSameDay:function(date1,date2){
            return  FR.Chart.ChartUtils.isBelongToSameMonth(date1,date2) &&
                (date1.getDate() === date2.getDate());
        },

        isBelongToSameHour:function(date1,date2){
            return  FR.Chart.ChartUtils.isBelongToSameDay(date1,date2) &&
                (date1.getHours() === date2.getHours());
        },

        isBelongToSameMinute:function(date1,date2){
            return  FR.Chart.ChartUtils.isBelongToSameHour(date1,date2) &&
                (date1.getMinutes() === date2.getMinutes());
        },

        isBelongToSameSecond:function(date1,date2){
            return  FR.Chart.ChartUtils.isBelongToSameMinute(date1,date2) &&
                (date1.getSeconds() === date2.getSeconds());
        },

        /**
         * 是否属于同一个年/月/日/时/分/秒
         */
        isBelongToSameUnit:function(date1,date2,dateType){
            if (dateType == FR.Chart.Constants.DAY) {
                return  FR.Chart.ChartUtils.isBelongToSameDay(date1,date2);
            } else if (dateType == FR.Chart.Constants.MONTH) {

                return  FR.Chart.ChartUtils.isBelongToSameMonth(date1,date2);
            } else if (dateType == FR.Chart.Constants.Minute) {

                return  FR.Chart.ChartUtils.isBelongToSameMinute(date1,date2);
            } else if (dateType == FR.Chart.Constants.Hour) {

                return  FR.Chart.ChartUtils.isBelongToSameHour(date1,date2);
            } else if (dateType == FR.Chart.Constants.Second) {

                return  FR.Chart.ChartUtils.isBelongToSameSecond(date1,date2);
            }else{
                return  FR.Chart.ChartUtils.isBelongToSameYear(date1,date2);
            }
        },

        /**
         * 获取date所在的datetype(时分秒，年月日)的中间的时间
         * @param date
         */
        getMiddleDate:function(date,dateType){
            if (dateType === FR.Chart.Constants.DAY) {
                return  new Date(date.getFullYear()+"-"+(date.getMonth()+1)+"-"+date.getDate()+" 12:00:00");
            } else if (dateType === FR.Chart.Constants.MONTH) {
                return new Date(date.getFullYear()+"-"+(date.getMonth()+1)+"-"+parseInt(date.getMonthDays()/2)+" 00:00:00");
            } else if (dateType === FR.Chart.Constants.Minute) {
                return  new Date(date.getFullYear()+"-"+(date.getMonth()+1)+"-"+date.getDate()+" "+date.getHours()+":"+date.getMinutes()+":30");
            } else if (dateType === FR.Chart.Constants.Hour) {
                return  new Date(date.getFullYear()+"-"+(date.getMonth()+1)+"-"+date.getDate()+" "+date.getHours()+":30:00");
            } else if (dateType === FR.Chart.Constants.Second) {
                return  date;
            }else{
                return  new Date(date.getFullYear()+"-06-30 00:00:00");
            }
        },

        /**
         * 获取date所在的datetype(时分秒，年月日)的开始的时间
         * @param date
         */
        getStartDate4Type:function(date,dateType){
            if (dateType === FR.Chart.Constants.DAY) {
                return  new Date(date.getFullYear()+"-"+(date.getMonth()+1)+"-"+date.getDate()+" 00:00:00");
            } else if (dateType === FR.Chart.Constants.MONTH) {
                return new Date(date.getFullYear()+"-"+(date.getMonth()+1)+"-01 00:00:00");
            } else if (dateType === FR.Chart.Constants.Minute) {
                return  new Date(date.getFullYear()+"-"+(date.getMonth()+1)+"-"+date.getDate()+" "+date.getHours()+":"+date.getMinutes()+":00");
            } else if (dateType === FR.Chart.Constants.Hour) {
                return  new Date(date.getFullYear()+"-"+(date.getMonth()+1)+"-"+date.getDate()+" "+date.getHours()+":00:00");
            } else if (dateType === FR.Chart.Constants.Second) {
                return  date;
            }else{
                return  new Date(date.getFullYear()+"-01-01 00:00:00");
            }
        },


        /**
         * 获取date所在的datetype(时分秒，年月日)的结束的时间
         * @param date
         */
        getEndDate4Type:function(date,dateType){
            if (dateType === FR.Chart.Constants.DAY) {
                return  new Date(date.getFullYear()+"-"+(date.getMonth()+1)+"-"+date.getDate()+" 23:59:59");
            } else if (dateType === FR.Chart.Constants.MONTH) {
                return new Date(date.getFullYear()+"-"+(date.getMonth()+1)+"-"+date.getMonthDays()+" 23:59:59");
            } else if (dateType === FR.Chart.Constants.Minute) {
                return  new Date(date.getFullYear()+"-"+(date.getMonth()+1)+"-"+date.getDate()+" "+date.getHours()+":"+date.getMinutes()+":59");
            } else if (dateType === FR.Chart.Constants.Hour) {
                return  new Date(date.getFullYear()+"-"+(date.getMonth()+1)+"-"+date.getDate()+" "+date.getHours()+":59:59");
            } else if (dateType === FR.Chart.Constants.Second) {
                return  date;
            }else{
                return  new Date(date.getFullYear()+"-12-31 23:59:59");
            }
        },


        /**
         * 根据调整的类型获得整体范围对应的年，月等，比如按年调整就返回2015,按月调整就返回2015-05
         * @param date 具体日期
         * @param dateType 调整范围的字符串
         */
        calculateAdjustDateRange:function(date,dateType){
            if (dateType === FR.Chart.Constants.DAY) {
                return FR.contentFormat(date, "Dyyyy-MM-dd");
            } else if (dateType === FR.Chart.Constants.MONTH) {
                return FR.contentFormat(date, "Dyyyy-MM");
            } else if (dateType === FR.Chart.Constants.Minute) {
                return FR.contentFormat(date, "Dyyyy-MM-dd hh:mm");
            } else if (dateType === FR.Chart.Constants.Hour) {
                return FR.contentFormat(date, "Dyyyy-MM-dd hh")
            } else if (dateType === FR.Chart.Constants.Second) {
                return FR.contentFormat(date, "Dyyyy-MM-dd hh:mm:ss");
            }else{
                return FR.contentFormat(date, "Dyyyy");
            }
        },


        /**
         * 将目标Object转为日期
         * @static
         * @param obj 目标Object
         * @param dayType 日期类型
         * @returns {Date} 日期.
         */
        object2Date: function (obj, dayType) {
            var ret = null;
            if (obj instanceof Date) {
                ret = obj;
            } else if (typeof(obj) == typeof("")) {
                if (obj.indexOf(".0") == obj.length - 2) {
                    obj = obj.substring(0, obj.length - 2);
                }
                ret = new Date(Date.parse(obj.replace(/-|\./g, "/")));
            } else if (typeof(obj) == typeof(0)) {
                ret = FR.Chart.ChartUtils.int2Date(obj, dayType);
            }
            return ret;
        },

        /**
         * 新需求:
         * var date = new Date();  2010-06-02 ---->  06/23/2010
         * 尽量支持: "2013/05/13 00:00:00.0"  类似这种不是标准的参数 转化为日期.
         * new Date(2013, 05, 13, 00, 00, 00, 0)  // Month 从0起, 05表示六月份, 所以在java端日期过来之后, 月份要做处理.
         * new Date(2013, 05, 13, 00, 00, 00)
         * new Date(2013, 05, 13, 00, 00)
         * new Date(2013, 05, 13, 00)
         * new Date(2013, 05, 13)
         * new Date(2013, 05)
         * new Date(2013, 0)
         * @static
         * @param {String} value 表示一个日期的字符串
         * @return {Date} 字符串转化后的日期
         */
        string2Date  :   function(value) {
            value = value.replace(/-/g, "/");// 2010-06-02 ---->  06/23/2010

            var date = new Date(value);
            if(!isNaN(date)) {
                return date;
            }

            var year = 1970;
            var month = 0;// 从0起, 表示1月份
            var day = 1;
            var hour = 0;
            var min = 0;
            var second = 0;

            var today = new Date();
            var offset = today.getTimezoneOffset(); // 本地计时和世界标准计时的区别.

            var beAndAf = value.split(" ");// 先试探空格, 分为前面的日期和后面的时分秒.

            var ymd = beAndAf[0];
            var yymmdd = ymd.split("/");// 已经处理过, '-' 需要替换为'/'统一;
            for(var i = 0, yl = yymmdd.length; i < yl; i++) {// 空格前 或者没有空格 只有日期.
                if(i === 0) {
                    year = yymmdd[0];
                } else if(i === 1) {
                    month = yymmdd[1];
                } else if(i === 2) {
                    day = yymmdd[2];
                }
            }

            if(beAndAf.length === 2) {// 空格后部
                var hms = beAndAf[1];
                var hhmmss = hms.split(":");
                for(var i = 0, ml = hhmmss.length; i < ml; i++) {
                    if(i === 0) {
                        hour = hhmmss[0];
                    } else if(i === 1) {
                        min = hhmmss[1];
                    } else if(i === 2) {
                        second = hhmmss[2];
                    } else if(i === 3) {
                        offset = hhmmss[3];
                    }
                }
            }

            year = year || today.getFullYear();
            month = (month - 1) || today.getMonth();
            day = day || today.getDay();
            hour = hour ? hour : 0;
            min = min ? min : 0;
            second = second ? second : 0;
            offset = offset ? offset : today.getTimezoneOffset();

            return new Date(year, month, day, hour, min, second, offset);
        },

        /**
         * 坐标提示时候的精确格式,用于时间坐标轴
         * @static
         * @param dayType 日期类型
         * @param increment 增长数
         * @returns {string} 返回日期格式字符串
         */
        accDateFormat: function (dayType, increment) {
            var ss = "Dyyyy-MM-dd HH:mm:ss";
            var mm = "Dyyyy-MM-dd HH:mm";
            var hh = "Dyyyy-MM-dd HH";
            var dd = "Dyyyy-MM-dd";
            var MM = "Dyyyy-MM";
            var yy = "Dyyyy";
            var acc = increment > 1 ? false : true;

            if (dayType == FR.Chart.Constants.Second) {
                return ss;
            } else if (dayType == FR.Chart.Constants.Minute) {
                if (acc) {
                    return ss;
                } else {
                    return mm;
                }
            } else if (dayType == FR.Chart.Constants.Hour) {
                if (acc) {
                    return mm;
                } else {
                    return hh;
                }
            } else if (dayType == FR.Chart.Constants.DAY) {
                if (acc) {
                    return hh;
                } else {
                    return dd;
                }
            } else if (dayType == FR.Chart.Constants.MONTH) {
                if (acc) {
                    return dd;
                } else {
                    return MM;
                }
            } else if (dayType == FR.Chart.Constants.YEAR) {
                if (acc) {
                    return MM;
                } else {
                    return yy;
                }
            }
        },

        /**
         * 坐标提示时候的精确格式
         * @static
         * @param format 日期格式
         * @returns {Object} 日期格式.
         */
        accFormat: function (format) {
            if (format == null) {
                return "#0.00";
            }

            format += "";
            if (format.indexOf(".") < 0) {
                return format + ".0";
            } else {
                return format.replace(/\./, ".0");
            }
        },

        /**
         * 返回最终的字符串, 去除末尾的无用0  20.0  20.4231000
         * @static
         * @param number 数字
         * @returns {string} 返回正常的数字 格式.
         */
        generalFormat: function (number) {
            return "" + parseFloat(number);
        },

        /**
         *  显示精确值 0.100002=0.1
         *  @static
         * @param value 数字
         * @returns {Number} 返回精确值
         */
        accNumber: function (value) {
            var sv = "" + parseFloat(value);
            var ev = "";
            if (sv.indexOf('.') < 0) {
                return value;
            }
            if (sv.indexOf('e') >= 0 || sv.indexOf('E') >= 0) {
                var e = sv.indexOf('e') >= 0 ? sv.indexOf('e') : sv.indexOf('E');
                ev = sv.substring(e);
                sv = sv.substring(0, e);
            }
            var s = sv.substring(sv.indexOf('.'));
            var l = s.length;
            var start = 1;
            while (start < l) {
                if (s.substr(start, 3) == '999') {
                    s = s.substring(0, start);
                    var d = '.000000000000000000000000000000000'.substring(0, s.length - 1) + '1';
                    s = '0' + s.substring(1);
                    if (d.indexOf('.') >= 0) {
                        d = '0' + d.substring(1);
                    }
                    var w = sv.substring(0, sv.indexOf('.'));
                    if (w == '') {
                        w = '0';
                    }
                    for (var i = 0; i < start - 1; i++) {
                        w += '0';
                    }
                    var op = '+';
                    if (value < 0) {
                        op = '-';
                    }

                    var tmp = eval(+s + "+" + +d);
                    var res = eval(+w + op + tmp).toString();
                    if (value < 0 && res.length < w.length) {
                        while (res.length < w.length) {
                            res = res.substr(0, 1) + '0' + res.substr(1);
                        }
                    } else if (value > 0 && res.length < w.length) {
                        while (res.length < w.length) {
                            res = '0' + res;
                        }
                    }
                    res = res.substr(0, res.length - start + 1) + '.' + res.substr(res.length - start + 1);
                    return res + ev;
                } else if (s.substr(start, 3) == '000') {
                    //值小余0.001的时候就去掉末尾的0
                    if(value < 0.001) {
                        while (s.charAt(start) == '0') {
                            start++;
                        }
                        while (start < s.length && s.charAt(start) != '0') {
                            start++;
                        }

                    }
                    return sv.substring(0, sv.indexOf('.') + start) + ev;
                }
                start++;
            }
            return value;
        },

        /**
         * 字符串 根据最大宽度 文本格式 转为换行结果
         * @static
         * @param labelString  字符串
         * @param textAttr 文本格式
         * @param maxWidth 最大宽度
         * @returns {String} 换行结果字符串
         */
        dealLabelStringAutoWrap: function (labelString, textAttr, maxWidth) {
            var splitString = "";
            var tmpString = "";
            for (var i = 0; i < labelString.length; i++) {
                var letter = labelString.charAt(i);
                tmpString += letter;
                var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(tmpString + " ", textAttr);
                if (dim.width > maxWidth && letter != '\\') {
                    splitString += (tmpString + "\n");
                    tmpString = "";
                }
            }
            splitString += tmpString;
            return splitString;
        },

        /**
         * 字符串 根据最大宽度 文本格式 转为换行结果   其中 不同的是最后一个字符串加入的位置不同. 标题加入下行.
         * @static
         * @param labelString  字符串
         * @param textAttr 文本格式
         * @param maxWidth 最大宽度
         * @returns {string} 换行结果字符串
         */
        dealTitleStringAutoWrap: function (labelString, textAttr, maxWidth) {
            var splitString = "";
            var tmpString = "";
            for (var i = 0; i < labelString.length; i++) {
                var letter = labelString.charAt(i);
                tmpString += letter;// 标题 某行过长, 最后一个字符加入下行.
                var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(tmpString, textAttr);
                if (dim.width > maxWidth && letter != '\\') {
                    splitString += (tmpString.substring(0, tmpString.length - 1) + "\n");
                    tmpString = tmpString.charAt(tmpString.length - 1);
                }
            }
            splitString += tmpString;
            return splitString;
        },

        /**
         * 根据填充方案生产填充的颜色数组
         * @static
         * @param fillStyle 填充风格
         * @param seriesCount 系列数组个数
         * @returns {Array} 颜色数组
         */
        createFillColorArray: function (fillStyle, seriesCount) {
            var colors = [seriesCount];

            if (fillStyle == null || !fillStyle.colorStyle || fillStyle.colorStyle == FR.Chart.Constants.Color_DEFAULT) {
                var currentFillStyle = FR.Chart.CurrentPreStyle;
                if (!currentFillStyle || currentFillStyle.colorStyle == FR.Chart.Constants.Color_DEFAULT) {
                    for (var i = 0; i < seriesCount; i++) {
                        colors[i] = FR.Chart.ChartCoreConstants.CHART_Color_ARRAY[i % FR.Chart.ChartCoreConstants.CHART_Color_ARRAY.length];
                    }
                } else {
                    return FR.Chart.ChartUtils.createFillColorArray(currentFillStyle, seriesCount);
                }
            } else if (fillStyle.colorStyle === FR.Chart.Constants.Color_ACC) {
                for (var i = 0; i < seriesCount; i++) {
                    colors[i] = fillStyle.getColorIndex(i);
                }
            } else if (fillStyle.colorStyle === FR.Chart.Constants.Color_GRADIENT && fillStyle.getColorSize() === 2) {
                for (var i = 0; i < seriesCount; i++) {
                    colors[i] = FR.Chart.ChartUtils.getSeriesColor(fillStyle.getColorIndex(0), fillStyle.getColorIndex(1), i, seriesCount);
                }
            }
            return colors;
        },

        /**
         * 饱和度变化方案
         * @static
         * @param color 颜色
         * @param index 序号
         * @param count 颜色数目
         * @returns {Color} 颜色
         */
        getSaturationSeriesColor: function (color, index, count) {
            if (count <= 1) {
                return color;
            }
            var colorArray = FR.Chart.Color.getColorArray(color);
            var hsb = FR.Chart.Color.RGB2HSB(colorArray[0], colorArray[1], colorArray[2]);
            if (colorArray[0] == colorArray[1] && colorArray[1] == colorArray[2]) {
                hsb[2] = 0.9 - (0.8 / (count - 1)) * index;
            } else {
                hsb[1] = 1 - (0.9 / (count - 1)) * index;
            }
            return FR.Chart.Color.getColor(FR.Chart.Color.HSB2RGB(hsb[0], hsb[1], hsb[2]));
        },

        /**
         * 2种颜色渐变方案
         * @static
         * @param startColor 起始颜色
         * @param endColor 结束颜色
         * @param index 颜色变化序号
         * @param count 总体颜色个数
         * @returns {Color} 返回颜色
         */
        getSeriesColor: function (startColor, endColor, index, count) {
            if (count <= 1) {
                return startColor;
            }
            var colorArray = FR.Chart.Color.getColorArray(startColor);
            var start = FR.Chart.Color.RGB2HSB(colorArray[0], colorArray[1], colorArray[2]);
            colorArray = FR.Chart.Color.getColorArray(endColor);
            var end = FR.Chart.Color.RGB2HSB(colorArray[0], colorArray[1], colorArray[2]);
            var HSB = [3];

            for (var i = 0; i < 3; i++) {
                HSB[i] = (start[i] - ((start[i] - end[i]) / (count - 1)) * index);
            }

            return FR.Chart.Color.getColor(FR.Chart.Color.HSB2RGB(HSB[0] - Math.floor(HSB[0]), HSB[1], HSB[2]));
        },

        /**
         * 判断两个矩形是否重叠的算法
         * @static
         * @param firstBounds 第一个矩形
         * @param secondBounds 第二个矩形
         * @returns {Boolean} 返回是否重叠，true重叠，false不重叠
         */
        rectangleOverlapped: function (firstBounds, secondBounds) {
            if (firstBounds == null || secondBounds == null) {
                return false;
            }
            var minx = Math.max(firstBounds.x, secondBounds.x);
            var miny = Math.max(firstBounds.y, secondBounds.y);
            var maxx = Math.min(firstBounds.x + firstBounds.width, secondBounds.x + secondBounds.width);
            var maxy = Math.min(firstBounds.y + firstBounds.height, secondBounds.y + secondBounds.height);
            return (minx <= maxx && miny <= maxy);
        },

        /**
         * 把整数或者字符串表示的位置统一转成数字表示的位置
         * @param position 位置
         */
        getPosition:function(position){
            if(isNaN(parseInt(position))){
                return FR.Chart.Position[position]
            }
            return position;
        },

        /**
         * 把整数或者字符串表示的填充方式统一转成数字表示的方式
         * @param colorStyle
         */
        getFillColorStyle:function(colorStyle){
            if(isNaN(parseInt(colorStyle))){
                return FR.Chart.FillStyle[colorStyle]
            }
            return colorStyle;
        },

        /**
         * 三维饼图 排序方式
         * @static
         * @param a 扇面A
         * @param b 扇面B
         * @returns {number} 返回排序结果
         */
        sortPie3DSide:function(a, b){
            if(!a || !b){
                return 1;
            }
            var angle_this = a.isStart ? a.cylinder.start : a.cylinder.start + a.cylinder.extent;
            var angle_that = b.isStart ? b.cylinder.start : b.cylinder.start + b.cylinder.extent;
            var value_this = Math.abs(angle_this - 180 / 2.0) % 360;
            var value_that =  Math.abs(angle_that - 180 / 2.0) % 360;
            value_this = value_this > 180 ? 360 - value_this : value_this;
            value_that = value_that > 180 ? 360 - value_that : value_that;
            if (Math.abs(value_this - value_that) < 0.05) {
                if (a.cylinder.extent !== 0) {
                    if (a.isStart) {
                        angle_this = a.cylinder.start
                            + a.cylinder.extent / Math.abs(a.cylinder.extent);
                    } else {
                        angle_this = a.cylinder.start + a.cylinder.extent
                            - a.cylinder.extent / Math.abs(a.cylinder.extent);
                    }
                }
                if (b.cylinder.extent !== 0) {
                    if (b.isStart) {
                        angle_that = b.cylinder.start
                            + b.cylinder.extent / Math.abs(b.cylinder.extent);
                    } else {
                        angle_that = b.cylinder.start + b.cylinder.extent
                            - b.cylinder.extent / Math.abs(b.cylinder.extent);
                    }
                }
                value_this = Math.abs(angle_this - 180 / 2.0) % 360;
                value_that =  Math.abs(angle_that - 180 / 2.0) % 360;
                value_this = value_this > 180 ? 360 - value_this : value_this;
                value_that = value_that > 180 ? 360 - value_that : value_that;
            }
            if (value_this < value_that) {
                return - 1;
            } else if(value_this > value_that) {
                return 1;
            } else {
                return 0;
            }
        }
    },

    /**
     * 缓动函数类
     * kunsnat:  可以对比 缓动速查表 http://easings.net/zh-cn
     * @class FR.Chart.ChartEase
     */
    ChartEase: /**@class FR.Chart.ChartEase*/{
        /**
         * @param t 缓动时间 current time
         * @param b beginValue  开始值
         * @param c changeInValue 变动中的值
         * @param d 缓动总体时间
         * @returns {*}
         */
        easeInBack: function (t, b, c, d) {
            var s = 1.70158;
            return c * (t /= d) * t * ((s + 1) * t - s) + b;
        },

        easeInCirc: function (t, b, c, d) {
            return -c * (Math.sqrt(1 - (t /= d) * t) - 1) + b;
        },

        easeOutExpo: function (t, b, c, d) {
            return (t==d) ? b+c : c * (-Math.pow(2, -10 * t/d) + 1) + b;
        },

        easeOutCirc: function (t, b, c, d) {
            return c * Math.sqrt(1 - (t = t / d - 1) * t) + b;
        },

        easeOutBounce: function (t, b, c, d) {
            if ((t /= d) < (1 / 2.75)) {
                return c * (7.5625 * t * t) + b;
            } else if (t < (2 / 2.75)) {
                return c * (7.5625 * (t -= (1.5 / 2.75)) * t + .75) + b;
            } else if (t < (2.5 / 2.75)) {
                return c * (7.5625 * (t -= (2.25 / 2.75)) * t + .9375) + b;
            } else {
                return c * (7.5625 * (t -= (2.625 / 2.75)) * t + .984375) + b;
            }
        },

        easeOutQuint: function (t, b, c, d) {
            return c * ((t = t / d - 1) * t * t * t * t + 1) + b;
        }
    },

    /**
     * 颜色类.
     * @class FR.Chart.Color
     */
    Color: /**FR.Chart.Color*/{
        /**
         * @static
         * 固定颜色变化 百分比比例
         */
        FACTOR: 0.7,

        //这个方法是因为ie678的fillstyle是rgb()的形式
        rgb2hex:function(rgb){
            if(rgb.charAt(0) == '#'){
                return rgb;
            }
            var newRgb = "0"+rgb;//兼容ie678
            var ds = newRgb.split(/\D+/);
            var decimal = Number(ds[1]) * 65536 + Number(ds[2]) * 256 + Number(ds[3]);

            var s = decimal.toString(16);
            while (s.length < 6){
                s = "0" + s;
            }
            return "#"+s;
        },

        /**
         * 将各种形式的颜色, 'red' 'rgb(255,0,0)' 等..
         * 转化为数组[255,0,0]
         * @static
         * @param color 目标颜色
         * @returns {array} 颜色数组格式
         */
        getColorArray: function (color) {
            if (/^#[0-9a-f]{6}$/i.test(color)) {//#rrggbb
                return FR.Chart.Color.Map([color.substr(1, 2), color.substr(3, 2), color.substr(5, 2)],
                    function (x) {
                        return parseInt(x, 16);
                    }
                )
            }
            else if (/^#[0-9a-f]{3}$/i.test(color)) {//#rgb
                return FR.Chart.Color.Map([color.substr(1, 1), color.substr(2, 1), color.substr(3, 1)],
                    function (x) {
                        return parseInt(x + x, 16);
                    }
                )
            }
            else if (/^rgb(.*)$/i.test(color)) {//rgb(n,n,n) or rgb(n%,n%,n%)
                return FR.Chart.Color.Map(color.match(/\d+(\.\d+)?\%?/g),
                    function (x) {
                        return parseInt(x.indexOf("%") > 0 ? parseFloat(x, 10) * 2.55 : x, 10);
                    }
                )
            }
            else {//color
                var mapping = {    "red": "#FF0000", "white": "#FFFFFF", "black": "#000000",
                    "gray": "#D0D0D0", "green": "#00EC00", "blue": "#0000C6"
                };//略
                color = mapping[color.toLowerCase()];
                if (color) {
                    return FR.Chart.Color.Map([color.substr(1, 2), color.substr(3, 2), color.substr(5, 2)],
                        function (x) {
                            return parseInt(x, 16);
                        }
                    )
                }
            }
        },

        /**
         * 返回颜色的H值
         * @static
         * @param color 颜色
         * @returns {number} H值
         */
        getHex: function (color) {
            var array = FR.Chart.Color.getColorArray(color);
            return ~~(array[0]) << 16 ^ ~~(array[1]) << 8 ^ ~~(array[2]);
        },

        /**
         *
         * 颜色Map
         * @static
         * @param array 颜色数组
         * @param callback callBack
         * @param thisObject 当前Object
         * @returns {Array} 数组
         * @constructor
         */
        Map: function (array, callback, thisObject) {
            if (array.map) {
                return array.map(callback, thisObject);
            } else {
                var res = [];
                for (var i = 0, len = array.length; i < len; i++) {
                    res.push(callback.call(thisObject, array[i], i, array));
                }
                return res;
            }
        },

        /**
         * 最终可以用于canvas的颜色结果
         * @static
         * @param colorArray 颜色数组
         * @returns {string} 颜色字符串
         */
        getColor: function (colorArray) {
            for(var i = 0; i < colorArray.length; i++){
                colorArray[i] = parseInt(colorArray[i]);//IE下面要想这么用的话只能是整数
            }
            if(colorArray.length === 4) {
                return 'rgba(' + colorArray + ')';
            } else {
                return 'rgb(' + colorArray + ')';
            }
        },

        /**
         * 根据颜色数组 和 透明度 转为颜色
         * @static
         * @param colorArray 颜色数组
         * @param alpha 透明度
         * @returns {string} 颜色字符串
         */
        getColorWithAlpha:function(colorArray, alpha){
            for(var i = 0; i < colorArray.length; i++){
                colorArray[i] = parseInt(colorArray[i]);//IE下面要想这么用的话只能是整数
            }
            return 'rgba(' + colorArray + ',' + alpha + ')';
        },

        /**
         * 深点的颜色
         * @static
         * @param color 目标颜色
         * @returns {string} 结果颜色
         */
        darker: function (color) {
            var colorArray = FR.Chart.Color.getColorArray(color);
            colorArray[0] = Math.max(Math.round(colorArray[0] * FR.Chart.Color.FACTOR), 0);
            colorArray[1] = Math.max(Math.round(colorArray[1] * FR.Chart.Color.FACTOR), 0);
            colorArray[2] = Math.max(Math.round(colorArray[2] * FR.Chart.Color.FACTOR), 0);
            return FR.Chart.Color.getColor(colorArray);
        },

        /**
         * 降低饱和度 获取新颜色
         * @static
         * @param color 目标颜色
         * @returns {string} 结果颜色字符串
         */
        saturationDown: function (color) {
            var colorArray = FR.Chart.Color.getColorArray(color);
            var hsb = FR.Chart.Color.RGB2HSB(colorArray[0], colorArray[1], colorArray[2]);
            hsb[1] *= 0.3;
            return FR.Chart.Color.getColor(FR.Chart.Color.HSB2RGB(hsb[0], hsb[1], hsb[2]));
        },

        /**
         * rgb 转为 hsb
         * @static
         * @param r  r值
         * @param g g值
         * @param b b值
         * @returns {number[]} hsb数组
         * @constructor
         */
        RGB2HSB: function (r, g, b) {
            var hsbvals = [3];

            var cmax = (r > g) ? r : g;
            if (b > cmax) {
                cmax = b;
            }
            var cmin = (r < g) ? r : g;
            if (b < cmin) {
                cmin = b;
            }

            brightness = cmax / 255.0;
            if (cmax !== 0) {
                saturation = (cmax - cmin) / cmax;
            }
            else {
                saturation = 0;
            }
            if (saturation === 0) {
                hue = 0;
            }
            else {
                var redc = (cmax - r) / (cmax - cmin);
                var greenc = (cmax - g) / (cmax - cmin);
                var bluec = (cmax - b) / (cmax - cmin);
                if (r == cmax) {
                    hue = bluec - greenc;
                }
                else if (g == cmax) {
                    hue = 2.0 + redc - bluec;
                }
                else {
                    hue = 4.0 + greenc - redc;
                }
                hue = hue / 6.0;
                if (hue < 0) {
                    hue = hue + 1.0;
                }
            }
            hsbvals[0] = hue;
            hsbvals[1] = saturation;
            hsbvals[2] = brightness;
            return hsbvals;
        },

        /**
         * hsb值 转为rgb
         * @static
         * @param hue h值
         * @param saturation s值
         * @param brightness b值
         * @returns {Array} 返回rgb数组
         * @constructor
         */
        HSB2RGB: function (hue, saturation, brightness) {
            if(saturation < 0){
                saturation = 0;
            }else if(saturation > 1){
                saturation = 1;
            }

            if(brightness < 0){
                brightness = 0;
            }else if(brightness > 1){
                brightness = 1;
            }

            var r = 0, g = 0, b = 0;
            if (saturation === 0) {
                r = g = b = brightness * 255.0 + 0.5;
            } else {
                var h = (hue - Math.floor(hue)) * 6.0;
                var f = h - Math.floor(h);
                var p = brightness * (1.0 - saturation);
                var q = brightness * (1.0 - saturation * f);
                var t = brightness * (1.0 - (saturation * (1.0 - f)));
                switch (Math.floor(h)) {
                    case 0:
                        r = brightness * 255.0 + 0.5;
                        g = t * 255.0 + 0.5;
                        b = p * 255.0 + 0.5;
                        break;
                    case 1:
                        r = q * 255.0 + 0.5;
                        g = brightness * 255.0 + 0.5;
                        b = p * 255.0 + 0.5;
                        break;
                    case 2:
                        r = p * 255.0 + 0.5;
                        g = brightness * 255.0 + 0.5;
                        b = t * 255.0 + 0.5;
                        break;
                    case 3:
                        r = p * 255.0 + 0.5;
                        g = q * 255.0 + 0.5;
                        b = brightness * 255.0 + 0.5;
                        break;
                    case 4:
                        r = t * 255.0 + 0.5;
                        g = p * 255.0 + 0.5;
                        b = brightness * 255.0 + 0.5;
                        break;
                    case 5:
                        r = brightness * 255.0 + 0.5;
                        g = p * 255.0 + 0.5;
                        b = q * 255.0 + 0.5;
                        break;
                }
            }
            var rgb = [];
            rgb.push(r);
            rgb.push(g);
            rgb.push(b);
            return rgb;
        },

        /**
         * hex 转为颜色数组
         * @static
         * @param fillStyle 颜色
         * @returns {Array} 颜色rgb数组
         * @constructor
         */
        Hex2RGB: function(fillStyle){
            var rgb = [];
            if(!!fillStyle ){
                if(fillStyle.charAt(0) == '#'){
                    var r = parseInt(fillStyle.substr(1,2), 16);
                    var g = parseInt(fillStyle.substr(3,2), 16);
                    var b = parseInt(fillStyle.substr(5,2), 16);
                    rgb.push(r);
                    rgb.push(g);
                    rgb.push(b);
                    return rgb;
                }else{
                    return FR.Chart.Color.getColorArray(fillStyle)
                }
            }
        },

        /**
         *H颜色转为rgb数组
         * @static
         * @param fillStyle 颜色
         * @returns {Array} rgb数组
         * @constructor
         */
        Hex2fRGB: function(fillStyle){
            var rgb = [];
            if(fillStyle && fillStyle.length === 7 && fillStyle.charAt(0) == '#'){
                var r = parseInt(fillStyle.substr(1,2), 16);
                var g = parseInt(fillStyle.substr(3,2), 16);
                var b = parseInt(fillStyle.substr(5,2), 16);
                rgb.push(r / 255);
                rgb.push(g / 255);
                rgb.push(b / 255);
                return rgb;
            }
        },

        /**
         * rgb,rgba表示的颜色转化成16进制的表示
         * @param rgb
         */
        rgb2Hex:function(rgb){
            var rRgba = /rgba?\((\d{1,3}),(\d{1,3}),(\d{1,3})(,([.\d]+))?\)/,
                r, g, b, a,
                rsa = rgb.replace(/\s+/g, "").match(rRgba);
            if (rsa) {
                r = (+rsa[1]).toString(16);
                r = r.length === 1 ? "0" + r : r;
                g = (+rsa[2]).toString(16);
                g = g.length === 1 ? "0" + g : g;
                b = (+rsa[3]).toString(16);
                b = b.length === 1 ? "0" + b : b;
                a = (+(rsa[5] ? rsa[5] : 1)) * 100
                return {hex: "#" + r + g + b, alpha: Math.ceil(a)};
            } else {
                return {hex: rgb, alpha: 100};
            }
        },

        /**
         * 传入颜色的格式是十六进制表示或者'rgb()'之类的形式
         * 返回 颜色渐变起始 颜色
         * @static
         * @param baseColor 基础颜色
         * @param alpha 透明度
         * @returns {string} 渐变起始颜色字符串
         */
        getShadeStartColor:function(baseColor, alpha){
            var frgbvalue = [];
            if(baseColor.charAt(0) == '#'){
                frgbvalue = FR.Chart.Color.Hex2fRGB(baseColor)
            }else{
                frgbvalue = FR.Chart.Color.getColorArray(baseColor);
                for(var index = 0; index < frgbvalue.length; index++){
                    frgbvalue[index] /= 255;
                }
            }
            var result = [];

            for(var i = 0; i < frgbvalue.length; i++){
                var comp = frgbvalue[i];
                if(comp <= 0.5){
                    result.push(2.0 * comp * (1.0 - alpha));
                }else{
                    result.push(1 - 2 * (1 - comp) * alpha);
                }
            }

            for(var index = 0; index < result.length; index++){
                result[index] = parseInt(result[index] * 255);
            }

            return FR.Chart.Color.getColor(result);
        },

        /**
         * 根据color 透明度 转为颜色渐变结束颜色
         * @static
         * @param baseColor 基础颜色
         * @param alpha 透明度
         * @returns {string} 颜色rgb
         */
        getShadeEndColor:function(baseColor, alpha){
            var frgbvalue = [];
            if(baseColor.charAt(0) == '#'){
                frgbvalue = FR.Chart.Color.Hex2fRGB(baseColor)
            }else{
                frgbvalue = FR.Chart.Color.getColorArray(baseColor);
                for(var index = 0; index < frgbvalue.length; index++){
                    frgbvalue[index] /= 255;
                }
            }
            var result = [];

            for(var i = 0; i < frgbvalue.length; i++){
                var comp = frgbvalue[i];
                if(comp <= 0.5){
                    result[i] = 2 * comp * alpha;
                }else{
                    result[i] = 1 - 2 * (1 - comp) * (1 - alpha);
                }
            }

            for(var index = 0; index < result.length; index++){
                result[index] = parseInt(result[index] * 255);
            }

            return FR.Chart.Color.getColor(result);
        },

        /**
         * 颜色变亮
         * @static
         * @param color 基础颜色
         * @returns {Color} 变亮的结果颜色
         */
        brighter: function (color) {
            return FR.Chart.Color.brighterWithFactor(color, FR.Chart.Color.FACTOR);
        },

        /**
         * 根据Factor 变化颜色的变亮程度
         * @static
         * @param color 颜色
         * @param factor 变亮程度
         * @returns {string} 颜色rgb数组
         */
        brighterWithFactor:function(color, factor){
            var colorArray = FR.Chart.Color.getColorArray(color);

            var r = colorArray[0];
            var g = colorArray[1];
            var b = colorArray[2];

            var i = 1.0 / (1.0 - factor);
            if (r === 0 && g === 0 && b === 0) {
                i = Math.round(i);
                colorArray[0] = i;
                colorArray[1] = i;
                colorArray[2] = i;
                return FR.Chart.Color.getColor(colorArray);
            }

            if (r > 0 && r < i) {
                r = i;
            }
            if (g > 0 && g < i) {
                g = i;
            }
            if (b > 0 && b < i) {
                b = i;
            }

            var resultColor = [];
            resultColor.push(Math.min(Math.round(r / factor), 255));
            resultColor.push(Math.min(Math.round(g / factor), 255));
            resultColor.push(Math.min(Math.round(b / factor), 255));

            return FR.Chart.Color.getColor(resultColor);

        },

        /**
         * 颜色转为Int值
         * @static
         * @param color 颜色
         * @returns {number} 颜色数值
         */
        getIntValue: function (color) {
            var colorArray = FR.Chart.Color.getColorArray(color);

            var r = colorArray[0];
            var g = colorArray[1];
            var b = colorArray[2];

            return ((255 & 0xFF) << 24) |
                ((r & 0xFF) << 16) |
                ((g & 0xFF) << 8) |
                ((b & 0xFF) << 0);
        },

        /**
         * 从int值的颜色中返回R值
         * @static
         * @param intColor int值的颜色
         * @returns {number} r值
         */
        getRint: function (intColor) {
            return (intColor >> 16) & 0xFF;
        },

        /**
         * 从int值的颜色终返回G值
         * @static
         * @param intColor int值的颜色
         * @returns {number} g值
         */
        getGint: function (intColor) {
            return (intColor >> 8) & 0xFF;
        },

        /**
         * 从int值的颜色中返回B值
         * @static
         * @param intColor int值的颜色
         * @returns {number} b值
         */
        getBint: function (intColor) {
            return (intColor >> 0) & 0xFF;
        },

        /**
         * 从int值的颜色中返回a值
         * @static
         * @param intColor int值颜色
         * @returns {number} a值
         */
        getAint: function (intColor) {
            return (intColor >> 24 & 0xff);
        },

        /**
         * 从颜色中返回R值
         * @static
         * @param color 颜色
         * @returns {number}R值
         */
        getR: function (color) {
            return (FR.Chart.Color.getIntValue(color) >> 16) & 0xFF;
        },

        /**
         * 从颜色中返回G值
         * @static
         * @param color 颜色
         * @returns {number} G值
         */
        getG: function (color) {
            return (FR.Chart.Color.getIntValue(color) >> 8) & 0xFF;
        },

        /**
         *从颜色中返回B值
         * @static
         * @param color 颜色
         * @returns {number} B值
         */
        getB: function (color) {
            return (FR.Chart.Color.getIntValue(color) >> 0) & 0xFF;
        },

        /**
         * 从颜色中返回A值
         * @static
         * @param color 颜色
         * @returns {number} A值
         */
        getA: function (color) {
            return (FR.Chart.Color.getIntValue(color) >> 24 & 0xff);
        }
    },

    /**
     * 画法 集合类
     * @class FR.Chart.PaintUtils
     */
    PaintUtils: /**@class FR.Chart.PaintUtils*/{

        /**
         * 画出文本内容
         * @static
         * @param context 画布
         * @param text 文本
         * @param textAttr 文本格式
         * @param startX 起始x值
         * @param startY 起始y值
         * @param width 文本宽度
         * @param height 文本高度
         * @param currentDivKey 当前divkey
         */
        paintContent: function (context, text, textAttr, startX, startY, width, height, currentDivKey) {
            context.save();

            this.setContextFont(context, textAttr);
            this.setContextBaseLine(context, textAttr);

            if (textAttr.alignText == FR.Chart.Style.VERTICALTEXT) {
                this.paintVerticalString(context, text, textAttr, startX, startY, width, height, currentDivKey);
            } else if (textAttr.rotation !== 0) {
                this.paintRotationString(context, text, textAttr, startX, startY, width, height, currentDivKey);
            } else {
                this.paintNormalString(context, text, textAttr, startX, startY, width, height, currentDivKey);
            }
            context.restore();
        },

        /**
         * 画出正常文本字符串
         * @static
         * @param context 画布
         * @param text 文本
         * @param textAttr 文本格式
         * @param startX 起始x值
         * @param startY 起始y值
         * @param width 文本宽度
         * @param height 文本高度
         * @param currentDivKey 当前divkey
         */
        paintNormalString: function (context, text, textAttr, startX, startY, width, height, currentDivKey) {
            var offY = this.isOffSetY4SomeFont(context, textAttr.frFont);

            if (context.useRight) {
                var old = context.textAlign || 'left';
                context.textAlign = 'right';
                context.fillText(text, startX + width, startY + offY);
                context.textAlign = old;
            } else if (context.notUseDivText) {
                context.fillText(text, startX, startY + offY);
            } else {
                //bug65220 第二个饼图里面%标签只显示一半
                var isContainsPercent = false;
                if(typeof(text) == "string"){
                    isContainsPercent = text.indexOf("%") !== -1
                }
                var offWidth = ((FR.Chart.ChartUtils.isIE9Later()) || isContainsPercent)? 2 : 0;

                var newdiv = $("<div>" + text + "</div>").css('position', 'absolute')
                    .css('font', context.font).css('color', context.fillStyle)
                    .css('white-space', 'nowrap')
                    .css('left', startX).css('top', startY + offY).css('width', width + offWidth).css('height', height);

                FR.Chart.DivCon.addDiv(currentDivKey, newdiv[0]);
            }
        },

        /**
         * 画出 旋转 文本
         * @static
         * @param context 画布
         * @param text 文本
         * @param textAttr 文本格式
         * @param startX 起始x值
         * @param startY 起始y值
         * @param width 宽度
         * @param height 高度
         * @param currentDivKey 当前divkey
         */
        paintRotationString: function (context, text, textAttr, startX, startY, width, height, currentDivKey) {
            var rotaAngle = textAttr.rotation;
            var toAngle = Math.PI / 180;
            var offY = this.isOffSetY4SomeFont(context, textAttr.frFont);

            context.translate(startX, startY);

            var preDim = FR.Chart.GraphHelper.stringDimension(text, textAttr.frFont, 0);// 未旋转时, 字体所占用的实际 宽度.
            var heightGap = preDim.width * Math.sin(rotaAngle * toAngle);
            if (rotaAngle > 0) {
                context.translate(0, heightGap);
            }

            var roValue = -rotaAngle * toAngle;
            context.rotate(roValue);
            context.fillText(text, 0, offY);// 旋转字体的时候 画两遍. 能清楚一些
            context.fillText(text, 0, offY);
            context.rotate(-roValue);

            if (rotaAngle > 0) {
                context.translate(0, -heightGap);
            }
            context.translate(-startX, -startY);
        },

        /**
         * 画出竖排 文本
         * @static
         * @param context 画布
         * @param text 文本
         * @param textAttr 文本格式
         * @param startX 起始x值
         * @param startY 起始y值
         * @param width 宽度
         * @param height 高度
         * @param currentDivKey 当前div key
         */
        paintVerticalString: function (context, text, textAttr, startX, startY, width, height, currentDivKey) {
            var offY = this.isOffSetY4SomeFont(context, textAttr.frFont);

            var y = 0;

            context.save();

            var rotateValue = 90 * Math.PI / 180;

            for (var i = 0; i < text.length; i++) {
                var tmp = text.charAt(i);
                var preDim = FR.Chart.GraphHelper.stringDimension(tmp, textAttr.frFont, 0);
                if(text.charCodeAt(i) > 127 && tmp != FR.Chart.Constants.ESP) {
                    context.fillText(tmp, startX , startY + y);
                    y += preDim.height;
                }else{
                    //数字旋转画
                    context.rotate(rotateValue);
                    context.fillText(tmp, startY + y, -startX - textAttr.frFont.size + offY);
                    y += preDim.width;
                    context.rotate(-rotateValue);
                }
            }
            context.restore();
        },

        /**
         * kunsnat: 某些字体 (比如微软雅黑时), 字体在某些浏览器有偏移Y, 需要手动调整.
         * @static
         * @param ctx 画布
         * @param frfont 字体
         * @returns {number} y偏移值
         */
        isOffSetY4SomeFont: function (ctx, frfont) {
            if (FR.Browser.r.webkit || FR.Browser.r.gecko || FR.Browser.isIE()) {
                var font = ctx.font;
                if (font && (font.indexOf('Microsoft YaHei') !== -1 || font.indexOf(FR.i18nText('Fine-Dec_Chart_Use_MSBold')) !== -1
                    || font.indexOf('Century Gothic') != -1)) {
                    var size = 12.0 / 5.0;
                    if (frfont && frfont.size) {
                        size = frfont.size / 5.0;
                    }

                    return -size;
                }
            }

            return 0;
        },

        /**
         * 根据文本格式 设置ctx对应字体
         * @param context 画布
         * @param textAttr 文本格式
         */
        setContextFont: function (context, textAttr) {
            if (textAttr.frFont == null) {
                context.font = "12px Simsun";
                context.fillStyle = 'black';
            } else {
                var font = textAttr.frFont;
                context.font = font.style + ' ' +  parseInt(font.size) + 'px  ' + font.fontName;// kunsnat: 大小取整, 因为大于半数, 容易有截取的现象.
                context.fillStyle = font.color;
            }
        },

        /***
         * 坐标轴标签 是否需要 热点提示
         * @static
         * @param text 文本
         * @param textAttr 文本格式
         * @param bounds 边界
         * @param shouldBeHeight 是否使用高度
         * @returns {boolean} 是否需要热点提示
         */
        isNeedToolTip4AxisLabel: function (text, textAttr, bounds, shouldBeHeight) {
            var dim = FR.Chart.TextUtils.calculateTextDimensionWithRotation(text, textAttr);

            var offset = 0;
            if (shouldBeHeight) {
                return parseFloat(Number(dim.height).toFixed(2)) > parseFloat(Number(bounds.height + offset).toFixed(2));
            } else {
                return parseFloat(Number(dim.width).toFixed(2)) > parseFloat(Number(bounds.width + offset).toFixed(2));
            }
        },

        /**
         * 画热点提示
         * @static
         * @param context 画布
         * @param tooltip 热点提示
         * @param startX 起始x值
         * @param startY 起始y值
         * @param width 宽度
         * @param height 高度
         * @param back 背景
         * @param alpha 透明度
         */
        paintTooltip: function (context, tooltip, startX, startY, backgroundColor) {
            var marginTop = 6;
            var marginLine = 5;
            context.save();
            var color = 'black';
            if (backgroundColor == 'black') {
                color = 'white';
            }
            context.fillStyle = color;
            context.textBaseline = "top";

            var x = startX;
            var y = startY + marginTop;

            var lineHeight = 0;
            for (var i = 0; i < tooltip.length; i++) {
                var tip = tooltip[i];

                context.font = tip.font.style + " " + tip.font.size + "px " + tip.font.fontName;

                var offY = this.isOffSetY4SomeFont(context, tip.font);

                if (tip.rightAlign) {
                    var tmpWidth = (tip.width - tip.realWidth) > 0 ? tip.width - tip.realWidth : 0;
                    context.fillText(tip.value, x + tmpWidth, y + offY);
                } else {
                    context.fillText(tip.value, x, y + offY);
                }
                x += tip.width;

                lineHeight = Math.max(lineHeight, tip.height);

                if (tip.newLine) {// 画完之后 换行
                    x = startX;
                    y += lineHeight + marginLine;
                }
            }

            context.restore();
        },


        /**
         * 画系列的数据点提示
         * @param context 画布
         * @param tooltip 数据提示内容
         * @param startX 开始X的位置
         * @param startY 开始的Y位置
         * @param backgroundColor {'black','white'}黑色背景或者白色背景
         * @param colors 系列颜色
         * @param alpha 透明度
         */
        paintTooltip4Series: function (context, tooltip, startX, startY, backgroundColor, colors) {
            var marginTop = 6;
            var marginLine = 5;
            context.save();

            var color = 'black';
            if (backgroundColor == 'black') {
                color = 'white';
            }

            context.fillStyle = color;
            context.textBaseline = "top";
            var x = startX;
            var y = startY + marginTop;

            var lineHeight = 0;
            var index = colors.length;
            for (var i = 0; i < tooltip.length; i++) {
                var tip = tooltip[i];

                context.font = tip.font.style + " " + tip.font.size + "px " + tip.font.fontName;

                var offY = this.isOffSetY4SomeFont(context, tip.font);

                if (tip.rightAlign) {
                    var tmpWidth = (tip.width - tip.realWidth) > 0 ? tip.width - tip.realWidth : 0;
                    context.fillText(tip.value, x + tmpWidth, y + offY);
                } else {
                    context.fillText(tip.value, x, y + offY);
                }
                x += tip.width;

                lineHeight = Math.max(lineHeight, tip.height);

                if (tip.newLine) {// 画完之后 换行
                    x = startX + 6;
                    y += lineHeight + marginLine;
                    var arc = new FR.Chart.Arc2D(x, y + lineHeight / 3, 3, 0, 360, FR.Chart.Arc2D.CHORD);
                    index--;
                    if (index >= 0) {
                        context.fillStyle = colors[index];
                    }
                    arc.paint(context);
                    x += 5;
                    context.fillStyle = color;
                }
            }

            context.restore();
        },

        /**
         * 设置画布 画文本的基准起始线
         * @param context 画布
         * @param textAttr 文本
         */
        setContextBaseLine: function (context, textAttr) {
            // The alignment is recorded here for purposes of Opera compatibility
            if (navigator.userAgent.indexOf('Opera') != -1) {
                context.canvas.__rgraph_valign__ = textAttr.verticalAlignment;
                context.canvas.__rgraph_halign__ = textAttr.horizontalAlignment;
            }

            // 先用这个
            context.textBaseline = 'top';

            var hAlign = textAttr.horizontalAlignment;
            if (hAlign == FR.Chart.Constants.LEFT) {
                context.textAlign = 'left';
            } else if (hAlign == FR.Chart.Constants.CENTER) {
                context.textAlign = 'center';
            } else if (hAlign == FR.Chart.Constants.RIGHT) {
                context.textAlign = 'right';
            }
        }
    },

    /**
     * 辅助类, 一些辅助的方法.
     * @class FR.Chart.GraphHelper
     */
    GraphHelper: /**@class FR.Chart.GraphHelper*/{

        /**
         * 暂存的 临时画布, 一些计算字体大小等 用到.
         */
        ctx: null,

        /**
         * 是否动态绘制 结束
         * 问题: 饼图边框. 在点击扇形分离时, 响应焦点. 导致有边框 视觉停留.
         * @static
         */
        isDrawingEnd  : true,

        /**
         * 判断边界是否可用
         * @static
         * @param bounds 边界
         * @returns {boolean} 边界是否可用
         */
        isBoundsNaN: function (bounds) {
            return !bounds || isNaN(bounds.x) || isNaN(bounds.y) || isNaN(bounds.width) || isNaN(bounds.height);
        },

        /**
         * 根据线型 画出shape形状
         * @static
         * @param ctx 画布
         * @param shape shape形状
         * @param borderStyle 边框线型
         */
        draw: function (ctx, shape, borderStyle) {
            if(!ctx){     return; } ctx.save();
            ctx.lineWidth = FR.Chart.GraphHelper.getLineStyleSize(borderStyle);

            shape.draw(ctx);
            ctx.restore();
        },

        /**
         * 矩形, 消除canvas的像素延伸情况.
         * @static
         * @param bounds 边界
         * @returns {FR.Chart.Bounds} 返回新的矩形
         */
        dealRectShape4JSDraw: function (bounds) {
            if (bounds) {
                var newBounds = new FR.Chart.Bounds();
                newBounds.x = Math.round(bounds.x) + 0.5;
                newBounds.y = Math.round(bounds.y) + 0.5;
                newBounds.width = Math.round(bounds.width - 0.5);
                newBounds.height = Math.round(bounds.height - 0.5);

                return newBounds;
            }
            return bounds;
        },

        /**
         * 处理矩形, 消除canvas的像素延伸, 适合paint
         * @static
         * @param bounds 矩形
         * @returns {FR.Chart.Bounds} 新的矩形.
         */
        dealRectShape4JSPaint: function (bounds) {
            if (bounds) {
                var newBounds = new FR.Chart.Bounds();
                newBounds.x = Math.round(bounds.x);
                newBounds.y = Math.round(bounds.y);
                newBounds.width = Math.round(bounds.width);
                newBounds.height = Math.round(bounds.height);

                return newBounds;
            }
            return bounds;
        },

        /**
         * 线型处理. 消除canvas像素延伸的影响  一般也只对水平 垂直线有效果.
         * @static
         * @param line2D 新的折线
         */
        dealLine4JSDraw: function (line2D) {
            if (line2D) {
                var startPoint = line2D.startPoint;
                var endPoint = line2D.endPoint;

                if (startPoint) {
                    startPoint.x = Math.round(startPoint.x) + 0.5;
                    startPoint.y = Math.round(startPoint.y) + 0.5;
                }
                if (endPoint) {
                    endPoint.x = Math.round(endPoint.x) + 0.5;
                    endPoint.y = Math.round(endPoint.y) + 0.5;
                }
            }
        },

        /**
         * 计算文本占用的空间大小.
         * @static
         * @param string 字符文本
         * @param font 字体
         * @param rotation 旋转角度
         * @returns {FR.Chart.Dimension2D} 返回占用空间大小
         */
        stringDimension: function (string, font, rotation) {
            if (string && font) {
                var norWidth = string.length * font.size;

                if (FR.Chart.GraphHelper.ctx) {//kunsnat: 37823  ie8效果极差(7,8)
                    var f = FR.Chart.GraphHelper.ctx.font;
                    var newFont = font.style + ' ' + font.size + 'px ' + font.fontName;
                    FR.Chart.GraphHelper.ctx.font = newFont;
                    if (FR.Chart.GraphHelper.ctx.measureText // kunsnat: bug 29805 数据分析下 方法计算结果为0
                    ) {
                        var tmpWidth = FR.Chart.GraphHelper.ctx.measureText(string).width;
                        if (!isNaN(tmpWidth)) {
                            norWidth = tmpWidth;
                        }
                    }
                    FR.Chart.GraphHelper.ctx.font = f;

                    if(norWidth === 0 && !FR.isEmpty(string)) {// kunsnat: 囧 计算字符结果失灵... 又回归到估算. bug: 29805 IE678模式
                        norWidth = FR.Chart.GraphHelper.stringDimensionWidthDiv(string, font).width;
                    }

                } else {
                    var count = this.getNumberInt(string);
                    if (!isNaN(count)) {
                        norWidth = norWidth - count * font.size * 0.3;// 估算字符宽度
                    }
                }
                var nRotation = rotation * Math.PI / 180;

                var dimension = new FR.Chart.Dimension2D();
                //kunsnat: 暂时用font.size 做高, 可能会有点大.  宽度则是默认的5
                dimension.width = norWidth * Math.abs(Math.cos(nRotation)) + font.size * Math.abs(Math.sin(nRotation));
                dimension.height = norWidth * Math.abs(Math.sin(nRotation)) + font.size * Math.abs(Math.cos(nRotation));
                return dimension;
            }

            return new FR.Chart.Dimension2D();
        },

        stringDimensionWidthDiv:function(text, font){
            return FR.Chart.WebUtils.stringDimensionWidthDiv(text, font);
        },

        /**
         获取字符串 非数字的个数
         @static
         * @param string 字符串
         * @returns {number} 非数字的个数
         */
        getNumberInt: function (string) {
            var count = 0;
            for (var i = 0, len = string.length; i < len; i++) {
                var achar = string.charAt(i);
                if (!isNaN(achar) || achar == ',' || achar == "." || achar == ";" || achar == "  " || achar == '_'
                    || (achar >= 'a' && achar <= 'z' || (achar >= 'A' && achar <= 'Z'))) {
                    count++;
                }
            }
            return count;
        },

        /**
         * 计算热点提示占用的空间大小
         *  "rightAlign"  右对齐参数, 处理冒号右对齐, 所以前面的文本参数用统一的所有中最大的.
         *  @static
         * @param tooltip 热点提示
         * @returns {FR.Chart.Dimension2D} 占用空间大小
         */
        calculateTooltipDimension: function (tooltip) {// 根据tip中是否标记换行 计算高度 宽度
            var dimension = new FR.Chart.Dimension2D();

            var rightAlighWidth = -1;
            for (var i = 0; i < tooltip.length; i++) {
                var tooltipContentDim = this.stringDimension(tooltip[i].value, tooltip[i].font, 0);
                tooltip[i].width = tooltipContentDim.width;
                tooltip[i].height = tooltipContentDim.height;
                if (tooltip[i].rightAlign) {
                    rightAlighWidth = Math.max(tooltip[i].width, rightAlighWidth);
                }

                if (FR.Browser.isIE8Before()) {
                    if (tooltip[i].value.indexOf(",") != -1) {
                        if (tooltip[i].width < tooltip[i].font.size - 1) {
                            tooltip[i].width = tooltip[i].font.size - 1;
                        }
                    }
                    else if (tooltip[i].value.indexOf(":") != -1) {
                        if (tooltip[i].width < tooltip[i].font.size - 1) {
                            tooltip[i].width = tooltip[i].font.size - 1;
                        }
                    }
                }
            }

            var marginLine = 4;

            var height = 0;
            var width = 0;

            var lineWidth = 0;
            var lineHeight = 0;

            if (rightAlighWidth != -1) {// 处理右对齐参数.
                for (var i = 0; i < tooltip.length; i++) {
                    if (tooltip[i].rightAlign) {
                        tooltip[i].realWidth = tooltip[i].width;
                        tooltip[i].width = rightAlighWidth;
                    }
                }
            }

            for (var i = 0; i < tooltip.length; i++) {
                var tip = tooltip[i];
                lineHeight = Math.max(lineHeight, tip.height);
                lineWidth += tip.width;

                if (tip.newLine) {
                    width = Math.max(width, lineWidth);
                    lineWidth = 0;
                    height += lineHeight + marginLine;
                } else {
                    if (i == tooltip.length - 1) {
                        height += lineHeight;
                    }
                    width = Math.max(width, lineWidth);
                }
            }

            dimension.width = width;
            dimension.height = height;
            return dimension;
        },

        /**
         * 计算热点提示占用的空间大小
         *  "rightAlign"  右对齐参数, 处理冒号右对齐, 所以前面的文本参数用统一的所有中最大的.
         *  @static
         * @param tooltip 热点提示
         * @returns {FR.Chart.Dimension2D} 占用空间大小
         */
        calculateTooltipDimension4Series: function (tooltip) {// 根据tip中是否标记换行 计算高度 宽度
            var dimension = new FR.Chart.Dimension2D();

            var rightAlighWidth = -1;
            for (var i = 0; i < tooltip.length; i++) {
                var tooltipContentDim = this.stringDimension(tooltip[i].value, tooltip[i].font, 0);
                tooltip[i].width = tooltipContentDim.width;
                tooltip[i].height = tooltipContentDim.height;
                if (tooltip[i].rightAlign) {
                    rightAlighWidth = Math.max(tooltip[i].width, rightAlighWidth);
                }

                if (FR.Browser.isIE8Before()) {
                    if (tooltip[i].value.indexOf(",") != -1) {
                        if (tooltip[i].width < tooltip[i].font.size - 1) {
                            tooltip[i].width = tooltip[i].font.size - 1;
                        }
                    }
                    else if (tooltip[i].value.indexOf(":") != -1) {
                        if (tooltip[i].width < tooltip[i].font.size - 1) {
                            tooltip[i].width = tooltip[i].font.size - 1;
                        }
                    }
                }
            }

            var marginLine = 4;

            var height = 0;
            var width = 0;

            var lineWidth = 0;
            var lineHeight = 0;

            if (rightAlighWidth != -1) {// 处理右对齐参数.
                for (var i = 0; i < tooltip.length; i++) {
                    if (tooltip[i].rightAlign) {
                        tooltip[i].realWidth = tooltip[i].width;
                        tooltip[i].width = rightAlighWidth;
                    }
                }
            }

            for (var i = 0; i < tooltip.length; i++) {
                var tip = tooltip[i];
                lineHeight = Math.max(lineHeight, tip.height);
                lineWidth += tip.width;

                if (tip.newLine) {
                    width = Math.max(width, lineWidth);
                    lineWidth = 0;
                    height += lineHeight + marginLine;
                } else {
                    if (i == tooltip.length - 1) {
                        height += lineHeight;
                    }
                    width = Math.max(width, lineWidth);
                }
            }

            dimension.width = width + 11;
            dimension.height = height;
            return dimension;
        },

        /**
         * 线型 对应的线宽度
         * @static
         * @param lineStyle 线型
         * @returns {number} 线宽度
         */
        getLineStyleSize: function (lineStyle) {
            if (lineStyle == FR.Chart.Constants.LINE_NONE) {
                return 0;
            } else if (lineStyle == FR.Chart.Constants.LINE_MEDIUM ||
                lineStyle == FR.Chart.Constants.LINE_CHART_MED_ARROW ||
                lineStyle == FR.Chart.Constants.LINE_DOUBLE ||
                lineStyle == FR.Chart.Constants.LINE_MEDIUM_DASH ||
                lineStyle == FR.Chart.Constants.LINE_MEDIUM_DASH_DOT ||
                lineStyle == FR.Chart.Constants.LINE_MEDIUM_DASH_DOT_DOT ||
                lineStyle == FR.Chart.Constants.LINE_SLANTED_DASH_DOT) {
                return 2;
            } else if (lineStyle == FR.Chart.Constants.LINE_THICK || lineStyle == FR.Chart.Constants.LINE_CHART_THICK_ARROW) {
                return 3;
            }

            return 1;
        }
    },

    /**
     * 文本辅助类
     * @class FR.Chart.TextUtils
     */
    TextUtils : /**@class FR.Chart.TextUtils*/{

        /**
         * 计算文本使用文本格式时 占用的空间大小
         * @static
         * @param text 文本
         * @param textAttr 文本格式.
         * @returns {*}
         */
        calculateTextDimensionWithRotation: function (text, textAttr) {
            var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(text, textAttr);
            return this._calDimWithRotation(dim, textAttr);
        },

        getTextWhenNotEnoughWidth: function (string, width, font4Label) {
            if (!string) {
                return FR.Chart.Constants.ESP;
            }
            var pointDim = FR.Chart.GraphHelper.stringDimension("…", font4Label, 0);

            if (pointDim.width > width) {
                return FR.Chart.Constants.ESP;
            }

            var preWidth = width - pointDim.width;

            var result = "";
            for (var i = 0; i < string.length; i++) {
                var tmp = string.charAt(i);
                var reDim = FR.Chart.GraphHelper.stringDimension(result, font4Label, 0);
                var tmpDim = FR.Chart.GraphHelper.stringDimension(tmp, font4Label, 0);
                if (reDim.width + tmpDim.width > preWidth) {
                    break;
                }
                result += tmp;
            }

            return result + FR.Chart.Constants.ESP;
        },

        _calDimWithRotation: function (dim, textAttr) {
            if (textAttr && textAttr.rotation !== 0) {
                var rotation = textAttr.rotation;
                var angle = rotation * Math.PI / 180;
                var width = Math.abs(Math.cos(angle) * dim.width) + Math.abs(Math.sin(angle) * dim.height);
                var height = Math.abs(Math.cos(angle) * dim.height) + Math.abs(Math.sin(angle) * dim.width);
                return new FR.Chart.Dimension2D(width, height);
            }

            return dim;
        },

        calOriginalDimWidthRotation:function(dim,textAttr){
            if(textAttr && textAttr.rotation !== 0){
                var rotation = textAttr.rotation;
                var angle = rotation * Math.PI / 180;
                var sin = Math.abs(Math.sin(angle));
                var cos = Math.abs(Math.cos(angle));
                var originalWidth =(dim.width*cos-dim.height*sin)/(cos*cos -sin*sin);
                var originalHeight = (dim.width * sin - dim.height*cos)/(sin*sin - cos*cos);
                return new FR.Chart.Dimension2D(originalWidth, originalHeight);
            }
            return nowLength;
        },

        /**
         * 计算文本在使用文本格式时(不旋转)占用的空间大小
         * @static
         * @param text 文本
         * @param textAttr 文本格式
         * @returns {FR.Chart.Dimension2D} 占用的空间大小
         */
        calculateTextDimensionWithNoRotation: function (text, textAttr) {
            if (FR.isEmpty(text)) {
                return new FR.Chart.Dimension2D();
            }

            text += "";//确保是字符串

            if (textAttr == null) {
                textAttr = new FR.Chart.TextAttr();
            }

            var font = textAttr.frFont;
            if (font == null) {
                font = new FR.Chart.FRFont();
            }
            text = FR.Chart.TextUtils.changeAllReline2Normal(text);
            var strs = text.split('\n');

            var dimension = new FR.Chart.Dimension2D();
            for (var i = 0; i < strs.length; i++) {
                if (textAttr.alignText == FR.Chart.Style.VERTICALTEXT) {
                    var d = this._calculateVerticalTextDimension(strs[i], textAttr);
                    dimension.width += d.width;
                    dimension.height = Math.max(dimension.height, d.height);
                } else {
                    var d = FR.Chart.GraphHelper.stringDimension(strs[i], font, 0);
                    dimension.width = Math.max(dimension.width, d.width);
                    dimension.height += d.height;
                }
            }

            return dimension;
        },

        //计算文字竖排时占据的区域大小
        _calculateVerticalTextDimension:function(string, textAttr){
            var height = 0;
            var width = 0;
            for (var i = 0; i < string.length; i++) {
                var letter = string.charAt(i);
                var letterDim = FR.Chart.GraphHelper.stringDimension(letter, textAttr.frFont, 0);

                if(string.charCodeAt(i) > 127 && letter != FR.Chart.Constants.ESP) {
                    width = Math.max(letterDim.width, width);
                    height += letterDim.height;
                } else {
                    width = Math.max(letterDim.height, width);
                    height += letterDim.width;
                }
            }
            return new FR.Chart.Dimension2D(width, height);
        },

        /**
         * 根据旋转情况 画出文本
         * @static
         * @param ctx 画布
         * @param text 文本
         * @param textAttr 文本格式
         * @param bounds 边界
         */
        drawLabelWithRotation: function (ctx, text, textAttr, bounds) {
            if (!text || bounds == null) {
                return;
            }

            this._drawStringWithTextAttr(ctx, text, textAttr, bounds);
        },

        /**
         * 统一所有换行符的格式使用
         * @static
         * @param text 文本
         * @returns {String} 返回结果字符串
         */
        changeAllReline2Normal: function (text) {
            if (!text) {
                return text;
            }

            text = text.toString();

            var result = "";
            for (var i = 0; i < text.length; i++) {
                if (text.charAt(i) == "\\" && i < text.length && text.charAt(i + 1) == "n") {
                    i++;
                    result += '\n';
                } else {
                    result += text.charAt(i);
                }
            }
            return result.toString();
        },

        /**
         * 画出文本情况  (不考虑旋转)
         * @static
         * @param ctx 画布
         * @param text 文本
         * @param textAttr 文本格式.
         * @param bounds 边界
         * @param currentDivKey 当前div key
         */
        drawLabelWithNoRotation: function (ctx, text, textAttr, bounds, currentDivKey) {
            if (!text || bounds == null) {
                return;
            }

            var noRotationTextAttr = this._createNoRotationTextAttr(textAttr);
            this._drawStringWithTextAttr(ctx, text, noRotationTextAttr, bounds, currentDivKey);
        },

        _drawStringWithTextAttr: function (ctx, text, textAttr, bounds, currentDivKey) {
            text = FR.Chart.TextUtils.changeAllReline2Normal(text);
            var relineStringS = text.split('\n');
            var y = bounds.y;
            var x = bounds.x;
            for (var i = 0; i < relineStringS.length; i++) {
                var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(relineStringS[i], textAttr);
                if (textAttr.alignText == FR.Chart.Style.VERTICALTEXT) {
                    FR.Chart.PaintUtils.paintContent(ctx, relineStringS[i], textAttr, x, y, bounds.width, bounds.height, currentDivKey);
                    x += dim.width;
                } else {
                    FR.Chart.PaintUtils.paintContent(ctx, relineStringS[i], textAttr, x, y, bounds.width, bounds.height, currentDivKey);
                    y += dim.height;
                }
            }
        },

        _createNoRotationTextAttr: function (textAttr) {
            if (textAttr == null) {
                textAttr = new FR.Chart.TextAttr();
            }
            if (textAttr.frFont == null) {
                textAttr.frFont = new FR.Chart.FRFont();
            }

            var result = textAttr.clone();
            result.rotation = 0;

            return result;
        }
    },

    /**
     * 各种 标记 工厂
     * @class FR.Chart.MarkerFactory
     */
    MarkerFactory : /**@class FR.Chart.MarkerFactory*/{
        /**
         * 创建 初始化所有标记类型
         * @static
         */
        createTypeArray: function () {
            if (FR.Chart.MarkerFactory.type && FR.Chart.MarkerFactory.index) {
                return;
            }

            var type = [];
            var index = [];
            FR.Chart.MarkerFactory.type = type;
            // index用于按照系列顺序生成标记类型
            FR.Chart.MarkerFactory.index = index;

            FR.Chart.MarkerFactory.addNewMarker(type, index, FR.Chart.Marker.RoundMarker, new FR.Chart.RoundMarker());
            FR.Chart.MarkerFactory.addNewMarker(type, index, FR.Chart.Marker.RoundFilledMarker, new FR.Chart.RoundFilledMarker());
            FR.Chart.MarkerFactory.addNewMarker(type, index, FR.Chart.Marker.SquareFilledMarker, new FR.Chart.SquareFilledMarker());
            FR.Chart.MarkerFactory.addNewMarker(type, index, FR.Chart.Marker.TriangleFilledMarker, new FR.Chart.TriangleFilledMarker());
            FR.Chart.MarkerFactory.addNewMarker(type, index, FR.Chart.Marker.DiamondFilledMarker, new FR.Chart.DiamondFilledMarker());
            FR.Chart.MarkerFactory.addNewMarker(type, index, FR.Chart.Marker.DiamondMarker, new FR.Chart.DiamondMarker());
            FR.Chart.MarkerFactory.addNewMarker(type, index, FR.Chart.Marker.SquareMarker, new FR.Chart.SquareMarker());
            FR.Chart.MarkerFactory.addNewMarker(type, index, FR.Chart.Marker.TriangleMarker, new FR.Chart.TriangleMarker());
            FR.Chart.MarkerFactory.addNewMarker(type, index, FR.Chart.Marker.NullMarker, new FR.Chart.NullMarker());
        },

        /**
         * 添加 标记类型, 在数组中
         * @static
         * @param typeArray 标记类型数组
         * @param makerArray 标记类型字符串数组
         * @param markerType 标记类型
         * @param marker 标记
         */
        addNewMarker: function (typeArray, makerArray, markerType, marker) {
            typeArray[markerType] = marker;
            makerArray.push(markerType);
        },

        /**
         * 根据类型 创建新的Marker
         * @static
         * @param markerType 标记类型
         * @returns {Object} 新的标记
         */
        createMarker: function (markerType) {
            FR.Chart.MarkerFactory.createTypeArray();

            if (FR.Chart.MarkerFactory.type[markerType]) {
                return FR.Chart.MarkerFactory.type[markerType].clone();
            }

            return new FR.Chart.DiamondFilledMarker();
        },

        /**
         * 根据序号 返回对应的标记类型 字符串
         * @static
         * @param index 标记的序号
         * @returns {String} 标记类型 字符串
         */
        markerIndex2String: function (index) {
            FR.Chart.MarkerFactory.createTypeArray();
            //散点图未设置标记点类型，循环所有标记点类型（不包括无标记点类型）
            return FR.Chart.MarkerFactory.index[index % (FR.Chart.MarkerFactory.index.length - 1)];
        }
    },

    /**
     * 条件属性 处理工厂
     * @class FR.Chart.ConditionAttrFactory
     */
    ConditionAttrFactory : /**@class FR.Chart.ConditionAttrFactory*/{

        /**
         * 创建条件属性
         * @static
         * @param conditionInstance 条件属性类型
         * @param obj 条件属性针对的Object
         * @param colors 颜色数组
         * @returns {Object} 条件属性
         */
        createConditionAttr: function (conditionInstance, obj, colors) {
            var index = -1;
            var seriesCondition = null;
            if (obj && obj.seriesIndex != undefined  // 计算在两个饼图并列时 不能识别DataPoint bug23360
                && (obj.jsType == "DataPoint" || obj.jsType == "DataSeries")) {
                index = obj.seriesIndex;
            } else if (!isNaN(obj)) {
                index = obj;
            }

            if (conditionInstance instanceof FR.Chart.AttrColor) {
                var seriesColor = new FR.Chart.AttrColor();
                seriesColor.seriesColor = colors[index % colors.length];
                seriesCondition = seriesColor;
            } else if (conditionInstance instanceof FR.Chart.AttrBackground) {
                var seriesBackground = new FR.Chart.AttrBackground();
                var colorBackground = FR.Chart.ColorBackground.getInstance(colors[index % colors.length]);
                colorBackground.isSimulation3D = true;
                seriesBackground.seriesBackground = colorBackground;
                seriesCondition = seriesBackground;
            } else if (conditionInstance instanceof FR.Chart.AttrLine) {
                var seriesLine = new FR.Chart.AttrLine();

                var color = colors[index % colors.length];
                seriesLine.seriesLineColor = color;
                seriesLine.seriesLineStyle = FR.Chart.Constants.LINE_THIN;
                seriesCondition = seriesLine;
            } else if (conditionInstance instanceof FR.Chart.AttrLineStyle) {
                return new FR.Chart.AttrLineStyle();
            } else if (conditionInstance instanceof FR.Chart.AttrMarkerType) {
                var markerType = new FR.Chart.AttrMarkerType();
                markerType.markerType = FR.Chart.MarkerFactory.markerIndex2String(index);
                seriesCondition = markerType;
            } else if (conditionInstance instanceof FR.Chart.AttrTrendLine) {
                seriesCondition = null;
            } else {
                seriesCondition = conditionInstance;
            }
            return seriesCondition;
        }
    },

    /**
     * 支持SVG扩展,解析svg图片，构造path
     */
    SVGExtension:{

        _toFloat:function(str){
            return parseFloat(str || 0);
        },

        _trim:function(str){
            return str.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
        },

        _parsePoints:function(str){
            var list = this._trim(str).replace(/,/g, ' ').split(/\s+/);
            var points = [];

            for (var i = 0; i < list.length;) {
                var x = parseFloat(list[i++]);
                var y = parseFloat(list[i++]);
                points.push([x, y]);
            }
            return points;
        },

        styleList:[
            'fill', 'stroke', 'stroke-width', 'opacity', 'stroke-dasharray',
            'stroke-linecap', 'stroke-linejoin', 'stroke-miterlimit',
            'font-style', 'font-weight', 'font-size', 'font-family'
        ],

        _parseStyleAttribute:function(xmlNode){
            var style = xmlNode.getAttribute('style');
            var styleRegex = /(\S*?):(.*?);/g;
            if (style) {
                var styleMap = {};
                style = style.replace(/\s*([;:])\s*/g, '$1');
                style.replace(styleRegex, function(str, key, val){
                    styleMap[key] = val;
                });
                return styleMap;
            }
            return {};
        },

        _parseAttributes:function(xmlNode){
            var styleMap = {}

            for (var i = 0; i < this.styleList.length; i++) {
                var styleName = this.styleList[i];
                var attrVal = xmlNode.getAttribute(styleName);
                if (attrVal) {
                    styleMap[styleName] = attrVal;
                }
            }
            // style属性里的样式拥有更高优先级
            var styleMap2 = this._parseStyleAttribute(xmlNode);
            for (var name in styleMap2) {
                styleMap[name] = styleMap2[name];
            }

            return styleMap;
        },

        //读取透明度等信息
        _extendCommonAttributes:function(obj, xmlNode){
            var attrbutes = this._parseAttributes(xmlNode);
            var color = attrbutes.fill;
            var strokeColor = attrbutes.stroke;
            var lineWidth = attrbutes['stroke-width'];
            var opacity = attrbutes.opacity;

            if (color && color != 'none') {
                obj.color = color;
                if (strokeColor) {
                    obj.brushType = 'both';
                    obj.strokeColor = strokeColor;
                } else {
                    obj.brushType = 'fill';
                }
            } else if (strokeColor && strokeColor != 'none') {
                obj.strokeColor = strokeColor;
                obj.brushType = 'stroke';
            }
            if (lineWidth && lineWidth != 'none') {
                obj.lineWidth = parseFloat(lineWidth);
            }
            if (opacity && opacity != 'none') {
                obj.opacity = parseFloat(opacity);
            }
        },

        /**
         * 返回svg图片的原始大小
         * @param root 根节点
         * @returns {{left: *, top: *, width: *, height: *}} 图片大小
         */
        getImageBounds:function(root){
            if(!root){
                return ;
            }
            var svgNode = root.firstChild;
            // Find the svg node
            while (!(svgNode.nodeName.toLowerCase() == 'svg' && svgNode.nodeType === 1)) {
                svgNode = svgNode.nextSibling;
            }

            var x = this._toFloat(svgNode.getAttribute('x'));
            var y = this._toFloat(svgNode.getAttribute('y'));
            var width = this._toFloat(svgNode.getAttribute('width'));
            var height = this._toFloat(svgNode.getAttribute('height'));

            if(FR.isEmpty(svgNode.getAttribute('width')) || FR.isEmpty(svgNode.getAttribute('height'))){
                var viewBox = svgNode.getAttribute('viewBox');
                var sizeArray = viewBox.split(' ');
                if(sizeArray.length === 4){
                    x = parseFloat(sizeArray[0]);
                    y = parseFloat(sizeArray[1]);
                    width = parseFloat(sizeArray[2]) - x;
                    height = parseFloat(sizeArray[3]) - y;
                }
            }

            return {
                x: x,
                y: y,
                width: width,
                height: height
            };
        },

        /**
         *
         * @param shape
         */
        getPath4Shape:function(shape){

            if(shape.shapePath){
                return shape.shapePath;
            }

            var shapePath;

            switch (shape.shapeType){
                case 'path':
                    shapePath = new FR.Chart.GeneralPath();
                    FR.Chart.SVGPathUtils.buildPath(shapePath, shape);
                    break;
                case 'rectangle':
                    shapePath = new FR.Chart.Bounds(shape.x, shape.y, shape.width, shape.height);
                    break;
                case 'line':
                    var startPoint = new FR.Chart.Point2D(shape.xStart, shape.yStart);
                    var endPoint = new FR.Chart.Point2D(shape.xEnd, shape.yEnd);
                    shapePath = new FR.Chart.Line2D(startPoint, endPoint);
                    break;
                case 'circle':
                    shapePath = new FR.Chart.Arc2D(shape.x, shape.y, shape.r, 0, 360);
                    break;
                case 'polygon':
                    shapePath = new FR.Chart.GeneralPath();
                    var pointList = shape.pointList;
                    for(var i = 0; i < pointList.length; i++){
                        var point = pointList[i];
                        if(i === 0){
                            shapePath.moveTo(point[0], point[1]);
                        }else if(i == pointList.length - 1){
                            shapePath.closePath();
                        }else{
                            shapePath.lineTo(point[0], point[1]);
                        }
                    }
                    break;
                default :
                    shapePath = new FR.Chart.GeneralPath();
            }

            shape.shapePath = shapePath;

            return shapePath;
        },

        drawNodeShape:function(ctx, shape, transform){
            var shapePath = this.getPath4Shape(shape).transform(transform);

            ctx.globalAlpha = shape.opacity ? shape.opacity : 1;
            ctx.lineWidth = shape.lineWidth ? shape.lineWidth : 0;
            ctx.strokeStyle = shape.strokeColor ? shape.strokeColor : 'rgba(255,255,255,0)';
            ctx.fillStyle = shape.color ? shape.color : 'rgba(255,255,255,0)';

            switch (shape.brushType){
                case 'both':
                    shapePath.paint(ctx);
                    shapePath.draw(ctx);
                    break;
                case 'stroke':
                    shapePath.draw(ctx);
                    break;
                case 'fill':
                    shapePath.paint(ctx);
                    break;
            }
        },

        /**
         * 递归将svg节点信息转化成path信息
         * @param root 根借点
         * @param scale [x,y] 横纵向的缩放比例
         */
        svgResolve:function(root, namedNodes, nameTo, layerTo, names){
            if(!root){
                return ;
            }
            var elList = [];
            var self = this;
            function _getShape(root) {
                var tagName = root.tagName;
                if (self.shapeBuilders[tagName]) {
                    var obj = self.shapeBuilders[tagName](root);

                    if (obj) {
                        self._extendCommonAttributes(obj, root);

                        var name = root.getAttribute('name') || '';
                        var useName = root.getAttribute('nameTo') || '';
                        var nextLayer = root.getAttribute('layerTo') || '';

                        if(!FR.isEmpty(name)){
                            namedNodes.push({name:name, path:obj});
                            nameTo[name] = useName;
                            layerTo[name] = nextLayer;
                            names.push(name)
                        }

                        elList.push(obj);
                    }
                }
                var shapes = root.childNodes;
                for (var i = 0, len = shapes.length; i < len; i++) {
                    _getShape(shapes[i]);
                }
            }
            _getShape(root);
            return elList;
        },

        shapeBuilders : {
            path: function(xmlNode) {
                var path = xmlNode.getAttribute('d');
                return {
                    shapeType: 'path',
                    path: path
                };
            },

            rect: function(xmlNode) {
                var x = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('x'));
                var y = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('y'));
                var width = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('width'));
                var height = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('height'));

                return {
                    shapeType: 'rectangle',
                    x: x,
                    y: y,
                    width: width,
                    height: height
                };
            },

            line: function(xmlNode) {
                var x1 = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('x1'));
                var y1 = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('y1'));
                var x2 = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('x2'));
                var y2 = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('y2'));

                return {
                    shapeType: 'line',
                    xStart: x1,
                    yStart: y1,
                    xEnd: x2,
                    yEnd: y2
                };
            },

            circle: function(xmlNode) {
                var cx = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('cx'));
                var cy = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('cy'));
                var r = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('r'));

                return {
                    shapeType: 'circle',
                    x: cx,
                    y: cy,
                    r: r
                };
            },

            ellipse: function(xmlNode) {
                var cx = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('cx'));
                var cy = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('cy'));
                var rx = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('rx'));
                var ry = FR.Chart.SVGExtension._toFloat(xmlNode.getAttribute('ry'));

                return {
                    shapeType: 'ellipse',
                    x: cx,
                    y: cy,
                    a: rx,
                    b: ry
                };
            },

            polygon: function(xmlNode) {
                var points = xmlNode.getAttribute('points');
                var min = [Infinity, Infinity];
                var max = [-Infinity, -Infinity];
                if (points) {
                    points = FR.Chart.SVGExtension._parsePoints(points);

                    for (var i = 0; i < points.length; i++) {
                        var p = points[i];

                        min[0] = Math.min(p[0], min[0]);
                        min[1] = Math.min(p[1], min[1]);

                        max[0] = Math.max(p[0], max[0]);
                        max[1] = Math.max(p[1], max[1]);

                    }
                    return {
                        shapeType: 'polygon',
                        pointList: points
                    };
                }
            },

            polyline: function(xmlNode) {
                var obj = this.polygon(xmlNode);
                return obj;
            }
        }

    },

    /**
     * SVG的path节点的构造工具方法
     * http://www.w3.org/TR/2011/REC-SVG11-20110816/paths.html#PathData
     */
    SVGPathUtils:{
        _convertPoint : function (x1, y1, x2, y2, fa, fs, rx, ry, psiDeg) {
            var psi = psiDeg * (Math.PI / 180.0);
            var xp = Math.cos(psi) * (x1 - x2) / 2.0
                + Math.sin(psi) * (y1 - y2) / 2.0;
            var yp = -1 * Math.sin(psi) * (x1 - x2) / 2.0
                + Math.cos(psi) * (y1 - y2) / 2.0;

            var lambda = (xp * xp) / (rx * rx) + (yp * yp) / (ry * ry);

            if (lambda > 1) {
                rx *= Math.sqrt(lambda);
                ry *= Math.sqrt(lambda);
            }

            var f = Math.sqrt((((rx * rx) * (ry * ry))
                - ((rx * rx) * (yp * yp))
                - ((ry * ry) * (xp * xp))) / ((rx * rx) * (yp * yp)
                + (ry * ry) * (xp * xp))
            );

            if (fa === fs) {
                f *= -1;
            }
            if (isNaN(f)) {
                f = 0;
            }

            var cxp = f * rx * yp / ry;
            var cyp = f * -ry * xp / rx;

            var cx = (x1 + x2) / 2.0
                + Math.cos(psi) * cxp
                - Math.sin(psi) * cyp;
            var cy = (y1 + y2) / 2.0
                + Math.sin(psi) * cxp
                + Math.cos(psi) * cyp;

            var theta = this.vAngle([ 1, 0 ], [ (xp - cxp) / rx, (yp - cyp) / ry ]);
            var u = [ (xp - cxp) / rx, (yp - cyp) / ry ];
            var v = [ (-1 * xp - cxp) / rx, (-1 * yp - cyp) / ry ];
            var dTheta = this.vAngle(u, v);

            if (this.vRatio(u, v) <= -1) {
                dTheta = Math.PI;
            }
            if (this.vRatio(u, v) >= 1) {
                dTheta = 0;
            }
            if (fs === 0 && dTheta > 0) {
                dTheta = dTheta - 2 * Math.PI;
            }
            if (fs === 1 && dTheta < 0) {
                dTheta = dTheta + 2 * Math.PI;
            }
            return [ cx, cy, rx, ry, theta, dTheta, psi, fs ];
        },

        //距离
        vMag: function(v){
            return Math.sqrt(v[0] * v[0] + v[1] * v[1]);
        },

        //弧度
        vRatio: function(u, v){
            return (u[0] * v[0] + u[1] * v[1]) / (this.vMag(u) * this.vMag(v));
        },

        //角度
        vAngle: function(u, v){
            return (u[0] * v[1] < u[1] * v[0] ? -1 : 1)
                * Math.acos(this.vRatio(u, v));
        },

        PathSegment:function(command, points){
            this.command = command;
            this.points = points || null;
        },

        //将svg的path定义转化为点的定义
        buildPathArray : function (data, x, y) {
            if (!data) {
                return [];
            }

            // 平移
            x = x || 0;
            y = y || 0;
            // command string
            var cs = data;

            // command chars
            var cc = [
                'm', 'M', 'l', 'L', 'v', 'V', 'h', 'H', 'z', 'Z',
                'c', 'C', 'q', 'Q', 't', 'T', 's', 'S', 'a', 'A'
            ];

            cs = cs.replace(/-/g, ' -');
            cs = cs.replace(/  /g, ' ');
            cs = cs.replace(/ /g, ',');
            cs = cs.replace(/,,/g, ',');

            var n;
            // create pipes so that we can split the data
            for (n = 0; n < cc.length; n++) {
                cs = cs.replace(new RegExp(cc[n], 'g'), '|' + cc[n]);
            }

            // create array
            var arr = cs.split('|');
            var ca = [];
            // init context point
            var cpx = 0;
            var cpy = 0;
            for (n = 1; n < arr.length; n++) {
                var str = arr[n];
                var c = str.charAt(0);
                str = str.slice(1);
                str = str.replace(new RegExp('e,-', 'g'), 'e-');

                var p = str.split(',');
                if (p.length > 0 && p[0] === '') {
                    p.shift();
                }

                for (var i = 0; i < p.length; i++) {
                    p[i] = parseFloat(p[i]);
                }
                while (p.length > 0) {
                    if (isNaN(p[0])) {
                        break;
                    }
                    var cmd = null;
                    var points = [];

                    var ctlPtx;
                    var ctlPty;
                    var prevCmd;

                    var rx;
                    var ry;
                    var psi;
                    var fa;
                    var fs;

                    var x1 = cpx;
                    var y1 = cpy;

                    // convert l, H, h, V, and v to L
                    switch (c) {
                        case 'l':
                            cpx += p.shift();
                            cpy += p.shift();
                            cmd = 'L';
                            points.push(cpx, cpy);
                            break;
                        case 'L':
                            cpx = p.shift();
                            cpy = p.shift();
                            points.push(cpx, cpy);
                            break;
                        case 'm':
                            cpx += p.shift();
                            cpy += p.shift();
                            cmd = 'M';
                            points.push(cpx, cpy);
                            c = 'l';
                            break;
                        case 'M':
                            cpx = p.shift();
                            cpy = p.shift();
                            cmd = 'M';
                            points.push(cpx, cpy);
                            c = 'L';
                            break;

                        case 'h':
                            cpx += p.shift();
                            cmd = 'L';
                            points.push(cpx, cpy);
                            break;
                        case 'H':
                            cpx = p.shift();
                            cmd = 'L';
                            points.push(cpx, cpy);
                            break;
                        case 'v':
                            cpy += p.shift();
                            cmd = 'L';
                            points.push(cpx, cpy);
                            break;
                        case 'V':
                            cpy = p.shift();
                            cmd = 'L';
                            points.push(cpx, cpy);
                            break;
                        case 'C':
                            points.push(p.shift(), p.shift(), p.shift(), p.shift());
                            cpx = p.shift();
                            cpy = p.shift();
                            points.push(cpx, cpy);
                            break;
                        case 'c':
                            points.push(
                                cpx + p.shift(), cpy + p.shift(),
                                cpx + p.shift(), cpy + p.shift()
                            );
                            cpx += p.shift();
                            cpy += p.shift();
                            cmd = 'C';
                            points.push(cpx, cpy);
                            break;
                        case 'S':
                            ctlPtx = cpx;
                            ctlPty = cpy;
                            prevCmd = ca[ca.length - 1];
                            if (prevCmd.command === 'C') {
                                ctlPtx = cpx + (cpx - prevCmd.points[2]);
                                ctlPty = cpy + (cpy - prevCmd.points[3]);
                            }
                            points.push(ctlPtx, ctlPty, p.shift(), p.shift());
                            cpx = p.shift();
                            cpy = p.shift();
                            cmd = 'C';
                            points.push(cpx, cpy);
                            break;
                        case 's':
                            ctlPtx = cpx, ctlPty = cpy;
                            prevCmd = ca[ca.length - 1];
                            if (prevCmd.command === 'C') {
                                ctlPtx = cpx + (cpx - prevCmd.points[2]);
                                ctlPty = cpy + (cpy - prevCmd.points[3]);
                            }
                            points.push(
                                ctlPtx, ctlPty,
                                cpx + p.shift(), cpy + p.shift()
                            );
                            cpx += p.shift();
                            cpy += p.shift();
                            cmd = 'C';
                            points.push(cpx, cpy);
                            break;
                        case 'Q':
                            points.push(p.shift(), p.shift());
                            cpx = p.shift();
                            cpy = p.shift();
                            points.push(cpx, cpy);
                            break;
                        case 'q':
                            points.push(cpx + p.shift(), cpy + p.shift());
                            cpx += p.shift();
                            cpy += p.shift();
                            cmd = 'Q';
                            points.push(cpx, cpy);
                            break;
                        case 'T':
                            ctlPtx = cpx, ctlPty = cpy;
                            prevCmd = ca[ca.length - 1];
                            if (prevCmd.command === 'Q') {
                                ctlPtx = cpx + (cpx - prevCmd.points[0]);
                                ctlPty = cpy + (cpy - prevCmd.points[1]);
                            }
                            cpx = p.shift();
                            cpy = p.shift();
                            cmd = 'Q';
                            points.push(ctlPtx, ctlPty, cpx, cpy);
                            break;
                        case 't':
                            ctlPtx = cpx, ctlPty = cpy;
                            prevCmd = ca[ca.length - 1];
                            if (prevCmd.command === 'Q') {
                                ctlPtx = cpx + (cpx - prevCmd.points[0]);
                                ctlPty = cpy + (cpy - prevCmd.points[1]);
                            }
                            cpx += p.shift();
                            cpy += p.shift();
                            cmd = 'Q';
                            points.push(ctlPtx, ctlPty, cpx, cpy);
                            break;
                        case 'A':
                            rx = p.shift();
                            ry = p.shift();
                            psi = p.shift();
                            fa = p.shift();
                            fs = p.shift();

                            x1 = cpx, y1 = cpy;
                            cpx = p.shift(), cpy = p.shift();
                            cmd = 'A';
                            points = this._convertPoint(
                                x1, y1, cpx, cpy, fa, fs, rx, ry, psi
                            );
                            break;
                        case 'a':
                            rx = p.shift();
                            ry = p.shift();
                            psi = p.shift();
                            fa = p.shift();
                            fs = p.shift();

                            x1 = cpx, y1 = cpy;
                            cpx += p.shift();
                            cpy += p.shift();
                            cmd = 'A';
                            points = this._convertPoint(
                                x1, y1, cpx, cpy, fa, fs, rx, ry, psi
                            );
                            break;
                    }

                    // 平移变换
                    for (var j = 0, l = points.length; j < l; j += 2) {
                        points[j] += x;
                        points[j + 1] += y;
                    }
                    ca.push(new FR.Chart.SVGPathUtils.PathSegment(
                        cmd || c, points
                    ));
                }

                if (c === 'z' || c === 'Z') {
                    ca.push(new FR.Chart.SVGPathUtils.PathSegment('z', []));
                }
            }

            return ca;
        },

        /**
         * 创建路径
         * @param  ctx
         * @param style
         */
        buildPath : function (ctx, style) {
            var path = style.path;

            // 平移坐标
            var x = style.x || 0;
            var y = style.y || 0;

            style.pathArray = style.pathArray || this.buildPathArray(path, x, y);
            var pathArray = style.pathArray;

            // 记录边界点，用于判断inside
            var pointList = style.pointList = [];
            var singlePointList = [];
            for (var i = 0, l = pathArray.length; i < l; i++) {
                if (pathArray[i].command.toUpperCase() == 'M') {
                    singlePointList.length > 0
                    && pointList.push(singlePointList);
                    singlePointList = [];
                }
                var p = pathArray[i].points;
                for (var j = 0, k = p.length; j < k; j += 2) {
                    singlePointList.push([p[j], p[j + 1]]);
                }
            }
            singlePointList.length > 0 && pointList.push(singlePointList);

            for (var i = 0, l = pathArray.length; i < l; i++) {
                var c = pathArray[i].command;
                var p = pathArray[i].points;
                switch (c) {
                    case 'L':
                        ctx.lineTo(p[0], p[1]);
                        break;
                    case 'M':
                        ctx.moveTo(p[0], p[1]);
                        break;
                    case 'C':
                        ctx.curveTo(p[0], p[1], p[2], p[3], p[4], p[5]);
                        break;
                    case 'Q':
                        ctx.quadTo(p[0], p[1], p[2], p[3]);
                        break;
                    case 'z':
                        ctx.closePath();
                        break;
                }
            }

            return;
        },

        /**
         * 计算返回Path包围盒矩形。
         * @param style path的样式
         * @return 矩形
         */
        getRect : function (style) {
            if (style.__rect) {
                return style.__rect;
            }

            var lineWidth;
            if (style.brushType == 'stroke' || style.brushType == 'fill') {
                lineWidth = style.lineWidth || 1;
            }
            else {
                lineWidth = 0;
            }

            var minX = Number.MAX_VALUE;
            var maxX = Number.MIN_VALUE;

            var minY = Number.MAX_VALUE;
            var maxY = Number.MIN_VALUE;

            // 平移坐标
            var x = style.x || 0;
            var y = style.y || 0;

            var pathArray = style.pathArray || this.buildPathArray(style.path);
            for (var i = 0; i < pathArray.length; i++) {
                var p = pathArray[i].points;

                for (var j = 0; j < p.length; j++) {
                    if (j % 2 === 0) {
                        if (p[j] + x < minX) {
                            minX = p[j];
                        }
                        if (p[j] + x > maxX) {
                            maxX = p[j];
                        }
                    }
                    else {
                        if (p[j] + y < minY) {
                            minY = p[j];
                        }
                        if (p[j] + y > maxY) {
                            maxY = p[j];
                        }
                    }
                }
            }

            var rect;
            if (minX === Number.MAX_VALUE
                || maxX === Number.MIN_VALUE
                || minY === Number.MAX_VALUE
                || maxY === Number.MIN_VALUE
            ) {
                rect = {
                    x : 0,
                    y : 0,
                    width : 0,
                    height : 0
                };
            }
            else {
                rect = {
                    x : Math.round(minX - lineWidth / 2),
                    y : Math.round(minY - lineWidth / 2),
                    width : maxX - minX + lineWidth,
                    height : maxY - minY + lineWidth
                };
            }
            style.__rect = rect;
            return rect;
        }

    },

    /**
     * 高亮 辅助类
     * @class FR.Chart.HighUtils
     */
    HighUtils: /**@class FR.Chart.HighUtils*/{

        /**
         * 响应焦点 高亮
         * @static
         * @param glyph 当前Glyph
         * @param ctx 画布
         */
        doFocus: function (glyph, ctx, mapOnMoveCtx) {
            if(!FR.Chart.GraphHelper.isDrawingEnd) {
                return;
            }
            if (glyph && ctx) {
                if(glyph.isFade && mapOnMoveCtx){//地图选中的阴影效果
                    mapOnMoveCtx.save();
                    mapOnMoveCtx.translate(0, -4);
                    mapOnMoveCtx.shadowBlur = 20;
                    mapOnMoveCtx.shadowOffsetX = 2;
                    mapOnMoveCtx.shadowOffsetY = 2;
                    mapOnMoveCtx.shadowColor = 'rgba(0,0,0,0.3)';
                    if(glyph.getFillStyleColor() && glyph.shape){
                        mapOnMoveCtx.fillStyle = glyph.getFillStyleColor();
                        glyph.shape.paint(mapOnMoveCtx);
                    }
                    mapOnMoveCtx.translate(0, 4);
                    mapOnMoveCtx.restore();
                }else{
                    this.doBorder(glyph, ctx);
                    this.doHighlight(glyph, ctx);
                }
            }
        },

        /**
         * 响应高亮边框
         * @static
         * @param glyph 当前Glyph
         * @param ctx 画布
         */
        doBorder: function (glyph, ctx) {
            if(!ctx){     return; } ctx.save();

            if (glyph) {
                var shape = glyph.getShape();
                if (shape) {// kunsnat: IE下 shape点太多,  花费时间较多, 体验较卡.
                    ctx.lineWidth = 2;
                    if(glyph.toolTipBackground) {
                        glyph.toolTipBackground.draw(ctx, shape);
                    } else {
                        FR.Chart.ColorBackground.getInstance("rgb(128,128,128)").draw(ctx, shape);
                    }
                }

                // 折线要整体上移1px.  // bounds 要整体外围放大一像素

                // shape 开始..
            }

            ctx.restore();
        },

        /**
         * 系列拖拽的时候需要画的边框,圆环图示doHeight
         * @static
         * @param glyph 响应的Glyph
         * @param ctx 画布
         */
        doSeriesDragBorder: function(glyph, ctx){
            if(!ctx){     return; } ctx.save();
            if (glyph) {
                if(glyph instanceof FR.Chart.MarkerGlyph){
                    var marker = glyph.marker;
                    if(marker){
                        ctx.lineWidth = 1;
                        ctx.strokeStyle = marker.background.color;
                        marker.drawDragBorder(ctx);
                    }

                }else{
                    var shape = glyph.getShape();
                    if (shape) {// kunsnat: IE下 shape点太多,  花费时间较多, 体验较卡.
                        if(shape instanceof FR.Chart.Ring){//圆环图是高亮
                            FR.Chart.HighUtils.doFocus(glyph, ctx);
                        }else if(glyph.toolTipBackground && shape.drawDragBorder) {
                            ctx.lineWidth = 1;
                            ctx.strokeStyle = glyph.toolTipBackground.color;
                            shape.drawDragBorder(ctx);
                        }
                    }
                }
            }
            ctx.restore();
        },

        /**
         * 响应外发光
         * @static
         * @param glyph 需要响应的Glyph
         * @param ctx 画布
         */
        doHighlight: function (glyph, ctx) {
            if (glyph instanceof FR.Chart.GeneralGlyph) {
                var oldInfo = glyph.info;

                var newInfo = new FR.Chart.GeneralInfo();
                newInfo.seriesAttrBackground.seriesBackground = FR.Chart.ColorBackground.getInstance("white");
                newInfo.seriesAttrAlpha.alpha = 0.15;

                glyph.info = newInfo;
                glyph.draw(ctx);
                glyph.info = oldInfo;
            }
        }
    }
});

// FRFont.start
/**
 * 字体类
 *    @example
 *    var font  = new FR.Chart.FRFont();
 *
 * @class FR.Chart.FRFont
 * @param {Number} [js.size = 12] 字体大小
 * @param {Color} [js.color = 'black'] 字体颜色
 * @param {String} [js.fontName = "simsun'] 字体名称
 * @param {String} [js.style = ''] 字体样式.
 */
FR.Chart.FRFont = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (js) {
        js = js || {};

        this.size = js.size || 12;
        this.color = js.color || 'black';
        this.fontName = js.fontName || "SimSun";
        // kunsnat: style --> 常规, 加粗, 倾斜, 加粗倾斜
        this.style = js.style || "";
    },

    getStringFont:function(){
        return this.style + ' ' + parseInt(this.size) + 'px ' + this.fontName;
    }
});
// FRFont.end


// AffineTransform.start
/**
 * @class FR.Chart.AffineTransform 缩放类  3x3 变换
 *
 *  [ x']   [  m00  m01  m02  ] [ x ]   [ m00x + m01y + m02 ]
 *	[ y'] = [  m10  m11  m12  ] [ y ] = [ m10x + m11y + m12 ]
 *	[ 1 ]   [   0    0    1   ] [ 1 ]   [         1         ]
 *
 * @param m00  x值
 * @param m10  y值
 * @param m01  x值
 * @param m11 y值
 * @param m02  x值
 * @param m12 y值
 * @constructor
 */
FR.Chart.AffineTransform = function (m00, m10, m01, m11, m02, m12) {
    this.m00 = m00;
    this.m10 = m10;
    this.m01 = m01;
    this.m11 = m11;
    this.m02 = m02;
    this.m12 = m12;

    this.updateState();
};
$.extend(FR.Chart.AffineTransform, {

    /**
     * This constant is used for the internal state variable to indicate
     * that no calculations need to be performed and that the source
     * coordinates only need to be copied to their destinations to
     * complete the transformation equation of this transform.
     */
    APPLY_IDENTITY: 0,

    /**
     *  This constant is used for the internal state variable to indicate
     * that the translation components of the matrix (m02 and m12) need
     * to be added to complete the transformation equation of this transform.
     */
    APPLY_TRANSLATE: 1,

    /**
     * This constant is used for the internal state variable to indicate
     * that the scaling components of the matrix (m00 and m11) need
     * to be factored in to complete the transformation equation of
     * this transform.  If the APPLY_SHEAR bit is also set then it
     * indicates that the scaling components are not both 0.0.  If the
     * APPLY_SHEAR bit is not also set then it indicates that the
     * scaling components are not both 1.0.  If neither the APPLY_SHEAR
     * nor the APPLY_SCALE bits are set then the scaling components
     * are both 1.0, which means that the x and y components contribute
     * to the transformed coordinate, but they are not multiplied by
     * any scaling factor.
     */
    APPLY_SCALE: 2,

    /**
     * This constant is used for the internal state variable to indicate
     * that the shearing components of the matrix (m01 and m10) need
     * to be factored in to complete the transformation equation of this
     * transform.  The presence of this bit in the state variable changes
     * the interpretation of the APPLY_SCALE bit as indicated in its
     * documentation.
     */
    APPLY_SHEAR: 4
});
$.extend(FR.Chart.AffineTransform.prototype, {
    /**
     * 确定变化状态
     * @static
     */
    updateState: function () {
        if (this.m01 === 0 && this.m10 === 0) {
            if (this.m00 === 1 && this.m11 === 1) {
                if (this.m02 === 0 && this.m12 === 0) {
                    this.state = FR.Chart.AffineTransform.APPLY_IDENTITY;
                } else {
                    this.state = FR.Chart.AffineTransform.APPLY_TRANSLATE;
                }
            } else {
                if (this.m02 === 0 && this.m12 === 0) {
                    this.state = FR.Chart.AffineTransform.APPLY_SCALE;
                } else {
                    this.state = (FR.Chart.AffineTransform.APPLY_SCALE | FR.Chart.AffineTransform.APPLY_TRANSLATE);
                }
            }
        } else {
            if (this.m00 === 0 && this.m11 === 0) {
                if (this.m02 === 0 && this.m12 === 0) {
                    this.state = FR.Chart.AffineTransform.APPLY_SHEAR;
                } else {
                    this.state = (FR.Chart.AffineTransform.APPLY_SHEAR | FR.Chart.AffineTransform.APPLY_TRANSLATE);
                }
            } else {
                if (this.m02 === 0 && this.m12 === 0) {
                    this.state = (FR.Chart.AffineTransform.APPLY_SHEAR | FR.Chart.AffineTransform.APPLY_SCALE);
                } else {
                    this.state = (FR.Chart.AffineTransform.APPLY_SHEAR | FR.Chart.AffineTransform.APPLY_SCALE | FR.Chart.AffineTransform.APPLY_TRANSLATE);
                }
            }
        }
    },

    /**
     * 变换
     * @param ptSrc 变换起始状态
     * @param ptDst 变换结果状态
     * @returns {{x: number, y: number}}
     */
    transform: function (ptSrc, ptDst) {
        ptSrc = ptSrc || {x: 0, y: 0};
        ptDst = ptDst || {x: 0, y: 0};
        var x = ptSrc.x || 0;
        var y = ptSrc.y || 0;

        switch (this.state) {
            default:
                return ptDst;
            case (FR.Chart.AffineTransform.APPLY_SHEAR | FR.Chart.AffineTransform.APPLY_SCALE | FR.Chart.AffineTransform.APPLY_TRANSLATE):
                ptDst.x = x * this.m00 + y * this.m01 + this.m02;
                ptDst.y = x * this.m10 + y * this.m11 + this.m12;
                return ptDst;
            case (FR.Chart.AffineTransform.APPLY_SHEAR | FR.Chart.AffineTransform.APPLY_SCALE):
                ptDst.x = x * this.m00 + y * this.m01;
                ptDst.y = x * this.m10 + y * this.m11;
                return ptDst;
            case (FR.Chart.AffineTransform.APPLY_SHEAR | FR.Chart.AffineTransform.APPLY_TRANSLATE):
                ptDst.x = y * this.m01 + this.m02;
                ptDst.y = x * this.m10 + this.m12;
                return ptDst;
            case (FR.Chart.AffineTransform.APPLY_SHEAR):
                ptDst.x = y * this.m01;
                ptDst.y = x * this.m10;
                return ptDst;
            case (FR.Chart.AffineTransform.APPLY_SCALE | FR.Chart.AffineTransform.APPLY_TRANSLATE):
                ptDst.x = x * this.m00 + this.m02;
                ptDst.y = y * this.m11 + this.m12;
                return ptDst;
            case (FR.Chart.AffineTransform.APPLY_SCALE):
                ptDst.x = x * this.m00;
                ptDst.y = y * this.m11;
                return ptDst;
            case (FR.Chart.AffineTransform.APPLY_TRANSLATE):
                ptDst.x = x + this.m02;
                ptDst.y = y + this.m12;
                return ptDst;
            case (FR.Chart.AffineTransform.APPLY_IDENTITY):
                ptDst.x = x;
                ptDst.y = y;
                return ptDst;
        }

        return ptDst;
    }
});
// AffineTransform.end

FR.Chart.HashMap = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (jsMap) {
        this.length = 0;

        jsMap = jsMap || {};
        if (jsMap.length > 0) {
            for (var i = 0; i < jsMap.length; i++) {
                this.add(new FR.Chart[jsMap[i].conditionAttrType](jsMap[i]));
            }
        }
    },

    add: function (value) {
        this[this.length] = value;
        this.length++;
    },

    put: function (key, value) {
        this[key] = value;
        this.length++;
    },

    get: function (key) {
        return this[key];
    },

    containsKey: function (key) {
        if (this[key] === undefined) {
            return false;
        }
        return true;
    }
});