/**
 * Created by mengao on 2017/11/6.
 */
/**
 *
 * 各种web静态的方法. webUtils
 * Web资源管理 辅助类.
 * @class FR.Chart.WebUtils
 *
 */

FR.Chart = FR.Chart || {};

var DIV_CONTAINER;

$.extend(FR.Chart, /**FR.Chart*/{

    WebUtils: /**@class FR.Chart.WebUtils*/{

        /**
         * 根据图表类型名称确定是否需要加载老图表用的的js
         * @static
         * @param chartName 图表类型名称
         */
        importFineChartJS: function (chartName) {
            //老图表预览，需要导入老图表相关的JS文件
            if (chartName === "FineChart") {
                FR.$defaultImport('/com/fr/web/core/js/chart.utils.js', 'js');
                FR.$defaultImport('/com/fr/web/core/js/chart.constants.js', 'js');
                FR.$defaultImport('/com/fr/web/core/js/chart.shape.js', 'js');
                FR.$defaultImport('/com/fr/web/core/js/chart.plotattr.js', 'js');
                FR.$defaultImport('/com/fr/web/core/js/chart.axisglyph.js', 'js');
                FR.$defaultImport('/com/fr/web/core/js/chart.datapoint.js', 'js');
                FR.$defaultImport('/com/fr/web/core/js/chart.dataseries.js', 'js');
                FR.$defaultImport('/com/fr/web/core/js/chart.glyph.js', 'js');
                FR.$defaultImport('/com/fr/web/core/js/chart.legend.js', 'js');
                FR.$defaultImport('/com/fr/web/core/js/chart.datasheet.js', 'js');
                FR.$defaultImport('/com/fr/web/core/js/chart.handler.js', 'js');
                FR.$defaultImport('/com/fr/web/core/js/chart.tooltip.js', 'js');
                FR.$defaultImport('/com/fr/web/core/js/chart.finechart.js', 'js');
            }
        },

        /**
         * 获取指定的图表集合中当前展示的图表
         * @para name 图表的名字
         * @returns {*} FineChart对象
         */
        getChart: function (name, ecName) {
            if (name.indexOf("__index__") >= 0) {
                name = name.substring(0, name.indexOf("__index__"));
            }
            var chartCollection = this._getChartCollection(name, ecName);
            if (chartCollection) {
                var index = chartCollection.Widget ? chartCollection.Widget.curChart.idxNumber : 0;
                if (chartCollection[index]) {
                    return chartCollection[index];
                }
            }
        },

        getChartWidget: function (name) {
            if (name.indexOf("__index__") >= 0) {
                name = name.substring(0, name.indexOf("__index__"));
            }
            var chartCollection = this._getChartCollection(name);
            if (chartCollection && chartCollection.Widget) {
                return chartCollection.Widget;
            }
        },

        _relateChart: function (params, fnName) {
            if (!params) {
                return;
            }

            if (params.idInfo) {
                var bridge = Van.Utils.getChartBridge(params.idInfo);
                if (bridge) {
                    bridge[fnName](params.para);
                    return;
                }
            }

            var chartID = params.chartID,
                para = params.para,
                ecName = params.ecName,
                sheetIndex = params.sheetIndex;


            if (!chartID && params.idInfo) {
                //兼容 9.0vancharts预览方式 或者是图表插件 联动
                chartID = params.idInfo.name;
                ecName = params.idInfo.ecName;
                sheetIndex = params.idInfo.sheetIndex;
            }

            if (!chartID) {
                return;
            }

            var chartSet = this._getChartCollection(chartID, ecName, sheetIndex);

            if (chartSet) {
                $.each(chartSet, function (id, chart) {
                    if (chart !== chartSet.Widget && chart[fnName]) {
                        chart[fnName](para);
                    }
                });
            }
        },


        /**
         * 图表用的参数变化
         */
        changeParameter: function () {

            var params = arguments[0];

            if (arguments.length === 2) {//兼容 用户模板 自定义js里面用到
                params = {chartID: arguments[0], para: arguments[1]};
            }

            if (!params) {
                return;
            }

            switch (params.animateType) {
                case 'reload'://增量刷新
                    this._relateChart(params, 'dealReloadChartAjax');
                    break;
                case 'increment'://重新加载
                    this._relateChart(params, 'dealIncrementChartAjax');
                    break;
                default:
                    this._relateChart(params, 'dealChartAjax');
                    break;

            }
        },

        clearChartsWithECName: function (ecName) {
            if (FR.isEmpty(ecName)) {
                return;
            }

            this._clearCharts(function (chart) {
                return !FR.isEmpty(chart.ecName) && chart.ecName.toUpperCase() === ecName.toUpperCase();});
        },

        clearChartsWithSheetIndex: function (sheetIndex) {
            this._clearCharts(function (chart) {return chart.sheetIndex === sheetIndex;});
        },

        clearCharts: function () {
            this._clearCharts(function () {return true;});
            FR.ChartManager = null;
        },

        _clearCharts: function(filterFn) {
            if (FR.ChartManager) {
                for (var chartID in FR.ChartManager) {
                    var chartSet = FR.ChartManager[chartID];
                    if (chartSet && chartSet.Widget) {
                        var curIndex = chartSet.Widget.curChart.idxNumber || 0;

                        if (chartSet[curIndex] && filterFn(chartSet[curIndex])) {
                            for (var index in chartSet) {
                                var chart = chartSet[index];
                                chart && chart.clearAll && chart.clearAll();
                            }

                            delete FR.ChartManager[chartID];
                        }
                    }
                }
            }

            //清空图表同时要把VanCharts.instances都清空
            if (VanCharts && VanCharts.instances) {
                var _instances = VanCharts.instances;

                for (var chartIndex in _instances) {
                    var vanCharts = _instances[chartIndex];

                    if (vanCharts && filterFn(vanCharts)) {
                        vanCharts.clear && vanCharts.clear();
                        delete _instances[chartIndex];
                    }
                }
            }

            if($.browser.msie) {
                if(CollectGarbage) {
                    CollectGarbage();
                }
            }
        },

        /**
         * 清空 图表资源
         * @static
         */
        chartDelete: function () {
            if (FR.ChartManager && FR.ChartManager.length > 0) {
                $.each(FR.ChartManager, function (id, chartSet) {
                    if (FR.ChartManager[id]) {
                        FR.ChartManager[id] = null;
                    }
                });
            }

            FR.ChartManager = null;
        },

        //判断图表是否能够接着钻取
        mapHasNextLayer: function (name) {

            var fineChart = this.getChart(name);
            return fineChart ? fineChart.hasNextLayer() : false;

        },

        //兼容之前的名字
        chart_Change_Parameter: function () {
            if (arguments.length !== 3) {
                return;
            }
            var arg = arguments[0] + "__" + arguments[1];
            if (arg.startWith('Elems')) {
                arg = arguments[1];
            }
            //后面这边不加"__"会把Cells_G14_也归纳为条件Cells_G1
            arg += "__";
            this.changeParameter(arg, arguments[2]);
        },

        _getChartCollection: function (name, ecName, sheetIndex) {
            if (!FR.ChartManager) {
                return {};
            }
            var chartCollection = this._findChartCollectionWithFilter(this._filterECName, name, ecName, sheetIndex);

            if (!chartCollection) {
                //单元格超链信息中没有sheetIndex
                chartCollection = this._findChartCollectionWithFilter(function () {return true;}, name);
            }
            return chartCollection;
        },

        _findChartCollectionWithFilter: function (filter, name, ecName, sheetIndex) {
            var result = null;

            $.each(FR.ChartManager, function (id, chartSet) {

                if (name.indexOf('__') != -1) {
                    //兼容老的模版
                    if (id.indexOf(name) !== -1 && filter(ecName, sheetIndex, chartSet)) {
                        result = chartSet;
                    }

                } else {
                    if ($.inArray(name, id.split('__')) !== -1 && filter(ecName, sheetIndex, chartSet)) {
                        result = chartSet;
                    }
                }
            });

            return result;
        },

        _filterECName: function (ecName, sheetIndex, chartSet) {
            if ((ecName || sheetIndex !== undefined) && chartSet.Widget) {
                var index = chartSet.Widget.curChart.idxNumber;

                if (ecName) {//frm 报表块名称
                    return ecName === chartSet[index].ecName;
                } else if (sheetIndex !== undefined) {//cpt 多sheet
                    return parseInt(sheetIndex) === chartSet[index].sheetIndex;
                }
            }
            return true;
        },

        _initChartManager: function() {
            if (!FR.ChartManager) {
                FR.ChartManager = {};

                var oldunload = window.onunload;
                if(typeof window.onunload != 'function') {
                    window.onunload = function() {
                        FR.Chart.WebUtils.clearCharts();
                    }
                } else {
                    window.onunload = function() {
                        oldunload();
                        FR.Chart.WebUtils.clearCharts();
                    }
                }
            }
        },

        //eg:FR.SimpleChart
        _storeChartWidget: function (chart, chartID) {
            this._initChartManager();

            var ids = FR.Chart.WebUtils._getChartIDAndIndex(chartID);

            if (FR.ChartManager[ids[0]]) {
                delete FR.ChartManager[ids[0]];
            }
            var chartSet = FR.ChartManager[ids[0]] = {};
            chartSet = chartSet || {};
            chartSet.Widget = chart;
        },

        //eg:VanChartWrapper
        _storeChart: function (chart, chartID) {
            this._initChartManager();

            var ids = FR.Chart.WebUtils._getChartIDAndIndex(chartID);

            var chartSet = FR.ChartManager[ids[0]] = FR.ChartManager[ids[0]] || {};
            chartSet[ids[1]] = chart;
        },

        /**
         * @Deprecated 用_storeChartWidget和_storeChart替代
         * 不能删除的原因：兼容老的第三方图表接口
         * 将图表,图表控件加入到ChartManager中
         * @static
         * @param chart 图表
         * @param chartID 图表id
         */
        _installChart: function (chart, chartID) {

            if (FR.Widget && chart instanceof FR.Widget){
                this._storeChartWidget(chart, chartID);
            } else {
                this._storeChart(chart, chartID);
                chart.inits();
            }
        },

        /**
         * 根据字符串得到图表的Cells__A1__A4 和 index
         * @static
         * @param id 图表id
         * @returns {Array} 返回对应的id, 序号资源数组.
         */
        _getChartIDAndIndex: function (id) {
            var str = id;
            var index = "";
            if (str.indexOf("__index__") >= 0) {
                index = str.substring(str.indexOf("__index__") + 9, str.length);
                str = str.substring(0, str.indexOf("__index__"));
            }
            if (str.indexOf("Chart") === 0 && str.indexOf("__") >= 0) {
                str = str.substring(str.indexOf("__") + 2, str.length);
            }
            return [str, index];
        },

        stringDimensionWidthDiv: function (text, font) {

            if (!DIV_CONTAINER) {
                DIV_CONTAINER = document.createElement("div");
                document.body.appendChild(DIV_CONTAINER);
            }

            DIV_CONTAINER.style.cssText = '';

            DIV_CONTAINER.style.visibility = "hidden";
            DIV_CONTAINER.style.whiteSpace = "nowrap";
            DIV_CONTAINER.style.position = 'absolute';
            DIV_CONTAINER.style.display = '';

            DIV_CONTAINER.style.fontFamily = font.fontName;
            DIV_CONTAINER.style.fontSize = font.size + 'px';
            DIV_CONTAINER.style.fontWeight = font.style;

            DIV_CONTAINER.innerHTML = text;

            var width = DIV_CONTAINER.offsetWidth || 0;
            var height = DIV_CONTAINER.offsetHeight || 0;
            var size = {width: width, height: height};

            DIV_CONTAINER.style.display = 'none';
            return size;
        }

    }
});