;
(function ($) {


    /**
     * 各种预览方式在浏览器中展示对象的基类
     * @abstract
     * @class FR.BasePane
     * @extends FR.Widget
     */
    FR.BasePane = FR.extend(FR.Widget, {
        type: "basepane",
        zoom: 1.0,

        _defaultConfig: function () {
            return $.extend(FR.BasePane.superclass._defaultConfig.apply(this, arguments), {
                loadTimeout: 3600000
            });
        },

        _init: function () {
            FR.BasePane.superclass._init.apply(this, arguments);
        },

        /**
         * 根据报表所在窗体的宽度和高度自适应的放大和缩小表内容
         */
        autoScale: function () {
            if (self !== top && this.$contentPane && this.options.autoScale) { // denny:
                // 说明是iframe集成
                var $contentDiv = $('.pageContentDIV', this.$contentPane);

                if ($contentDiv) {
                    // var bHeight = window.document.body.scrollHeight;
                    // var dHeight = window.document.documentElement.scrollHeight;
                    var height = $contentDiv.height();

                    // var bWidth = window.document.body.scrollWidth;
                    // var dWidth = window.document.documentElement.scrollWidth;
                    // var width = $contentDiv.width();

                    var iframe_height = FR.windowHeight;
                    // var iframe_width = FR.windowWidth;

                    this.scale(iframe_height / height);
                }
            }
        },

        /**
         * 缩放报表页面
         *
         *     @example
         *     contentPane.scale('+');//放大
         *     contentPane.scale(1.5);//縮放到150%
         *
         * @param {'+'/'-'/Number} 按照指定的参数进行缩放
         */
        scale: (function () {
            // ["25%", "50%", "75%", "100%", "120%", "150%", "200%"]
            var zooms = [0.25, 0.5, 0.75, 1.0, 1.2, 1.5, 2.0];

            return function (zoom) {
                if (zoom === '+') {
                    for (var j = 0, len = zooms.length; j < len; j++) {
                        if (zooms[j] > this.zoom) {
                            return this.scale(zooms[j]);
                        }
                    }
                    if (j === zooms.length) {
                        return this.scale(zooms[zooms.length - 1]);
                    }
                } else if (zoom === '-') {
                    for (var i = (zooms.length - 1); i >= 0; i--) {
                        if (zooms[i] < this.zoom) {
                            return this.scale(zooms[i]);
                        }
                    }
                    if (i < 0) {
                        return this.scale(zooms[0]);
                    }
                } else if (typeof zoom === 'number') {
                    if (this.$contentPane) {
                        var $contentDiv = $('.pageContentDIV', this.$contentPane);
                        if (FR.Browser.r.gecko) {
                            $contentDiv.css('-moz-transform', 'scale(' + zoom + ',' + zoom + ')');
                            $contentDiv.css('-moz-transform-origin', 'top left');
                        } else if (FR.Browser.r.webkit) {
                            $contentDiv.css('-webkit-transform', 'scale(' + zoom + ',' + zoom + ')');
                            $contentDiv.css('-webkit-transform-origin', 'top left');
                        } else if (FR.Browser.r.opera) {
                            $contentDiv.css('-o-transform', 'scale(' + zoom + ',' + zoom + ')');
                            $contentDiv.css('-o-transform-origin', 'top left');
                        } else {
                            $contentDiv.css('zoom', zoom);
                        }
                        this.zoom = zoom;

                        this.fireEvent("zoomchanged");
                    }
                } else {
                    return this.zoom;
                }
            };
        })(),


        /**
         * 整体缩放contentDiv
         * @param $contentDiv
         * @param zoom
         */
        zoomContent: function ($contentDiv, zoom) {
            if ($contentDiv) {
                if (FR.Browser.r.gecko) {
                    $contentDiv.css('-moz-transform', 'scale(' + zoom.x + ',' + zoom.y + ')');
                    $contentDiv.css('-moz-transform-origin', 'top left');
                } else if (FR.Browser.r.webkit) {
                    $contentDiv.css('-webkit-transform', 'scale(' + zoom.x + ',' + zoom.y + ')');
                    $contentDiv.css('-webkit-transform-origin', 'top left');
                } else if (FR.Browser.r.opera) {
                    $contentDiv.css('-o-transform', 'scale(' + zoom.x + ',' + zoom.y + ')');
                    $contentDiv.css('-o-transform-origin', 'top left');
                } else if (FR.Browser.isIE9Later()) {
                    $contentDiv.css('-ms-transform', 'scale(' + zoom.x + ',' + zoom.y + ')');
                    $contentDiv.css('-ms-transform-origin', 'top left');
                }
            }
        },


        extendPara: function (para) {
            var documentHeight = FR.windowHeight;
            var pageContentPane = this.$contentPane;
            if (documentHeight === 0) {
                return;
            }
            var toolBarHeight = FR.hasTopToolbar ? FR.toolbarHeight : 0;
            var extraHeight = 0;
            extraHeight += toolBarHeight;
            var tabPane = pageContentPane.data('TabPane');
            //直接取height，sougou高速模式还是有滚动条，额外加1px

            if (tabPane !== undefined) {
                extraHeight += tabPane.options.tabHeight;
            }
            extraHeight += FR.parameterContainerHeight;
            documentHeight -= extraHeight;
            $.extend(para, {
                __webpage__: true,
                _paperWidth: FR.windowWidth,
                _paperHeight: documentHeight,
                __fit__: this.__fit__
            });

            if (!pageContentPane.data("preWidth")) {
                pageContentPane.data("preWidth", pageContentPane.width());

            }
            if (!pageContentPane.data("preHeight")) {
                pageContentPane.data("preHeight", pageContentPane.height());
            }
        },

        /**
         * 根据配置文件加载ReportPane。
         * 如果有参数，就加载ParameterPane + ContentPane,如果没有参数则加载ContentPane
         * @param options 配置
         * @protected
         */
        loadReportPane: function (options) {
            FR.$defaultImport('/com/fr/web/core/js/paramtemplate.js', 'js');
            this.fireEvent(FR.Events.INIT);
            options = options || {};
            options.param = $.extend({
                showType: 1
            }, options.param); // 默认不显示参数面板,showType为1:embed

            var items = [];
            var paramsTemplate = getParamsTemplate(this.currentSessionID);
            if (options.param.html) {
                this.parameterEl = this._createFormParameterPane(options, items, paramsTemplate);
            }

            this.$contentPane = this._createContentPane(options.sheets, options.browserbg, options.$paraContainer);
            items.push({
                region: 'center',
                el: this.$contentPane
            });

            // alex:设置布局为BorderLayout
            this._doReportPaneLayout(items, options, paramsTemplate);

            // james:初始化ContentPane的一些事件，目前主要是Tab的一些切换操作内容
            this.initContentPane();

            // 最后loadContentPane
            if (this.parameterEl === undefined || this.parameterEl === null) {
                this.loadContentPane();
            } else if (options.param.delay === false) {
                this.parameterCommit();
            }

            this.delay = options.param.delay;
        },

        _doReportPaneLayout: function (items, options, paramsTemplate) {
            var self = this;
            var borderLayout = new FR.BorderLayout({
                items: items,
                renderEl: this.options.renderEl
            });
            var showParamsTemplate = options.param.useParamsTemplate && paramsTemplate !== null;
            $(window).bind("resize", function () {
                if (self.resizeid !== undefined && self.resizeid !== null) {
                    clearTimeout(self.resizeid);
                    self.resizeid = null;
                }
                self.resizeid = setTimeout(function () {
                    if (options.$paraContainer) {
                        options.$paraContainer.width = document.body.offsetWidth;
                    }
                    var paramWidget = borderLayout.getWidgetByName(options.alignLayoutName);

                    if (paramWidget) {
                        paramWidget.element.css('width', options.param.width);
                        self.addParaPaneScrollBar(options.$paraContainer, paramWidget.element, options.param.height + (showParamsTemplate ? 30 : 0), self.scrollHeight);
                    }
                    borderLayout.doLayout();
                    $('[widgetname^=sortable]').removeAttr('style');
                    self.doResize();
                }, 200);
            });
            var paramWidget = borderLayout.getWidgetByName(options.alignLayoutName);
            if (paramWidget) {
                this.addParaPaneScrollBar(options.$paraContainer, paramWidget.element, options.param.height + (showParamsTemplate ? 30 : 0), this.scrollHeight);
            }
            options.onCollapse = function (isCollapsed, layoutName, paraHeight) {
                borderLayout.getWidgetByName(layoutName).setVisible(!isCollapsed);
                if (borderLayout.getWidgetByName('paramsTemplatePane') !== undefined) {
                    borderLayout.getWidgetByName('paramsTemplatePane').setVisible(!isCollapsed);
                    borderLayout.getWidgetByName('saveBtn').setVisible(!isCollapsed);
                    borderLayout.getWidgetByName('newParamsBtn').setVisible(!isCollapsed);
                }
                borderLayout.setRegionWH('north', paraHeight);
                borderLayout.doLayout();
                $('[widgetname^=sortable]').removeAttr('style');
            };
            options.onCalculate = function (maxHeight, collapseHeight, $container) {
                options.param.height = maxHeight;
                $container.height(maxHeight + collapseHeight);
                borderLayout.doLayout();
                $('[widgetname^=sortable]').removeAttr('style');
            };
            borderLayout.doLayout();
            if (options.param.html) {
                initParameterTemplate(options.param, options.param.html.items, this.currentSessionID, borderLayout, paramsTemplate);
            }
        },

        _createContentPane: function (sheetsO, browserbg, $paraContainer) {
            sheetsO = sheetsO || [];
            var $contentPane = $("div.content-container");
            if ($contentPane.length !== 1) {
                $contentPane = $("<div id='content-container'>")
                        .appendTo(this.element).addClass("content-container");
            }
            $contentPane.css({overflow: 'hidden'});
            if (sheetsO && sheetsO.length > 1) {
                this._initSheetTabPane($contentPane, sheetsO, browserbg);
            } else {// 普通的单Sheet，就不需要用tabs啦
                // alex: overflow:auto 以显示滚动条查看所有报表内容
                // p: '0px solid #97A5B0'我改成了0px,这样既去掉了上边框,也能够显示滚动条了.
                $contentPane.css("overflow", "auto").css('border-top', '0px');
            }
            // 画网页背景
            // 为使背景固定，对不同浏览器使用不同策略
            if (browserbg.background !== null && browserbg.background !== undefined) {
                if (FR.Browser.isIE8Before()) {
                    $contentPane.css('background', browserbg.background + " fixed");
                    if (browserbg.filter !== undefined) {
                        $contentPane.css('filter', browserbg.filter);
                    }
                    if (FR.Browser.isIE6() && $paraContainer) {
                        $contentPane.css('background-position', '0px ' + $paraContainer.height() + 'px 0px 0px');
                    }
                } else {
                    $contentPane.css('background', browserbg.background);
                    var bgSize = browserbg["background-size"];
                    if (bgSize !== undefined) {
                        $contentPane.css('background-size', bgSize);
                    }
                }
            }
            return $contentPane;
        },

        addParaPaneScrollBar: function (container, mover, top, height) {
            if (container) {
                container.hScrollPane({
                    mover: mover, //指定container对象下的哪个元素需要滚动位置 | 必传项;
                    showArrow: false, //指定是否显示左右箭头，默认不显示 | 可选项;
                    top: top, //滚动条所在的top位置
                    height: height,//滚动条的高度
                    //moverW:function(){return $(".press").width();}(), //传入水平滚动对象的长度值,不传入的话默认直接获取mover的宽度值 | 可选项;
                    //handleMinWidth:100,//指定handle的最小宽度,要固定handle的宽度请在css中设定handle的width属性（如 width:28px!important;），不传入则不设定最小宽度 | 可选项;
                    dragable: false, //指定是否要支持拖动效果，默认可以拖动 | 可选项;
                    easing: false, //滚动是否需要滑动效果,默认有滑动效果 | 可选项;
                    // handleCssAlter:"draghandlealter", //指定拖动鼠标时滚动条的样式，不传入该参数则没有变化效果 | 可选项;
                    mousewheel: {bind: false, moveLength: 500} //mousewheel: bind->'true',绑定mousewheel事件; ->'false',不绑定mousewheel事件；moveLength是指定鼠标滚动一次移动的距离,默认值：{bind:true,moveLength:300} | 可选项;
                });
            }
        },
        /**
         * 展示页面设置的对话框
         * @param o 对话框的配置
         */
        showPageSetupDialog: function (o) {
            o = o || {};
            o.width = o.width || 400;
            o.height = o.height || 295;
            o.title = FR.i18nText("Fine-Engine_Report_PageSetup_Page_Setup");
            var fontsize = 12, label_color = '#4C4C4C', fontfamily = "simsun", sessionID = o.sessionID || '',
                    reportIndex = o.reportIndex,
                    widgetPadding = 10, northHeight = 25, mediumRightPadding = 162, rh = 21, radioRightPadding = 234;
            var paperSize_items = [
                {text: "Letter", value: "Letter"},
                {text: "A2", value: "A2"},
                {text: "Legal", value: "Legal"},
                {text: "A0", value: "A0"},
                {text: "A1", value: "A1"},
                {text: "Note", value: "Note"},
                {text: "A3", value: "A3"},
                {text: "A4", value: "A4"},
                {text: "A5", value: "A5"},
                {text: "A6", value: "A6"},
                {text: "A7", value: "A7"},
                {text: "A8", value: "A8"},
                {text: "B0", value: "B0"},
                {text: "B1", value: "B1"},
                {text: "B2", value: "B2"},
                {text: "B3", value: "B3"},
                {text: "B4", value: "B4"},
                {text: "B5", value: "B5"},
                {text: "Tabloid", value: "Tabloid"},
                {text: "Ledger", value: "Ledger"},
                {text: "Halfletter", value: "Halfletter"}
            ];
            var tableLayout = FR.createWidget({
                type: 'tablelayout',
                columnSize: [72, 16, 312],
                rowSize: [rh, 52, 52],
                vgap: 20,
                items: [
                    [
                        {
                            el: {
                                type: 'label',
                                value: FR.i18nText("Fine-Engine_PageSetup_Orientation"),
                                fontsize: fontsize,
                                verticalcenter: true,
                                textalign: 'right',
                                fontfamily: fontfamily
                            }
                        },
                        {el: $('<div>')},
                        {
                            el: {
                                type: 'border', items: [
                                    {
                                        region: 'center', el: {
                                            type: 'radio',
                                            widgetName: 'portrait',
                                            text: FR.i18nText("Fine-Engine_PageSetup_Portrait"),
                                            fontSize: fontsize,
                                            listeners: [
                                                {
                                                    eventName: 'statechange',
                                                    action: function () {
                                                        if (this.getValue()) {
                                                            var radio_landscape = tableLayout.getWidgetByName("landscape");
                                                            if (radio_landscape) {
                                                                radio_landscape.setValue(false);
                                                            }
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    },
                                    {
                                        region: 'east', width: radioRightPadding, el: {
                                            type: 'radio',
                                            widgetName: 'landscape',
                                            text: FR.i18nText("Fine-Engine_PageSetup_Landscape"),
                                            fontSize: fontsize,
                                            listeners: [
                                                {
                                                    eventName: 'statechange',
                                                    action: function () {
                                                        if (this.getValue()) {
                                                            var radio_portrait = tableLayout.getWidgetByName("portrait");
                                                            if (radio_portrait) {
                                                                radio_portrait.setValue(false);
                                                            }
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                ]
                            }
                        }
                    ],
                    [
                        {
                            el: {
                                type: 'label',
                                value: FR.i18nText("Fine-Engine_PageSetup_Paper_Size"),
                                fontsize: fontsize,
                                textalign: 'right',
                                fontfamily: fontfamily
                            }
                        },
                        {el: $('<div>')},
                        {
                            el: {
                                type: 'border', items: [
                                    {
                                        region: 'north', height: rh, el: {
                                            type: 'border', items: [
                                                {
                                                    region: 'center', el: {
                                                        type: 'radio',
                                                        widgetName: 'predefined',
                                                        text: FR.i18nText("Fine-Engine_PageSetup_Predefined"),
                                                        fontSize: fontsize,
                                                        listeners: [
                                                            {
                                                                eventName: 'afteredit', action: function () {
                                                                    var value = this.getValue();
                                                                    var tableWidget = tableLayout.getWidgetByName('papaersize_setup');
                                                                    if (value === false) {
                                                                        tableWidget.setRowVisible(0, false);
                                                                    } else {
                                                                        tableWidget.setRowVisible(0, true);
                                                                    }
                                                                    tableWidget.doLayout();
                                                                }
                                                            },
                                                            {
                                                                eventName: 'statechange', action: function () {
                                                                    var tableWidget = tableLayout.getWidgetByName('papaersize_setup');
                                                                    if (this.getValue()) {
                                                                        tableWidget.setRowVisible(1, false);
                                                                        var radio_customed = tableLayout.getWidgetByName("customed");
                                                                        if (radio_customed) {
                                                                            radio_customed.setValue(false);
                                                                        }
                                                                    } else {
                                                                        tableWidget.setRowVisible(1, true);
                                                                    }
                                                                    tableWidget.doLayout();
                                                                }
                                                            }
                                                        ]
                                                    }
                                                },
                                                {
                                                    region: 'east', width: radioRightPadding, el: {
                                                        type: 'radio',
                                                        widgetName: 'customed',
                                                        text: FR.i18nText("Fine-Engine_Custom"),
                                                        fontSize: fontsize,
                                                        listeners: [
                                                            {
                                                                eventName: 'afteredit', action: function () {
                                                                    var value = this.getValue();
                                                                    var tableWidget = tableLayout.getWidgetByName('papaersize_setup');
                                                                    if (value === false) {
                                                                        tableWidget.setRowVisible(1, false);
                                                                    } else {
                                                                        tableWidget.setRowVisible(1, true);
                                                                    }
                                                                    tableWidget.doLayout();
                                                                }
                                                            },
                                                            {
                                                                eventName: 'statechange', action: function () {
                                                                    var tableWidget = tableLayout.getWidgetByName('papaersize_setup');
                                                                    if (this.getValue()) {
                                                                        tableWidget.setRowVisible(0, false);
                                                                        var radio_predefined = tableLayout.getWidgetByName("predefined");
                                                                        if (radio_predefined) {
                                                                            radio_predefined.setValue(false);
                                                                        }
                                                                    } else {
                                                                        tableWidget.setRowVisible(0, true);
                                                                    }
                                                                    tableWidget.doLayout();
                                                                }
                                                            }
                                                        ]
                                                    }
                                                }
                                            ]
                                        }
                                    },
                                    {region: 'center', height: widgetPadding, el: $('<div>')},
                                    {
                                        region: 'south', height: rh, el: {
                                            type: 'tablelayout',
                                            widgetName: 'papaersize_setup',
                                            columnSize: [302],
                                            rowSize: [rh, rh],
                                            vgap: 0,
                                            items: [
                                                [
                                                    {
                                                        el: {
                                                            type: 'border', items: [
                                                                {
                                                                    region: 'center', el: {
                                                                        type: 'combo',
                                                                        widgetName: 'paperSize',
                                                                        allowBlank: false,
                                                                        directEdit: true,
                                                                        items: paperSize_items,
                                                                        fontSize: fontsize,
                                                                        listeners: [
                                                                            {
                                                                                eventName: 'afteredit',
                                                                                action: function () {
                                                                                    FR.ajax({
                                                                                        url: FR.servletURL,
                                                                                        type: 'POST',
                                                                                        data: {
                                                                                            op: 'fr_utils',
                                                                                            cmd: 'utils_papersize',
                                                                                            paperSize: this.getValue()
                                                                                        },
                                                                                        complete: function (res) {
                                                                                            var paper = FR.jsonDecode(res.responseText);
                                                                                            tableLayout.getWidgetByName("user_width").setValue(paper.width);
                                                                                            tableLayout.getWidgetByName("user_height").setValue(paper.height);
                                                                                            tableLayout.getWidgetByName("predefined").setValue(true);
                                                                                            tableLayout.getWidgetByName("customed").reset();
                                                                                        }
                                                                                    });
                                                                                }
                                                                            }
                                                                        ]
                                                                    }
                                                                },
                                                                {
                                                                    region: 'east',
                                                                    width: mediumRightPadding,
                                                                    el: $('<div>')
                                                                }
                                                            ]
                                                        }
                                                    }
                                                ],
                                                [
                                                    {
                                                        el: {
                                                            type: 'tablelayout',
                                                            columnSize: [12, 12, 60, 5, 21, 14, 12, 12, 60, 5, 21, 68],
                                                            rowSize: [rh],
                                                            vgap: 0,
                                                            items: [
                                                                [
                                                                    {
                                                                        el: {
                                                                            type: 'label',
                                                                            value: FR.i18nText("Fine-Engine_Report_Width"),
                                                                            fontsize: fontsize
                                                                        }
                                                                    },
                                                                    {el: $('<div>')},
                                                                    {
                                                                        el: {
                                                                            type: 'number',
                                                                            widgetName: 'user_width',
                                                                            fontSize: fontsize
                                                                        }
                                                                    },
                                                                    {el: $('<div>')},
                                                                    {
                                                                        el: {
                                                                            type: 'label',
                                                                            value: "mm",
                                                                            fontsize: fontsize,
                                                                            color: label_color,
                                                                            fontfamily: 'tahoma'
                                                                        }
                                                                    },
                                                                    {el: $('<div>')},
                                                                    {
                                                                        el: {
                                                                            type: 'label',
                                                                            value: FR.i18nText("Fine-Engine_Height"),
                                                                            fontsize: fontsize
                                                                        }
                                                                    },
                                                                    {el: $('<div>')},
                                                                    {
                                                                        el: {
                                                                            type: 'number',
                                                                            widgetName: 'user_height',
                                                                            fontSize: fontsize
                                                                        }
                                                                    },
                                                                    {el: $('<div>')},
                                                                    {
                                                                        el: {
                                                                            type: 'label',
                                                                            value: "mm",
                                                                            fontsize: fontsize,
                                                                            color: label_color,
                                                                            fontfamily: 'tahoma'
                                                                        }
                                                                    },
                                                                    {el: $('<div>')}
                                                                ]
                                                            ]
                                                        }
                                                    }
                                                ]
                                            ]
                                        }
                                    }
                                ]
                            }
                        }
                    ],
                    [
                        {
                            el: {
                                type: 'label',
                                value: FR.i18nText("Fine-Engine_PageSetup_Margin"),
                                fontsize: fontsize,
                                textalign: 'right',
                                fontfamily: fontfamily
                            }
                        },
                        {el: $('<div>')},
                        {
                            el: {
                                type: 'tablelayout',
                                columnSize: [30, 12, 60, 5, 21, 14, 36, 12, 60, 5, 21, 68],
                                rowSize: [rh, rh],
                                vgap: 10,
                                items: [
                                    [
                                        {
                                            el: {
                                                type: 'label',
                                                value: FR.i18nText("Fine-Engine_Report_Top"),
                                                fontsize: fontsize
                                            }
                                        },
                                        {el: $('<div>')},
                                        {el: {type: 'number', widgetName: 'margin_top', fontSize: fontsize}},
                                        {el: $('<div>')},
                                        {
                                            el: {
                                                type: 'label',
                                                value: "mm",
                                                fontsize: fontsize,
                                                color: label_color,
                                                fontfamily: 'tahoma'
                                            }
                                        },
                                        {el: $('<div>')},
                                        {
                                            el: {
                                                type: 'label',
                                                value: FR.i18nText("Fine-Engine_Bottom"),
                                                fontsize: fontsize
                                            }
                                        },
                                        {el: $('<div>')},
                                        {el: {type: 'number', widgetName: 'margin_bottom', fontSize: fontsize}},
                                        {el: $('<div>')},
                                        {
                                            el: {
                                                type: 'label',
                                                value: "mm",
                                                fontsize: fontsize,
                                                color: label_color,
                                                fontfamily: 'tahoma'
                                            }
                                        },
                                        {el: $('<div>')}
                                    ],
                                    [
                                        {
                                            el: {
                                                type: 'label',
                                                value: FR.i18nText("Fine-Engine_Report_Left"),
                                                fontsize: fontsize
                                            }
                                        },
                                        {el: $('<div>')},
                                        {el: {type: 'number', widgetName: 'margin_left', fontSize: fontsize}},
                                        {el: $('<div>')},
                                        {
                                            el: {
                                                type: 'label',
                                                value: "mm",
                                                fontsize: fontsize,
                                                color: label_color,
                                                fontfamily: 'tahoma'
                                            }
                                        },
                                        {el: $('<div>')},
                                        {
                                            el: {
                                                type: 'label',
                                                value: FR.i18nText("Fine-Engine_Report_Right"),
                                                fontsize: fontsize
                                            }
                                        },
                                        {el: $('<div>')},
                                        {el: {type: 'number', widgetName: 'margin_right', fontSize: fontsize}},
                                        {el: $('<div>')},
                                        {
                                            el: {
                                                type: 'label',
                                                value: "mm",
                                                fontsize: fontsize,
                                                color: label_color,
                                                fontfamily: 'tahoma'
                                            }
                                        },
                                        {el: $('<div>')}
                                    ]
                                ]
                            }
                        }
                    ]
                ]
            });
            var widget_predefined = tableLayout.getWidgetByName('predefined');
            var widget_customed = tableLayout.getWidgetByName('customed');
            var widget_portrait = tableLayout.getWidgetByName('portrait');
            var widget_landscape = tableLayout.getWidgetByName('landscape');
            var widget_paperSize = tableLayout.getWidgetByName('paperSize');
            var widget_user_width = tableLayout.getWidgetByName('user_width');
            var widget_user_height = tableLayout.getWidgetByName('user_height');
            var widget_margin_top = tableLayout.getWidgetByName('margin_top');
            var widget_margin_bottom = tableLayout.getWidgetByName('margin_bottom');
            var widget_margin_left = tableLayout.getWidgetByName('margin_left');
            var widget_margin_right = tableLayout.getWidgetByName('margin_right');
            // 异步获取页面设置的信息,给控件赋默认值
            FR.ajax({
                url: FR.servletURL,
                type: 'POST',
                data: {
                    op: 'fr_dialog',
                    cmd: 'read_page_setup_tpl',
                    sessionID: sessionID,
                    reportIndex: reportIndex
                },
                complete: function (res) {
                    var result = FR.jsonDecode(res.responseText);
                    if (result !== null && result !== undefined) {
                        widget_predefined.setValue(result.predefined);
                        if (widget_predefined.getValue()) {
                            tableLayout.getWidgetByName('papaersize_setup').setRowVisible(0, true);
                            tableLayout.getWidgetByName('papaersize_setup').setRowVisible(1, false);
                        } else {
                            tableLayout.getWidgetByName('papaersize_setup').setRowVisible(0, false);
                            tableLayout.getWidgetByName('papaersize_setup').setRowVisible(1, true);
                        }
                        tableLayout.getWidgetByName('papaersize_setup').doLayout();
                        widget_customed.setValue(result.customed);
                        widget_portrait.setValue(result.portrait);
                        widget_landscape.setValue(result.landscape);
                        widget_paperSize.setValue(result.paperSize);
                        widget_user_width.setValue(result.user_width);
                        widget_user_height.setValue(result.user_height);
                        widget_margin_top.setValue(result.margin_top);
                        widget_margin_bottom.setValue(result.margin_bottom);
                        widget_margin_left.setValue(result.margin_left);
                        widget_margin_right.setValue(result.margin_right);
                    }
                }
            });
            o.onOK = function () {
                FR.ajax({
                    url: FR.servletURL,
                    type: 'POST',
                    data: {
                        op: 'fr_dialog',
                        cmd: 'change_page_setup',
                        sessionID: sessionID,
                        reportIndex: reportIndex,
                        predefined: widget_predefined.getValue(),
                        customed: widget_customed.getValue(),
                        portrait: widget_portrait.getValue(),
                        landscape: widget_landscape.getValue(),
                        paperSize: widget_paperSize.getValue(),
                        user_width: widget_user_width.getValue(),
                        user_height: widget_user_height.getValue(),
                        margin_top: widget_margin_top.getValue(),
                        margin_bottom: widget_margin_bottom.getValue(),
                        margin_left: widget_margin_left.getValue(),
                        margin_right: widget_margin_right.getValue()
                    },
                    complete: function () {
                        FR.closeDialog();
                    }
                });
            };
            o.contentWidget = {
                type: 'border',
                items: [
                    {region: 'north', el: $('<div>'), height: northHeight},
                    {region: 'center', el: tableLayout.element}
                ]
            };
            FR.showUpdateDialog(o);
        },

        _sheetPositionPane: function () {
            return "bottom";
        },

        _initSheetTabPane: function ($contentPane, sheetsO, bg) {
            var sheetPosition = this._sheetPositionPane();
            var writeTab = new FR.TabPane({
                tabBackground: bg,
                items: sheetsO,
                tabPosition: sheetPosition,
                renderEl: $contentPane
            });
            $contentPane.data("TabPane", writeTab);
            // 先构建出来隐藏掉，当需要展示报表时再显示
            writeTab.setVisible(false);
        },

        _createFormParameterPane: function (options, layout, paramsTemplate) {
            var collapsePaneHeight = 8;
            options.alignLayoutName = '__layout4align__';
            var self = this, paramO = options.param;
            var config = paramO.html;
            self.scrollHeight = 6;
            //这边maxWidth 7.1的时候被改成了paramO.width, 导致web端的居右居中效果很奇怪
            var maxWidth = 0, maxHeight = 0;
            for (var i = 0; i < config.items.length; i++) {
                var item = config.items[i];
                maxWidth = Math.max(maxWidth, item.x + item.width);
                maxHeight = Math.max(maxHeight, item.y + item.height);
            }
            var deltaHeight = paramO.height - maxHeight;
            var formLayout;
            var barHeight = paramO.width > document.body.offsetWidth ? self.scrollHeight : 0;
            var $paraContainer = $("<div class='parameter-container' style='overflow: auto;'>").css({
                'height': paramO.height + barHeight + collapsePaneHeight,
                'width': document.body.offsetWidth
            });
            var paramsTemplateHeight = (paramO.useParamsTemplate && paramsTemplate !== null ? 30 : 0);
            FR.parameterContainerHeight = paramO.height + barHeight + collapsePaneHeight + paramsTemplateHeight;
            options.$paraContainer = $paraContainer;
            config.onCalculate = function (currentMaxHeight) {
                if ($.isFunction(options.onCalculate)) {
                    options.onCalculate.call(this, currentMaxHeight + deltaHeight, collapsePaneHeight, $paraContainer);
                }
            };
            var $expandEl = $("<div class='parameter-container-collapseimg-up' style='cursor: pointer'></div>").click(function () {
                var resizeTime = new Date();
                self.lastResizeTime = resizeTime;
                setTimeout(function () {
                    if (resizeTime === self.lastResizeTime) {
                        delete self.lastResizeTime;
                        $paraContainer.animate({
                            height: (self.hasCollapsed ? "+=" : "-=") + (paramO.height + paramsTemplateHeight)
                        }, "fast", function () {
                            if (self.hasCollapsed) {
                                FR.parameterContainerHeight += (paramO.height + paramsTemplateHeight);
                            } else {
                                FR.parameterContainerHeight -= (paramO.height + paramsTemplateHeight);
                            }
                            self.hasCollapsed = !self.hasCollapsed;
                            $expandEl.switchClass('parameter-container-collapseimg-down', 'parameter-container-collapseimg-up');
                            if ($.isFunction(options.onCollapse)) {
                                var height = $paraContainer.height();
                                options.onCollapse.call(this, self.hasCollapsed, options.alignLayoutName, height);
                                self.doAfterCollapse();
                                self.afterLoad();
                            }
                        });
                    }
                }, 222);
            });

            var alignment = 'left';
            switch (paramO.alignLocation) {
                case 0 :
                    alignment = 'left';
                    break;
                case 1 :
                    alignment = 'center';
                    break;
                case 2 :
                    alignment = 'right';
                    break;
                default :
                    alignment = 'left';
            }
            var $center = $('<div class="pmeter-container"></div>').css({
                "height": paramO.height,
                "width": paramO.width
            });
            var paramstemplateHeight = 30;
            var paramstemplateContainer = {
                region: 'north',
                height: paramstemplateHeight,
                el: {
                    type: 'border',
                    widgetName: 'paramsTemplate',
                    items: [
                        {
                            region: 'east',
                            width: 138,
                            el: {
                                type: 'horizontal',
                                alignment: 'right',
                                items: [{
                                    el: {
                                        type: 'button',
                                        widgetName: 'saveBtn',
                                        text: FR.i18nText("Fine-Engine_Report_Designer_Save_As_Params_Template")
                                    }, width: 128
                                }, {
                                    el: $('<div>'), width: 10
                                }]
                            }
                        }, {
                            region: 'center',
                            el: {
                                type: 'horizontal',
                                widgetName: 'paramsTemplatePane',
                                alignment: 'left',
                                items: []
                            }
                        }, {
                            region: 'west',
                            width: 30,
                            el: {
                                type: 'horizontal',
                                alignment: 'left',
                                items: [
                                    {
                                        el: {
                                            type: 'iconbutton',
                                            widgetName: 'newParamsBtn',
                                            width: 30,
                                            baseClass: 'fs-new-params-template-blue'

                                        },
                                        width: 30
                                    }
                                ]
                            }
                        }
                    ]
                }
            };

            if (!options.param.useParamsTemplate || paramsTemplate === null) {
                paramstemplateHeight = 0;
                paramstemplateContainer = {};
            }
            //定义一下用来滚动的对象
            var paraConfig = {
                renderEl: $paraContainer,
                type: 'border',
                items: [
                    paramstemplateContainer,
                    {
                        region: 'south', height: collapsePaneHeight, el: {
                            type: 'horizontal',
                            items: [
                                {el: $expandEl, width: 120}
                            ]
                        }
                    },
                    {
                        region: 'center', el: {
                            type: 'para_horizontal',
                            width: paramO.width,
                            minWidth: paramO.width,
                            widgetName: options.alignLayoutName,
                            alignment: alignment,
                            items: [
                                {el: $center, width: maxWidth}
                            ]
                        }
                    }
                ]
            };
            this._setBackground(paramO, $paraContainer);
            if (paramO.showType === 1) {
                layout.push({
                    region: 'north',
                    el: paraConfig,
                    height: paramO.height + collapsePaneHeight + barHeight + paramstemplateHeight
                });
            }
            if (paramO.showType === 0) {
                $paraContainer.css({
                    position: "absolute",
                    width: paramO.width,
                    height: paramO.height
                }).appendTo("body");
                config.listeners = [
                    {
                        once: true,
                        eventName: 'submitcomplete',
                        action: function () {
                            FR.closeDialog();
                        }
                    }
                ];
                config.renderEl = $paraContainer;
                formLayout = new FR.contentForm(config);
            } else {
                config.renderEl = $center;
                config.container = $paraContainer;
                formLayout = new FR.contentForm(config);
            }
            if (paramO.showType === 0) {
                formLayout.doLayout();
                FR.showDialog({
                    title: paramO.paramWindowTitle,
                    width: paramO.width + 20,
                    height: paramO.height + 45,
                    contentHtml: $paraContainer
                });
            }
            return formLayout;
        },

        _setBackground: function (paramO, $dom) {
            var bg = paramO.parambg;
            if (bg !== null && bg !== undefined) {
                FR.setBackground($dom, bg);
            }
        },

        doAfterCollapse: function () {
        },

        afterLoad: function () {
            this.doResize();
        },

        doResize: function () {
            var fitConfig = FR.jsonDecode(this.__fitConfig__);
            var isFit = fitConfig.__fit__;
            var fitState = fitConfig.__fitState__;
            var $frozenDIV = $(".frozen-table");
            if (isFit && !FR.Browser.isIE8Before() && $frozenDIV.length === 0) {
                var container = this.$contentPane;
                if (container !== undefined) {
                    var preBounds = container.data("preBounds");
                    if (preBounds !== undefined && preBounds.height === container.height() && preBounds.width === container.width()) {
                        return;
                    }
                    container.data("preBounds", container.bounds());
                    this.scaleContent(container, fitState);
                }

            }
        },

        scaleContent: function (container, fitState) {
            var $contentDIV = $(".pageContentDIV").length > 0 ? ".pageContentDIV" : ".sheet-container";
            var widthScale = parseFloat(container.width() / container.data("preWidth"));
            var heightScale = parseFloat(container.height() / container.data("preHeight"));
            if (fitState === 1) {
                heightScale = widthScale;
            }
            this.processScrollWidth(container, widthScale, heightScale);
        },


        processScrollWidth: function (container, widthScale, heightScale) {
            var $contentDIV = $(".pageContentDIV").length > 0 ? ".pageContentDIV" : ".sheet-container";
            for (var i = 0; i < container.length; i++) {
                var container = $(container[i]);
                var contentDIV = $($contentDIV, container[i]);
                contentDIV.removeClass("main-content-left");
                var tableDiv = $(".x-table", container[i]);
                if (contentDIV.height() > 0) {
                    if (tableDiv.height() * heightScale > container.height() && tableDiv.width() * widthScale <= container.width() && $(".frozen-table").length === 0) {
                        widthScale = parseFloat((container.width() - 18) / container.data("preWidth"));
                        container.css({
                            overflowX: 'hidden',
                            overflowY: 'auto'
                        });
                    } else if (tableDiv.width() * widthScale <= (container.width()) && tableDiv.height() * heightScale <= container.height() && $(".frozen-table").length === 0) {
                        container.css({
                            overflow: 'hidden',
                            overflowX: 'hidden',
                            overflowY: 'hidden'
                        });
                    } else if ($(".frozen-table").length === 0) {
                        container.css({
                            overflow: 'auto',
                            overflowX: 'auto',
                            overflowY: 'auto'
                        });
                    }

                }
                var zoom = {x: widthScale, y: heightScale};
                this.zoomContent(contentDIV, zoom);
            }
        },

        /**
         * 提交参数并加载报表主体内容
         *
         *     @example
         *     contentPane.parameterCommit();
         *
         * @param {Object} reloadOthers 配置
         */
        parameterCommit: function (reloadOthers) {
            if ($.isFunction(this.parameterEl.formSubmit)) {
                this.parameterEl.formSubmit({
                    url: FR.servletURL + "?op=fr_dialog&cmd=parameters_d&sessionID=" + _g().currentSessionID,
                    callback: function () {
                        _g().loadContentPane(reloadOthers);
                    }
                });
            }
        },

        /**
         * 改变在鼠标经过或者点击报表行时行的颜色
         *
         *     @example
         *     contentPane.makeHighlight('red', 'click');
         *
         * @param {String} color 需要变换成的颜色
         * @param {'click'/'mouseover'} [op='click'] 鼠标动作
         * @param deep 是否是加在td上面
         */
        makeHighlight: function (color, op, deep) {
            var self = this;
            if (!op) {
                op = 'click';
            }
            var container = $('div.content-container');
            var tr = $("tr", container);
            var changeTds = function (currentTR, lastTDColors) {
                var tds = currentTR.children();
                for (var i = 0; i < tds.length; i++) {
                    var td = tds.eq(i);
                    lastTDColors.push(td[0].style.backgroundColor);
                    td[0].style.backgroundColor = color;
                }
            };
            var highLightFun = function (e) {
                if (!$(e.target).is("td")) {
                    return;
                }
                var currentTR = $(e.target).parent();
                if ($(e.target).attr("hv")) {
                    // marro:如果是格子套格子的,PageCellWriter.getValueWrapper
                    currentTR = currentTR.closest("td").parent();
                }
                if (lastSelectedTR != null) {
                    if (deep === true) {
                        var tds = lastSelectedTR.children();
                        for (var i = 0; i < tds.length; i++) {
                            var td = tds.eq(i);
                            td[0].style.backgroundColor = lastTDColors[i] === "" ? "White" : lastTDColors[i];
                        }
                        lastTDColors.length = 0;
                        if (currentTR[0] !== lastSelectedTR[0]) {
                            changeTds(currentTR, lastTDColors);
                            lastSelectedTR = currentTR;
                            return;
                        }
                    } else {
                        lastSelectedTR[0].style.backgroundColor = 'White';
                        if (currentTR[0] !== lastSelectedTR[0]) {
                            currentTR[0].style.backgroundColor = (currentTR[0].style.backgroundColor === color ? 'White' : color);
                            lastSelectedTR = currentTR;
                            return;
                        }
                    }
                    lastSelectedTR = null;
                    return;
                }
                lastSelectedTR = currentTR;

                if (deep === true) {
                    changeTds(currentTR, lastTDColors);
                } else {
                    currentTR[0].style.backgroundColor = (currentTR[0].style.backgroundColor === color ? 'White' : color);
                }
            };

            var lastSelectedTR = null;
            var lastTDColors = [];
            var action = op.indexOf('over') !== -1 ? "mouseover" : "click";
            if (window.highfun !== null && window.highfun !== undefined) {
                container.unbind(action, window.highfun);
            }
            window.highfun = function (e) {
                highLightFun(e);
            };
            container.bind(action, window.highfun);
        },

        initContentPane: function () {
            var events = FR.Report.Plugin.Panel.Events;
            if ($.isArray(events)) {
                for (var i = 0, len = events.length; i < len; i++) {
                    var evt = events[i];
                    this.on(evt.name, evt.action.createDelegate(this));
                }
            }
        },

        /**
         * 导出csv
         */
        exportReportToCSV: function () {
            window.location = FR.servletURL + "?op=export&sessionID=" + this.currentSessionID + "&format=csv";
        },

        directExportToExcel: function (dsName, fileName, params, colNames) {
            if (this.fireEvent(FR.Events.BTOEXCEL) === false) {
                return;
            }

            var url = FR.servletURL + "?op=export&cmd=direct_excel&sessionID=" + this.currentSessionID + "&dsName=" + dsName;
            if (fileName !== undefined) {
                url += "&fileName=" + fileName
            }
            if (params !== undefined) {
                url += "&params=" + params;
            }
            if (colNames !== undefined) {
                url += "&colNames=" + colNames;
            }

            this.downloadExportFile(url, 'excel');

            this.fireEvent(FR.Events.ATOEXCEL);
        },

        /**
         * 零客户端打印
         */
        noClientPrint: function (popupMarginSetup, ieQuietPrint) {
            FR.doNoClientPrint(this.currentSessionID, popupMarginSetup, ieQuietPrint);
        },

        /**
         * (新)本地打印
         */
        newNativePrint: function (popupDialog, serverURL) {
            var self = this;
            var config = {
                isPopUp: popupDialog,  // 是否弹窗
                currentPageIndex: self.currentPageIndex == null ? self.currentPageIndex : self.currentPageIndex.toString(),
                currentSheetIndex: self.selectedIndex + 1,  // 从 1 开始计数
                serverURL: serverURL
            };
            if (self.selectedIndex == null) {  // 是分页预览
                // 分页预览不支持选择 sheet
                config.needSelectSheet = false;
            }
            FR.doNewNativePrint(this.currentSessionID, config);
        },

        pdfPrint: function (isPopup) {
            if (this.fireEvent(FR.Events.BPDFPRINT) === false) {
                return;
            }
            if (FR.Browser.isIE()) {
                FR.doPDFPrint(this.currentSessionID, isPopup);
            } else {
                // james：pdfPrint需要用format=pdfPrint，服务器才能识别为是打印状态啊
                var url = FR.servletURL + "?sessionID=" + this.currentSessionID;
                FR.doURLPDFPrint(url);
                this.fireEvent(FR.Events.APDFPRINT);
            }
        },
//wei:移动端支持自动刷新，pc不做事
        setAppearRefresh: function () {

        },
        /**
         * 根据sheet的序号加载sheet
         * @param index sheet序号
         */
        loadSheetByIndex: function (index) {
            var $contentPane = $("div.content-container");
            if (!FR.isEmpty(index) && $contentPane && $contentPane.data && $contentPane.data('TabPane')) {
                $contentPane.data('TabPane').selectTabAt(index);
            }
        },
        /**
         * 根据sheet的名称加载sheet
         * @param sheetName sheet名称
         */
        loadSheetByName: function (sheetName) {
            var $contentPane = $("div.content-container");
            if (!FR.isEmpty(sheetName) && $contentPane && $contentPane.data && $contentPane.data('TabPane')) {
                $contentPane.data('TabPane').selectTabByName(sheetName);
            }
        },
        /**
         * 文件下载
         * @param url 下载地址
         * @param type
         */
        downloadExportFile: function (url, type) {
            //新导出方式插件
            if (FR.Report.Plugin.XHR2FileSaveProcessor.item && FR.Plugin.validLevel(FR.Report.Plugin.XHR2FileSaveProcessor, FR.Report.Plugin.XHR2FileSaveProcessor.item)) {
                FR.Report.Plugin.XHR2FileSaveProcessor.item.action.call(this, url, type);
                return;
            }
            FR.openUrlByForm(url);
            FR.progressBar.call(this, this.currentSessionID, type);
        },

        checkExportRegister: function () {
            function checkRegister() {
                var result = "";
                FR.ajax({
                    url: FR.servletURL,
                    data: {
                        op: 'export',
                        cmd: 'check_register',
                        sessionID: this.currentSessionID
                    },
                    async: false,
                    complete: function (res, status) {
                        try {
                            var returnData = FR.jsonDecode(res.responseText);
                            result = returnData.exception === 'FAILPASS';
                            if (result) {
                                FR.Msg.toast(FR.i18nText("Fine-Engine_Export_Print_Not_Support"));
                            }
                        } catch (Error) {
                            FR.Msg.toast("AnalysisData Error!");
                        }

                    }
                });
                return !result;
            }

            this.on(FR.Events.BTOEXCEL, checkRegister);
            this.on(FR.Events.BTOIMAGE, checkRegister);
            this.on(FR.Events.BTOPDF, checkRegister);
            this.on(FR.Events.BTOWORD, checkRegister);
            this.on(FR.Events.BTOHTML, checkRegister);
        },

        createProgressBar: function () {
            if ($('#progressBar').length <= 0) {
                var progressBar = $('<div id = ' + '\"' + 'progressBar' + '\"' + '>' + '</div>').appendTo("body");
                var tips = $('<div>' + FR.i18nText("Fine-Engine_Exporting") + '...' + '</div>').appendTo($('#progressBar'));
                var $maskDiv = $('<div id="transparent_mask" style="position:absolute;filter:alpha(opacity=50);opacity:0.5;"></div>');
                $maskDiv.css('left', "0px").css('top', "0px").css('width', "100%").css('height', "100%").css('z-index', 2001).appendTo("body");
                progressBar.addClass("progressBar");
                tips.addClass("progressBarTips");
            }
        },
        removeProgressBar: function () {
            $('#progressBar').remove();
            $('#transparent_mask').remove();
        },

        /**
         * 邮件发送
         * @param jo
         */
        emailReport: function (jo) {
            if (this.fireEvent(FR.Events.BEMAIL) === false) {
                return;
            }
            var self = this;
            FR.showEmailDialog({
                sessionID: this.currentSessionID,
                emailAttr: FR.jsonDecode(jo),
                onFinish: function () {
                    self.fireEvent(FR.Events.AEMAIL);
                }
            });
        }
    });
})(jQuery);
