/**
 * 用于放置ChartGlyph. js端各种用到的shape
 *
 * shape的initConstants(初始化),
 */

FR.Chart = FR.Chart || {};

/**
 * 图表使用的2D对象（点）
 * @class FR.Chart.Point2D
 * @param x 横坐标
 * @param y 纵坐标
 * @constructor
 */
FR.Chart.Point2D = function (x, y) {
    this.x = x;
    this.y = y;
};
$.extend(FR.Chart.Point2D.prototype, /**@class FR.Chart.Point2D*/{
    /**
     * @property 横坐标
     */
    x: 0,

    /**
     * @property 纵纵坐标
     */
    y: 0,

    /**
     * 判断两个点是否相同
     * @param point 要比较的点
     * @returns {Boolean} 如何相同则返回true，否则返回false
     */
    equals: function (point) {
        if (point instanceof FR.Chart.Point2D) {
            return (this.x == point.x) && (this.y == point.y);
        }
        return false;
    },

    /**
     * 改变点的坐标
     * @param x 横坐标
     * @param y 纵坐标
     */
    setLocation: function (x, y) {
        this.x = x;
        this.y = y;
    }
});
// Point2D.end

/**
 * 图表中使用的尺寸
 * @class FR.Chart.Dimension2D
 * @param width 宽度
 * @param height 高度
 * @constructor
 */
FR.Chart.Dimension2D = function (width, height) {
    this.initConstants(width, height);
};
$.extend(FR.Chart.Dimension2D.prototype, /**@class FR.Chart.Dimension2D*/{
    initConstants: function (width, height) {
        this.width = FR.pick(width, 0);
        this.height = FR.pick(height, 0);
    }
});
// Dimension2D.end

//MapReturnButton.start
FR.Chart.MapReturnButton = function(bounds) {
    this.bounds = bounds;
    this.clickable = true;
};

$.extend(FR.Chart.MapReturnButton.prototype, {
    onClick: function (mousePos, event) {
        event.returnBack = true;
        event.needRefresh = true;
    },

    containsPoint   :   function(mouseX, mouseY) {
        return this.bounds.containsPoint(mouseX, mouseY);
    }
});
//MapReturnButton.end

//HeatMapControlBar.start
FR.Chart.HeatMapControlBar = function(bounds, position, mapHotAreaColor, seriesList, textAttr){
    this.bounds = bounds;
    this.position = position;
    this.mapHotAreaColor = mapHotAreaColor;
    this.seriesList = seriesList;

    this.startItemIndex = 0;
    this.endItemIndex = mapHotAreaColor.getAreaNumber() - 1;

    this.areaColorBlock = [];

    this.jsType = "controlBar";
    this.textAttr = textAttr;

    this.clickable = true;

    //初始化色块位置
    this._layoutAreaColorBlock();
};

$.extend(FR.Chart.HeatMapControlBar,{
    BAR_LONG:26,
    BAR_SHORT:13,
    BAR_GAP:1
});

$.extend(FR.Chart.HeatMapControlBar.prototype,{
    _layoutAreaColorBlock:function(){

        var barStartX = 0, barStartY = 0;
        var barNumber = this.mapHotAreaColor.getAreaNumber();
        var barLongSide = FR.Chart.HeatMapControlBar.BAR_LONG;
        var barShortSide = FR.Chart.HeatMapControlBar.BAR_SHORT;

        var startLabel = this.mapHotAreaColor.getMaxLabelWithIndex(0);
        var startWidth = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(startLabel, this.textAttr).width;
        var fontHeight = this.textAttr.frFont.size;
        var x = this.bounds.x;
        var y = this.bounds.y;

        switch (this.position){
            case FR.Chart.Constants.LEFT:
                barStartX = x + startWidth + 5;
                barStartY = y + fontHeight / 2;
                break;
            case FR.Chart.Constants.RIGHT:
                barStartX = x;
                barStartY = y + fontHeight / 2;
                break;
            default:
                barStartX = x + startWidth / 2;
                barStartY = y + fontHeight + 5;
        }

        if(this.isInTopOrBottom()){
            for(var barIndex = 0; barIndex < barNumber; barIndex++){
                var x = barStartX + barLongSide * barIndex + barIndex;
                this.areaColorBlock.push(new FR.Chart.Bounds(x, barStartY, barLongSide, barShortSide));
            }
        }else{
            for(var barIndex = 0; barIndex < barNumber; barIndex++){
                var y = barStartY + barLongSide * barIndex + barIndex;
                this.areaColorBlock.push(new FR.Chart.Bounds(barStartX, y, barShortSide, barLongSide));
            }
        }

    },

    draw:function(ctx){
        if(this.isInTopOrBottom()){
            this._drawTopOrBottom(ctx);
        }else{
            this._drawLeftOrRight(ctx);
        }
    },

    _drawTopOrBottom:function(ctx){
        this._drawColorBlocks(ctx);
        var preWidth = FR.Chart.HeatMapControlBar.BAR_LONG * 2 + 1;
        var labels = this.mapHotAreaColor.getNumberLabels();
        var fontHeight = this.textAttr.frFont.size;
        var step = FR.Chart.HeatMapControlBar.BAR_LONG + 1;
        var startX = this.bounds.x;
        var topY = this.bounds.y;
        var bottomY = this.bounds.y + fontHeight + 5 + FR.Chart.HeatMapControlBar.BAR_SHORT + 5;
        for(var i = 0; i < labels.length; i++){
            var label = this._getFinalDrawLabel(labels[i], preWidth);
            var y = i % 2 === 0 ? topY : bottomY;
            this._drawLabel(label, startX, y);
            startX += step;
        }
    },

    _drawLeftOrRight:function(ctx){
        this._drawColorBlocks(ctx);
        var preWidth = this.bounds.width - FR.Chart.HeatMapControlBar.BAR_SHORT - 5;
        var startY = this.bounds.y;
        var startX = this.position == FR.Chart.Constants.RIGHT ? (this.bounds.x + FR.Chart.HeatMapControlBar.BAR_SHORT + 5) : this.bounds.x;
        var labels = this.mapHotAreaColor.getNumberLabels();
        var step = FR.Chart.HeatMapControlBar.BAR_LONG + 1;
        for(var i = 0; i < labels.length; i++){
            var label = this._getFinalDrawLabel(labels[i], preWidth);
            this._drawLabel(label, startX, startY);
            startY += step;
        }
    },

    _drawColorBlocks:function(ctx){
        if(!ctx){     return; } ctx.save();
        for(var i = 0; i < this.areaColorBlock.length; i++){
            var color = this.mapHotAreaColor.getColorByIndex(i);
            if(this._isSeriesEnable(i)){
                ctx.fillStyle = color;
            }else{
                ctx.fillStyle = 'rgba(128,128,128,0.8)';
            }

            this.areaColorBlock[i].paint(ctx);
        }
        ctx.restore();
    },

    _getFinalDrawLabel:function(label, preWidth){
        var labelDim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(label, this.textAttr);
        if(labelDim.width > preWidth){
            return FR.Chart.TextUtils.getTextWhenNotEnoughWidth(label, preWidth, this.textAttr.frFont);
        }
        return label;
    },

    _drawLabel:function(label, x, y){
        var newDiv = $("<div>" + label + "</div>").css('position', 'absolute')
            .css('font', this.textAttr.frFont.getStringFont()).css('z-index', 1).css('color', this.textAttr.frFont.color)
            .css('left', x).css('top', y);

        FR.Chart.DivCon.addDiv(FR.Chart.currentDivKey, newDiv[0]);
    },

    containsPoint:function(mousePos){
        for(var i = 0, count = this.areaColorBlock.length; i < count; i++){
            var bounds = this.areaColorBlock[i];
            if(bounds && bounds.containsPoint(mousePos.x, mousePos.y)){
                return true;
            }
        }
        return false;
    },

    findOnMoveGlyph:function(mousePos, event){
        if(this.containsPoint(mousePos)){
            event._hasfound = true;
            event._lastHover = this;
        }
    },

    onClick:function(mousePos, event){
        var seriesIndex = this._getItemIndex(mousePos);
        if(seriesIndex !== -1){
            event.needRefresh = true;
            var series = this.seriesList[seriesIndex];
            if(series){
                if (series.paintState >= FR.Chart.ChartPaintConstants.STATE_ENABLE) {
                    series.paintState = FR.Chart.ChartPaintConstants.STATE_DISAPPEAR;
                } else if (series.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                    series.paintState = FR.Chart.ChartPaintConstants.STATE_APPEAR;
                }
            }
        }
    },

    doOnMove:function(ctx4OnMove, mousePos, plotGlyph, mapOnMoveCtx){
        var seriesIndex = this._getItemIndex(mousePos);
        if(this._isSeriesEnable(seriesIndex)){
            var bounds = this.areaColorBlock[seriesIndex];
            var color = this.mapHotAreaColor.getColorByIndex(seriesIndex);
            var newBounds = new FR.Chart.Bounds(bounds.x-2, bounds.y-2, bounds.width+4, bounds.height+4);
            ctx4OnMove.save();

            ctx4OnMove.fillStyle = color;
            ctx4OnMove.shadowBlur = 10;
            ctx4OnMove.shadowColor= 'rgba(0,0,0, 0.2)';
            newBounds.paint(ctx4OnMove);

            ctx4OnMove.restore();

            var ctxUse = ctx4OnMove;
            if(plotGlyph && plotGlyph.mapType != FR.Chart.MapType.Map_Normal){
                ctxUse = mapOnMoveCtx;
            }

            var dataPoints = plotGlyph.getSeriesPointsOfCurrentMap(seriesIndex);
            //截取绘图区原因：放大后悬浮在某区域，超出绘图区范围到图表区了。bug：74495
            ctxUse.save();// 整个动态开始前 保存, 以便于clip截取绘图区. 然后在整个动态(包括after 最后的标记结束后) 再复位.
            plotGlyph._saveClip4SeriesDrawThrowOutOfPlot(ctxUse, plotGlyph);
            for(var i = 0; i < dataPoints.length; i++){
                var dataPoint = dataPoints[i];
                dataPoint.doOnMoveWithoutTooltip(ctx4OnMove, plotGlyph, ctxUse);
            }
            ctxUse.restore();// 整个动态结束 包括最后的标记类型, 复位.
        }
    },

    _getItemIndex:function(mousePos){
        for(var i = 0, count = this.areaColorBlock.length; i < count; i++){
            var bounds = this.areaColorBlock[i];
            if(bounds && bounds.containsPoint(mousePos.x, mousePos.y)){
                return i;
            }
        }
        return -1;
    },

    _isSeriesEnable:function(seriesIndex){
        if(this.seriesList && this.seriesList[seriesIndex]){
            var series = this.seriesList[seriesIndex];
            return series.paintState >= FR.Chart.ChartPaintConstants.STATE_ENABLE;
        }
        return false;
    },

    isInTopOrBottom:function(){
        return this.position == FR.Chart.Constants.TOP || this.position == FR.Chart.Constants.BOTTOM;
    }

});
//HeatMapControlBar.end

// Line2D.start
FR.Chart.Line2D = function (startPoint, endPoint) {
    this.startPoint = startPoint;
    this.endPoint = endPoint;
};
$.extend(FR.Chart.Line2D.prototype, {
    startPoint: new FR.Chart.Point2D(),
    endPoint: new FR.Chart.Point2D(),

    getBounds2D: function () {
        var minX = Math.min(this.startPoint.x, this.endPoint.x);
        var maxX = Math.max(this.startPoint.x, this.endPoint.x);

        var minY = Math.min(this.startPoint.y, this.endPoint.y);
        var maxY = Math.max(this.startPoint.y, this.endPoint.y);

        return new FR.Chart.Bounds(minX, minY, maxX - minX, maxY - minY);
    },

    isNeedRadialGradient:function(){
        return false;
    },

    transform: function (at) {
        var leftUp = at.transform(this.startPoint);
        var rightDown = at.transform(this.endPoint);
        var line = new FR.Chart.Line2D(leftUp, rightDown);
        return line;
    },

    draw: function (ctx, lineStyle) {
        if (lineStyle) {
            // kunsnat: TODO 虚线部分的处理, 还有各种类型的虚线样式.
            if (lineStyle == FR.Chart.Constants.LINE_THIN) {
                ctx.lineWidth = 1;
                this.draw(ctx);
                return;
            } else if (lineStyle == FR.Chart.Constants.LINE_MEDIUM) {
                ctx.lineWidth = 2;
                this.draw(ctx);
                return;
            } else if (lineStyle == FR.Chart.Constants.LINE_DASH || lineStyle == FR.Chart.Constants.LINE_HAIR
                || lineStyle == FR.Chart.Constants.LINE_HAIR2) {
                ctx.lineWidth = 1;
            } else if (lineStyle == FR.Chart.Constants.LINE_THICK) {
                ctx.lineWidth = 3;
                this.draw(ctx);
                return;
            } else if (lineStyle == FR.Chart.Constants.LINE_DOT) {
                ctx.lineWidth = 1;
            } else if (lineStyle == FR.Chart.Constants.LINE_MEDIUM_DASH) {
                ctx.lineWidth = 2;
            } else if (lineStyle == FR.Chart.Constants.LINE_DASH_DOT) {
                ctx.lineWidth = 1;
            } else if (lineStyle == FR.Chart.Constants.LINE_MEDIUM_DASH_DOT) {
                ctx.lineWidth = 2;
            } else if (lineStyle == FR.Chart.Constants.LINE_DASH_DOT_DOT) {
                ctx.lineWidth = 1;
            } else if (lineStyle == FR.Chart.Constants.LINE_MEDIUM_DASH_DOT_DOT) {
                ctx.lineWidth = 2;
            } else if (lineStyle == FR.Chart.Constants.LINE_SLANTED_DASH_DOT) {
                ctx.lineWidth = 2;
            }
            this.drawDotLine(ctx);
        } else {
            ctx.beginPath();
            ctx.moveTo(this.startPoint.x, this.startPoint.y);
            ctx.lineTo(this.endPoint.x, this.endPoint.y);
            ctx.stroke();
        }
    },

    // kunsnat: 画虚线---  lineStyle  各种虚线. 这边先简单处理, 只提供一种..
    drawDotLine: function (ctx) {
        /**
         * 注释6为虚线的总长度, 包括中间的空余部分, 一半为虚线的长度
         */
        var dotLength = 6;

        var x = this.startPoint.x;
        var y = this.startPoint.y;
        var ex = this.endPoint.x;
        var ey = this.endPoint.y;

        // kunsnat: 宽度固定在4, 间隔2  计算总长度, 分割, 然后斜率
        var width = x - ex;
        var height = y - ey;
        // 虚线的数目
        var dotNumber = 0;
        var xIncrement = 0;
        var yIncrement = 0;
        // 垂直的. 不存在斜率
        if (width === 0) {
            dotNumber = Math.abs(height) / dotLength;
            xIncrement = 0;
            yIncrement = (y - ey) > 0 ? -dotLength : dotLength;
        } else {
            var alpha = Math.atan(height / width);

            xIncrement = dotLength * Math.cos(alpha);
            yIncrement = dotLength * Math.sin(alpha);

            // dotNumber 需要多少个虚线来完成.
            dotNumber = Math.sqrt(width * width + height * height) / dotLength;
        }
        ctx.beginPath();

        // TODO 不理想的虚线... 特别是趋势线的时候, 已经分到每个像素了..
        if (dotNumber < 1) {
            ctx.moveTo(x + (ex - x) * 0.95, y + (ey - y) * 0.95);
            ctx.lineTo(ex, ey);
        } else {
            for (var i = 0; i < dotNumber - 0.5; i++) {
                ctx.moveTo(x + xIncrement * i, y + yIncrement * i);
                ctx.lineTo(x + xIncrement * (i + 0.5), y + yIncrement * (i + 0.5));
            }
        }

        ctx.stroke();
    }
});
FR.Chart.Line2D.relativeCCW = function (X1, Y1, X2, Y2, PX, PY) {
    X2 -= X1;
    Y2 -= Y1;
    PX -= X1;
    PY -= Y1;
    var ccw = PX * Y2 - PY * X2;
    if (ccw === 0.0) {
        ccw = PX * X2 + PY * Y2;
        if (ccw > 0.0) {
            PX -= X2;
            PY -= Y2;
            ccw = PX * X2 + PY * Y2;
            if (ccw < 0.0) {
                ccw = 0.0;
            }
        }
    }
    return (ccw < 0.0) ? -1 : ((ccw > 0.0) ? 1 : 0);
};
// Line2D.end

// Bounds.start
FR.Chart.Bounds = function (x, y, width, height) {
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
    this.rect = true;
};
$.extend(FR.Chart.Bounds.prototype, {
    initConstants: function (js) {
        js = js || {};

        this.x = js.x || 0;
        this.y = js.y || 0;
        this.width = js.width || 0;
        this.height = js.height || 0;

        // 柱形的负值倒立画法. 传入bounds才会需要在这边初始化
        this.isNigativeValue = js.isNigativeValue || false;
        // 用于条形图: 横向增长方式
        this.isHorizontal4Bounds = js.isHorizontal4Bounds || false;
        this.is4RangePlot = js.is4RangePlot || false;
        this.isUnanimate = js.isUnanimate || false;

        // 股价图的动态方式. 滑行的平移的方式. (预留)
        this.is4StockPlotGlyph = false;
    },

    getBounds2D: function () {
        return this;
    },

    isNeedRadialGradient:function(){
        return false;
    },

    getMiddleX:function(){
        return this.x + this.width / 2;
    },

    getMiddleY:function(){
        return this.y + this.height / 2;
    },

    transform: function (at) {
        var bounds = new FR.Chart.Bounds();

        var leftUp = at.transform({x:this.x, y:this.y});
        var rightDown = at.transform({x:this.x + this.width, y:this.y+this.height});

        bounds.setRect(leftUp.x, leftUp.y, rightDown.x - leftUp.x, rightDown.y - leftUp.y);

        return bounds;
    },

    clone: function () {
        var bounds = new FR.Chart.Bounds(this.x, this.y, this.width, this.height);
        bounds.isNigativeValue = this.isNigativeValue;
        bounds.isHorizontal4Bounds = this.isHorizontal4Bounds;
        bounds.is4RangePlot = this.is4RangePlot;
        bounds.isUnanimate = this.isUnanimate;

        return bounds;
    },

    getLineGradient: function (ctx, isHorizontal) {
        if (this.lineGradient) {
            return this.lineGradient;
        }

        if (isHorizontal) {
            // 后面的xy等 IE不是必须的 只要表示方向就可, FF必须..
            this.lineGradient = ctx.createLinearGradient(this.x, this.y, this.x, this.y + this.height);
        } else {
            this.lineGradient = ctx.createLinearGradient(this.x, this.y, this.x + this.width, this.y);
        }

        return this.lineGradient;
    },

    paint: function (ctx) {
        if (this.width <= 0 || this.height <= 0) {
            return;
        }

        var newBounds = FR.Chart.GraphHelper.dealRectShape4JSPaint(this);

        ctx.beginPath();
        ctx.moveTo(newBounds.x, newBounds.y);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y + newBounds.height);
        ctx.lineTo(newBounds.x, newBounds.y + newBounds.height);
        ctx.closePath();
        ctx.fill();
    },

    draw: function (ctx) {
        if (this.width <= 0 || this.height <= 0) {
            return;
        }

        var newBounds = FR.Chart.GraphHelper.dealRectShape4JSDraw(this);

        ctx.beginPath();
        ctx.moveTo(newBounds.x, newBounds.y);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y + newBounds.height);
        ctx.lineTo(newBounds.x, newBounds.y + newBounds.height);
        ctx.closePath();
        ctx.stroke();
    },

    drawWithOutBottom:function(ctx){
        if (this.width <= 0 || this.height <= 0) {
            return;
        }

        var newBounds = FR.Chart.GraphHelper.dealRectShape4JSDraw(this);

        ctx.beginPath();
        ctx.moveTo(newBounds.x, newBounds.y + newBounds.height);
        ctx.lineTo(newBounds.x, newBounds.y);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y + newBounds.height);
        ctx.stroke();
    },

    drawWithOutTop:function(ctx){
        if (this.width <= 0 || this.height <= 0) {
            return;
        }

        var newBounds = FR.Chart.GraphHelper.dealRectShape4JSDraw(this);

        ctx.beginPath();
        ctx.moveTo(newBounds.x, newBounds.y );
        ctx.lineTo(newBounds.x, newBounds.y + newBounds.height);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y+ newBounds.height);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y);
        ctx.stroke();
    },

    drawWithOutRight:function(ctx){
        if (this.width <= 0 || this.height <= 0) {
            return;
        }

        var newBounds = FR.Chart.GraphHelper.dealRectShape4JSDraw(this);

        ctx.beginPath();
        ctx.moveTo(newBounds.x + newBounds.width, newBounds.y);
        ctx.lineTo(newBounds.x, newBounds.y);
        ctx.lineTo(newBounds.x, newBounds.y+ newBounds.height);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y+ newBounds.height);
        ctx.stroke();

    },

    drawWithOutLeft:function(ctx){
        if (this.width <= 0 || this.height <= 0) {
            return;
        }

        var newBounds = FR.Chart.GraphHelper.dealRectShape4JSDraw(this);

        ctx.beginPath();
        ctx.moveTo(newBounds.x, newBounds.y);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y + newBounds.height);
        ctx.lineTo(newBounds.x, newBounds.y + newBounds.height);
        ctx.stroke();
    },

    drawLeftAndRight:function(ctx){
        if (this.width <= 0 || this.height <= 0) {
            return;
        }

        var newBounds = FR.Chart.GraphHelper.dealRectShape4JSDraw(this);

        ctx.beginPath();
        ctx.moveTo(newBounds.x, newBounds.y + newBounds.height);
        ctx.lineTo(newBounds.x, newBounds.y);
        ctx.moveTo(newBounds.x + newBounds.width, newBounds.y);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y + newBounds.height);
        ctx.stroke();

    },

    drawTopAndBottom:function(ctx){
        if (this.width <= 0 || this.height <= 0) {
            return;
        }

        var newBounds = FR.Chart.GraphHelper.dealRectShape4JSDraw(this);

        ctx.beginPath();
        ctx.moveTo(newBounds.x, newBounds.y);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y);
        ctx.moveTo(newBounds.x, newBounds.y + newBounds.height);
        ctx.lineTo(newBounds.x + newBounds.width, newBounds.y + newBounds.height);
        ctx.stroke();
    },

    //系列拖拽时的外边框的要求
    drawDragBorder: function(ctx){
        if(this.isHorizontal4Bounds){

            var bounds = new FR.Chart.Bounds(this.x, this.y-3, this.width+3, this.height+6);
            if (bounds.width <= 0 || bounds.height <= 0) {
                return;
            }
            var newBounds = FR.Chart.GraphHelper.dealRectShape4JSDraw(bounds);

            ctx.beginPath();
            ctx.moveTo(newBounds.x, newBounds.y);
            ctx.lineTo(newBounds.x + newBounds.width, newBounds.y);
            ctx.lineTo(newBounds.x + newBounds.width, newBounds.y + newBounds.height);
            ctx.lineTo(newBounds.x, newBounds.y + newBounds.height);
            ctx.stroke();

        }else{

            var bounds = new FR.Chart.Bounds(this.x-3, this.y-3, this.width+6, this.height+3);
            if (bounds.width <= 0 || bounds.height <= 0) {
                return;
            }
            var newBounds = FR.Chart.GraphHelper.dealRectShape4JSDraw(bounds);

            ctx.beginPath();
            ctx.moveTo(newBounds.x, newBounds.y + newBounds.height);
            ctx.lineTo(newBounds.x, newBounds.y);
            ctx.lineTo(newBounds.x + newBounds.width, newBounds.y);
            ctx.lineTo(newBounds.x + newBounds.width, newBounds.y + newBounds.height);
            ctx.stroke();
        }


    },

    setRect: function (x, y, w, h) {
        this.x = x;
        this.y = y;
        this.width = w;
        this.height = h;
    },

    containsPoint: function (x, y) {
        var newBounds = new FR.Chart.Bounds(this.x - 2, this.y - 2, this.width + 4, this.height + 4);
        return newBounds.contains(x, y, 1, 1); // 扩大响应区域
    },

    contains: function (x, y, w, h) {
        if (this.x <= x && this.y <= y
            && ((this.x + this.width) >= (x + w))
            && ((this.y + this.height) >= (y + h))) {
            return true;
        }
        return false;
    },

    getStockAnimateShape: function (step) {
        var shape = [];
        for (var i = 0; i < step; i++) {
            var x = this.x;
            var width = this.width;
            var height = FR.Chart.ChartEase.easeInCirc(i + 1, 0, this.height, step);

            if (this.upDown < 0) {
                var y = this.y;

                shape[i] = new FR.Chart.Bounds(x, y, width, height);
            } else {
                var y = this.y + this.height - height;

                shape[i] = new FR.Chart.Bounds(x, y, width, height);
            }
        }
        return shape;
    },

    getActionChangeAnimateShape: function (step, lastShape, paintState) {
        if (paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return [];
        }
        if (paintState == FR.Chart.ChartPaintConstants.STATE_APPEAR || !lastShape) {
            return this.getActionDefaultAnimateShape(step);
        }
        var shape = [];

        var detX = parseInt(this.x - lastShape.x);
        var detY = parseInt(this.y - lastShape.y);
        var detWidth = parseInt(this.width - lastShape.width);
        var detHeight = parseInt(this.height - lastShape.height);

        for (var i = 0; i < step; i++) {
            if (this.isUnanimate) {
                // 甘特图,特殊动态,背景条不动
                shape[i] = this;
            } else {
                var x = FR.Chart.ChartEase.easeInCirc(i + 1, lastShape.x, detX , step);
                var y = FR.Chart.ChartEase.easeInCirc(i + 1, lastShape.y, detY, step);
                var width = FR.Chart.ChartEase.easeInCirc(i + 1, lastShape.width, detWidth, step);
                var height = FR.Chart.ChartEase.easeInCirc(i + 1, lastShape.height, detHeight, step);

                shape[i] = new FR.Chart.Bounds(x, y, width, height);
            }
        }
        return shape;
    },

    getActionDefaultAnimateShape: function (step) {
        var shape = [];

        for (var i = 0; i < step; i++) {
            if (this.isUnanimate) {
                // 甘特图,特殊动态,背景条不动
                shape[i] = this;
            } else if (this.is4StockPlotGlyph) {// 股价图的收盘块滑行. TODO 预留

            } else {
                // kunsnat: 用于决定画的方式. 横向增长
                if (this.isHorizontal4Bounds) {
                    var actionWidth = FR.Chart.ChartEase.easeOutQuint(i + 1, 0, this.width, step);

                    if (this.isNigativeValue !== this.axisReversed) {
                        shape[i] = new FR.Chart.Bounds(this.x - actionWidth + this.width, this.y, actionWidth, this.height);
                    } else {
                        shape[i] = new FR.Chart.Bounds(this.x, this.y, actionWidth, this.height);
                    }
                } else if (this.is4RangePlot) {
                    shape[i] = new FR.Chart.Bounds(FR.Chart.ChartEase.easeOutQuint(i + 1, 0, this.x, step), this.y, this.width, this.height);
                }
                else {
                    var actionHeight = FR.Chart.ChartEase.easeOutQuint(i + 1, 0, this.height, step);
                    // kunsnat: 判断是否0值以下, 倒立的画法
                    if (this.isNigativeValue !== this.axisReversed) {
                        shape[i] = new FR.Chart.Bounds(this.x, this.y, this.width, actionHeight);
                    } else {
                        shape[i] = new FR.Chart.Bounds(this.x, this.y + this.height - actionHeight, this.width, actionHeight);
                        // TODO
                    }
                }
            }
        }

        return shape;
    },

    // TODO actionModel 动态变化的方式
    getAnimateShape: function (step, actionModel, lastShape, paintState) {
        if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE) {
            return this.getActionChangeAnimateShape(step, lastShape, paintState);
        } else if (this.upDown) {
            // 股价图根据涨跌动态不同
            return this.getStockAnimateShape(step);
        } else {
            return this.getActionDefaultAnimateShape(step);
        }
    },

    largerThan:function(bounds){
        if(bounds.width && bounds.height){
            return this.width * this.height > bounds.width * bounds.height;
        }
    }
});
// Bounds.end

// RoundBounds.start
FR.Chart.RoundBounds = function (x, y, width, height, arcWidth, arcHeight) {
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
    this.arcWidth = arcWidth;
    this.arcHeight = arcHeight;
};
$.extend(FR.Chart.RoundBounds.prototype, {
    x: 0,
    y: 0,
    width: 0,
    height: 0,
    arcWidth: 0,
    arcHeight: 0,

    paint: function (ctx) {
        if (this.width === 0 || this.height === 0) {
            return;
        }

        this.roundPath(ctx);
        ctx.fill();
    },

    draw: function (ctx) {
        if (this.width === 0 || this.height === 0) {
            return;
        }

        this.roundPath(ctx);
        ctx.stroke();
    },

    containsPoint: function (x, y) {
        var newBounds = new FR.Chart.Bounds(this.x - 2, this.y - 2, this.width + 4, this.height + 4);
        return newBounds.contains(x, y, 1, 1); // 扩大响应区域
    },

    contains: function (x, y, w, h) {
        if (this.x <= x && this.y <= y
            && ((this.x + this.width) >= (x + w))
            && ((this.y + this.height) >= (y + h))) {
            return true;
        }
        return false;
    },

    roundPath: function (ctx) {
        var x = Math.round(this.x), y = Math.round(this.y),
            height = Math.round(this.height), width = Math.round(this.width);

        ctx.beginPath();
        ctx.moveTo(x, y + this.arcHeight);
        ctx.lineTo(x, y + height - this.arcHeight);
        ctx.quadraticCurveTo(x, y + height, x + this.arcWidth, y + height);
        ctx.lineTo(x + width - this.arcWidth, y + height);
        ctx.quadraticCurveTo(x + width, y + height, x + width, y + height - this.arcHeight);
        ctx.lineTo(x + width, y + this.arcHeight);
        ctx.quadraticCurveTo(x + width, y, x + width - this.arcWidth, y);
        ctx.lineTo(x + this.arcWidth, y);
        ctx.quadraticCurveTo(x, y, x, y + this.arcHeight);
    },

    drawRightBottomShadow:function(ctx){
        var x = Math.round(this.x), y = Math.round(this.y),
            height = Math.round(this.height), width = Math.round(this.width);

        ctx.beginPath();
        ctx.moveTo(x, y + height - this.arcHeight);
        ctx.quadraticCurveTo(x, y + height, x + this.arcWidth, y + height);
        ctx.lineTo(x + width - this.arcWidth, y + height);
        ctx.quadraticCurveTo(x + width, y + height, x + width, y + height - this.arcHeight);
        ctx.lineTo(x + width, y + this.arcHeight);
        ctx.quadraticCurveTo(x + width, y, x + width - this.arcWidth, y);

        ctx.stroke();
    },

    getBounds2D: function () {
        return new FR.Chart.Bounds(this.x, this.y, this.width, this.height);
    },

    isNeedRadialGradient:function(){
        return false;
    }

});
// RoundBounds.end

// Marker.start
FR.Chart.Marker = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (js) {
        js = js || {};

        this.size = FR.pick(js.size, 4.5);
        this.gapSize = 2.0;
        this.color = js.color;
        this.background = FR.Chart.AbstractBackground.initBackground(js.background);
        this.x = 0;
        this.y = 0;
    },


    paintBackgroundBiggerMarker:function(ctx,x,y){
        if (this.background && !isNaN(x) && !isNaN(y)) {
            this.paintBackroungMarker(ctx,x,y);
            //画完记录一下x,y吧
            this.x = x;
            this.y = y;
        }
    },

    paintBackroungMarker:function(ctx,x,y){
        this.paintMarker(ctx,x,y);
    },

    paint: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y)) {
            if(!ctx){     return; } ctx.save();
            ctx.lineWidth = 2;
            this.paintMarker(ctx, x, y);
            ctx.restore();

            //画完记录一下x,y吧
            this.x = x;
            this.y = y;
        }
    },

    getMarkerType: function () {
        return "Maker";
    },

    drawDragBorder:function(ctx){
        ctx.lineWidth = 2;
        this.paintMarker(ctx, this.x, this.y);
    }

});
$.extend(FR.Chart.Marker, {
    RoundMarker: "RoundMarker",
    RoundFilledMarker: "RoundFilledMarker",
    SquareFilledMarker: "SquareFilledMarker",
    TriangleFilledMarker: "TriangleFilledMarker",
    DiamondFilledMarker: "DiamondFilledMarker",
    CrossMarker: "CrossMarker",
    PlusSignMarker: "PlusSignMarker",
    CircleFilledMarker: "CircleFilledMarker",
    MinusSignMarker: "MinusSignMarker",
    DiamondMarker: "DiamondMarker",
    SquareMarker: "SquareMarker",
    TriangleMarker: "TriangleMarker",
    CircleMarker: "CircleMarker",
    AllEqualMarker: "AllEqualMarker",
    ClockLineMarker: "ClockLineMarker",
    AntiClockLineMarker: "AntiClockLineMarker",
    StarMarker: "StarMarker",
    XMarker: "XMarker",
    NullMarker: "NullMarker",
    PointMarker: "PointMarker",
    DowJonesMarker: "DowJonesMarker"
});
// Marker.end

// AntiClockLineMarker.start
FR.Chart.AntiClockLineMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.AntiClockLineMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.AntiClockLineMarker();
    },

    paintMarker: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            var size = this.size;

            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - size * 0.5, y - size), new FR.Chart.Point2D(x + size, y + size * 0.5)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x, y - size), new FR.Chart.Point2D(x + size, y)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - size, y - size), new FR.Chart.Point2D(x + size, y + size)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x + size * 0.5, y - size), new FR.Chart.Point2D(x + size, y - size * 0.5)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - size, y - size * 0.5), new FR.Chart.Point2D(x + size * 0.5, y + size)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - size, y), new FR.Chart.Point2D(x, y + size)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - size, y + size * 0.5), new FR.Chart.Point2D(x - size * 0.5, y + size)));
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.AntiClockLineMarker;
    }
});
// AntiClockLineMarker.end

// ClockLineMarker.start
FR.Chart.ClockLineMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.ClockLineMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.ClockLineMarker();
    },

    paintMarker: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            var size = this.size;

            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - size, y - size * 0.5), new FR.Chart.Point2D(x - size * 0.5, y - size)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - size, y), new FR.Chart.Point2D(x, y - size)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - size, y + size * 0.5), new FR.Chart.Point2D(x + size * 0.5, y - size)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - size, y + size), new FR.Chart.Point2D(x + size, y - size)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - size * 0.5, y + size), new FR.Chart.Point2D(x + size, y - size * 0.5)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x, y + size), new FR.Chart.Point2D(x + size, y)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x + size * 0.5, y + size), new FR.Chart.Point2D(x + size, y + size * 0.5)));
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.ClockLineMarker;
    }
});
// ClockLineMarker.end

// PointMarker.start
FR.Chart.PointMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        js = js || {};

        FR.Chart.PointMarker.superclass.initConstants.apply(this, [js]);
    },

    paintMarker: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            var size = this.size;
            var pointNumber = 4;
            for (var i = 0; i < pointNumber; i++) {
                for (var j = 0; j < pointNumber; j++) {
                    var x1 = x - size + 2 * size * i / pointNumber;
                    var y1 = y - size + 2 * size * j / pointNumber;
                    this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x1, y1), new FR.Chart.Point2D(x1, y1)));
                }
            }
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.RoundMarker;
    },

    drawDragBorder:function(ctx){

    }
});
// PointMarker.end

// AllEqualMarker.start
FR.Chart.AllEqualMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.AllEqualMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.AllEqualMarker();
    },

    paintMarker: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - this.size, y - this.size),
                new FR.Chart.Point2D(x + this.size, y - this.size)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - this.size, y),
                new FR.Chart.Point2D(x + this.size, y)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - this.size, y + this.size),
                new FR.Chart.Point2D(x + this.size, y + this.size)));
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.AllEqualMarker;
    }
});
// AllEqualMarker.end

// MinusSignMarker.start
FR.Chart.MinusSignMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.MinusSignMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.MinusSignMarker();
    },

    paintMarker: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - this.size, y - this.size / 2),
                new FR.Chart.Point2D(x + this.size, y - this.size / 2)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - this.size, y + this.size / 2),
                new FR.Chart.Point2D(x + this.size, y + this.size / 2)));
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.MinusSignMarker;
    }
});
// MinusSignMarker.end

// CircleFilledMarker.start
FR.Chart.CircleFilledMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.CircleFilledMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.CircleFilledMarker();
    },

    paintMarker: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.paint(ctx, new FR.Chart.Arc2D(x, y, this.size / 2, 0, 360, FR.Chart.Arc2D.CHORD));
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.CircleFilledMarker;
    },

    drawDragBorder:function(ctx){
        var arc = new FR.Chart.Arc2D(this.x, this.y, this.size / 2 + 3, 0, 360, FR.Chart.Arc2D.CHORD);
        arc.draw(ctx);
    }
});
// CircleFilledMarker.end

// CircleMarker.start
FR.Chart.CircleMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.CircleMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.CircleMarker();
    },

    paintMarker: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.draw(ctx, new FR.Chart.Arc2D(x, y, this.size / 2, 0, 360, FR.Chart.Arc2D.CHORD));
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.CircleMarker;
    },

    drawDragBorder:function(ctx){
        var arc = new FR.Chart.Arc2D(this.x, this.y, this.size / 2 + 3, 0, 360, FR.Chart.Arc2D.CHORD);
        arc.draw(ctx);
    }
});
// CircleMarker.end

// RoundMarker.start
FR.Chart.RoundMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.RoundMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.RoundMarker();
    },

    paintMarker: function (ctx, x, y) {
        var isNeedPaint = this.background && !isNaN(x) && !isNaN(y) && this.size;
        if (!isNeedPaint) {
            return;
        }

        this.background.draw(ctx, new FR.Chart.Arc2D(x, y, this.size, 0, 360, FR.Chart.Arc2D.CHORD));
    },

    paintBackroungMarker:function(ctx,x,y){
        var isNeedPaint = this.background && !isNaN(x) && !isNaN(y) && this.size;
        if (!isNeedPaint) {
            return;
        }

        this.background.paint(ctx, new FR.Chart.Arc2D(x, y, this.size, 0, 360, FR.Chart.Arc2D.CHORD));
    },

    getMarkerType: function () {
        return FR.Chart.Marker.RoundMarker;
    },

    drawDragBorder:function(ctx){
        var arc = new FR.Chart.Arc2D(this.x, this.y, this.size + 3, 0, 360, FR.Chart.Arc2D.CHORD);
        arc.draw(ctx);
    }
});
// RoundMarker.end

// RoundFilledMarker.start
FR.Chart.RoundFilledMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.RoundFilledMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.RoundFilledMarker();
    },

    paintMarker: function (ctx, x, y) {

        if (this.background && !isNaN(x) && !isNaN(y) && this.size){
            this.background.paint(ctx, new FR.Chart.Arc2D(x, y, this.size, 0, 360, FR.Chart.Arc2D.CHORD));
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.RoundFilledMarker;
    },

    drawDragBorder:function(ctx){
        var arc = new FR.Chart.Arc2D(this.x, this.y, this.size + 3, 0, 360, FR.Chart.Arc2D.CHORD);
        arc.draw(ctx);
    }
});
// RoundFilledMarker.end

// CrossMarker.start
FR.Chart.CrossMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.CrossMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.CrossMarker();
    },

    paintMarker: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - this.size, y), new FR.Chart.Point2D(x + this.size, y)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x, y + this.size), new FR.Chart.Point2D(x, y - this.size)));
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.CrossMarker;
    }
});
// CrossMarker.end

// DiamondFilledMarker.start
FR.Chart.DiamondFilledMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.DiamondFilledMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.DiamondFilledMarker();
    },

    paintMarker: function (ctx, x, y) {

        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.paint(ctx, this.getPath(x,y,this.size));
        }
    },


    getPath:function(x,y,size){
        var px = [x - size, x, x + size, x];
        var py = [y, y - size, y, y + size];

        var path = new FR.Chart.GeneralPath();

        path.moveTo(px[px.length - 1], py[px.length - 1]);
        for (var i = 0; i < px.length; i++) {
            path.lineTo(px[i], py[i]);
        }
        return path;
    },

    getMarkerType: function () {
        return FR.Chart.Marker.DiamondFilledMarker;
    },

    drawDragBorder:function(ctx){
        var px = [this.x - this.size -3*Math.sqrt(2), this.x, this.x + this.size + 3*Math.sqrt(2), this.x];
        var py = [this.y, this.y - this.size - 3*Math.sqrt(2), this.y, this.y + this.size + 3*Math.sqrt(2)];

        var path = new FR.Chart.GeneralPath();

        path.moveTo(px[px.length - 1], py[px.length - 1]);
        for (var i = 0; i < px.length; i++) {
            path.lineTo(px[i], py[i]);
        }

        path.draw(ctx);
    }
});
// DiamondFilledMarker.end

// DiamondMarker.start
FR.Chart.DiamondMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.DiamondMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.DiamondMarker();
    },

    paintMarker: function (ctx, x, y) {
        var isNeedPaint = this.background && !isNaN(x) && !isNaN(y) && this.size;
        if (!isNeedPaint) {
            return;
        }
        this.background.draw(ctx, this.getPath(x,y,this.size));

    },


    paintBackroungMarker:function(ctx,x,y){
        var isNeedPaint = this.background && !isNaN(x) && !isNaN(y) && this.size;
        if (!isNeedPaint) {
            return;
        }
        this.background.paint(ctx, this.getPath(x,y,this.size));
    },


    getPath:function(x,y,size){
        var px = [x - size, x, x + size, x];
        var py = [y, y - size, y, y + size];

        var path = new FR.Chart.GeneralPath();

        path.moveTo(px[px.length - 1], py[px.length - 1]);
        for (var i = 0; i < px.length; i++) {
            path.lineTo(px[i], py[i]);
        }
        return path;
    },

    getMarkerType: function () {
        return FR.Chart.Marker.DiamondMarker;
    },

    drawDragBorder:function(ctx){
        var px = [this.x - this.size -3*Math.sqrt(2), this.x, this.x + this.size + 3*Math.sqrt(2), this.x];
        var py = [this.y, this.y - this.size - 3*Math.sqrt(2), this.y, this.y + this.size + 3*Math.sqrt(2)];

        var path = new FR.Chart.GeneralPath();

        path.moveTo(px[px.length - 1], py[px.length - 1]);
        for (var i = 0; i < px.length; i++) {
            path.lineTo(px[i], py[i]);
        }

        path.draw(ctx);
    }
});
// DiamondMarker.end

// DowJonesMarker.start
FR.Chart.DowJonesMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.DowJonesMarker.superclass.initConstants.apply(this, [js]);
    },

    paintMarker: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.size = this.size / 2;

            ctx.lineWidth = FR.Chart.Constants.LINE_THIN;

            var path = new FR.Chart.GeneralPath();
            path.moveTo(x, y);
            path.lineTo(x + this.size, y);

            this.background.draw(ctx, path);
        }

    },

    getMarkerType: function () {
        return FR.Chart.Marker.RoundMarker;
    }
});
// DowJonesMarker.end

// NullMarker.start
FR.Chart.NullMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.NullMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.NullMarker();
    },

    paintMarker: function (ctx, x, y) {

    },

    getMarkerType: function () {
        return FR.Chart.Marker.NullMarker;
    },

    drawDragBorder:function(ctx){
        var bounds = new FR.Chart.Bounds(this.x - 4, this.y - 4, 8, 8);
        bounds.draw(ctx);
    }

});
// NullMarker.end

// PlusSignMarker.start
FR.Chart.PlusSignMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.PlusSignMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.PlusSignMarker();
    },

    paintMarker: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - this.size, y + this.size), new FR.Chart.Point2D(x + this.size, y + this.size)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x, y + this.size), new FR.Chart.Point2D(x, y - this.size)));
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.PlusSignMarker;
    }
});
// PlusSignMarker.end

// SquareFilledMarker.start
FR.Chart.SquareFilledMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.SquareFilledMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.SquareFilledMarker();
    },

    paintMarker: function (ctx, x, y) {

        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.paint(ctx, new FR.Chart.Bounds(x - 0.75 * this.size, y - 0.75 * this.size, 1.5 * this.size, 1.5 * this.size));
        }
    },


    paintBackroungMarker:function(ctx,x,y){
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.paint(ctx, new FR.Chart.Bounds(x - 0.75 *(this.size-2)-2, y - 0.75 *(this.size-2)-2, 1.5 *(this.size-2)+4, 1.5 *(this.size-2)+4));
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.SquareFilledMarker;
    },

    drawDragBorder:function(ctx){
        var bounds = new FR.Chart.Bounds(this.x - 0.75 * this.size - 3, this.y - 0.75 * this.size - 3,
            1.5 * this.size + 6, 1.5 * this.size + 6);
        bounds.draw(ctx);
    }

});
// SquareFilledMarker.end

// SquareMarker.start
FR.Chart.SquareMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.SquareMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.SquareMarker();
    },

    paintMarker: function (ctx, x, y) {
        var isNeedPaint =this.background && !isNaN(x) && !isNaN(y) && this.size;
        if (!isNeedPaint) {
            return ;
        }

        this.background.draw(ctx, new FR.Chart.Bounds(x - 0.75 * this.size, y - 0.75 * this.size, 1.5 * this.size, 1.5 * this.size));
    },

    paintBackroungMarker:function(ctx,x,y){
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.paint(ctx, new FR.Chart.Bounds(x - 0.75 *(this.size-2)-2, y - 0.75 *(this.size-2)-2, 1.5 *(this.size-2)+4, 1.5 *(this.size-2)+4));
        }
    },


    getMarkerType: function () {
        return FR.Chart.Marker.SquareMarker;
    },

    drawDragBorder:function(ctx){
        var bounds = new FR.Chart.Bounds(this.x - 0.75 * this.size - 3, this.y - 0.75 * this.size - 3,
            1.5 * this.size + 6, 1.5 * this.size + 6);
        bounds.draw(ctx);
    }
});
// SquareMarker.end

// StarMarker.start
FR.Chart.StarMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.StarMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.StarMarker();
    },

    paintMarker: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - this.size, y),
                new FR.Chart.Point2D(x + this.size, y)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x, y + this.size),
                new FR.Chart.Point2D(x, y - this.size)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - this.size, y - this.size),
                new FR.Chart.Point2D(x + this.size, y + this.size)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - this.size, y + this.size),
                new FR.Chart.Point2D(x + this.size, y - this.size)));
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.StarMarker;
    }
});
// StarMarker.end

// TriangleFilledMarker.start
FR.Chart.TriangleFilledMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.TriangleFilledMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.TriangleFilledMarker();
    },

    paintMarker: function (ctx, x, y) {

        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.paint(ctx, this.getPath(x,y,this.size));
        }
    },

    getPath:function(x,y,size){
        var px = [x - size, x + size, x];
        var py = [y + size, y + size, y -size];

        var path = new FR.Chart.GeneralPath();

        path.moveTo(px[px.length - 1], py[py.length - 1]);
        for (var i = 0; i < px.length; i++) {
            path.lineTo(px[i], py[i]);
        }
        return path;
    },

    getMarkerType: function () {
        return FR.Chart.Marker.TriangleFilledMarker;
    },

    drawDragBorder:function(ctx){
        var px = [this.x - this.size - 3 * Math.sqrt(3), this.x + this.size + 3 * Math.sqrt(3), this.x];
        var py = [this.y + this.size + 3, this.y + this.size + 3, this.y - this.size - 6];

        var path = new FR.Chart.GeneralPath();

        path.moveTo(px[px.length - 1], py[py.length - 1]);
        for (var i = 0; i < px.length; i++) {
            path.lineTo(px[i], py[i]);
        }

        path.draw(ctx);
    }
});
// TriangleFilledMarker.end

// TriangleMarker.start
FR.Chart.TriangleMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.TriangleMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.TriangleMarker();
    },

    paintMarker: function (ctx, x, y) {
        var isNeedPaint = this.background && !isNaN(x) && !isNaN(y) && this.size;
        if (!isNeedPaint) {
            return ;
        }
        this.background.draw(ctx, this.getPath(x,y,this.size));
    },

    paintBackroungMarker:function(ctx,x,y){
        var isNeedPaint = this.background && !isNaN(x) && !isNaN(y) && this.size;
        if (!isNeedPaint) {
            return ;
        }
        this.background.paint(ctx, this.getPath(x,y,this.size));
    },


    getPath:function(x,y,size){
        var px = [x - size, x + size, x];
        var py = [y + size, y + size, y -size];

        var path = new FR.Chart.GeneralPath();

        path.moveTo(px[px.length - 1], py[py.length - 1]);
        for (var i = 0; i < px.length; i++) {
            path.lineTo(px[i], py[i]);
        }
        return path;
    },

    getMarkerType: function () {
        return FR.Chart.Marker.TriangleMarker;
    },

    drawDragBorder:function(ctx){
        var px = [this.x - this.size - 3 * Math.sqrt(3), this.x + this.size + 3 * Math.sqrt(3), this.x];
        var py = [this.y + this.size + 3, this.y + this.size + 3, this.y - this.size - 6];

        var path = new FR.Chart.GeneralPath();

        path.moveTo(px[px.length - 1], py[py.length - 1]);
        for (var i = 0; i < px.length; i++) {
            path.lineTo(px[i], py[i]);
        }

        path.draw(ctx);
    }
});
// TriangleMarker.end

// XMarker.start
FR.Chart.XMarker = FR.extend(FR.Chart.Marker, {
    initConstants: function (js) {
        FR.Chart.XMarker.superclass.initConstants.apply(this, [js]);
    },

    clone: function () {
        return new FR.Chart.XMarker();
    },

    paintMarker: function (ctx, x, y) {
        if (this.background && !isNaN(x) && !isNaN(y) && this.size) {
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - this.size, y - this.size),
                new FR.Chart.Point2D(x + this.size, y + this.size)));
            this.background.draw(ctx, new FR.Chart.Line2D(new FR.Chart.Point2D(x - this.size, y + this.size),
                new FR.Chart.Point2D(x + this.size, y - this.size)));
        }
    },

    getMarkerType: function () {
        return FR.Chart.Marker.XMarker;
    }
});
// XMarker.end

// GeneralPath.start
FR.Chart.GeneralPath = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (js) {
        this.x = [];
        this.y = [];

        // kunsnat: 记录是否是起始点, 为linePlot中可能有断点做准备
        // 属性只做内部处理应用.
        this.type = [];

        // 曲线记录贝塞尔控制点.二次贝塞尔里为一个point2D,三次贝塞尔为point2D的双值数组,其他为null
        this.ctrl = [];

        js = js || {};

        if (js.x) {
            for (var i = 0; i < js.x.length; i++) {
                if (js.type[i] == FR.Chart.GeneralPath.MOVE_TO) {
                    this.moveTo(js.x[i], js.y[i]);
                } else if (js.type[i] == FR.Chart.GeneralPath.LINE_TO) {
                    this.lineTo(js.x[i], js.y[i]);
                } else if (js.type[i] == FR.Chart.GeneralPath.QUAD_TO) {
                    this.quadTo(js.ctrl[i].x, js.ctrl[i].y, js.x[i], js.y[i]);
                } else if (js.type[i] == FR.Chart.GeneralPath.CURVE_TO) {
                    this.curveTo(js.ctrl[i].x0, js.ctrl[i].y0,
                        js.ctrl[i].x1, js.ctrl[i].y1,
                        js.x[i], js.y[i]);
                } else if (js.type[i] == FR.Chart.GeneralPath.CLOSE) {
                    this.closePath();
                }
            }
        }
    },

    transform: function (at) {
        var newPath = new FR.Chart.GeneralPath();

        for (var i = 0; i < this.x.length; i++) {
            if (this.type[i] == FR.Chart.GeneralPath.MOVE_TO) {
                var ptDst = at.transform({x: this.x[i], y: this.y[i]});
                newPath.moveTo(ptDst.x, ptDst.y);
            } else if (this.type[i] == FR.Chart.GeneralPath.LINE_TO) {
                var ptDst = at.transform({x: this.x[i], y: this.y[i]});
                newPath.lineTo(ptDst.x, ptDst.y);
            } else if (this.type[i] == FR.Chart.GeneralPath.QUAD_TO) {
                var ptDst = at.transform({x: this.x[i], y: this.y[i]});
                var ctrl = at.transform({x: this.ctrl[i].x, y: this.ctrl[i].y});
                newPath.quadTo(ctrl.x, ctrl.y, ptDst.x, ptDst.y);
            } else if (this.type[i] == FR.Chart.GeneralPath.CURVE_TO) {
                var ptDst = at.transform({x: this.x[i], y: this.y[i]});
                var ctrl0 = at.transform({x: this.ctrl[i][0].x, y: this.ctrl[i][0].y});
                var ctrl1 = at.transform({x: this.ctrl[i][1].x, y: this.ctrl[i][1].y});
                newPath.curveTo(ctrl0.x, ctrl0.y,
                    ctrl1.x, ctrl1.y,
                    ptDst.x, ptDst.y);
            } else if (this.type[i] == FR.Chart.GeneralPath.CLOSE) {
                newPath.closePath();
            }
        }

        return newPath;
    },

    //  拆分成两点之间. 支持虚线
    //kunsnat: 性能改进, 应该改变 画法:  直接一个beginPath() 完整的画出路径.
    draw: function (ctx, lineStyle) {
        if(this.x.length <= 0) {
            return;
        }

        if(lineStyle) {
            ctx.lineWidth = FR.Chart.GraphHelper.getLineStyleSize(lineStyle);
        }
        ctx.beginPath();
        for (var i = 0; i < this.x.length; i++) {
            if (i + 1 < this.x.length) {
                if (this.type[i + 1] == FR.Chart.GeneralPath.LINE_TO
                    || this.type[i + 1] == FR.Chart.GeneralPath.CLOSE) {

                    ctx.moveTo(this.x[i], this.y[i]);
                    ctx.lineTo(this.x[i + 1], this.y[i + 1]);
                } else if (this.type[i + 1] == FR.Chart.GeneralPath.QUAD_TO) {
                    ctx.moveTo(this.x[i], this.y[i]);
                    ctx.quadraticCurveTo(this.ctrl[i + 1].x, this.ctrl[i + 1].y,
                        this.x[i + 1], this.y[i + 1]);
                } else if (this.type[i + 1] == FR.Chart.GeneralPath.CURVE_TO) {
                    ctx.moveTo(this.x[i], this.y[i]);
                    ctx.bezierCurveTo(this.ctrl[i + 1][0].x, this.ctrl[i + 1][0].y,
                        this.ctrl[i + 1][1].x, this.ctrl[i + 1][1].y,
                        this.x[i + 1], this.y[i + 1]);
                }
            }
        }
        ctx.stroke();
    },

    drawDragBorder: function(ctx){

    },

    paint: function (ctx) {
        ctx.beginPath();
        for (var i = 0; i < this.x.length; i++) {
            if (this.type[i] == FR.Chart.GeneralPath.MOVE_TO || i === 0) {// kunsnat: IE下需要严谨的起始点位置, 所有必须i==0时有个判断. 37393
                ctx.moveTo(this.x[i], this.y[i]);
            } else if (this.type[i] == FR.Chart.GeneralPath.LINE_TO
                || this.type[i] == FR.Chart.GeneralPath.CLOSE) {
                ctx.lineTo(this.x[i], this.y[i]);
            } else if (this.type[i] == FR.Chart.GeneralPath.QUAD_TO) {
                ctx.quadraticCurveTo(this.ctrl[i].x, this.ctrl[i].y,
                    this.x[i], this.y[i]);
            } else if (this.type[i] == FR.Chart.GeneralPath.CURVE_TO) {
                ctx.bezierCurveTo(this.ctrl[i][0].x, this.ctrl[i][0].y,
                    this.ctrl[i][1].x, this.ctrl[i][1].y,
                    this.x[i], this.y[i]);
            }
        }
        ctx.fill();
    },

    // 用于Area图 拆分
    getAnimateShape: function (step, actionModel, lastShape, paintState) {
        // 雷达图动态
        if (this.centerPoint) {
            return this.getAnimateRadarPath(step, paintState);
        }
        if (paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return [];
        }
        var pathArray = [];

        for (var actionStep = 0; actionStep < step; actionStep++) {

            var generalPath = new FR.Chart.GeneralPath();
            var curvePath = new FR.Chart.GeneralPath();
            pathArray[actionStep] = generalPath;

            var pathLength = this.x.length;
            if (this.type[pathLength - 1] == FR.Chart.GeneralPath.CLOSE) {
                pathLength--;
            }

            for (var i = 0; i < pathLength; i++) {
                if (i < pathLength / 2) {
                    var x1 = this.x[i];
                    var y1 = this.y[i];
                    var x2 = this.x[pathLength - 1 - i];
                    var y2 = this.y[pathLength - 1 - i];
                    if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE && lastShape &&
                        lastShape.x[i] && lastShape.y[i]) {
                        x2 = lastShape.x[i];
                        y2 = lastShape.y[i];
                    }
                    var x = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, x2, x1 - x2, step);
                    var y = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, y2, y1 - y2, step);

                    if (this.type[i] == FR.Chart.GeneralPath.MOVE_TO) {
                        FR.Chart.FoldLine.curveTo(generalPath, curvePath);
                        curvePath.clear();
                        generalPath.moveTo(x, y);
                    } else if (this.type[i] == FR.Chart.GeneralPath.LINE_TO) {
                        FR.Chart.FoldLine.curveTo(generalPath, curvePath);
                        curvePath.clear();
                        generalPath.lineTo(x, y);
                    } else if (this.type[i] == FR.Chart.GeneralPath.CLOSE) {
                        FR.Chart.FoldLine.curveTo(generalPath, curvePath);
                        curvePath.clear();
                        generalPath.closePath();
                    } else if (this.type[i] == FR.Chart.GeneralPath.QUAD_TO) {
                        if (curvePath.x.length === 0) {
                            curvePath.moveTo(generalPath.x[generalPath.x.length - 1], generalPath.y[generalPath.y.length - 1]);
                        }
                        curvePath.lineTo(x, y);
                    } else if (this.type[i] === FR.Chart.GeneralPath.CURVE_TO) {
                        if (curvePath.x.length === 0) {
                            curvePath.moveTo(generalPath.x[generalPath.x.length - 1], generalPath.y[generalPath.y.length - 1]);
                        }
                        curvePath.lineTo(x, y);
                    }
                } else {
                    FR.Chart.FoldLine.curveTo(generalPath, curvePath);
                    curvePath.clear();
                    var x = this.x[i];
                    var y = this.y[i];
                    if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE && lastShape &&
                        lastShape.x[i] && lastShape.y[i]) {
                        x = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, lastShape.x[i], x - lastShape.x[i], step);
                        y = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, lastShape.y[i], y - lastShape.y[i], step);
                    }
                    if (this.type[i] == FR.Chart.GeneralPath.LINE_TO) {
                        generalPath.lineTo(x, y);
                    } else if (this.type[i] == FR.Chart.GeneralPath.CLOSE) {
                        generalPath.closePath();
                    } else if (this.type[i] == FR.Chart.GeneralPath.QUAD_TO) {
                        var c = {x: this.ctrl[i].x,
                            y: this.ctrl[i].y};
                        if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE && lastShape) {
                            c.x = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, lastShape.ctrl[i].x, this.ctrl[i].x - lastShape.ctrl[i].x, step);
                            c.y = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, lastShape.ctrl[i].y, this.ctrl[i].y - lastShape.ctrl[i].y, step);
                        }
                        generalPath.quadTo(c.x, c.y, x, y);
                    } else if (this.type[i] == FR.Chart.GeneralPath.CURVE_TO) {
                        var c0 = {x: this.ctrl[i][0].x,
                            y: this.ctrl[i][0].y};
                        var c1 = {x: this.ctrl[i][1].x,
                            y: this.ctrl[i][1].y};
                        if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE && lastShape) {
                            c0.x = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, lastShape.ctrl[i][0].x, this.ctrl[i][0].x - lastShape.ctrl[i][0].x, step);
                            c0.y = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, lastShape.ctrl[i][0].y, this.ctrl[i][0].y - lastShape.ctrl[i][0].y, step);
                            c1.x = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, lastShape.ctrl[i][1].x, this.ctrl[i][1].x - lastShape.ctrl[i][1].x, step);
                            c1.y = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, lastShape.ctrl[i][1].y, this.ctrl[i][1].y - lastShape.ctrl[i][1].y, step);
                        }
                        generalPath.curveTo(c0.x, c0.y, c1.x, c1.y, x, y);
                    }
                }
            }
        }

        return pathArray;
    },

    // 用于Area图 拆分上边缘
    getTopFoldLineAnimateShape: function (step, actionModel, lastShape, paintState) {
        var pathArray = [];
        for (var actionStep = 0; actionStep < step; actionStep++) {

            var generalPath = new FR.Chart.GeneralPath();
            var curvePath = new FR.Chart.GeneralPath();
            pathArray[actionStep] = generalPath;

            var pathLength = this.x.length;
            if (this.type[pathLength - 1] == FR.Chart.GeneralPath.CLOSE) {
                pathLength--;
            }

            for (var i = 0; i < pathLength / 2; i++) {
                var x1 = this.x[i];
                var y1 = this.y[i];
                var x2 = this.x[pathLength - 1 - i];
                var y2 = this.y[pathLength - 1 - i];
                if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE && lastShape && lastShape.x[i] && lastShape.y[i]) {
                    x2 = lastShape.x[i];
                    y2 = lastShape.y[i];
                }
                var x = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, x2, x1 - x2, step);
                var y = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, y2, y1 - y2, step);

                if (this.type[i] == FR.Chart.GeneralPath.MOVE_TO) {
                    FR.Chart.FoldLine.curveTo(generalPath, curvePath);
                    curvePath.clear();
                    generalPath.moveTo(x, y);
                } else if (this.type[i] == FR.Chart.GeneralPath.LINE_TO) {
                    FR.Chart.FoldLine.curveTo(generalPath, curvePath);
                    curvePath.clear();
                    generalPath.lineTo(x, y);
                } else if (this.type[i] == FR.Chart.GeneralPath.CLOSE) {
                    FR.Chart.FoldLine.curveTo(generalPath, curvePath);
                    curvePath.clear();
                    generalPath.closePath();
                } else if (this.type[i] == FR.Chart.GeneralPath.QUAD_TO) {
                    if (curvePath.x.length === 0) {
                        curvePath.moveTo(generalPath.x[generalPath.x.length - 1], generalPath.y[generalPath.y.length - 1]);
                    }
                    curvePath.lineTo(x, y);
                } else if (this.type[i] == FR.Chart.GeneralPath.CURVE_TO) {
                    if (curvePath.x.length === 0) {
                        curvePath.moveTo(generalPath.x[generalPath.x.length - 1], generalPath.y[generalPath.y.length - 1]);
                    }
                    curvePath.lineTo(x, y);
                }
            }

            FR.Chart.FoldLine.curveTo(generalPath, curvePath);
            curvePath.clear();
        }

        return pathArray;
    },

    // 折线拆分.从0值开始 起落那种 传参数 Y轴中间值的坐标y, (用于折线, 散点)
    getAnimatePath: function (step, actionModel, lastShape, paintState) {
        // 雷达图动态
        if (this.centerPoint) {
            return this.getAnimateRadarPath(step, paintState);
        }

        if (paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return [];
        }

        if (this.isXYScatter) {
            return this.getAnimateXYScatterPath(step, actionModel, lastShape, paintState);
        }

        // 股价图高低线
        if (this.upDown) {
            return this.getAnimateHighLowLinePath(step);
        }

        // 折线图
        if (!this.yZero) {
            return this.getAnimateLinePath(step, actionModel, lastShape, paintState);
        }

        var pathArray = [];

        for (var actionStep = 0; actionStep < step; actionStep++) {
            var path = new FR.Chart.GeneralPath();

            for (var i = 0; i < this.x.length; i++) {
                var newY = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, this.yZero, this.y[i] - this.yZero, step);
                if (this.type[i] == FR.Chart.GeneralPath.MOVE_TO) {
                    path.moveTo(this.x[i], newY);
                } else if (this.type[i] == FR.Chart.GeneralPath.LINE_TO) {
                    path.lineTo(this.x[i], newY);
                } else if (this.type[i] == FR.Chart.GeneralPath.CLOSE) {
                    path.closePath();
                } else if (this.type[i] == FR.Chart.GeneralPath.QUAD_TO) {
                    path.quadTo(this.ctrl[i].x, FR.Chart.ChartEase.easeOutQuint(actionStep + 1, this.yZero, this.ctrl[i].y - this.yZero, step),
                        this.x[i], newY);
                } else if (this.type[i] == FR.Chart.GeneralPath.CURVE_TO) {
                    path.curveTo(this.ctrl[i][0].x, FR.Chart.ChartEase.easeOutQuint(actionStep + 1, this.yZero, this.ctrl[i][0].y - this.yZero, step),
                        this.ctrl[i][1].x, FR.Chart.ChartEase.easeOutQuint(actionStep + 1, this.yZero, this.ctrl[i][1].y - this.yZero, step),
                        this.x[i], newY);
                }
            }

            pathArray[actionStep] = path;
        }

        return pathArray;
    },

    // 散点图折线
    getAnimateXYScatterPath: function (step, actionModel, lastShape, paintState) {
        if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE && lastShape
            && paintState == FR.Chart.ChartPaintConstants.STATE_ENABLE) {
            return this.getAnimateLinePath(step, actionModel, lastShape, paintState);
        }
        var pathArray = [];

        for (var actionStep = 0; actionStep < step; actionStep++) {
            var path = new FR.Chart.GeneralPath();

            if (paintState == FR.Chart.ChartPaintConstants.STATE_ENABLE && actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE) {
                pathArray[actionStep] = this;
            } else if (paintState == FR.Chart.ChartPaintConstants.STATE_DISAPPEAR) {
                pathArray[actionStep] = path;
            } else {
                pathArray[actionStep] = path;
                if (actionStep == step - 1) {
                    pathArray[actionStep] = this;
                }
            }
        }

        return pathArray;
    },

    // 折线图动态
    getAnimateLinePath: function (step, actionModel, lastShape, paintState) {
        var pathArray = [];

        var lastShpeUsable = lastShape && lastShape.x.length>0 && lastShape.x.length > 0;
        if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE && lastShpeUsable
            && paintState == FR.Chart.ChartPaintConstants.STATE_ENABLE) {

            for (var actionStep = 0; actionStep < step; actionStep++) {
                var path = new FR.Chart.GeneralPath();

                for (var i = 0; i < this.x.length; i++) {
                    var newY = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, lastShape.y[i], this.y[i] - lastShape.y[i], step);
                    if (this.type[i] == FR.Chart.GeneralPath.MOVE_TO) {
                        path.moveTo(this.x[i], newY);
                    } else if (this.type[i] == FR.Chart.GeneralPath.LINE_TO) {
                        path.lineTo(this.x[i], newY);
                    } else if (this.type[i] == FR.Chart.GeneralPath.CLOSE) {
                        path.closePath();
                    } else if (this.type[i] == FR.Chart.GeneralPath.QUAD_TO) {
                        path.quadTo(this.ctrl[i].x, FR.Chart.ChartEase.easeOutQuint(actionStep + 1, lastShape.ctrl[i].y, this.ctrl[i].y - lastShape.ctrl[i].y, step),
                            this.x[i], newY);
                    } else if (this.type[i] == FR.Chart.GeneralPath.CURVE_TO) {
                        path.curveTo(this.ctrl[i][0].x, FR.Chart.ChartEase.easeOutQuint(actionStep + 1, lastShape.ctrl[i][0].y, this.ctrl[i][0].y - lastShape.ctrl[i][0].y, step),
                            this.ctrl[i][1].x, FR.Chart.ChartEase.easeOutQuint(actionStep + 1, lastShape.ctrl[i][1].y, this.ctrl[i][1].y - lastShape.ctrl[i][1].y, step),
                            this.x[i], newY);
                    }
                }

                pathArray[actionStep] = path;
            }
        } else {
            for (var actionStep = 0; actionStep < step; actionStep++) {
                var path = new FR.Chart.GeneralPath();
                var curvePath = new FR.Chart.GeneralPath();

                //折线图设置分类轴逆序影响动画效果
                if(this.axisReversed){
                    var length = this.x[0] - this.x[this.x.length - 1] ;
                    length = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, 0, length, step);
                    for (var i = 0; i < this.x.length; i++) {
                        var x = this.x[i];
                        var y = this.y[i];

                        if (this.x[i] < this.x[0] - length
                            && this.x[i + 1] > this.x[0] - length) {
                            if (this.type[i] != FR.Chart.GeneralPath.MOVE_TO) {
                                var x = this.x[0] - length;
                                var temp = (x - this.x[i + 1]) / (this.x[i] - x);
                                var y = (temp * this.y[i] + this.y[i + 1]) / (1 + temp);
                            }
                        }

                        if (this.x[i] < this.x[0] - length && this.x[i + 1] <= this.x[0] - length) {
                            break;
                        }

                        if (this.type[i] == FR.Chart.GeneralPath.MOVE_TO) {
                            FR.Chart.FoldLine.curveTo(path, curvePath);
                            curvePath.clear();
                            path.moveTo(x, y);
                        } else if (this.type[i] == FR.Chart.GeneralPath.LINE_TO) {
                            FR.Chart.FoldLine.curveTo(path, curvePath);
                            curvePath.clear();
                            path.lineTo(x, y);
                        } else if (this.type[i] == FR.Chart.GeneralPath.CLOSE) {
                            FR.Chart.FoldLine.curveTo(path, curvePath);
                            curvePath.clear();
                            path.closePath();
                        } else if (this.type[i] == FR.Chart.GeneralPath.QUAD_TO) {
                            if (curvePath.x.length === 0) {
                                curvePath.moveTo(path.x[path.x.length - 1], path.y[path.y.length - 1]);
                            }
                            curvePath.lineTo(x, y);
                        } else if (this.type[i] == FR.Chart.GeneralPath.CURVE_TO) {
                            if (curvePath.x.length === 0) {
                                curvePath.moveTo(path.x[path.x.length - 1], path.y[path.y.length - 1]);
                            }
                            curvePath.lineTo(x, y);
                        }

                    }
                }else{
                    var length = this.x[this.x.length - 1] - this.x[0];
                    length = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, 0, length, step);
                    for (var i = 0; i < this.x.length; i++) {
                        var x = this.x[i];
                        var y = this.y[i];

                        if (this.x[i] > this.x[0] + length
                            && this.x[i - 1] < this.x[0] + length) {
                            if (this.type[i] != FR.Chart.GeneralPath.MOVE_TO) {
                                var x = this.x[0] + length;
                                var temp = (x - this.x[i - 1]) / (this.x[i] - x);
                                var y = (temp * this.y[i] + this.y[i - 1]) / (1 + temp);
                            }
                        }

                        if (this.x[i] > this.x[0] + length && this.x[i - 1] >= this.x[0] + length) {
                            break;
                        }

                        if (this.type[i] == FR.Chart.GeneralPath.MOVE_TO) {
                            FR.Chart.FoldLine.curveTo(path, curvePath);
                            curvePath.clear();
                            path.moveTo(x, y);
                        } else if (this.type[i] == FR.Chart.GeneralPath.LINE_TO) {
                            FR.Chart.FoldLine.curveTo(path, curvePath);
                            curvePath.clear();
                            path.lineTo(x, y);
                        } else if (this.type[i] == FR.Chart.GeneralPath.CLOSE) {
                            FR.Chart.FoldLine.curveTo(path, curvePath);
                            curvePath.clear();
                            path.closePath();
                        } else if (this.type[i] == FR.Chart.GeneralPath.QUAD_TO) {
                            if (curvePath.x.length === 0) {
                                curvePath.moveTo(path.x[path.x.length - 1], path.y[path.y.length - 1]);
                            }
                            curvePath.lineTo(x, y);
                        } else if (this.type[i] == FR.Chart.GeneralPath.CURVE_TO) {
                            if (curvePath.x.length === 0) {
                                curvePath.moveTo(path.x[path.x.length - 1], path.y[path.y.length - 1]);
                            }
                            curvePath.lineTo(x, y);
                        }
                    }
                }
                FR.Chart.FoldLine.curveTo(path, curvePath);
                pathArray[actionStep] = path;
            }
        }
        return pathArray;
    },


    // 雷达图动态
    getAnimateRadarPath: function (step, paintState) {
        var pathArray = [];

        for (var actionStep = 0; actionStep < step; actionStep++) {
            var path = new FR.Chart.GeneralPath();

            for (var i = 0; i < this.x.length; i++) {
                var newX, newY;
                if (paintState != FR.Chart.ChartPaintConstants.STATE_DISAPPEAR) {
                    newX = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, this.centerPoint.x, this.x[i] - this.centerPoint.x, step);
                    newY = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, this.centerPoint.y, this.y[i] - this.centerPoint.y, step);
                } else {
                    newX = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, this.x[i], this.centerPoint.x - this.x[i], step);
                    newY = FR.Chart.ChartEase.easeOutQuint(actionStep + 1, this.y[i], this.centerPoint.y - this.y[i], step);
                }

                if (this.type[i] == FR.Chart.GeneralPath.MOVE_TO) {
                    path.moveTo(newX, newY);
                } else if (this.type[i] == FR.Chart.GeneralPath.LINE_TO) {
                    path.lineTo(newX, newY);
                } else if (this.type[i] == FR.Chart.GeneralPath.CLOSE) {
                    path.closePath();
                }
            }

            pathArray[actionStep] = path;
        }

        return pathArray;
    },

    // 高低线动态
    getAnimateHighLowLinePath: function (step, paintState) {
        var pathArray = [];

        for (var actionStep = 0; actionStep < step; actionStep++) {
            var path = new FR.Chart.GeneralPath();

            var x = this.x[0], miny, maxy;
            if (this.y[0] > this.y[1]) {
                miny = this.y[1];
                maxy = this.y[0];
            } else {
                miny = this.y[0];
                maxy = this.y[1];
            }

            // 这里的min和max是坐标的大小和值的大小相反
            if (this.upDown < 0) {
                path.moveTo(x, miny);
                path.lineTo(x, FR.Chart.ChartEase.easeOutQuint(actionStep + 1, miny, maxy - miny, step));
                path.closePath();
            } else {
                path.moveTo(x, maxy);
                path.lineTo(x, FR.Chart.ChartEase.easeOutQuint(actionStep + 1, maxy, miny - maxy, step));
                path.closePath();
            }

            pathArray[actionStep] = path;
        }

        return pathArray;
    },

    moveTo: function (x, y) {
        this.type[this.type.length] = FR.Chart.GeneralPath.MOVE_TO;
        this.x[this.x.length] = x;
        this.y[this.y.length] = y;
        this.ctrl[this.ctrl.length] = null;
    },

    lineTo: function (x, y) {
        this.type[this.type.length] = FR.Chart.GeneralPath.LINE_TO;
        this.x[this.x.length] = x;
        this.y[this.y.length] = y;
        this.ctrl[this.ctrl.length] = null;
    },

    quadTo: function (aCPx, aCPy, x, y) {
        this.type[this.type.length] = FR.Chart.GeneralPath.QUAD_TO;
        this.x[this.x.length] = x;
        this.y[this.y.length] = y;
        this.ctrl[this.ctrl.length] = new FR.Chart.Point2D(aCPx, aCPy);
    },

    curveTo: function (aCP1x, aCP1y, aCP2x, aCP2y, x, y) {
        this.type[this.type.length] = FR.Chart.GeneralPath.CURVE_TO;
        this.x[this.x.length] = x;
        this.y[this.y.length] = y;
        var cps = [];
        cps[0] = new FR.Chart.Point2D(aCP1x, aCP1y);
        cps[1] = new FR.Chart.Point2D(aCP2x, aCP2y);
        this.ctrl[this.ctrl.length] = cps;
    },

    closePath: function () {
        for (var i = this.type.length - 1; i >= 0; i--) {
            if (this.type[i] == FR.Chart.GeneralPath.MOVE_TO) {
                this.type[this.type.length] = FR.Chart.GeneralPath.CLOSE;
                this.x[this.x.length] = this.x[i];
                this.y[this.y.length] = this.y[i];
                this.ctrl[this.ctrl.length] = null;
                return;
            }
        }
        if (this.type.length > 1) {
            this.type[this.type.length] = FR.Chart.GeneralPath.CLOSE;
            this.x[this.x.length] = this.x[0];
            this.y[this.y.length] = this.y[0];
            this.ctrl[this.ctrl.length] = null;
        }
    },

    clear: function () {
        this.initConstants();
    },

    isNeedRadialGradient:function(){
        return false;
    },

    //计算地图地方最大的图块的边界，预防那种一个低于分好几个块的，比如俄罗斯，取最大的块的边界
    getLargeShapeBounds4Map: function(){
        var smallPaths = [];
        var path = null;
        for (var i = 0; i < this.x.length; i++) {
            if (this.type[i] == FR.Chart.GeneralPath.MOVE_TO) {
                if(i !== 0 && path !== null){
                    path.closePath();
                    smallPaths[smallPaths.length] = path;
                }
                path = new FR.Chart.GeneralPath();
                path.moveTo(this.x[i], this.y[i]);
            }else if(this.type[i] == FR.Chart.GeneralPath.CLOSE){
                path.closePath();
                smallPaths[smallPaths.length] = path;
            }else{
                path.lineTo(this.x[i],this.y[i]);
            }
        }

        var maxWith = 0;
        var maxHeight = 0;
        var largeIndex = 0;

        for(var count =0 ;count < smallPaths.length;count++){
            var smallBounds = smallPaths[count].getBounds2D();
            if(smallBounds.width * smallBounds.height > maxWith*maxHeight) {
                largeIndex = count;
                maxWith = smallBounds.width;
                maxHeight = smallBounds.height;
            }
        }

        if(smallPaths.length <= 0){
            return this.getBounds2D();
        }

        return smallPaths[largeIndex].getBounds2D();
    },

    // 计算矩形 范围.
    getBounds2D: function () {
        var x = this.x[0] || 0;
        var y = this.y[0] || 0;
        var width = this.x[0] || 0;
        var height = this.y[0] || 0;
        for (var i = 0; i < this.x.length; i++) {
            x = Math.min(this.x[i], x);
            y = Math.min(this.y[i], y);
            width = Math.max(this.x[i], width);
            height = Math.max(this.y[i], height);

            // 控制点也计算在内
            if (this.type[i] == FR.Chart.GeneralPath.QUAD_TO) {
                x = Math.min(this.ctrl[i].x, x);
                y = Math.min(this.ctrl[i].y, y);
                width = Math.max(this.ctrl[i].x, width);
                height = Math.max(this.ctrl[i].y, height);
            } else if (this.type[i] == FR.Chart.GeneralPath.CURVE_TO) {
                x = Math.min(this.ctrl[i][0].x, x);
                y = Math.min(this.ctrl[i][0].y, y);
                width = Math.max(this.ctrl[i][0].x, width);
                height = Math.max(this.ctrl[i][0].y, height);

                x = Math.min(this.ctrl[i][1].x, x);
                y = Math.min(this.ctrl[i][1].y, y);
                width = Math.max(this.ctrl[i][1].x, width);
                height = Math.max(this.ctrl[i][1].y, height);
            }
        }
        return new FR.Chart.Bounds(x, y, width - x, height - y)
    },

    append: function (newPath, combine) {
        if (newPath instanceof FR.Chart.GeneralPath) {
            for (var i = 0; i < newPath.x.length; i++) {
                if (i === 0 && combine) {
                    this.type[this.type.length] = FR.Chart.GeneralPath.LINE_TO;
                } else {
                    this.type[this.type.length] = newPath.type[i];
                }
                this.x[this.x.length] = newPath.x[i];
                this.y[this.y.length] = newPath.y[i];
                this.ctrl[this.ctrl.length] = newPath.ctrl[i];
            }
        } else if (newPath instanceof FR.Chart.Bounds) {
            this.moveTo(newPath.x, newPath.y);
            this.lineTo(newPath.x + newPath.width, newPath.y);
            this.lineTo(newPath.x + newPath.width, newPath.y + newPath.height);
            this.lineTo(newPath.x, newPath.y + newPath.height);
            this.closePath();
        }
    },

    // 把折线的相应区域扩展为面积方式,即折线上移5像素,下移5像素组成的面积
    getInteractivePath: function () {
        if (this.type[this.type.length - 1] == FR.Chart.GeneralPath.CLOSE) {
            return this;
        }
        var path = new FR.Chart.GeneralPath();
        var offset = 5;
        for (var i = 0; i < this.type.length; i++) {
            path.x[i] = this.x[i];
            path.y[i] = this.y[i] - offset;
            path.type[i] = this.type[i];
            if (path.type[i] == FR.Chart.GeneralPath.QUAD_TO) {
                path.ctrl[i] = new FR.Chart.Point2D(this.ctrl[i].x, this.ctrl[i].y - offset);
            } else if (path.type[i] == FR.Chart.GeneralPath.CURVE_TO) {
                var cps = [];
                cps[0] = new FR.Chart.Point2D(this.ctrl[i][0].x, this.ctrl[i][0].y - offset);
                cps[1] = new FR.Chart.Point2D(this.ctrl[i][1].x, this.ctrl[i][1].y - offset);
                path.ctrl[i] = cps;
            } else {
                path.ctrl[i] = this.ctrl[i];
            }
        }

        path.lineTo(this.x[this.x.length - 1], this.y[this.y.length - 1] + offset);

        var i = this.type.length - 2;
        while (i >= 0) {
            switch (this.type[i + 1]) {
                case FR.Chart.GeneralPath.MOVE_TO:
                    path.moveTo(this.x[i], this.y[i] + offset);
                    i--;
                    break;
                case FR.Chart.GeneralPath.LINE_TO:
                    path.lineTo(this.x[i], this.y[i] + offset);
                    i--;
                    break;
                case FR.Chart.GeneralPath.QUAD_TO:
                    path.quadTo(this.ctrl[i + 1].x, this.ctrl[i + 1].y + offset, this.x[i], this.y[i] + offset);
                    i--;
                    break;
                case FR.Chart.GeneralPath.CURVE_TO:
                    path.curveTo(this.ctrl[i + 1][1].x, this.ctrl[i + 1][1].y + offset, this.ctrl[i + 1][0].x, this.ctrl[i + 1][0].y + 1, this.x[i], this.y[i] + offset);
                    i--;
                    break;
                case FR.Chart.GeneralPath.CLOSE:
                    path.type[path.type.length] = this.type[i];
                    path.x[path.x.length] = this.x[i];
                    path.y[path.y.length] = this.y[i] + offset;
                    path.ctrl[path.ctrl.length] = this.ctrl[i];
                    i--;
                    break;
            }
        }

        path.lineTo(this.x[0], this.y[0] + offset);

        return path;
    },

    contains: function (x, y, w, h) {
        if (isNaN(x + w) || isNaN(y + h)) {
            return false;
        }
        if (w <= 0 || h <= 0) {
            return false;
        }
        var crossings = this.rectCrossings(x, y, x + w, y + h);
        return (crossings != FR.Chart.GeneralPath.RECT_INTERSECTS && (crossings & -1) !== 0);
    },

    containsPoint: function (x, y) {
        return this.getInteractivePath().contains(x, y, 1, 1);
    },

    intersects: function (x, y, w, h) {
        if (isNaN(x + w) && isNaN(y + h)) {
            return false;
        }
        if (w <= 0 || h <= 0) {
            return false;
        }
        var crossings = this.rectCrossings(x, y, x + w, y + h);
        return (crossings == FR.Chart.GeneralPath.RECT_INTERSECTS
        || (crossings & -1) !== 0);
    },

    verticalIntersects: function (y,h) {
        if (isNaN(y + h)) {
            return false;
        }
        if (h <= 0) {
            return false;
        }
        var crossings = this.verticalCrossings(y,y + h);
        return (crossings == FR.Chart.GeneralPath.RECT_INTERSECTS
        || (crossings & -1) !== 0);
    },

    verticalCrossings: function (rymin,rymax){
        var cury,movy, endy;
        cury = movy = this.y[0];
        var crossings = 0;

        for (var i = 1; i < this.y.length && crossings != FR.Chart.GeneralPath.RECT_INTERSECTS; i++) {
            switch (this.type[i]) {
                case FR.Chart.GeneralPath.MOVE_TO:
                    if (cury != movy) {
                        crossings = this.verticalCrossingsForLine(crossings,
                            rymin,rymax, cury,  movy);
                    }
                    movy = cury = this.y[i];
                    break;
                case FR.Chart.GeneralPath.LINE_TO:
                    endy = this.y[i];
                    crossings = this.verticalCrossingsForLine(crossings,
                        rymin,  rymax, cury,  endy);
                    cury = endy;
                    break;
                case FR.Chart.GeneralPath.CLOSE:
                    if ( cury != movy) {
                        crossings = this.verticalCrossingsForLine(crossings,
                            rymin,  rymax, cury,  movy);
                    }
                    cury = movy;
                    break;
            }
        }
        if (crossings != FR.Chart.GeneralPath.RECT_INTERSECTS && cury != movy) {
            crossings = this.rectCrossingsForLine(crossings,
                rymin,  rymax, cury, movy);
        }
        return crossings;
    },

    rectCrossings: function (rxmin, rymin, rxmax, rymax) {
        var curx, cury, movx, movy, endx, endy;
        curx = movx = this.x[0];
        cury = movy = this.y[0];
        var crossings = 0;

        for (var i = 1; i < this.x.length && crossings != FR.Chart.GeneralPath.RECT_INTERSECTS; i++) {
            switch (this.type[i]) {
                case FR.Chart.GeneralPath.MOVE_TO:
                    if (curx != movx || cury != movy) {
                        crossings = this.rectCrossingsForLine(crossings,
                            rxmin, rymin, rxmax, rymax,
                            curx, cury, movx, movy);
                    }
                    movx = curx = this.x[i];
                    movy = cury = this.y[i];
                    break;
                case FR.Chart.GeneralPath.LINE_TO:
                    endx = this.x[i];
                    endy = this.y[i];
                    crossings = this.rectCrossingsForLine(crossings,
                        rxmin, rymin, rxmax, rymax,
                        curx, cury, endx, endy);
                    curx = endx;
                    cury = endy;
                    break;
                case FR.Chart.GeneralPath.QUAD_TO:
                    crossings = this.rectCrossingsForQuad(crossings,
                        rxmin, rymin, rxmax, rymax,
                        curx, cury,
                        this.ctrl[i].x, this.ctrl[i].y,
                        endx = this.x[i], endy = this.y[i],
                        0);
                    curx = endx;
                    cury = endy;
                    break;
                case FR.Chart.GeneralPath.CURVE_TO:
                    crossings = this.rectCrossingsForCubic(crossings,
                        rxmin, rymin, rxmax, rymax,
                        curx, cury,
                        this.ctrl[i][0].x, this.ctrl[i][0].y,
                        this.ctrl[i][1].x, this.ctrl[i][1].y,
                        endx = this.x[i], endy = this.y[i],
                        0);
                    curx = endx;
                    cury = endy;
                    break;
                case FR.Chart.GeneralPath.CLOSE:
                    if (curx != movx || cury != movy) {
                        crossings = this.rectCrossingsForLine(crossings,
                            rxmin, rymin, rxmax, rymax,
                            curx, cury, movx, movy);
                    }
                    curx = movx;
                    cury = movy;
                    break;
            }
        }
        if (crossings != FR.Chart.GeneralPath.RECT_INTERSECTS && (curx != movx || cury != movy)) {
            crossings = this.rectCrossingsForLine(crossings,
                rxmin, rymin, rxmax, rymax,
                curx, cury, movx, movy);
        }
        return crossings;
    },

    verticalCrossingsForLine: function (crossings, rymin,  rymax,  y0,  y1) {
        if ((y0 >= rymin && y0 <= rymax)
            || (y1 >= rymin && y1 <= rymax)) {
            return FR.Chart.GeneralPath.RECT_INTERSECTS;
        }else{
            return crossings;
        }
    },

    rectCrossingsForLine: function (crossings, rxmin, rymin, rxmax, rymax, x0, y0, x1, y1) {
        if (y0 >= rymax && y1 >= rymax) {
            return crossings;
        }
        if (y0 <= rymin && y1 <= rymin) {
            return crossings;
        }
        if (x0 <= rxmin && x1 <= rxmin) {
            return crossings;
        }
        if (x0 >= rxmax && x1 >= rxmax) {
            if (y0 < y1) {
                if (y0 <= rymin) {
                    crossings++;
                }
                if (y1 >= rymax) {
                    crossings++;
                }
            } else if (y1 < y0) {
                if (y1 <= rymin) {
                    crossings--;
                }
                if (y0 >= rymax) {
                    crossings--;
                }
            }
            return crossings;
        }

        if ((x0 > rxmin && x0 < rxmax && y0 > rymin && y0 < rymax)
            || (x1 > rxmin && x1 < rxmax && y1 > rymin && y1 < rymax)) {
            return FR.Chart.GeneralPath.RECT_INTERSECTS;
        }

        var xi0 = x0;
        if (y0 < rymin) {
            xi0 += ((rymin - y0) * (x1 - x0) / (y1 - y0));
        } else if (y0 > rymax) {
            xi0 += ((rymax - y0) * (x1 - x0) / (y1 - y0));
        }
        var xi1 = x1;
        if (y1 < rymin) {
            xi1 += ((rymin - y1) * (x0 - x1) / (y0 - y1));
        } else if (y1 > rymax) {
            xi1 += ((rymax - y1) * (x0 - x1) / (y0 - y1));
        }
        if (xi0 <= rxmin && xi1 <= rxmin) {
            return crossings;
        }
        if (xi0 >= rxmax && xi1 >= rxmax) {
            if (y0 < y1) {
                if (y0 <= rymin) {
                    crossings++;
                }
                if (y1 >= rymax) {
                    crossings++;
                }
            } else if (y1 < y0) {
                if (y1 <= rymin) {
                    crossings--;
                }
                if (y0 >= rymax) {
                    crossings--;
                }
            }
            return crossings;
        }
        return FR.Chart.GeneralPath.RECT_INTERSECTS;
    },

    rectCrossingsForCubic: function (crossings, rxmin, rymin, rxmax, rymax, x0, y0, xc0, yc0, xc1, yc1, x1, y1, level) {
        if (y0 >= rymax && yc0 >= rymax && yc1 >= rymax && y1 >= rymax) {
            return crossings;
        }
        if (y0 <= rymin && yc0 <= rymin && yc1 <= rymin && y1 <= rymin) {
            return crossings;
        }
        if (x0 <= rxmin && xc0 <= rxmin && xc1 <= rxmin && x1 <= rxmin) {
            return crossings;
        }
        if (x0 >= rxmax && xc0 >= rxmax && xc1 >= rxmax && x1 >= rxmax) {
            if (y0 < y1) {
                if (y0 <= rymin && y1 > rymin) {
                    crossings++;
                }
                if (y0 < rymax && y1 >= rymax) {
                    crossings++;
                }
            } else if (y1 < y0) {
                if (y1 <= rymin && y0 > rymin) {
                    crossings--;
                }
                if (y1 < rymax && y0 >= rymax) {
                    crossings--;
                }
            }
            return crossings;
        }

        if ((x0 > rxmin && x0 < rxmax && y0 > rymin && y0 < rymax)
            || (x1 > rxmin && x1 < rxmax && y1 > rymin && y1 < rymax)) {
            return FR.Chart.GeneralPath.RECT_INTERSECTS;
        }

        if (level > 52) {
            return this.rectCrossingsForLine(crossings,
                rxmin, rymin, rxmax, rymax,
                x0, y0, x1, y1);
        }
        var xmid = (xc0 + xc1) / 2;
        var ymid = (yc0 + yc1) / 2;
        xc0 = (x0 + xc0) / 2;
        yc0 = (y0 + yc0) / 2;
        xc1 = (xc1 + x1) / 2;
        yc1 = (yc1 + y1) / 2;
        var xc0m = (xc0 + xmid) / 2;
        var yc0m = (yc0 + ymid) / 2;
        var xmc1 = (xmid + xc1) / 2;
        var ymc1 = (ymid + yc1) / 2;
        xmid = (xc0m + xmc1) / 2;
        ymid = (yc0m + ymc1) / 2;
        if (isNaN(xmid) || isNaN(ymid)) {
            return 0;
        }
        crossings = this.rectCrossingsForCubic(crossings,
            rxmin, rymin, rxmax, rymax,
            x0, y0, xc0, yc0,
            xc0m, yc0m, xmid, ymid, level + 1);
        if (crossings != FR.Chart.GeneralPath.RECT_INTERSECTS) {
            crossings = this.rectCrossingsForCubic(crossings,
                rxmin, rymin, rxmax, rymax,
                xmid, ymid, xmc1, ymc1,
                xc1, yc1, x1, y1, level + 1);
        }
        return crossings;
    },

    rectCrossingsForQuad: function (crossings, rxmin, rymin, rxmax, rymax, x0, y0, xc, yc, x1, y1, level) {
        if (y0 >= rymax && yc >= rymax && y1 >= rymax) {
            return crossings;
        }
        if (y0 <= rymin && yc <= rymin && y1 <= rymin) {
            return crossings;
        }
        if (x0 <= rxmin && xc <= rxmin && x1 <= rxmin) {
            return crossings;
        }

        if (x0 >= rxmax && xc >= rxmax && x1 >= rxmax) {
            if (y0 < y1) {
                if (y0 <= rymin && y1 > rymin) {
                    crossings++;
                }
                if (y0 < rymax && y1 >= rymax) {
                    crossings++;
                }
            } else if (y1 < y0) {
                if (y1 <= rymin && y0 > rymin) {
                    crossings--;
                }
                if (y1 < rymax && y0 >= rymax) {
                    crossings--;
                }
            }
            return crossings;
        }

        if ((x0 < rxmax && x0 > rxmin && y0 < rymax && y0 > rymin)
            || (x1 < rxmax && x1 > rxmin && y1 < rymax && y1 > rymin)) {
            return FR.Chart.GeneralPath.RECT_INTERSECTS;
        }

        if (level > 52) {
            return this.rectCrossingsForLine(crossings,
                rxmin, rymin, rxmax, rymax,
                x0, y0, x1, y1);
        }
        var x0c = (x0 + xc) / 2;
        var y0c = (y0 + yc) / 2;
        var xc1 = (xc + x1) / 2;
        var yc1 = (yc + y1) / 2;
        xc = (x0c + xc1) / 2;
        yc = (y0c + yc1) / 2;
        if (isNaN(xc) || isNaN(yc)) {
            return 0;
        }
        crossings = this.rectCrossingsForQuad(crossings,
            rxmin, rymin, rxmax, rymax,
            x0, y0, x0c, y0c, xc, yc,
            level + 1);
        if (crossings != FR.Chart.GeneralPath.RECT_INTERSECTS) {
            crossings = this.rectCrossingsForQuad(crossings,
                rxmin, rymin, rxmax, rymax,
                xc, yc, xc1, yc1, x1, y1,
                level + 1);
        }
        return crossings;
    },

    getVerticalMiddlePointY:function(){
        var bounds = this.getBounds2D();
        var x = bounds.x + bounds.width / 2;
        var count = 0;
        var y = [];
        var midY = bounds.y + bounds.height / 2;
        for (var i = 0; i < this.x.length; i ++) {
            var j = (i + 1) % this.x.length;
            if (this.x[i] <= x && this.x[j] >= x || this.x[i] >= x && this.x[j] <= x) {
                if (this.x[i] === this.x[j]) {
                    y[count++] = (this.y[i] + this.y[j]) / 2;
                } else {
                    y[count++] = (this.y[j] - this.y[i]) * (x - this.x[i]) / (this.x[j] - this.x[i]) + this.y[i];
                }
            }
        }
        if (count <= 0) {
            return midY;
        }
        y.sort();
        var num = count / 2;
        if (num > 0) {
            var max = 0;
            for (i = 0; i < num; i++) {
                j = i * 2;
                var k = i * 2 + 1;
                var gap = Math.abs(y[j] - y[k]);
                if (gap > max) {
                    max = gap;
                    midY = (y[j] + y[k]) / 2;
                }
            }
        }
        if (this.containsPoint(x, midY)) {
            return midY;
        }
        return bounds.y + bounds.height / 2;
    }
});
$.extend(FR.Chart.GeneralPath, {
    MOVE_TO: 0,
    LINE_TO: 1,
    QUAD_TO: 2,
    CURVE_TO: 3,
    CLOSE: 4,
    RECT_INTERSECTS: 0x80000000
});
// GeneralPath.end

/**
 * 折线
 * @class FR.Chart.FoldLine
 * @extends FR.Chart.ChartJsInit
 */
FR.Chart.FoldLine = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (generalPath) {
        this.lineStyleInfo = new FR.Chart.LineStyleInfo();

        this.generalPath = generalPath || null;
    },

    draw: function (ctx, time) {
        if (this.lineStyleInfo) {
            this.lineStyleInfo.paint(ctx, this.generalPath, time);
        }
    },

    getShadeColor:function(){
        var rgb = FR.Chart.Color.getColorArray(this.lineStyleInfo.attrLineColor.seriesColor);
        var hsb = FR.Chart.Color.RGB2HSB(rgb[0], rgb[1], rgb[2]);
        return FR.Chart.Color.getColor(FR.Chart.Color.HSB2RGB(hsb[0], hsb[1] * 2, hsb[2] / 2));
    },

    setAlpha: function(alpha){
        this.lineStyleInfo.seriesAttrAlpha.alpha = alpha;
    },

    getAlpha:function(){
        return this.lineStyleInfo.seriesAttrAlpha.alpha;
    },

    draw4Demo: function (ctx) {
        if (this.generalPath) {
            this.generalPath.draw(ctx);
        }
    },

    getShape: function () {
        return this.generalPath;
    },

    getAnimateGlyph: function (step, actionModel, lastGlyph, paintState) {
        var lineArray = [];

        if (this.generalPath) {
            var lastShape = null;
            if (lastGlyph && actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE) {
                lastShape = lastGlyph.generalPath;
            }
            var pathArray = this.generalPath.getAnimatePath(step, actionModel, lastShape, paintState);

            for (var i = 0; i < pathArray.length; i++) {
                if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL) {
                    lineArray[i] = this;
                } else {
                    var line = new FR.Chart.FoldLine(pathArray[i]);
                    lineArray[i] = line;

                    line.lineStyleInfo = this.lineStyleInfo;
                }

            }
        }

        return lineArray;
    }
});
$.extend(FR.Chart.FoldLine, /**@class FR.Chart.FoldLine*/{
    curveTo: function (appendTo, point) {
        if (point.x.length <= 2) {
            appendTo.append(point, true);
        } else {
            var scale = 0.6;
            var midX = [];
            var midY = [];

            for (var i = 0; i < point.x.length - 1; i++) {
                midX[i] = (point.x[i] + point.x[i + 1]) / 2;
                midY[i] = (point.y[i] + point.y[i + 1]) / 2;
            }

            var ctrlX = [];
            var ctrlY = [];

            for (var i = 1; i < midX.length; i++) {
                var offsetX = point.x[i] - (midX[i - 1] + midX[i]) / 2;
                var offsetY = point.y[i] - (midY[i - 1] + midY[i]) / 2;

                ctrlX[(i - 1) * 2] = point.x[i] + (midX[i - 1] + offsetX - point.x[i]) * scale;
                ctrlY[(i - 1) * 2] = point.y[i] + (midY[i - 1] + offsetY - point.y[i]) * scale;

                ctrlX[(i - 1) * 2 + 1] = point.x[i] + (midX[i] + offsetX - point.x[i]) * scale;
                ctrlY[(i - 1) * 2 + 1] = point.y[i] + (midY[i] + offsetY - point.y[i]) * scale;
            }

            var path = new FR.Chart.GeneralPath();
            path.moveTo(point.x[0], point.y[0]);
            for (var i = 1; i < point.x.length; i++) {
                if (i === 1) {
                    path.quadTo(ctrlX[0], ctrlY[0], point.x[i], point.y[i]);
                } else if (i == point.x.length - 1) {
                    path.quadTo(ctrlX[ctrlX.length - 1], ctrlY[ctrlY.length - 1], point.x[i], point.y[i]);
                } else {
                    path.curveTo(ctrlX[(i - 1) * 2 - 1], ctrlY[(i - 1) * 2 - 1], ctrlX[(i - 1) * 2], ctrlY[(i - 1) * 2], point.x[i], point.y[i]);
                }
            }

            appendTo.append(path, true);
        }
    }
});
// FoldLine.end

/**
 * 趋势线
 * @class FR.Chart.TrendLine
 * @extends FR.Chart.ChartJsInit
 */
FR.Chart.TrendLine = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (js) {
        js = js || {};

        this.equation = FR.pick(js.equation, FR.Chart.Constants.TRENDLINE_POLYNOMIAL);
        this.square = FR.pick(js.square, 6);
        this.moveAverage = FR.pick(js.moveAverage, 5);

        this.lineStyleInfo = js.lineStyleInfo == undefined ? null : new FR.Chart.LineStyleInfo(js.lineStyleInfo);

        // 提示 + 图例 相关
        this.trendLineName = FR.pick(js.trendLineName, FR.i18nText('Fine-Dec_Chart_Trend_Line'));
    },

    createTrendLineGlyph: function () {
        var glyph = new FR.Chart.TrendLineGlyph();

        glyph.equation = this.equation;
        glyph.square = this.square;
        glyph.moveAverage = this.moveAverage;
        glyph.trendLineName = this.trendLineName;

        glyph.lineStyleInfo.attrLineColor.seriesColor = this.lineStyleInfo.attrLineColor.seriesColor;
        glyph.lineStyleInfo.attrLineStyle.lineStyle = this.lineStyleInfo.attrLineStyle.lineStyle;

        glyph.lineStyleInfo.seriesAttrAlpha.alpha = this.lineStyleInfo.seriesAttrAlpha.alpha;

        glyph.lineStyleInfo.seriesAttrBorder.borderStyle = this.lineStyleInfo.seriesAttrBorder.borderStyle;
        glyph.lineStyleInfo.seriesAttrBorder.borderBackground = this.lineStyleInfo.seriesAttrBorder.borderBackground;

        return glyph;
    }
});
//TrendLine.end

/**
 * 立方体
 * @param x 横坐标
 * @param y 纵坐标
 * @param width 宽度
 * @param height 高度
 * @constructor
 */
FR.Chart.Cubic = function(x, y, width, height){
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
};

$.extend(FR.Chart.Cubic.prototype, /**@class FR.Chart.Cubic*/{
    x: 0,
    y: 0,
    z: 0,
    width: 0,
    height: 0,
    deep: 0,
    projection: null,
    points:null,
    covers: [0, 3, 4],
    horizontalDrawBar: false, //用于条形图，增长方向判断

    setCubicWidthHeight: function(width, height){
        this.width = width;
        this.height = height;
        if(this.points){
            this.points = null;
            this.init();
        }
    },

    setCubicPosition: function(x, y){
        this.x = x;
        this.y = y;
        if(this.points){
            this.points = null;
            this.init();
        }
    },

    getGapWidth: function(){
        var points = this.getPoints();
        return Math.abs(points[4].x - points[0].x);
    },

    getGapHeight: function(){
        var points = this.getPoints();
        return Math.abs(points[0].y - points[4].y);
    },

    getPoints: function(){
        this.init();
        return this.points;
    },

    init: function(){
        if(this.projection && !this.points){
            this.points = [];
            var x = this.x;
            var y = this.y;
            var z = this.z;
            var width = this.width;
            var height = this.height;
            var deep = this.deep;
            this.points[0] = this.projection.projectee(x, y, z);
            this.points[1] = this.projection.projectee(x + width, y, z);
            this.points[2] = this.projection.projectee(x, y + height, z);
            this.points[3] = this.projection.projectee(x + width, y + height, z);
            this.points[4] = this.projection.projectee(x, y, z - deep);
            this.points[5] = this.projection.projectee(x + width, y, z - deep);
            this.points[6] = this.projection.projectee(x, y + height, z - deep);
            this.points[7] = this.projection.projectee(x + width, y + height, z - deep);
        }
    },

    top: function(){
        if(this.points){
            var topArea = new FR.Chart.GeneralPath();
            var points = this.points;
            topArea.moveTo(points[0].x, points[0].y);
            topArea.lineTo(points[1].x, points[1].y);
            topArea.lineTo(points[5].x, points[5].y);
            topArea.lineTo(points[4].x, points[4].y);
            topArea.closePath();
            return topArea;
        }
    },

    bottom: function(){
        if(this.points){
            var bottomArea = new FR.Chart.GeneralPath();
            var points = this.points;
            bottomArea.moveTo(points[2].x, points[2].y);
            bottomArea.lineTo(points[3].x, points[3].y);
            bottomArea.lineTo(points[7].x, points[7].y);
            bottomArea.lineTo(points[6].x, points[6].y);
            bottomArea.closePath();
            return bottomArea;
        }
    },

    right: function(){
        if(this.points){
            var rightArea = new FR.Chart.GeneralPath();
            var points = this.points;
            rightArea.moveTo(points[1].x, points[1].y);
            rightArea.lineTo(points[3].x, points[3].y);
            rightArea.lineTo(points[7].x, points[7].y);
            rightArea.lineTo(points[5].x, points[5].y);
            rightArea.closePath();
            return rightArea;
        }
    },

    left: function(){
        if(this.points){
            var leftArea = new FR.Chart.GeneralPath();
            var points = this.points;
            leftArea.moveTo(points[0].x, points[0].y);
            leftArea.lineTo(points[2].x, points[2].y);
            leftArea.lineTo(points[6].x, points[6].y);
            leftArea.lineTo(points[4].x, points[4].y);
            leftArea.closePath();
            return leftArea;
        }
    },

    front: function(){
        if(this.points){
            var frontArea = new FR.Chart.GeneralPath();
            var points = this.points;
            frontArea.moveTo(points[0].x, points[0].y);
            frontArea.lineTo(points[1].x, points[1].y);
            frontArea.lineTo(points[3].x, points[3].y);
            frontArea.lineTo(points[2].x, points[2].y);
            frontArea.closePath();
            return frontArea;
        }
    },

    back: function(){
        if(this.points){
            var backArea = new FR.Chart.GeneralPath();
            var points = this.points;
            backArea.moveTo(points[4].x, points[4].y);
            backArea.lineTo(points[5].x, points[5].y);
            backArea.lineTo(points[7].x, points[7].y);
            backArea.lineTo(points[6].x, points[6].y);
            backArea.closePath();
            return backArea;
        }
    },
    getShape: function(){
        this.init();
        if(this.points && this.covers){
            var shape = new FR.Chart.GeneralPath();
            for(var i = 0; i < this.covers.length; i++){
                switch(this.covers[i]){
                    case FR.Chart.Cubic.TOP:
                        shape.append(this.top(), false);
                        break;

                    case FR.Chart.Cubic.BOTTOM:
                        shape.append(this.bottom(), false);
                        break;

                    case FR.Chart.Cubic.LEFT:
                        shape.append(this.left(), false);
                        break;

                    case FR.Chart.Cubic.RIGHT:
                        shape.append(this.right(), false);
                        break;

                    case FR.Chart.Cubic.FRONT:
                        shape.append(this.front(), false);
                        break;

                    case FR.Chart.Cubic.BACK:
                        shape.append(this.back(), false);
                        break;
                    default :
                }
            }
            return shape;
        }
    },

    paint: function(ctx){ //只画确定的三个面，上，前，右,画法设计文档上规定
        if(!ctx){     return; } ctx.save();
        this.init();
        var origin = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
        if(origin){
            //背面的阴影
            if(!ctx){     return; } ctx.save();
            ctx.shadowBlur=6;
            ctx.shadowOffsetX=2;
            ctx.shadowColor= 'rgba(0,0,0, 0.3)';
            this.back().paint(ctx);
            ctx.restore();

            //正面的效果
            var hsb = FR.Chart.Color.RGB2HSB(origin[0], origin[1], origin[2]);
            var startColor = ctx.fillStyle;

            var frontEnd = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1] - 0.3, hsb[2] + 0.35);
            var fEndColor = FR.Chart.Color.getColor(frontEnd);
            var mid = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1]-0.1, hsb[2]+0.1)
            var midColor = FR.Chart.Color.getColor(mid);

            var grd;
            if(this.horizontalDrawBar){
                grd = ctx.createLinearGradient(this.points[2].x, this.points[2].y, this.points[3].x, this.points[3].y);
            }else{
                grd = ctx.createLinearGradient(this.points[2].x, this.points[2].y, this.points[0].x, this.points[0].y);
            }
            grd.addColorStop(0, startColor);
            grd.addColorStop(0.2,midColor);
            grd.addColorStop(1, fEndColor);
            ctx.fillStyle = grd;
            this.front().paint(ctx);

            //右面
            var first = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1] + 0.1, hsb[2] - 0.1);
            var firstColor = FR.Chart.Color.getColor(first);

            var second = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1] - 0.05, hsb[2] + 0.2);
            var secondColor = FR.Chart.Color.getColor(second);

            var third = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1] + 0.05, hsb[2] + 0.1);
            var thirdColor = FR.Chart.Color.getColor(third);

            grd = ctx.createLinearGradient(this.points[3].x, this.points[3].y, this.points[5].x, this.points[5].y);
            grd.addColorStop(0, firstColor);
            grd.addColorStop(0.7, secondColor);
            grd.addColorStop(1, thirdColor);
            ctx.fillStyle = grd;
            this.right().paint(ctx);

            //上面
            var tStartArray = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1]-0.05, hsb[2]+0.05);

            var tStartColor = FR.Chart.Color.getColor(tStartArray);

            var tEndArray = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1]-0.2, hsb[2]+0.3);
            var tEndColor = FR.Chart.Color.getColor(tEndArray);
            grd = ctx.createLinearGradient(this.points[0].x, this.points[0].y, this.points[0].x, this.points[5].y);
            grd.addColorStop(0, tStartColor);
            grd.addColorStop(1, tEndColor);
            ctx.fillStyle = grd;
            this.top().paint(ctx);
        }

        ctx.restore();
    },

    //边框还不确定要不要，暂时保留画法
    draw: function(ctx){
//        if (this.width === 0 || this.height === 0) {
//            return;
//        }
//
//        if(!this.shape){
//            this.shape = this.getShape();
//        }
//
//        this.shape.draw(ctx);
    },

    getAnimateShape: function (step, actionModel, lastShape, paintState) {
        var cubicArray = [];
        if(paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE){
            return cubicArray;
        }

        for (var i = 0; i < step; i++) {
            var cubic;
            if (this.horizontalDrawBar) {
                var actionWidth = FR.Chart.ChartEase.easeOutQuint(i + 1, 0, this.width, step);
                cubic = new FR.Chart.Cubic(this.x, this.y, actionWidth, this.height);
            }else{
                var actionHeight = FR.Chart.ChartEase.easeOutQuint(i + 1, 0, this.height, step);
                cubic = new FR.Chart.Cubic(this.x, this.y + this.height - actionHeight, this.width, actionHeight);
            }
            cubic.projection = this.projection;
            cubic.z = this.z;
            cubic.deep = this.deep;
            cubic.horizontalDrawBar = this.horizontalDrawBar;
            cubicArray[i] = cubic;
        }
        return cubicArray;
    },

    containsPoint: function (x, y) {
        if(!this.shape){
            this.shape = this.getShape();
        }
        return this.shape.containsPoint(x, y);
    },

    draw3DBase: function(ctx, face){
        if(!ctx){     return; } ctx.save();
        ctx.globalAlpha = 0.18;
        this.init();
        if(face == FR.Chart.Cubic.BOTTOM){
            var grd=ctx.createLinearGradient(this.points[2].x,this.points[2].y, this.points[2].x, this.points[6].y);
            grd.addColorStop(0,'rgb(145,151,156)');
            grd.addColorStop(1,'rgb(198, 205, 210)');
            ctx.fillStyle=grd;
            this.bottom().paint(ctx);
        }else if(face == FR.Chart.Cubic.LEFT){
            var grd=ctx.createLinearGradient(this.points[0].x, this.points[4].y, this.points[4].x, this.points[4].y);
            grd.addColorStop(0,'rgb(145,151,156)');
            grd.addColorStop(1,'rgb(198, 205, 210)');
            ctx.fillStyle=grd;
            this.left().paint(ctx);
        }

        ctx.restore();
    },

    isNeedRadialGradient:function(){
        return false;
    },

    getBounds2D:function(){
        this.init();
        return this.front().getBounds2D();
    }
});

$.extend(FR.Chart.Cubic, /**@class FR.Chart.Cubic*/{

    /**
     * @property 顶部
     */
    TOP: 0,

    /**
     * @property 底部
     */
    BOTTOM: 1,

    /**
     * @property 左侧
     */
    LEFT: 2,

    /**
     * @property 右侧
     */
    RIGHT: 3,
    /**
     * @property 靠前
     */
    FRONT: 4,

    /**
     * @property 靠后
     */
    BACK: 5
});
//Cubic.end

//Area3D.start
/**
 * 3D面积图图形
 * @param z
 * @param deep
 * @constructor
 */
FR.Chart.Area3D = function(z, deep){
    this.z = z;
    this.deep = deep;
    this.point_list = [];
    this.type_list = [];
    this.positivePaths = []; //迎光面的path
    this.backPaths = []; //背光面的path
    this.flatPathArray = [];//一段一段平的上面
    this.topSize = 0;
    this.preSize = 0;
    this.isFirst = false;
    this.positiveStartPoint = [];
    this.positiveEndPoint = [];
    this.backStartPoint = [];
    this.backEndPoint = [];
};

$.extend(FR.Chart.Area3D.prototype, /**@class FR.Chart.Area3D*/{
    z: 0,
    deep: 0,
    projection: null,
    point_list:[],
    type_list:[],
    positivePaths:[],
    backPaths:[],
    flatPathArray:[],
    topSize:0,
    preSize:0,
    isFirst: false,
    positiveStartPoint : [],
    positiveEndPoint : [],
    backStartPoint : [],
    backEndPoint : [],

    moveTo: function(x, y){
        this.type_list.push(FR.Chart.Area3D.MOVE_TO);
        this.point_list.push(new FR.Chart.Point2D(x, y));
    },

    lineTo: function(x, y){
        this.type_list.push(FR.Chart.Area3D.LINE_TO);
        this.point_list.push(new FR.Chart.Point2D(x, y));
    },

    closePath: function(){
        var i = this.type_list.length - 1;
        while (i >= 0 && this.type_list[i] == FR.Chart.Area3D.LINE_TO) {
            i--;
        }
        if (i >=0 ) {
            this.type_list.push(FR.Chart.Area3D.LINE_TO);
            this.point_list.push(this.point_list[i]);
        }
    },

    getFrontArea: function(){
        var areaPath = new FR.Chart.GeneralPath();
        for(var i = 0; i < this.point_list.length; i++){
            var p = this.projection.projectee(this.point_list[i].x, this.point_list[i].y, this.z);
            if(i === 0){
                areaPath.moveTo(p.x, p.y);
            }else{
                areaPath.lineTo(p.x, p.y);
            }
        }

        areaPath.closePath();
        return areaPath;
    },

    getBottomPoint: function(){
        var bottomPoint  = this.point_list[0];
        var y = bottomPoint.y;
        for(var i = 0; i < this.point_list.length; i++){
            if(y < this.point_list[i].y){
                bottomPoint = this.point_list[i];
                y = bottomPoint.y;
            }
        }

        return bottomPoint;

    },

    getTopPoint: function(){
        var topPoint = this.point_list[0];
        var y = topPoint.y;
        for(var i = 0; i < this.point_list.length; i++){
            if(y > this.point_list[i].y){
                topPoint = this.point_list[i];
                y = topPoint.y;
            }
        }

        return topPoint;
    },

    //面积图折线的shape,上面的线的shape
    getLineShape: function(paths){
        var linePath = new FR.Chart.GeneralPath();
        if(paths && paths.length !== 0){
            for(var i = 0; i < paths.length; i++){
                linePath.append(paths[i], false);
            }
        }

        return linePath;
    },

    getShape: function(){
        var lineShape = this.getLineShape(this.getTopPaths());
        lineShape.append(this.getFrontArea(), false);
        return lineShape;
    },

    getInteractivePath:function(){
        return this.getShape();
    },

    getTopPaths: function(){
        var pathList = [];
        var flatPath = [];
        var lastIndex = -1;
        for(var i = 0; i < this.topSize-1; i++){
            if(this.type_list[i+1] == FR.Chart.Area3D.MOVE_TO){
                continue;
            }
            var x1 = this.point_list[i].x;
            var y1 = this.point_list[i].y;
            var x2 = this.point_list[i+1].x;
            var y2 = this.point_list[i+1].y;

            var p_1 = this.projection.projectee(x1, y1, this.z);
            var p_2 = this.projection.projectee(x2, y2, this.z);
            var p_3 = this.projection.projectee(x2, y2, this.z - this.deep);
            var p_4 = this.projection.projectee(x1, y1, this.z - this.deep);

            var area = new FR.Chart.GeneralPath();
            area.moveTo(p_1.x, p_1.y);
            area.lineTo(p_2.x, p_2.y);
            area.lineTo(p_3.x, p_3.y);
            area.lineTo(p_4.x, p_4.y);
            area.closePath();
            pathList.push(area);

            if(y1 > y2){//y1在y2下面，迎光面
                this.positivePaths.push(area);
                this.positiveStartPoint.push(this.point_list[i]);
                this.positiveEndPoint.push(this.point_list[i+1]);
            }else if(y1 < y2){//背光面
                this.backPaths.push(area);
                this.backStartPoint.push(this.point_list[i]);
                this.backEndPoint.push(this.point_list[i+1]);
            }else{
                if(lastIndex == i - 1){
                    flatPath.push(area);
                    if(i == this.topSize-2){
                        this.flatPathArray.push(flatPath);
                        flatPath = [];
                    }
                }else{
                    if(lastIndex === -1){
                        flatPath.push(area);
                        lastIndex = i;
                        continue;
                    }
                    this.flatPathArray.push(flatPath);
                    flatPath = [];
                }
                lastIndex = i;
            }

        }

        return pathList;
    },

    getAllPaths: function(){
        var pathList = [];
        for(var i = 0; i < this.point_list.length - 1; i++){
            if(this.type_list[i+1] == FR.Chart.Area3D.MOVE_TO){
                continue;
            }
            var x1 = this.point_list[i].x;
            var y1 = this.point_list[i].y;
            var x2 = this.point_list[i+1].x;
            var y2 = this.point_list[i+1].y;

            var p_1 = this.projection.projectee(x1, y1, this.z);
            var p_2 = this.projection.projectee(x2, y2, this.z);
            var p_3 = this.projection.projectee(x2, y2, this.z - this.deep);
            var p_4 = this.projection.projectee(x1, y1, this.z - this.deep);

            var area = new FR.Chart.GeneralPath();
            area.moveTo(p_1.x, p_1.y);
            area.lineTo(p_2.x, p_2.y);
            area.lineTo(p_3.x, p_3.y);
            area.lineTo(p_4.x, p_4.y);
            area.closePath();
            pathList.push(area);
        }

        return pathList;
    },

    paint: function(ctx){
        var generalPaths = this.getTopPaths();
        this.paintLeftRight(ctx);
        this.paintFlat(ctx);
        this.paintPositive(ctx);
        this.paintBack(ctx);
        if(this.isFirst){
            this.paintBottom(ctx);
        }
        this.paintFront(ctx);
    },

    paintFlat:function(ctx){
        for(var i = 0; i < this.flatPathArray.length; i++){
            var flatPaths = this.flatPathArray[i];
            this.getLineShape(flatPaths).paint(ctx);
        }
    },

    paintPositive:function(ctx){//画迎光面的path
        var rgb = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
        var hsb = FR.Chart.Color.RGB2HSB(rgb[0], rgb[1], rgb[2]);

        var start = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1], hsb[2]);
        var startColor = FR.Chart.Color.getColor(start);

        var end = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1]-0.2, hsb[2]+0.35);
        var endColor = FR.Chart.Color.getColor(end);

        for(var i = 0; i < this.positivePaths.length; i++){
            var path = this.positivePaths[i];
            var bounds = path.getBounds2D();
            if(!ctx){     return; } ctx.save();
            var grd = ctx.createLinearGradient(this.positiveStartPoint[i].x, this.positiveStartPoint[i].y,
                this.positiveEndPoint[i].x, this.positiveEndPoint[i].y);
            grd.addColorStop(0, startColor);
            grd.addColorStop(1, endColor);
            ctx.fillStyle = grd;
            path.paint(ctx);
            ctx.restore();
        }
    },

    paintBottom: function(ctx){
        var bottomShape = this.getBottomShape();
        bottomShape.paint(ctx);
    },

    getBottomShape:function(){
        var x1 = this.point_list[this.point_list.length - 2].x;
        var y1 = this.point_list[this.point_list.length - 2].y;
        var x2 = this.point_list[this.topSize].x;
        var y2 = this.point_list[this.topSize].y;

        var p_1 = this.projection.projectee(x1, y1, this.z);
        var p_2 = this.projection.projectee(x2, y2, this.z);
        var p_3 = this.projection.projectee(x2, y2, this.z - this.deep);
        var p_4 = this.projection.projectee(x1, y1, this.z - this.deep);

        var area = new FR.Chart.GeneralPath();
        area.moveTo(p_1.x, p_1.y);
        area.lineTo(p_2.x, p_2.y);
        area.lineTo(p_3.x, p_3.y);
        area.lineTo(p_4.x, p_4.y);
        area.closePath();
        return area;
    },

    paintFront: function(ctx){
        var front = this.getFrontArea();
        var top = this.getTopPoint();
        var bottom = this.getBottomPoint();

        var rgb = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
        var hsb = FR.Chart.Color.RGB2HSB(rgb[0], rgb[1], rgb[2]);

        var startColor = ctx.fillStyle;

        var frontEnd = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1] - 0.2, hsb[2] + 0.25);
        var fEndColor = FR.Chart.Color.getColor(frontEnd);
        var grd = ctx.createLinearGradient(bottom.x, bottom.y, bottom.x, top.y);
        grd.addColorStop(0, startColor);
        grd.addColorStop(1, fEndColor);
        ctx.fillStyle = grd;

        front.paint(ctx);
    },

    paintBack:function(ctx){//画背光面的path
        var rgb = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
        var hsb = FR.Chart.Color.RGB2HSB(rgb[0], rgb[1], rgb[2]);

        var start = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1]-0.05, hsb[2]+0.1);
        var startColor = FR.Chart.Color.getColor(start);

        var end = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1]-0.2, hsb[2]+0.25);
        var endColor = FR.Chart.Color.getColor(end);

        for(var i = 0; i < this.backPaths.length; i++){
            var path = this.backPaths[i];
            if(!ctx){     return; } ctx.save();
            var grd = ctx.createLinearGradient(this.backStartPoint[i].x, this.backStartPoint[i].y,
                this.backEndPoint[i].x, this.backEndPoint[i].y);
            grd.addColorStop(0, startColor);
            grd.addColorStop(1, endColor);
            ctx.fillStyle = grd;
            path.paint(ctx);
            ctx.restore();
        }
    },

    paintLeftRight: function(ctx){
        if(!ctx){     return; } ctx.save();
        var allPaths = this.getAllPaths();

        var leftPath = allPaths[allPaths.length - 1];
        var rightPath = allPaths[this.topSize - 1];

        var rgb = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
        var hsb = FR.Chart.Color.RGB2HSB(rgb[0], rgb[1], rgb[2]);

        var start = FR.Chart.Color.HSB2RGB(hsb[0], hsb[1]+0.1, hsb[2]-0.1);
        var startColor = FR.Chart.Color.getColor(start);
        ctx.fillStyle = startColor;

        leftPath.paint(ctx);
        rightPath.paint(ctx);

        ctx.restore();
    },

    draw: function(ctx){

    },

    getAnimateShape: function (step, actionModel, lastShape, paintState) {
        if (paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return [];
        }

        var pathArray = [];
        for(var i = 0; i < step; i++){
            var area3D = new FR.Chart.Area3D(this.z, this.deep);
            area3D.projection = this.projection;
            area3D.type_list = this.type_list;
            area3D.topSize = this.topSize;
            area3D.preSize = this.preSize;
            area3D.isFirst = this.isFirst;
            area3D.point_list = [];

            for(var pointIndex = 0; pointIndex < this.topSize; pointIndex++){
                var height = this.getAreaHeight(pointIndex);
                var actionHeight = FR.Chart.ChartEase.easeOutQuint(i + 1, 0, height, step);
                var prePoint = this.point_list[2 * this.topSize - 1 - pointIndex];
                var y = prePoint.y - actionHeight;
                if(pointIndex !== 0){
                    if(this.point_list[pointIndex].y == this.point_list[pointIndex-1].y){//本来就是相等的
                        y = area3D.point_list[pointIndex-1].y;
                    }
                }
                area3D.point_list.push(new FR.Chart.Point2D(prePoint.x, y));
            }

            for(var pointIndex = this.topSize; pointIndex < this.point_list.length; pointIndex++){
                area3D.point_list.push(this.point_list[pointIndex]);
            }

            pathArray[i] = area3D;
        }

        return pathArray;

    },

    getAreaHeight:function(index){
        return this.point_list[2 * this.topSize - 1 - index].y - this.point_list[index].y;
    },

    containsPoint: function(){
        return false;
    }

});

$.extend(FR.Chart.Area3D, /**@class FR.Chart.Area3D*/{
    MOVE_TO: "move_to",
    LINE_TO: "line_to"
});
//Area3D.end

//Pie3D.start
/**
 * 3D饼图图形
 * @param start_y
 * @param depth
 * @param radius
 * @constructor
 */
FR.Chart.Pie3D = function(start_y, depth, radius){
    this.start_y = start_y;  //左上角的y
    this.depth = depth;  //饼图的厚度
    this.radius = radius;  //饼图的半径
    this.info = new FR.Chart.GeneralInfo();
    this.startEnable = true;
    this.endEnable = true;
    this.paintCount = 0;
};
$.extend(FR.Chart.Pie3D.prototype, /**@class FR.Chart.Pie3D*/{
    x:0,
    y:0,
    start_y : 0,
    depth : 0,
    radius : 0,
    start: 0,
    extent: 360,
    projection : null,
    info: null,
    startEnable: true,
    endEnable: true,
    paintCount:0,

    setAttribute: function(x, y, start, extent, projection){
        this.x = x;
        this.y = y;
        this.start = start;
        this.extent = extent;
        this.projection = projection;
    },

    paint: function (ctx) {
        this.paintBack(ctx);
        this.paintTop(ctx);
    },

    paintSides: function(ctx){
        if(!ctx){     return; } ctx.save();
        var sides = this.sides();
        var rgbArray = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
        var hsbArray = FR.Chart.Color.RGB2HSB(rgbArray[0], rgbArray[1], rgbArray[2]);
        var endArray = FR.Chart.Color.HSB2RGB(hsbArray[0], hsbArray[1] + 0.1, hsbArray[2] - 0.15);
        var endColor = FR.Chart.Color.getColor(endArray);
        ctx.fillStyle = endColor;
        sides[0].paint(ctx);
        sides[1].paint(ctx);
        ctx.restore();
    },

    paintStartSide: function(ctx){
        if(!ctx){     return; } ctx.save();
        var side = this.getStartSide();
        var rgbArray = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
        var hsbArray = FR.Chart.Color.RGB2HSB(rgbArray[0], rgbArray[1], rgbArray[2]);
        var endArray = FR.Chart.Color.HSB2RGB(hsbArray[0], hsbArray[1] + 0.1, hsbArray[2] - 0.15);
        var endColor = FR.Chart.Color.getColor(endArray);
        ctx.fillStyle = endColor;
        side.paint(ctx)
        ctx.restore();

    },

    paintEndSide: function(ctx){
        if(!ctx){     return; } ctx.save();
        var side = this.getEndSide();
        var rgbArray = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
        var hsbArray = FR.Chart.Color.RGB2HSB(rgbArray[0], rgbArray[1], rgbArray[2]);
        var endArray = FR.Chart.Color.HSB2RGB(hsbArray[0], hsbArray[1] + 0.1, hsbArray[2] - 0.15);
        var endColor = FR.Chart.Color.getColor(endArray);
        ctx.fillStyle = endColor;
        side.paint(ctx)
        ctx.restore();
    },

    paintTop: function(ctx){
        if(!ctx){     return; } ctx.save();
        var top = this.top();
        //相当于压缩y方向上的像素
        var r = top.width / 2;
        var radioX = top.width / 2 / r;
        var radioY = top.height / 2 / r;
        ctx.scale(radioX, radioY);
        //从圆心开始向四周做径向渐变
        var rgbArray = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
        var hsbArray = FR.Chart.Color.RGB2HSB(rgbArray[0], rgbArray[1], rgbArray[2]);
        var startArray = FR.Chart.Color.HSB2RGB(hsbArray[0], hsbArray[1] - 0.2, hsbArray[2] + 0.25);
        var startColor = FR.Chart.Color.getColor(startArray);
        var endColor = ctx.fillStyle;
        var grd = ctx.createRadialGradient(top.x / radioX, top.y / radioY, 0, top.x / radioX, top.y / radioY, r);
        grd.addColorStop(0, startColor);
        grd.addColorStop(1, endColor);
        ctx.fillStyle= grd;
        var start = (180 - top.start - top.extent + 360) % 360;
        var arc = new FR.Chart.Arc2D(top.x / radioX, top.y / radioY, r, start, top.extent);
        arc.paint(ctx);
        ctx.restore();
    },

    //pie3d的前面的部分
    getFrontPie3d:function(){
        var frontPie = [];

        var theta = this.getTheta();
        var arc_top = this.top();
        var arc_bottom = this.bottom();
        var start = arc_top.start % 360;
        var end = (arc_top.start + arc_top.extent) % 360;

        //右边界是theta, 左边界是180 - theta
        var endAnlge = (theta < 0 ? theta + 360 : theta);

        if (this.isAngleAtFront(start)) {  // 正面开始
            if (!this.isAngleAtBack(end)) {  // 正面结束
                if (this.extent < (180 + 2 * theta)) {  // 只在前面，没有绕到后面
                    var pie = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
                    pie.x = this.x;
                    pie.y = this.y;
                    pie.start = this.start;
                    pie.extent = this.extent;
                    pie.projection = this.projection;
                    pie.info = this.info;
                    frontPie.push(pie);
                    return frontPie;
                } else {  // segments wraps right around the back...
                    var pie = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
                    pie.x = this.x;
                    pie.y = this.y;
                    pie.start = this.start
                    pie.extent = (360 - this.start) % 360;
                    pie.projection = this.projection;
                    pie.info = this.info;
                    pie.startEnable = false;

                    var secondPie = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
                    secondPie.x = this.x;
                    secondPie.y = this.y;
                    secondPie.start = 180;
                    secondPie.extent = (this.start + this.extent) % 360 - 180;
                    secondPie.projection = this.projection;
                    secondPie.info = this.info;
                    secondPie.endEnable = false;

                    frontPie.push(pie);
                    frontPie.push(secondPie);
                    return frontPie;
                }
            } else {  // segments starts at front and finishes at back...
                var pie = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
                pie.x = this.x;
                pie.y = this.y;
                pie.start = this.start;
                pie.extent = (360 - this.start % 360);
                pie.projection = this.projection;
                pie.info = this.info;
                frontPie.push(pie);
                return frontPie;
            }
        }
        else {  // segment starts at back
            if (!this.isAngleAtFront(end)) {
                if (this.extent < (180 - 2 * theta)) {  // and finishes at back
                    return frontPie;
                } else {  // starts at back and wraps right around to the back
                    var pie = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
                    pie.x = this.x;
                    pie.y = this.y;
                    pie.start = 180;
                    pie.extent = 180;
                    pie.projection = this.projection;
                    pie.info = this.info;
                    frontPie.push(pie);
                    return frontPie;
                }
            } else {  // starts at the back and finishes at the front
                // (wrapping the left side)
                var pie = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
                pie.x = this.x;
                pie.y = this.y;
                pie.start = 180;
                pie.extent = (this.start + this.extent) % 360 - 180;
                pie.projection = this.projection;
                pie.info = this.info;
                frontPie.push(pie);
                return frontPie;
            }
        }
    },


    //pie3d的后面的部分
    getBackPie3d: function(){

        var backPie = [];

        var theta = this.getTheta();
        var arc_top = this.top();
        var arc_bottom = this.bottom();
        var start = arc_top.start % 360;
        var end = (arc_top.start + arc_top.extent) % 360;

        if (this.isAngleAtFront(start)) {  // 正面开始
            if (!this.isAngleAtBack(end)) {  // 正面结束
                if (this.extent < (180 + 2 * theta)) {
                    return backPie;
                } else {  // 正面开始，正面结束，并且是从后面绕过来
                    var pie = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
                    pie.x = this.x;
                    pie.y = this.y;
                    pie.start = 0;
                    pie.extent = 180;
                    pie.projection = this.projection;
                    pie.info = this.info;
                    pie.startEnable = false;
                    pie.endEnable = false;
                    backPie.push(pie);
                    return backPie;
                }
            } else {  // 背面结束
                var pie = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
                pie.x = this.x;
                pie.y = this.y;
                pie.start = 0;
                pie.extent = this.extent - (360 - this.start % 360);
                pie.projection = this.projection;
                pie.info = this.info;
                pie.startEnable = false;
                backPie.push(pie);

                return backPie;
            }
        }
        else {  // 背面开始
            if (!this.isAngleAtFront(end)) { //背面结束
                if (this.extent < (180 - 2 * theta)) {  // 只在背面
                    var pie = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
                    pie.x = this.x;
                    pie.y = this.y;
                    pie.start = this.start;
                    pie.extent = this.extent;
                    pie.projection = this.projection;
                    pie.info = this.info;
                    backPie.push(pie);
                    return backPie;

                } else {  // 背面开始，背面结束，并且是从前面绕过来
                    var pie = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
                    pie.x = this.x;
                    pie.y = this.y;
                    pie.start = 360;
                    pie.extent = (this.start + this.extent - 360) % 360;
                    pie.projection = this.projection;
                    pie.info = this.info;
                    pie.startEnable = false;

                    var secondPie = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
                    secondPie.x = this.x;
                    secondPie.y = this.y;
                    secondPie.start = this.start;
                    secondPie.extent = (180 - this.start) % 360;
                    secondPie.projection = this.projection;
                    secondPie.info = this.info;
                    secondPie.endEnable = false;

                    backPie.push(pie);
                    backPie.push(secondPie);
                    return backPie;
                }
            } else {  //正面结束
                // (wrapping the left side)
                var pie = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
                pie.x = this.x;
                pie.y = this.y;
                pie.start = this.start;
                pie.extent = (180 - this.start) % 360;
                pie.projection = this.projection;
                pie.info = this.info;
                pie.endEnable = false;
                backPie.push(pie);
                return backPie;
            }
        }

    },

    paintNegtiveBack: function(ctx){
        if(!ctx){     return; } ctx.save();
        var arc_top = this.top();
        var arc_bottom = this.bottom();
        var start = arc_top.start % 360;
        var end = (arc_top.start + arc_top.extent) % 360;
        var theta = this.getTheta();
        var front_side = new FR.Chart.GeneralPath();


        if (this.isAngleAtFront(start)) {  // 正面开始
            if (!this.isAngleAtBack(end)) {  // 正面结束
                if (this.extent < (180 + 2 * theta)) {  // 只在前面，没有绕到后面
                } else {  // 正面开始，正面结束，并且是从后面绕过来
                    front_side = this.getSideArea(arc_top, arc_bottom, theta, 180-theta);
                }
            } else {  // 背面结束
                front_side = this.getSideArea(arc_top, arc_bottom, theta, end);
            }
        }
        else {  // 背面开始
            if (!this.isAngleAtFront(end)) { //背面结束
                if (this.extent < (180 - 2 * theta)) {  // 只在背面
                    front_side = this.getSideArea(arc_top, arc_bottom, start, end);
                } else {  // 背面开始，背面结束，并且是从前面绕过来
                    var side1 = this.getSideArea(arc_top, arc_bottom, theta, end);
                    var side2 = this.getSideArea(arc_top, arc_bottom, start, 180 - theta);
                    front_side.append(side1, false);
                    front_side.append(side2, false);
                }
            } else {  //正面结束
                // (wrapping the left side)
                front_side = this.getSideArea(arc_top, arc_bottom, start, 180 - theta);
            }
        }

        front_side.paint(ctx);
        ctx.restore();
    },

    paintBack: function(ctx){
        if(!ctx){     return; } ctx.save();
        var rgbArray = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
        var hsbArray = FR.Chart.Color.RGB2HSB(rgbArray[0], rgbArray[1], rgbArray[2]);
        var endArray = FR.Chart.Color.HSB2RGB(hsbArray[0], hsbArray[1] + 0.1, hsbArray[2] - 0.1);
        var endColor = FR.Chart.Color.getColor(endArray);
        ctx.fillStyle = endColor;

        var arc_top = this.top();
        var arc_bottom = this.bottom();
        var start = arc_top.start % 360;
        var end = (arc_top.start + arc_top.extent) % 360;
        var theta = this.getTheta();
        var front_side = new FR.Chart.GeneralPath();


        //右边界是theta, 左边界是180 - theta
        var endAnlge = (theta < 0 ? theta + 360 : theta);

        if (this.isAngleAtFront(start)) {  // 正面开始
            if (!this.isAngleAtBack(end)) {  // 正面结束
                if (this.extent < (180 + 2 * theta)) {  // 只在前面，没有绕到后面
                    front_side = this.getSideArea(arc_top, arc_bottom, start, end);
                } else {  // segments wraps right around the back...
                    var side1 = this.getSideArea(arc_top, arc_bottom, 180 - theta, end);
                    var side2 = this.getSideArea(arc_top, arc_bottom, start, endAnlge);
                    front_side.append(side1, false);
                    front_side.append(side2, false);
                }
            } else {  // segments starts at front and finishes at back...
                front_side = this.getSideArea(arc_top, arc_bottom, start, endAnlge);
            }
        }
        else {  // segment starts at back
            if (!this.isAngleAtFront(end)) {
                if (this.extent < (180 - 2 * theta)) {  // and finishes at back
                } else {  // starts at back and wraps right around to the
                    front_side = this.getSideArea(arc_top, arc_bottom, 180 - theta, endAnlge);
                }
            } else {  // starts at the back and finishes at the front
                // (wrapping the left side)
                front_side = this.getSideArea(arc_top, arc_bottom, 180 - theta, end);
            }
        }

        front_side.paint(ctx);
        ctx.restore();
    },

    getSideArea: function(top, bottom, startAngle, endAngle){
        var side = new FR.Chart.GeneralPath();
        var topArc = top.getShapeWithStartEnd(startAngle, endAngle, false);
        var bottomArc = bottom.getCounterclockwiseShapeWithStartEnd(startAngle, endAngle, false);
        var topStart = top.getPointWithAngle(startAngle);
        var bottomEnd = bottom.getPointWithAngle(endAngle);
        side.moveTo(topStart.x, topStart.y);
        side.append(topArc, true);
        side.lineTo(bottomEnd.x, bottomEnd.y);
        side.append(bottomArc, true);
        side.closePath();
        return side;
    },

    getTheta: function() {
        /*
         *  theta 为负值，-90 ~ 90
         */
        return Math.asin(- this.get_sin_theta()) * 180 / Math.PI;
    },

    isAngleAtFront: function(angle) {
        angle = angle / 180.0 * Math.PI;
        return (Math.sin(angle) < - this.get_sin_theta());
    },


    isAngleAtBack: function(angle) {
        return !this.isAngleAtFront(angle);
    },

    get_sin_theta : function() {
        var elevation = (this.projection.elevation) / 180.0 * Math.PI;
        var focus = this.projection.focus;

        return this.radius * (Math.pow(Math.sin(elevation), 2)) / (Math.cos(elevation) * focus);
    },

    //暂时不要边框
    draw: function (ctx) {
//        this.top().draw(ctx);
//        this.bottom().draw(ctx);
//        this.back().draw(ctx);
//        var sides = this.sides();
//        if(sides && sides.length === 2){
//            sides[0].draw(ctx);
//            sides[1].draw(ctx);
//        }
    },

    getActionDefaultAnimateShape:function(step, paintState){
        var arcArray = [];

        for (var i = 0; i < step; i++) {
            var pie3D = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
            pie3D.x = this.x;
            pie3D.y = this.y;
            pie3D.start = this.start;
            pie3D.extent = FR.Chart.ChartEase.easeOutQuint(i + 1, 0, this.extent, step);
            pie3D.projection = this.projection;
            arcArray.push(pie3D);
        }

        return arcArray;
    },

    getActionChangeAnimateShape: function (step, lastShape, paintState) {
        if (paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return [];
        }
        if (paintState == FR.Chart.ChartPaintConstants.STATE_APPEAR) {
            return this.getActionDefaultAnimateShape(step, paintState);
        }

        var arcArray = [];
        for (var i = 0; i < step; i++) {
            var lastStart = lastShape.start;
            var lastExtent = lastShape.extent;
            var thisStart = this.start;
            var thisExtent = this.extent;

            if (Math.abs(thisStart - lastStart) > Math.abs(thisStart - lastStart + 360)) {
                thisStart += 360
            }
            if (Math.abs(thisStart - lastStart) > Math.abs(thisStart - lastStart - 360)) {
                thisStart -= 360
            }

            var x = FR.Chart.ChartEase.easeOutQuint(i + 1, lastShape.x, this.x - lastShape.x, step);
            var y = FR.Chart.ChartEase.easeOutQuint(i + 1, lastShape.y, this.y - lastShape.y, step);

            var pie3D = new FR.Chart.Pie3D(this.start_y, this.depth, this.radius);
            pie3D.x = x;
            pie3D.y = y;
            pie3D.start = this.start;
            pie3D.extent = this.extent;
            pie3D.projection = this.projection;

            arcArray.push(pie3D);
        }

        return arcArray;
    },

    /**
     * 返回动态
     * @param step 动态步骤.
     * @param actionModel 动态方式
     * @param lastShape 上一状态shape
     * @param paintState 当前画法.
     * @returns {Array} 动态数组.
     */
    getAnimateShape: function (step, actionModel, lastShape, paintState) {
        if (lastShape && actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE) {
            return this.getActionChangeAnimateShape(step, lastShape, paintState);
        } else {
            return this.getActionDefaultAnimateShape(step);
        }
    },

    /**
     * 判断是否包含 坐标(x,y)
     * @param x 坐标x
     * @param y 坐标y
     * @returns {扩大响应区域|*|Boolean} 判断是否包含.
     */
    containsPoint:function(x, y){
        return this.top().getShape().containsPoint(x, y);
    },

    /**
     * 顶面
     * @returns {FR.Chart.Arc2D} 顶面.
     */
    top: function(){
        return this.getArc2D(0, true);
    },

    /**
     * 底面
     * @returns {.Chart.Arc2D} 底面.
     */
    bottom: function(){
        return this.getArc2D(this.depth, true);
    },

    /**
     * 背面
     * @returns {FR.Chart.GeneralPath}
     */
    back: function(){
        var top = this.getArc2D(0, false);
        var bottom = this.getArc2D(this.depth, false);

        var back = new FR.Chart.GeneralPath();
        var topStart = top.getStartPoint();
        var topEnd = top.getEndPoint();
        var bottomEnd = bottom.getEndPoint();
        var bottomStart = bottom.getStartPoint();
        back.moveTo(topStart.x, topStart.y);
        back.append(top.getShape(), true);
        back.lineTo(bottomEnd.x, bottomEnd.y);
        back.append(bottom.getCounterclockwiseShape(), true);
        back.closePath();
        return back;
    },

    /**
     * 开始面.
     * @returns {FR.Chart.GeneralPath}
     */
    getStartSide: function(){
        var top = this.getArc2D(0, false);
        var bottom = this.getArc2D(this.depth, false);

        var topStart = top.getStartPoint();
        var topCenter = top.getCenterPoint();

        var bottomStart = bottom.getStartPoint();
        var bottomCenter = bottom.getCenterPoint();

        var first = new FR.Chart.GeneralPath();
        first.moveTo(topStart.x, topStart.y);
        first.lineTo(bottomStart.x, bottomStart.y);
        first.lineTo(bottomCenter.x, bottomCenter.y);
        first.lineTo(topCenter.x, topCenter.y);
        first.closePath();

        return first;
    },

    /**
     * 结束面.
     * @returns {FR.Chart.GeneralPath}
     */
    getEndSide: function(){
        var top = this.getArc2D(0, false);
        var bottom = this.getArc2D(this.depth, false);

        var topEnd = top.getEndPoint();
        var topCenter = top.getCenterPoint();

        var bottomEnd = bottom.getEndPoint();
        var bottomCenter = bottom.getCenterPoint();

        var second = new FR.Chart.GeneralPath();
        second.moveTo(topEnd.x, topEnd.y);
        second.lineTo(bottomEnd.x, bottomEnd.y);
        second.lineTo(bottomCenter.x, bottomCenter.y);
        second.lineTo(topCenter.x, topCenter.y);
        second.closePath();

        return second;
    },

    /**
     * 起始 结束面.
     * @returns {Array}
     * @private
     */
    sides: function(){
        var sizes = [];
        sizes.push(this.getStartSide());
        sizes.push(this.getEndSide());
        return sizes;
    },

    /**
     * 返回椭圆的一部分, type表示是否为Arc2D.PIE
     * @param depth 深度
     * @param type
     * @returns {*}
     */
    getArc2D:function(depth, type) {
        var arc = this.projection.projecteeArc(this.start_y + depth, this.radius, this.start, this.extent, !!type);
        arc.x += this.x + arc.width / 2;
        arc.y += this.y + arc.height / 2;
        return arc;
    },

    /**
     * 返回顶层弧线的中间点
     * @returns {FR.Chart.Point2D} 中间点.
     */
    getTopArcCenterPoint:function(){
        var top = this.top();
        var middleEdge =  top.getMiddlePoint();
        return new FR.Chart.Point2D((top.x + middleEdge.x) / 2, (top.y + middleEdge.y) / 2);
    }
});
//pie3D.end

//pie3dSide.start
/**
 * 三维饼图 面
 *          @example
 *          var 3dside = new FR.Chart.Pie3DSide();
 *          @class FR.Chart.Pie3DSide
 * @param shapeGlyph
 * @param isStart 是否为开始面
 * @constructor
 */
FR.Chart.Pie3DSide = function(shapeGlyph, isStart){
    if(shapeGlyph instanceof  FR.Chart.ShapeGlyph){
        this.shapeGlyph = shapeGlyph;
        this.cylinder = shapeGlyph.shape;
    }else{
        this.cylinder = shapeGlyph;
    }
    this.isStart = isStart;
};
//pie3dSide.end

//ellipse.start 椭圆的形状(x,y)圆心,type是否闭合
/**
 * 椭圆形状
 *     @example
 *     var ellipse = new FR.Chart.Ellipse();
 *     @class FR.Chart.Ellipse
 *
 * @param x 椭圆的中心位置x
 * @param y 椭圆的中心位置y
 * @param width 椭圆宽度
 * @param height 椭圆高度
 * @param start 起始角度
 * @param extent 角度范围
 * @param type 是否闭合
 * @constructor
 */
FR.Chart.Ellipse = function(x, y, width, height, start, extent, type){
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
    this.start = start;
    this.extent = extent;
    this.type = !!type;
};
$.extend(FR.Chart.Ellipse.prototype,{
    x:0,
    y:0,
    width:0,
    height:0,
    start:0,
    extent:0,
    type: true,

    /**
     * 画出椭圆填充
     * @param ctx 画布
     */
    paint:function(ctx){
        this.getShape().paint(ctx);
    },

    /**
     * 画出椭圆边框
     * @param ctx 画布
     */
    draw: function(ctx){
        this.getShape().draw(ctx);
    },

    /**
     * 返回 shape形状.
     * @returns {FR.Chart.GeneralPath}
     */
    getShape: function(){
        return this.getShapeWithStartEnd(this.start, this.extent + this.start, this.type);
    },

    /**
     * 逆时针方向画一段弧线
     * @returns {FR.Chart.GeneralPath}
     */
    getCounterclockwiseShape: function(){
        return this.getCounterclockwiseShapeWithStartEnd(this.start, this.extent + this.start, false);
    },

    /**
     * 顺时针方向画一段弧线
     * @returns {FR.Chart.GeneralPath}
     */
    getClockwiseShape: function(){
        return this.getShapeWithStartEnd(this.start, this.extent + this.start, false);
    },

    /**
     * 返回起始结束 连接的shape
     * @param startAngle 起始角度
     * @param endAngle 结束角度
     * @param type 类型 type是否闭合
     * @returns {FR.Chart.GeneralPath} 路径.
     */
    getShapeWithStartEnd: function(startAngle, endAngle, type){
        var shape = new FR.Chart.GeneralPath();
        var inc = Math.PI / 90;
        var start = (startAngle / 180) * Math.PI + Math.PI
        var end = ((endAngle - startAngle) / 180) * Math.PI + start;
        var angle = start;
        var a = this.width / 2;
        var b = this.height / 2;
        type = !!type;


        if(type){
            shape.moveTo(this.x, this.y);
        }else{
            shape.moveTo(this.x + a * Math.cos(start), this.y + b * Math.sin(start));
        }

        while (angle <= end) {
            shape.lineTo(this.x + a * Math.cos(angle), this.y + (b * Math.sin(angle)));
            angle += inc;
        }

        shape.lineTo(this.x + a * Math.cos(end), this.y + (b * Math.sin(end)));

        if(type){
            shape.closePath();
        }

        return shape;
    },

    /**
     * TODO 代码质量
     * 逆时针 画一段shape
     * @param startAngle 开始角度
     * @param endAngle 结束角度
     * @param type 类型 type是否闭合
     * @returns {FR.Chart.GeneralPath}
     */
    getCounterclockwiseShapeWithStartEnd: function(startAngle, endAngle, type){
        var shape = new FR.Chart.GeneralPath();
        var inc = Math.PI / 90;
        var start = (startAngle / 180) * Math.PI + Math.PI;
        var end = ((endAngle - startAngle) / 180) * Math.PI + start;
        var angle = end;
        var a = this.width / 2;
        var b = this.height / 2;
        type = !!type;


        if(type){
            shape.moveTo(this.x, this.y);
        }else{
            shape.moveTo(this.x + a * Math.cos(end), this.y + b * Math.sin(end));
        }

        while (angle >= start) {
            shape.lineTo(this.x + a * Math.cos(angle), this.y + (b * Math.sin(angle)));
            angle -= inc;
        }

        shape.lineTo(this.x + a * Math.cos(start), this.y + (b * Math.sin(start)));

        if(type){
            shape.closePath();
        }

        return shape;

    },

    /**
     * 根据角度 返回弧线上的 点
     * @param angle 角度
     * @returns {FR.Chart.Point2D} 弧线点
     */
    getPointWithAngle: function(angle){
        var a = this.width / 2;
        var b = this.height / 2;
        angle = (angle / 180) * Math.PI + Math.PI;
        return new FR.Chart.Point2D(this.x + a * Math.cos(angle), this.y + b * Math.sin(angle));
    },

    /**
     * 返回 起始点
     * @returns {FR.Chart.Point2D} 起始点
     */
    getStartPoint: function(){
        return this.getPointWithAngle(this.start);
    },

    /**
     * 返回中间 点
     * @returns {FR.Chart.Point2D} 中间点.
     */
    getMiddlePoint: function(){
        return this.getPointWithAngle(this.start + this.extent / 2);
    },

    /**
     * 返回结束 点
     * @returns {FR.Chart.Point2D} 结束点.
     */
    getEndPoint: function(){
        return this.getPointWithAngle(this.start + this.extent);
    },

    /**
     * 返回中心点
     * @returns {FR.Chart.Point2D} 中心点.
     */
    getCenterPoint: function(){
        return new FR.Chart.Point2D(this.x, this.y);
    },

    /**
     * 返回中间角度值
     * @returns {Number} 返回中间角度
     */
    getMiddleAngle: function(){
        return this.start + this.extent / 2;
    },

    /**
     * 最右边的点
     * @returns {FR.Chart.Point2D}
     * TODO 删掉
     */
    getMaxXPoint: function(){
        var a = this.width / 2;
        return new FR.Chart.Point2D(this.x + a, this.y);
    },

    /**
     * 最左边的点
     * @returns {FR.Chart.Point2D}
     * TODO 删掉.
     */
    getMinXPoint: function(){
        var a = this.width / 2;
        return new FR.Chart.Point2D(this.x - a, this.y);
    }

});
//ellipse.end

//Ring3D.start 相当于是两个Pie3D可以计算出来的形状
/**
 * 三维圆环扇形
 *     @example
 *     var ring3d = new FR.Chart.Ring3D();
 *
 *     @class FR.Chart.Ring3D
 *
 * @param projection 三维信息.
 * @property {Array} outerCylinderList 外三维环形数组
 * @property {Array} innerCylinderList 内三维环形数组
 * @constructor
 */
FR.Chart.Ring3D = function(projection){
    this.projection = projection;
    this.outerCylinderList = [];
    this.innerCylinderList = [];
    this.origin = []; //切割之前的完整的pie形状
};

$.extend(FR.Chart.Ring3D.prototype,{
    projection:null,
    outerCylinderList:null,
    innerCylinderList:null,

    /**
     * 画 暂时无. 因为每个层面是分开步骤画的.
     * @param ctx 画布.
     */
    paint: function(ctx){

    },

    /**
     * 画边框.
     * @param ctx 画布.
     */
    draw: function(ctx){
//        this.top().draw(ctx);
//        this.front().draw(ctx);
//        this.back().draw(ctx);
//        this.getStartSide().draw(ctx);
//        this.getEndSide().draw(ctx);
    },

    /**
     * 画顶层.
     * @param ctx 画布.
     */
    drawTop: function(ctx){
        if(!ctx){     return; } ctx.save();
        for (var i = 0 ; i < this.getCylinderCount(); i ++) {
            var pie = new FR.Chart.Ring3D(this.projection);
            var outerCylinder = this.getOuterCylinder(i);
            var fillStyle = outerCylinder.info.getBackground().color;
            pie.addCylinder(this.getOuterCylinder(i), this.getInnerCylinder(i));
            ctx.fillStyle = fillStyle;
            var rgbArray = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
            var hsbArray = FR.Chart.Color.RGB2HSB(rgbArray[0], rgbArray[1], rgbArray[2]);
            var endArray = FR.Chart.Color.HSB2RGB(hsbArray[0], hsbArray[1] - 0.2, hsbArray[2] + 0.25);
            var endColor = FR.Chart.Color.getColor(endArray);

            var top = this.origin[i].top();
            var startPoint = top.getMiddlePoint();
            var grd=ctx.createLinearGradient(top.x,top.y, startPoint.x, startPoint.y);

            grd.addColorStop(0, endColor);
            grd.addColorStop(1, ctx.fillStyle);
            ctx.fillStyle=grd;
            pie.top().paint(ctx);
        }
        ctx.restore();
    },

    /**
     * 小圆的back (内圆环)
     * @param ctx
     */
    drawBack:function(ctx){
        if(!ctx){     return; } ctx.save();
        for (var i = 0 ; i < this.getCylinderCount(); i ++) {
            var cylinder = this.getInnerCylinder(i);
            var fillStyle = cylinder.info.getBackground().color;
            ctx.fillStyle = fillStyle;
            cylinder.paintNegtiveBack(ctx);
        }
        ctx.restore();
    },

    /**
     * 大圆的back
     * @param ctx 画布
     */
    drawPositive:function(ctx){
        if(!ctx){     return; } ctx.save();
        for (var i = 0 ; i < this.getCylinderCount(); i++) {
            var cylinder = this.getOuterCylinder(i);
            var fillStyle = cylinder.info.getBackground().color;
            ctx.fillStyle = fillStyle;
            cylinder.paintBack(ctx);
        }
        ctx.restore();
    },

    /**
     * 画出 扇面.
     * @param ctx 画布
     */
    drawSide: function(ctx){
        if(!ctx){     return; } ctx.save();
        var outerSideList = [];
        var innerSideList = [];
        for(var i = 0; i < this.getCylinderCount(); i++){
            if(this.getOuterCylinder(i).startEnable){
                outerSideList.push(new FR.Chart.Pie3DSide(this.getOuterCylinder(i), true));
                innerSideList.push(new FR.Chart.Pie3DSide(this.getInnerCylinder(i), true));
            }

            if(this.getOuterCylinder(i).endEnable){
                outerSideList.push(new FR.Chart.Pie3DSide(this.getOuterCylinder(i), false));
                innerSideList.push(new FR.Chart.Pie3DSide(this.getInnerCylinder(i), false));
            }
        }

        outerSideList.sort(FR.Chart.ChartUtils.sortPie3DSide);
        innerSideList.sort(FR.Chart.ChartUtils.sortPie3DSide);

        for(var index = 0; index < outerSideList.length; index++){
            var outerSide = outerSideList[index];
            var innerSide = innerSideList[index];

            var ring3D = new FR.Chart.Ring3D(this.projection);
            ring3D.addCylinder(outerSide.cylinder, innerSide.cylinder);
            var fillStyle = outerSide.cylinder.info.getBackground().color;
            ctx.fillStyle = fillStyle;
            if(outerSide.isStart && outerSide.cylinder.startEnable){
                ring3D.paintStartSide(ctx);
            }else if(!outerSide.isStart && outerSide.cylinder.endEnable){
                ring3D.paintEndSide(ctx);
            }
        }
        ctx.restore();
    },

    /**
     * 返回 三维扇形个数.
     * @returns {Number} 个数.
     * @private
     */
    getCylinderCount:function() {
        return this.innerCylinderList.length;
    },

    /**
     * 返回三维动态步骤  不支持.
     * @param step 动态步骤
     * @param actionModel 动态方式
     * @param lastShape 上一步shape
     * @param paintState 画法状态
     * @returns {Array} 动态数组
     */
    getAnimateShape: function (step, actionModel, lastShape, paintState) {
        var ringArray = [];
        return ringArray;
    },

    /**
     * top是两个pie3D的top的重叠部分
     * @returns {FR.Chart.GeneralPath} 返回路径
     */
    top: function(){
        var innerCylinder = this.getInnerCylinder(0);
        var outerCylinder = this.getOuterCylinder(0);
        var innerTop = innerCylinder.top();
        var outerTop = outerCylinder.top();

        var outerStart = outerTop.getStartPoint();
        var innerEnd = innerTop.getEndPoint();

        var top = new FR.Chart.GeneralPath();
        top.moveTo(outerStart.x, outerStart.y);
        top.append(outerTop.getClockwiseShape(), true);
        top.lineTo(innerEnd.x, innerEnd.y);
        top.append(innerTop.getCounterclockwiseShape(), true);
        top.closePath();

        return top;
    },

    /**
     * 圆环小半径的饼状的back
     * @returns {*|*|void|void}
     * TODO 删掉 感觉没用到.
     */
    front: function(){
        var innerCylinder = this.getInnerCylinder(0);
        return innerCylinder.back();
    },

    /**
     * 圆环大半径的饼状的back
     * @returns {*|void}
     * TODO 删掉 感觉有用到
     */
    back: function(){
        var outerCylinder = this.getOuterCylinder(0);
        return outerCylinder.back();
    },

    /**
     * 画开始面.
     * @param ctx
     */
    paintStartSide: function(ctx){
        if(!ctx){     return; } ctx.save();
        var rgbArray = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
        var hsbArray = FR.Chart.Color.RGB2HSB(rgbArray[0], rgbArray[1], rgbArray[2]);
        var endArray = FR.Chart.Color.HSB2RGB(hsbArray[0], hsbArray[1] + 0.1, hsbArray[2] - 0.15);
        var endColor = FR.Chart.Color.getColor(endArray);
        ctx.fillStyle = endColor;
        var path = this.getStartSide();
        path.paint(ctx);
        ctx.restore();
    },

    /**
     * 返回开始面
     * @returns {FR.Chart.GeneralPath}
     * @private
     */
    getStartSide:function(){

        var outerCylinder = this.getOuterCylinder(0);
        var innerCylinder = this.getInnerCylinder(0);

        var topStart = outerCylinder.top().getStartPoint();
        var topEnd = innerCylinder.top().getStartPoint();

        var bottomStart = outerCylinder.bottom().getStartPoint();
        var bottomEnd = innerCylinder.bottom().getStartPoint();

        var first = new FR.Chart.GeneralPath();
        first.moveTo(topStart.x, topStart.y);
        first.lineTo(bottomStart.x, bottomStart.y);
        first.lineTo(bottomEnd.x, bottomEnd.y);
        first.lineTo(topEnd.x, topEnd.y);
        first.closePath();

        return first;
    },

    /**
     * 画出结束面
     * @param ctx 画布
     */
    paintEndSide: function(ctx){
        if(!ctx){     return; } ctx.save();
        var rgbArray = FR.Chart.Color.Hex2RGB(ctx.fillStyle);
        var hsbArray = FR.Chart.Color.RGB2HSB(rgbArray[0], rgbArray[1], rgbArray[2]);
        var endArray = FR.Chart.Color.HSB2RGB(hsbArray[0], hsbArray[1] + 0.1, hsbArray[2] - 0.15);
        var endColor = FR.Chart.Color.getColor(endArray);
        ctx.fillStyle = endColor;
        var path = this.getEndSide();
        path.paint(ctx);
        ctx.restore();
    },

    /**
     * 返回结束面
     * @returns {FR.Chart.GeneralPath}
     * @private
     */
    getEndSide: function(){
        var outerCylinder = this.getOuterCylinder(0);
        var innerCylinder = this.getInnerCylinder(0);

        var topStart = outerCylinder.top().getEndPoint();
        var topEnd = innerCylinder.top().getEndPoint();

        var bottomStart = outerCylinder.bottom().getEndPoint();
        var bottomEnd = innerCylinder.bottom().getEndPoint();

        var second = new FR.Chart.GeneralPath();
        second.moveTo(topStart.x, topStart.y);
        second.lineTo(bottomStart.x, bottomStart.y);
        second.lineTo(bottomEnd.x, bottomEnd.y);
        second.lineTo(topEnd.x, topEnd.y);
        second.closePath();

        return second;
    },

    /**
     * 计算背面的三维圆环
     * @returns {FR.Chart.Ring3D} 三维圆环扇形
     */
    calculateBackHalf: function(){
        var backRing = new FR.Chart.Ring3D(this.projection);
        for(var i = 0; i< this.getCylinderCount(); i++){
            var outerBack = this.getOuterCylinder(i).getBackPie3d();
            var innerBack = this.getInnerCylinder(i).getBackPie3d();
            if(outerBack.length == innerBack.length){
                for(var j = 0; j < outerBack.length; j++){
                    backRing.addCylinder(outerBack[j], innerBack[j]);
                    backRing.addOrigin(this.getOuterCylinder(i));
                }
            }
        }
        return backRing;
    },

    /**
     * 计算正面的三维扇形
     * @returns {FR.Chart.Ring3D}
     */
    calculateFrontHalf:function(){
        var frontRing = new FR.Chart.Ring3D(this.projection);
        for(var i = 0; i< this.getCylinderCount(); i++){
            var outerBack = this.getOuterCylinder(i).getFrontPie3d();
            var innerBack = this.getInnerCylinder(i).getFrontPie3d();
            if(outerBack.length == innerBack.length){
                for(var j = 0; j < outerBack.length; j++){
                    frontRing.addCylinder(outerBack[j], innerBack[j]);
                    frontRing.addOrigin(this.getOuterCylinder(i));
                }
            }
        }
        return frontRing;
    },

    /**
     * 添加外三维扇形
     * @param cylinder 三维扇形
     */
    addOuterCylinder: function(cylinder){
        this.outerCylinderList.push(cylinder);
    },

    /**
     * 添加 内三维扇形
     * @param cylinder 三维扇形
     */
    addInnerCylinder: function(cylinder){
        this.innerCylinderList.push(cylinder);
    },

    /**
     * 返回外 三维扇形
     * @param index 序号
     * @returns {Object} 三维扇形
     */
    getOuterCylinder: function(index){
        if(this.outerCylinderList && index < this.outerCylinderList.length){
            return this.outerCylinderList[index];
        }
    },

    /**
     * 返回内 三维扇形
     * @param index 序号
     * @returns {Object} 三维扇形
     */
    getInnerCylinder: function(index){
        if(this.innerCylinderList && index < this.innerCylinderList.length){
            return this.innerCylinderList[index];
        }
    },

    /**
     * 返回三维圆环的开始角度
     * @returns {Number} 开始角度
     */
    getRingStartAngle:function(){
        return this.getOuterCylinder(0).top().start;
    },

    /**
     * 返回三维圆环的结束角度
     * @returns {Number} 结束角度
     */
    getRingEndAngle:function(){
        var top = this.getOuterCylinder(0).top();
        return top.start + top.extent;
    },

    /**
     * 添加三维扇形
     * @param outerCylinder 外部扇形
     * @param innerCylinder 内部扇形
     */
    addCylinder: function(outerCylinder, innerCylinder){
        this.outerCylinderList.push(outerCylinder);
        this.innerCylinderList.push(innerCylinder);
    },

    /**
     * @param cylinder
     * @private
     */
    addOrigin:function(cylinder){
        this.origin.push(cylinder);
    },

    /**
     * 返回上层弧形的中心 点
     * @returns {FR.Chart.Point2D}
     */
    getTopArcCenterPoint:function(){
        var innerCylinder = this.getInnerCylinder(0);
        var outerCylinder = this.getOuterCylinder(0);
        var innerTop = innerCylinder.top().getMiddlePoint();
        var outerTop = outerCylinder.top().getMiddlePoint();
        return new FR.Chart.Point2D((innerTop.x + outerTop.x)/2, (innerTop.y+outerTop.y)/2);
    },

    /**
     * 判断是否包含坐标(x, y) 扩大响应区域
     * @param x 坐标x
     * @param y 坐标y
     * @returns {Boolean} 判断是否包含.
     */
    containsPoint: function(x, y){
        return this.top().containsPoint(x, y);
    }

});
// Ring3D.end

// Ring.start 参数这么多，是因为这个圆环不是一个标准的圆环
/**
 * 圆环
 *     @example
 *     var ring = new FR.Chart.Ring(x, y, oR, oS, oE, iR, iS, iE);
 *
 *     @class FR.Chart.Ring
 * @param x 中心点x
 * @param y 中心点y
 * @param outerRadius 外弧形半径
 * @param outerStart  外弧形起始角度
 * @param outerExtent 外弧形角度范围
 * @param innerRadius 内弧形半径
 * @param innerStart  内弧形起始角度
 * @param innerExtent 内弧形角度范围
 * @constructor
 */
FR.Chart.Ring = function (x, y, outerRadius, outerStart, outerExtent, innerRadius, innerStart, innerExtent){
    this.x = x;
    this.y = y;
    this.outerRadius = outerRadius;
    this.outerStart = outerStart;
    this.outerExtent = outerExtent;

    this.innerRadius = innerRadius;
    this.innerStart = innerStart;
    this.innerExtent = innerExtent;

    if (this.innerExtent === 360 || this.outerExtent === 360) {
        this.outerStart = this.innerStart = 0;
        this.outerExtent = this.innerExtent = 360;
    }
    this.outerCircle = new FR.Chart.Ellipse(this.x, this.y, this.outerRadius * 2, this.outerRadius * 2, this.outerStart, this.outerExtent);
    this.innerCircle = new FR.Chart.Ellipse(this.x, this.y, this.innerRadius * 2, this.innerRadius * 2, this.innerStart, this.innerExtent);
};

$.extend(FR.Chart.Ring.prototype,{
    x: 0,
    y: 0,
    outerRadius: 0,
    outerStart: 0,
    outerExtent: 0,

    innerRadius: 0,
    innerStart: 0,
    innerExtent: 0,
    outerCircle: null,
    innerCircle: null,

    /**
     * 填充圆环
     * @param ctx 画布
     */
    paint: function(ctx){
        this.doWithTheRing(ctx, true);
    },

    /**
     * 画边框圆环
     * @param ctx 画布
     */
    draw: function(ctx){
        this.doWithTheRing(ctx, false)
    },

    /**
     * 圆环画法
     * @param ctx 画布
     * @param isFill 是否填充
     * @private
     */
    doWithTheRing: function(ctx, isFill){
        var ring = this._getRingShape();
        if(isFill){
            ring.paint(ctx);
        }else{
            ring.draw(ctx);
        }
    },

    getBounds2D: function () {
        return new FR.Chart.Bounds(this.x,this.y,this.outerRadius,this.outerRadius);
    },

    isNeedRadialGradient:function(){
        return true;
    },

    /**
     *  返回中心点
     * @returns {FR.Chart.Point2D} 中心点
     */
    getCenterPoint:function(){
        return new FR.Chart.Point2D(this.x, this.y);
    },

    getInnerRadius:function(){
        return this.innerRadius;
    },

    getOuterRadius:function(){
        return this.outerRadius;
    },

    /**
     * 返回 外弧形 中间角度的点
     * @returns {FR.Chart.Point2D } 外弧形中间的点
     */
    getHalfAnglePoint:function(){
        return this.outerCircle.getMiddlePoint();
    },

    /**
     * 返回 外弧形 开始和结束点中 底下的点
     * @returns {*|FR.Chart.Point2D} 底下的点
     */
    getBottomPoint:function(){
        var start = this.outerCircle.getStartPoint();
        var end = this.outerCircle.getEndPoint();
        return start.y > end.y ? start : end;
    },

    /**
     * 返回外弧形 开始和结束点 中 左侧的点
     * @returns {*|FR.Chart.Point2D} 左侧点
     */
    getLeftMostPoint:function(){
        var start = this.outerCircle.getStartPoint();
        var end = this.outerCircle.getEndPoint();
        return start.x > end.x ? end : start;
    },

    /**
     * 返回外弧形 开始和结束点 中 右侧的点
     * @returns {*|FR.Chart.Point2D} 右侧点
     */
    getRightMostPoint:function(){
        var start = this.outerCircle.getStartPoint();
        var end = this.outerCircle.getEndPoint();
        return start.x > end.x ? start : end;
    },

    /**
     * 返回外弧形 起始 结束点 上侧的点
     * @returns {*|FR.Chart.Point2D} 上点
     */
    getTopPoint:function(){
        var start = this.outerCircle.getStartPoint();
        var end = this.outerCircle.getEndPoint();
        return start.y > end.y ? end : start;
    },

    /**
     * 返回环形占用的shape
     * @returns {FR.Chart.GeneralPath} 路径
     * @private
     */
    _getRingShape: function(){
        var ring = new FR.Chart.GeneralPath();
        var outerArc = this.outerCircle.getClockwiseShape();
        var innerArc = this.innerCircle.getCounterclockwiseShape();
        var outerStart = this.outerCircle.getStartPoint();
        var innerEnd = this.innerCircle.getEndPoint();
        ring.moveTo(outerStart.x, outerStart.y);
        ring.append(outerArc, true);
        ring.lineTo(innerEnd.x, innerEnd.y);
        ring.append(innerArc, true);
        ring.closePath();
        return ring;
    },

    /**
     * 返回动态数组shape
     * @param step 动态步骤
     * @param actionModel 动态方式
     * @param lastShape 上一次动态shape
     * @param paintState 画法状态
     * @returns {Array} 动态数组
     */
    getAnimateShape: function (step, actionModel, lastShape, paintState) {
        if (lastShape && actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE) {
            return this.getActionChangeAnimateShape(step, lastShape, paintState);
        } else {
            return this.getActionDefaultAnimateShape(step, paintState);
        }
    },

    getActionChangeAnimateShape:function(step, lastShape, paintState){
        if (paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return [];
        }

        if (paintState == FR.Chart.ChartPaintConstants.STATE_APPEAR) {
            return this.getActionDefaultAnimateShape(step);
        }

        var arcArray = [];
        for (var i = 0; i < step; i++) {
            var lastOutStart = lastShape.outerStart;
            var lastOutExtent = lastShape.outerExtent;
            var thisOutStart = this.outerStart;
            var thisOutExtent = this.outerExtent;

            var lastInStart = lastShape.innerStart;
            var lastInExtent = lastShape.innerExtent;
            var thisInStart = this.innerStart;
            var thisInExtent = this.innerExtent;

            if (Math.abs(thisOutStart - lastOutStart) > Math.abs(thisOutStart - lastOutStart + 360)) {
                thisOutStart += 360;
            }
            if (Math.abs(thisOutStart - lastOutStart) > Math.abs(thisOutStart - lastOutStart - 360)) {
                thisOutStart -= 360;
            }

            if (Math.abs(thisInStart - lastInStart) > Math.abs(thisInStart - lastInStart + 360)) {
                thisInStart += 360;
            }
            if (Math.abs(thisInStart - lastInStart) > Math.abs(thisInStart - lastInStart - 360)) {
                thisInStart -= 360;
            }

            var outStart = FR.Chart.ChartEase.easeOutQuint(i + 1, lastOutStart, thisOutStart - lastOutStart, step);
            var outExtent = FR.Chart.ChartEase.easeOutQuint(i + 1, lastOutExtent, thisOutExtent - lastOutExtent, step);

            var inStart = FR.Chart.ChartEase.easeOutQuint(i + 1, lastInStart, thisInStart - lastInStart, step);
            var inExtent = FR.Chart.ChartEase.easeOutQuint(i + 1, lastInExtent, thisInExtent - lastInExtent, step);

            arcArray[i] = new FR.Chart.Ring(this.x, this.y, this.outerRadius, outStart, outExtent, this.innerRadius, inStart, inExtent);
        }

        return arcArray;
    },

    getActionDefaultAnimateShape:function(step, paintState){
        var arcArray = [];
        if(paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE){
            return arcArray;
        }

        if (this.innerExtent === 360 || this.outerExtent === 360) {
            this.outerStart = this.innerStart = 0;
            this.outerExtent = this.innerExtent = 360;
        }

        for (var i = 0; i < step; i++) {
            var tmpOuterExtent = Math.min(FR.Chart.ChartEase.easeOutQuint(i+1, 0, this.outerExtent, step), this.outerExtent);

            var tmpInnerExtent = Math.min(FR.Chart.ChartEase.easeOutQuint(i+1, 0, this.innerExtent, step), this.innerExtent);
            arcArray[i] = new FR.Chart.Ring(this.x, this.y, this.outerRadius, this.outerStart, tmpOuterExtent,
                this.innerRadius, this.innerStart, tmpInnerExtent);
        }
        return arcArray;
    },

    /**
     * 拖拽时 画边框等.
     * @param ctx 画布
     */
    drawDragBorder:function(ctx){
        // 暂时不支持边框画法. 太丑.
    },

    /**
     * 判断是否包含坐标(x,y)
     * @param x 坐标x
     * @param y 坐标y
     * @returns {*|扩大响应区域|Boolean}判断是否包含.
     */
    containsPoint: function (x, y) {
        return this._getRingShape().containsPoint(x, y);
    },

    /**
     * 根据值返回外弧线上的点
     * @param angle 角度
     * @returns {*|FR.Chart.Point2D} 弧线上的点.
     */
    getPointWithAngle:function(angle){
        return this.outerCircle.getPointWithAngle(angle);
    },

    /**
     * 返回中间角度值
     * @returns {Number} 中间角度值
     */
    getHalfAngle:function(){
        return this.outerStart + this.outerExtent / 2;
    },

    /**
     * 返回弧形角度范围
     * @returns {Number} 角度范围大小
     */
    getExtent:function(){
        return this.outerExtent;
    },

    /**
     * 返回外围半径
     * @returns {Number} 返回外围半径
     */
    getRadius:function(){
        return this.outerRadius;
    }

});
//Ring.end

// Arc2D.start
/**
 *
 * 弧形.
 *     @example
 *     var arc = new FR.Chart.Arc2D(x, y, radius, start, extent, type);
 *
 *     @class FR.Chart.Arc2D
 *
 * @param x 弧形圆心的位置x
 * @param y 弧形圆心的位置y
 * @param radius 半径
 * @param start 起始角度
 * @param extent 角度范围
 * @param type 弧形类型
 * @constructor
 */
FR.Chart.Arc2D = function (x, y, radius, start, extent, type) {
    this.x = x;
    this.y = y;
    this.start = start;
    this.extent = extent;
    this.radius = radius;
    this.width = radius * 2;
    this.height = radius * 2;
    this.type = FR.pick(type, FR.Chart.Arc2D.PIE);
};
$.extend(FR.Chart.Arc2D, {
    CHORD: 0,
    OPEN: 1,
    PIE: 2
});
$.extend(FR.Chart.Arc2D.prototype, {
    x: 0,
    y: 0,
    start: 0,
    extent: 0,
    radius: 0,
    width: 0,
    height: 0,

    /**
     * 填充弧形
     * @param ctx 画布
     */
    paint: function (ctx) {
        this.doWithTheArc(ctx, true);
    },

    /**
     * 画出弧形边框
     * @param ctx 画布
     */
    draw: function (ctx) {
        this.doWithTheArc(ctx, false);
    },

    transform: function (at) {
        var centerPoint = at.transform({x:this.x, y:this.y});
        var otherPoint = at.transform({x:this.x + this.radius, y:this.y+this.radius});
        var detX = Math.abs(centerPoint.x - otherPoint.x);
        var detY = Math.abs(centerPoint.y - otherPoint.y);
        var radius = Math.sqrt(detX * detX + detY + detY);
        return new FR.Chart.Arc2D(centerPoint.x, centerPoint.y, radius, this.start, this.extent);
    },

    /**
     * 拖拽时 画边框 基本UI规定
     * @param ctx 画布
     */
    drawDragBorder: function(ctx){
        //坐标转来转去，真麻烦
        var start = (180 - this.start - this.extent + 360) % 360;
        var extent = this.extent;

        var middleAngle = start + extent / 2;
        middleAngle = (middleAngle / 180) * Math.PI + Math.PI;
        var centerX = this.x + -5 * Math.cos(middleAngle);
        var centerY = this.y + -5 * Math.sin(middleAngle);

        var newArc = new FR.Chart.Arc2D(centerX, centerY, this.radius + 8, this.start, this.extent, this.type);
        newArc.draw(ctx);
    },


    /**
     * arc对应的画法
     * @param ctx 画布
     * @param isFill 是否填充
     * @private
     */
    doWithTheArc: function (ctx, isFill) {
        var counterclockwise = true;
        var type = this.type;
        var start = Number(360 - this.start).toFixed(2);
        var extent = - Number(this.extent).toFixed(2);

        start = Number(start);
        extent = Number(extent);

        if (Math.abs(extent) === 360) {
            counterclockwise = false;
            type = FR.Chart.Arc2D.OPEN;
            extent = 360;
            start = 0; // kunsnat: 特殊点: 270开始 270 + 360 在IE下为空 所以强制为从0开始
        } else if (extent > 0) {
            counterclockwise = false;
        } else if (extent === 0) {
            return;
        }

        var startAngle = start * Math.PI / 180;
        var endAngle = (extent + start) * Math.PI / 180;
        var startPoint = new FR.Chart.Point2D(this.x + this.radius * Math.cos(startAngle),
            this.y + this.radius * Math.sin(startAngle));
        var endPoint = new FR.Chart.Point2D(this.x + this.radius * Math.cos(endAngle),
            this.y + this.radius * Math.sin(endAngle));
        if (isFill) {
            ctx.beginPath();
            switch (type) {
                case FR.Chart.Arc2D.CHORD :
                case FR.Chart.Arc2D.OPEN :
                    ctx.moveTo(startPoint.x, startPoint.y);
                    break;
                case FR.Chart.Arc2D.PIE :
                    ctx.moveTo(this.x, this.y);
                    break;
            }
            ctx.arc(this.x, this.y, this.radius, startAngle, endAngle, counterclockwise);
            ctx.fill();
        } else {
            if(extent < 360 && this.type != FR.Chart.Arc2D.OPEN){
                ctx.beginPath();
                ctx.moveTo(startPoint.x, startPoint.y);
                if(this.type == FR.Chart.Arc2D.PIE){
                    ctx.lineTo(this.x, this.y);
                }
                ctx.lineTo(endPoint.x, endPoint.y);
                ctx.stroke();
            }

            ctx.beginPath();
            ctx.globalAlpha = 0.5;
            ctx.arc(this.x, this.y, this.radius, startAngle, endAngle, counterclockwise);
            ctx.stroke();
        }
    },


    /**
     * 返回动态改变时 动态shape数组
     * @param step 动态步骤
     * @param lastShape 上一个动态shape
     * @param paintState 画的状态
     * @returns {Array} 返回动态数组
     * @private
     */
    getActionChangeAnimateShape: function (step, lastShape, paintState) {
        if (paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return [];
        }
        if (paintState == FR.Chart.ChartPaintConstants.STATE_APPEAR) {
            return this.getActionDefaultAnimateShape(step);
        }

        var arcArray = [];
        for (var i = 0; i < step; i++) {
            var lastStart = lastShape.start;
            var lastExtent = lastShape.extent;
            var thisStart = this.start;
            var thisExtent = this.extent;

            if (Math.abs(thisStart - lastStart) > Math.abs(thisStart - lastStart + 360)) {
                thisStart += 360;
            }
            if (Math.abs(thisStart - lastStart) > Math.abs(thisStart - lastStart - 360)) {
                thisStart -= 360;
            }

            var start = FR.Chart.ChartEase.easeOutQuint(i + 1, lastStart, thisStart - lastStart, step);
            var extent = FR.Chart.ChartEase.easeOutQuint(i + 1, lastExtent, thisExtent - lastExtent, step);
            var x = FR.Chart.ChartEase.easeOutQuint(i + 1, lastShape.x, this.x - lastShape.x, step);
            var y = FR.Chart.ChartEase.easeOutQuint(i + 1, lastShape.y, this.y - lastShape.y, step);
            arcArray[i] = new FR.Chart.Arc2D(x, y, this.radius, start, extent, this.type);
        }

        return arcArray;
    },

    /**
     * 返回默认的动态 方式数组.
     * @param step 动态步骤
     * @returns {Array} 动态shape数组
     * @private
     */
    getActionDefaultAnimateShape: function (step) {
        var arcArray = [];
        for (var i = 0; i < step; i++) {
            arcArray[i] = new FR.Chart.Arc2D(this.x, this.y, this.radius,
                this.start, FR.Chart.ChartEase.easeOutQuint(i + 1, 0, this.extent, step), this.type);
        }

        return arcArray;
    },

    /**
     * 返回动态步骤的Shape数组
     * @param step 步骤
     * @param actionModel 动态方式
     * @param lastShape 上一状态的shape
     * @param paintState 显示状态
     * @returns {Array} 返回shape数组
     */
    getAnimateShape: function (step, actionModel, lastShape, paintState) {
        if (lastShape && actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE) {
            return this.getActionChangeAnimateShape(step, lastShape, paintState);
        } else {
            return this.getActionDefaultAnimateShape(step);
        }
    },

    /**
     * 返回起始角度对应的坐标点
     * @returns {FR.Chart.Point2D} 坐标点
     */
    getStartPoint: function () {
        return this.getArcPoint(-this.start);
    },

    /**
     * 返回角度所对应的 坐标点
     * 在使用此方法时: 传入的都应该是canvas中的角度, (顺时针x轴方向起始的角度)
     * @param angle 角度
     * @returns {FR.Chart.Point2D} 坐标点
     */
    getArcPoint: function (angle) {
        angle = angle * Math.PI / 180;
        // 同java. 计算角度, 参见饼图getDataPointShape, 计算的都是逆时针角度, 4476,
        var x = this.x + (Math.cos(angle) * 0.5) * this.width;
        var y = this.y + (Math.sin(angle) * 0.5) * this.height;
        return new FR.Chart.Point2D(x, y);
    },

    /**
     * 返回结束角度的 坐标点
     * @returns {FR.Chart.Point2D} 结束点的位置.
     */
    getEndPoint: function () {
        return this.getArcPoint(-(this.start + this.extent));
    },

    /**
     * 返回半角度的坐标点
     * @returns {FR.Chart.Point2D} 坐标点
     */
    getHalfAnglePoint: function () {
        return this.getArcPoint(-(this.start + this.extent / 2));
    },

    isNeedRadialGradient:function(){
        return false;
    },

    /**
     * 返回中心点
     * @returns {FR.Chart.Point2D} 中心点
     */
    getCenterPoint:function(){
        return new FR.Chart.Point2D(this.x, this.y);
    },

    /**
     * 返回弧形 占用的矩形面积
     * @returns {FR.Chart.Bounds} 矩形面积
     */
    getBounds2D: function () {
        var x1, y1, x2, y2;

        if (this.type == FR.Chart.Arc2D.PIE) {
            x1 = y1 = x2 = y2 = 0.0;
        } else {
            x1 = y1 = 1.0;
            x2 = y2 = -1.0;
        }
        var angle = 0.0;
        for (var i = 0; i < 6; i++) {
            if (i < 4) {
                angle += 90.0;
                if (!this.containsAngle(angle)) {
                    continue;
                }
            } else if (i === 4) {
                angle = this.start;
            } else {
                angle += this.extent;
            }
            var rads = -angle * Math.PI / 180.0;
            var xe = Math.cos(rads);
            var ye = Math.sin(rads);
            x1 = Math.min(x1, xe);
            y1 = Math.min(y1, ye);
            x2 = Math.max(x2, xe);
            y2 = Math.max(y2, ye);
        }
        var w = this.width;
        var h = this.height;
        x2 = (x2 - x1) * 0.5 * w;
        y2 = (y2 - y1) * 0.5 * h;
        x1 = this.x + (x1 * 0.5) * w;
        y1 = this.y + (y1 * 0.5) * h;
        return new FR.Chart.Bounds(x1, y1, x2, y2);
    },

    /**
     * 是否包含坐标(,y)
     * @param x 坐标x
     * @param y 坐标y
     * @returns {Boolean} 是否包含
     */
    containsPoint: function (x, y) {
        var ellw = this.width;
        if (ellw <= 0.0) {
            return false;
        }
        var normx = (x - this.x + this.width / 2) / ellw - 0.5;
        var ellh = this.height;
        if (ellh <= 0.0) {
            return false;
        }
        var normy = (y - this.y + this.height / 2) / ellh - 0.5;
        var distSq = (normx * normx + normy * normy);
        if (distSq >= 0.25) {
            return false;
        }
        var angExt = Math.abs(this.extent);
        if (angExt >= 360.0) {
            return true;
        }
        var inarc = this.containsAngle(-Math.atan2(normy, normx) * 180 / Math.PI);
        if (this.type == FR.Chart.Arc2D.PIE) {
            return inarc;
        }

        if (inarc) {
            if (angExt >= 180.0) {
                return true;
            }

        } else {
            if (angExt <= 180.0) {
                return false;
            }
        }

        var angle = -this.start / 180 * Math.PI;
        var x1 = Math.cos(angle);
        var y1 = Math.sin(angle);
        angle += -this.extent / 180 * Math.PI;
        var x2 = Math.cos(angle);
        var y2 = Math.sin(angle);
        var inside = (FR.Chart.Line2D.relativeCCW(x1, y1, x2, y2, x, y) *
        FR.Chart.Line2D.relativeCCW(x1, y1, x2, y2, 0, 0) >= 0);
        return inarc ? !inside : inside;
    },

    /**
     * 常规化角度 (0--360) TOOD 取模是否可行
     *
     * @param angle 目标角度
     * @returns {Number} 角度
     */
    normalizeDegrees: function (angle) {
        if (angle > 180.0) {
            if (angle <= (180.0 + 360.0)) {
                angle = angle - 360.0;
            } else {
                angle = angle - (360.0 * Math.Round(angle / 360.0));
                if (angle == -180.0) {
                    angle = 180.0;
                }
            }
        } else if (angle <= -180.0) {
            if (angle > (-180.0 - 360.0)) {
                angle = angle + 360.0;
            } else {
                angle = angle - (360.0 * Math.Round(angle / 360.0));
                if (angle == -180.0) {
                    angle = 180.0;
                }
            }
        }
        return angle;
    },

    /**
     * 判断弧形是否包含 指定角度
     * @param angle 指定角度
     * @returns {Boolean} 是否包含指定角度
     */
    containsAngle: function (angle) {
        var angExt = this.extent;
        var backwards = (angExt < 0.0);
        if (backwards) {
            angExt = -angExt;
        }
        if (angExt >= 360.0) {
            return true;
        }
        angle = this.normalizeDegrees(angle) - this.normalizeDegrees(this.start);
        if (backwards) {
            angle = -angle;
        }
        if (angle < 0.0) {
            angle += 360.0;
        }


        return (angle >= 0.0) && (angle < angExt);
    },

    /**
     * 返回角度对应弧上的点
     * /传入java那边的坐标
     * @param angle 角度
     * @returns {FR.Chart.Point2D} 返回角度对应弧上的点
     */
    getPointWithAngle:function(angle){
        return this.getArcPoint(angle + 180);
    },

    /**
     * 返回圆弧 中间的角度
     * x负方向，顺时针
     * @returns {Number} 角度
     */
    getHalfAngle:function(){
        var start = (180 - this.start - this.extent + 360) % 360;
        var extent = this.extent;
        return (start + extent / 2) % 360;
    },

    /**
     * 选取开始, 结束 下侧的点
     * @returns {FR.Chart.Point2D} 下侧的点
     */
    getBottomPoint:function(){
        var start = this.getStartPoint();
        var end = this.getEndPoint();
        return start.y > end.y ? start : end;
    },

    /**
     * 选取开始, 结束, 左侧的店
     * @returns {FR.Chart.Point2D} 左侧的店
     */
    getLeftMostPoint:function(){
        var start = this.getStartPoint();
        var end = this.getEndPoint();
        return start.x > end.x ? end : start;
    },

    /**
     * 从开始结束点中 选取右侧的点
     * @returns {FR.Chart.Point2D} 右侧的店
     */
    getRightMostPoint:function(){
        var start = this.getStartPoint();
        var end = this.getEndPoint();
        return start.x > end.x ? start : end;
    },

    /**
     * 从开始 结束点中选取上点
     * @returns {FR.Chart.Point2D} 上点
     */
    getTopPoint:function(){
        var start = this.getStartPoint();
        var end = this.getEndPoint();
        return start.y > end.y ? end : start;
    },

    /**
     * 返回角度范围
     * @returns {Number} 角度范围
     */
    getExtent:function(){
        return this.extent;
    },

    /**
     * 返回圆弧半径
     * @returns {Number} 半径长度
     */
    getRadius:function(){
        return this.radius;
    }

});
// Arc2D.end

// MoveButton.start
/**
 * 缩略图的移动按钮
 *     @example
 *     var button = new FR.Chart.MoveButton(x, y);
 *
 *     @class FR.Chart.MoveButton
 * @param x 左上角坐标x
 * @param y 左上角坐标y
 * @constructor
 */
FR.Chart.MoveButton = function (x, y) {
    this.bounds = new FR.Chart.Bounds(x, y, FR.Chart.MoveButton.WIDTH, FR.Chart.MoveButton.HEIGHT);
};

$.extend(FR.Chart.MoveButton.prototype, {

    /**
     * 画出按钮
     * @param ctx 画布
     */
    draw: function (ctx) {
        if (this.bounds) {
            var x = this.bounds.x;
            var y = this.bounds.y;
            var height = this.bounds.height;
            var width = this.bounds.width;

            var roundBorder = new FR.Chart.RoundBounds(x, y, width, height, 5, 5);

            ctx.fillStyle = 'rgb(255, 255, 255)';
            roundBorder.paint(ctx);

            ctx.lineWidth = 1;
            ctx.strokeStyle = 'rgb(102,102,102)';
            roundBorder.draw(ctx);

            var path = new FR.Chart.GeneralPath();

            path.moveTo(x + width / 2 - 1, y + height / 2 - 2.5);
            path.lineTo(x + width / 2 - 1, y + height / 2 + 2.5);

            path.moveTo(x + width / 2 + 1, y + height / 2 - 2.5);
            path.lineTo(x + width / 2 + 1, y + height / 2 + 2.5);

            path.draw(ctx);
        }
    },

    /**
     * 判断是否包含坐标(x, y)
     * @param x 坐标x
     * @param y 坐标y
     * @returns {Boolean} 扩大响应区域 是否包含
     */
    containsPoint: function (x, y) {
        var newBounds = new FR.Chart.Bounds(this.bounds.x - 10, this.bounds.y - 10, this.bounds.width + 20, this.bounds.height + 20);
        return this.bounds ? newBounds.containsPoint(x, y) : false;
    }
});
$.extend(FR.Chart.MoveButton, {
    /**
     * 按钮宽度
     * @static
     */
    WIDTH: 11,
    /**
     * 按钮高度
     * @static
     */
    HEIGHT: 19
});
// MoveButton.end

// LegendButton.start
/**
 * 图例按钮
 *     @example
 *     var button = new FR.Chart.LegendButton(x, y, position);
 *
 * @class FR.Chart.LegendButton
 * @param x 位置x
 * @param y 位置y
 * @param position 图例位置 上下左右
 *
 * @property {Boolean} onPress 是否鼠标按压状态
 * @property {Boolean} isDisable 按钮是否可用
 * @property {Boolean} onMove 是否鼠标悬浮状态
 * @constructor
 */
FR.Chart.LegendButton = function (x, y, position) {
    this.position = FR.pick(position, FR.Chart.Constants.TOP);// 位置决定形状
    this.initButtonBounds(x, y);

    this.onPress = false;
    this.isDisable = false;
    this.onMove = false;
};

$.extend(FR.Chart.LegendButton.prototype, {

    /**
     * 根据x y的位置 初始化按钮位置
     * @param x 位置x
     * @param y 位置y
     */
    initButtonBounds: function (x, y) {
        switch (this.position) {
            case FR.Chart.Constants.TOP :
            {
                this.bounds = new FR.Chart.Bounds(x, y, FR.Chart.LegendButton.TOP_WIDTH, FR.Chart.LegendButton.TOP_HEIGHT);
                break;
            }
            case FR.Chart.Constants.LEFT :
            {
                this.bounds = new FR.Chart.Bounds(x, y, FR.Chart.LegendButton.RIGHT_WIDTH, FR.Chart.LegendButton.RIGHT_HEIGHT);
                break;
            }
            case FR.Chart.Constants.RIGHT :
            {
                this.bounds = new FR.Chart.Bounds(x, y, FR.Chart.LegendButton.RIGHT_WIDTH, FR.Chart.LegendButton.RIGHT_HEIGHT);
                break;
            }
            default :
            {
                this.bounds = new FR.Chart.Bounds(x, y, FR.Chart.LegendButton.TOP_WIDTH, FR.Chart.LegendButton.TOP_HEIGHT);
            }
        }
    },

    /**
     * 画出图例按钮
     * @param ctx 画布
     */
    draw: function (ctx) {
        if (!this.bounds) {
            return;
        }
        if(!ctx){     return; } ctx.save();

        if (this.isDisable) {
            ctx.strokeStyle = "rgb(139, 192, 241)";
        } else {
            ctx.strokeStyle = "rgb(221, 221, 221)";// 悬浮. 不能点击状态
        }

        ctx.lineWidth = 3;

        this._drawButtonPath(ctx);

        ctx.restore();
    },

    /**
     * 画出按钮路径
     * @param ctx 画布
     * @private
     */
    _drawButtonPath: function (ctx) {
        if (this.bounds) {
            ctx.translate(this.bounds.x, this.bounds.y);
            var path = this._getButtonPath();
            path.draw(ctx);
            ctx.translate(-this.bounds.x, -this.bounds.y);
        }
    },

    /**
     * 返回按钮在不同位置的路径.
     * @returns {FR.Chart.GeneralPath} 按钮形状
     * @private
     */
    _getButtonPath: function () {
        var path = new FR.Chart.GeneralPath();
        var width = this.bounds.width;
        var height = this.bounds.height;

        if (this.position == FR.Chart.Constants.TOP) {
            path.moveTo(1, height - 1);
            path.lineTo(width / 2, 1);
            path.lineTo(width - 1, height - 1);
        } else if (this.position == FR.Chart.Constants.BOTTOM) {
            path.moveTo(1, 1);
            path.lineTo(width / 2, height - 1);
            path.lineTo(width - 1, 1);
        } else if (this.position == FR.Chart.Constants.RIGHT) {
            path.moveTo(0, 0);
            path.lineTo(width, height / 2);
            path.lineTo(0, height);
        } else if (this.position == FR.Chart.Constants.LEFT) {
            path.moveTo(width, 0);
            path.lineTo(0, height / 2);
            path.lineTo(width, height);
        }

        return path;
    },

    /**
     * 鼠标按压时 画.
     * @param ctx 画布
     */
    drawOnPress: function (ctx) {
        if (this.bounds) {
            if(!ctx){     return; } ctx.save();

            ctx.lineWidth = 3;
            ctx.strokeStyle = "rgb(75, 145, 194)";

            this._drawButtonPath(ctx);

            ctx.restore();
        }
    },

    /**
     * 鼠标悬浮时 画
     * @param ctx 画布
     */
    draw4OnMove: function (ctx) {
        if (!this.bounds || !this.onMove) {
            return;
        }
        if(!ctx){     return; } ctx.save();

        ctx.strokeStyle = "rgb(85,166,222)";
        ctx.lineWidth = 3;

        this._drawButtonPath(ctx);

        ctx.restore();
    },

    /**
     * 判断是否包含坐标(x,y)
     * @param x 坐标x
     * @param y 坐标y
     * @returns {Boolean} 判断是否包含(x,y)
     */
    containsPoint: function (x, y) {
        return this.bounds == null ? false : this.bounds.containsPoint(x, y);
    }
});
$.extend(FR.Chart.LegendButton, {
    TOP_WIDTH: 13,
    TOP_HEIGHT: 9,
    RIGHT_WIDTH: 9,
    RIGHT_HEIGHT: 13
});
// LegendButton.end

// Bubble.start
/**
 * 气泡
 *     @example
 *     var bubble = new FR.Chart.Bubble();
 *
 *     @class FR.Chart.Bubble 气泡
 * @param x 中心点x位置
 * @param y 中心点y位置
 * @param radius 半径
 * @constructor
 */
FR.Chart.Bubble = function (x, y, radius) {
    this.centerX = x;
    this.centerY = y;
    this.radius = radius;

    this.bounds = new FR.Chart.Bounds(x - radius, y - radius, radius * 2, radius * 2);
    this.colorInfo = new FR.Chart.ColorInfo();
    this.jsType = "bubble";
};
$.extend(FR.Chart.Bubble.prototype, {

    getTooltipBounds4Shape:function(bounds){
        var x = this.centerX + this.radius / Math.sqrt(2) + 1;
        var y = this.centerY + this.radius / Math.sqrt(2) + 1;
        bounds.x = x;
        bounds.y = y;
        return bounds;
    },

    /**
     * 画出气泡
     * @param ctx 画布
     */
    draw: function (ctx) {
        if (this.radius <= 1){
            return;
        }

        if(!ctx){     return; } ctx.save();

        ctx.globalAlpha = this.colorInfo.seriesAttrAlpha.alpha;
        var colorArray = FR.Chart.Color.getColorArray(this.colorInfo.seriesAttrColor.seriesColor);
        ctx.fillStyle = 'rgba(' + colorArray + ',0.6)';
        var arc = new FR.Chart.Arc2D(this.centerX, this.centerY, this.radius+1, 0, 360, FR.Chart.Arc2D.OPEN);
        arc.paint(ctx);

        //优化边缘模糊
        ctx.globalAlpha = 0.5;
        ctx.strokeStyle = 'rgb(' + colorArray + ')';
        arc.draw(ctx);

        ctx.restore();
    },

    /**
     * 返回边界形状
     * @returns {FR.Chart.Bubble} 气泡形状
     */
    getShape: function () {
        return this;
    },

    getBounds2D:function(){
        var arc = new FR.Chart.Arc2D(this.centerX, this.centerY, this.radius, 0, 360, FR.Chart.Arc2D.PIE);
        return arc.getBounds2D();
    },

    isNeedRadialGradient:function(){
        return false;
    },

    /**
     * 拖拽时的边框
     * @param ctx 画布
     */
    drawDragBorder: function(ctx){
        var arc = new FR.Chart.Arc2D(this.centerX, this.centerY, this.radius + 3, 0, 360, FR.Chart.Arc2D.PIE);
        arc.draw(ctx);
    },

    /**
     * 设置透明度
     * @param alpha 透明度
     */
    setAlpha: function(alpha){
        this.colorInfo.seriesAttrAlpha.alpha = alpha;
    },

    /**
     * 返回透明度
     * @returns {FR.Chart.ColorInfo.seriesAttrAlpha.alpha|*} 透明度
     */
    getAlpha: function(){
        return this.colorInfo.seriesAttrAlpha.alpha;
    },

    /**
     * 返回动态数组
     * @param step 步骤个数
     * @param actionModel 动态方式
     * @param lastGlyph 上一个Glyph状态
     * @param paintState 画法状态
     * @returns {Array} 动态数组
     */
    getAnimateGlyph: function (step, actionModel, lastGlyph, paintState) {
        if (paintState == FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return [];
        }

        if (paintState == FR.Chart.ChartPaintConstants.STATE_DISAPPEAR) {
            var dis = new FR.Chart.BubbleDisappear(this.centerX, this.centerY, this.radius);
            dis.colorInfo = this.colorInfo;

            return dis.getAnimateGlyph(step);
        }

        var array = [];

        for (var i = 0; i < step; i++) {
            if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL
                || (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE && paintState == FR.Chart.ChartPaintConstants.STATE_ENABLE)) {

                if (lastGlyph) {
                    var bubble = new FR.Chart.Bubble(FR.Chart.ChartEase.easeOutQuint(i + 1, lastGlyph.centerX, this.centerX - lastGlyph.centerX, step),
                        FR.Chart.ChartEase.easeOutQuint(i + 1, lastGlyph.centerY, this.centerY - lastGlyph.centerY, step), this.radius);
                    array[i] = bubble;
                    bubble.colorInfo = this.colorInfo;
                } else {
                    var bubble = new FR.Chart.Bubble(this.centerX, this.centerY, FR.Chart.ChartEase.easeOutBounce(i + 1, 0, this.radius, step));
                    array[i] = bubble;
                    bubble.colorInfo = this.colorInfo;
                }
            } else if ((actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_DEFAULT && paintState == FR.Chart.ChartPaintConstants.STATE_ENABLE)
                || paintState == FR.Chart.ChartPaintConstants.STATE_APPEAR) {
                var bubble = new FR.Chart.Bubble(this.centerX, this.centerY, FR.Chart.ChartEase.easeOutBounce(i + 1, 0, this.radius, step));
                array[i] = bubble;

                bubble.colorInfo = this.colorInfo;
            }
        }

        return array;
    },

    /**
     * 判断是否包含坐标 (x,y)
     * @param x 坐标x
     * @param y 坐标y
     * @returns {Boolean} 是否包含(x,y)
     */
    containsPoint:function(x,y){
        var arc = new FR.Chart.Arc2D(this.centerX, this.centerY, this.radius, 0, 360, FR.Chart.Arc2D.PIE);
        return arc.containsPoint(x,y);
    }
});
// Bubble.end

// BubbleDisappear.start

/**
 * 气泡图消失
 *    @example
 *    var disapp = new FR.Chart.BubbleDisappear();
 *
 *    @class FR.Chart.BubbleDisappear 气泡图消失.
 *    @extends FR.Chart.Bubble
 *
 */
FR.Chart.BubbleDisappear = FR.extend(FR.Chart.Bubble, {
    constructor: function (x, y, radius) {
        FR.Chart.BubbleDisappear.superclass.constructor.apply(this, [x, y, radius]);
    },

    /**
     * 气泡消失
     * @param ctx 画法.
     */
    draw: function (ctx) {
        if (this.actionStep > this.step / 2 || (this.step < 2)) {
            return;
        }

        this.step /= 2;

        if(!ctx){     return; } ctx.save();
        var bubble = new FR.Chart.Bubble();
        bubble.colorInfo = this.colorInfo;

        bubble.radius = this.radius / 9;
        bubble.centerX = this.centerX - this.radius * 0.75;
        bubble.centerX = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerX, this.centerX - this.radius - bubble.centerX, this.step);
        bubble.centerY = this.centerY - this.radius * 0.75;
        bubble.centerY = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerY, this.centerY - bubble.centerY, this.step);
        bubble.draw(ctx);

        bubble.radius = this.radius / 9;
        bubble.centerX = this.centerX - this.radius * 0.25;
        bubble.centerY = this.centerY;
        bubble.centerY = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerY, this.centerY + this.radius - bubble.centerY, this.step);
        bubble.draw(ctx);

        bubble.radius = this.radius / 5;
        bubble.centerX = this.centerX - this.radius * 0.5;
        bubble.centerX = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerX, this.centerX - this.radius * 1.25 - bubble.centerX, this.step);
        bubble.centerY = this.centerY + this.radius * 0.5;
        bubble.centerY = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerY, this.centerY + this.radius - bubble.centerY, this.step);
        bubble.draw(ctx);

        bubble.radius = this.radius / 8;
        bubble.centerX = this.centerX - this.radius * 0.25;
        bubble.centerX = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerX, this.centerX - this.radius * 1.25 - bubble.centerX, this.step);
        bubble.centerY = this.centerY + this.radius * 0.4;
        bubble.centerY = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerY, this.centerY + this.radius * 1.25 - bubble.centerY, this.step);
        bubble.draw(ctx);

        bubble.radius = this.radius / 8;
        bubble.centerX = this.centerX + this.radius * 0.25;
        bubble.centerX = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerX, this.centerX + this.radius * 0.2 - bubble.centerX, this.step);
        bubble.centerY = this.centerY - this.radius;
        bubble.centerY = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerY, this.centerY - this.radius * 1.25 - bubble.centerY, this.step);
        bubble.draw(ctx);

        bubble.radius = this.radius / 7;
        bubble.centerX = this.centerX + this.radius * 0.5;
        bubble.centerX = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerX, this.centerX + this.radius * 0.75 - bubble.centerX, this.step);
        bubble.centerY = this.centerY - this.radius * 0.5;
        bubble.centerY = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerY, this.centerY - bubble.centerY, this.step);
        bubble.draw(ctx);

        bubble.radius = this.radius / 7;
        bubble.centerX = this.centerX + this.radius;
        bubble.centerX = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerX, this.centerX + this.radius * 1.3 - bubble.centerX, this.step);
        bubble.centerY = this.centerY - this.radius * 0.25;
        bubble.centerY = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerY, this.centerY - bubble.centerY, this.step);
        bubble.draw(ctx);

        bubble.radius = this.radius / 7;
        bubble.centerX = this.centerX + this.radius;
        bubble.centerX = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerX, this.centerX + this.radius * 1.5 - bubble.centerX, this.step);
        bubble.centerY = this.centerY + this.radius * 0.25;
        bubble.centerY = FR.Chart.ChartEase.easeOutCirc(this.actionStep, bubble.centerY, this.centerY + this.radius * 0.75 - bubble.centerY, this.step);
        bubble.draw(ctx);

        ctx.restore();
    },

    /**
     * 返回动态步骤的Glyph数组
     * @param stepNumber 动态步骤个数.
     * @returns {Array} 动态步骤数组.
     */
    getAnimateGlyph: function (stepNumber) {
        var array = [];

        for (var i = 0; i < stepNumber; i++) {
            var dis = new FR.Chart.BubbleDisappear(this.centerX, this.centerY, this.radius);
            dis.colorInfo = this.colorInfo;
            dis.step = stepNumber;
            dis.actionStep = i;

            array[i] = dis;
        }

        return array;
    }
});
// BubbleDisappear.end

//Meter.start
/**
 * 数据小于0.09 时候 数据可能无法显示. 产品UI...待续.
 * 仪表盘
 *    @example
 *    var meter = new FR.Chart.Meter();
 *
 *    @class FR.Chart.Meter 仪表盘
 *
 * @param titleGlyph 标题
 * @param value 值
 * @param meterStyle 仪表盘格式
 * @constructor
 */
FR.Chart.Meter = function (titleGlyph, value, meterStyle) {
    this.titleGlyph = titleGlyph;
    this.value = value;
    this.meterStyle = meterStyle;
    this.shapeType = "meter";
};
$.extend(FR.Chart.Meter.prototype, {
    /**
     * 值
     */
    value: 0,
    /**
     * 标题
     */
    titleGlyph: null,
    /**
     * 风格
     */
    meterStyle: null,

    /**
     * 边界
     */
    bounds: null,

    /**
     * 仪表盘中心x
     */
    meterMiddleX: 0,
    /**
     * 仪表盘中心y
     */
    meterMiddleY: 0,
    /**
     * 仪表盘圆形半径
     */
    radius: 0,
    /**
     * 起始角度
     */
    startAngle: 0,
    /**
     * 角度大小范围
     */
    extentAngle: 0,

    /**
     *环的大小
     */
    arcWidth:0,

    /**
     * 指针的颜色
     */
    arrowColor:null,

    /**
     * 返回边界.
     * @returns {FR.Chart.Bounds} 边界
     */
    getBounds: function () {
        return this.bounds;
    },

    /**
     * 画仪表盘
     * @param ctx 画布
     */
    draw: function (ctx) {

        this._initTransientValue();

        //半径小余0直接return掉
        if(this.radius <= 0){
            return ;
        }

        this._drawArrow(ctx);
    },

    /**
     * 画仪表盘背景
     * @param ctx 画布
     */
    drawMeterBottom: function (ctx, needReset) {
        if(needReset){
            this.resetBeforeDraw();
        }
        this._initTransientValue();

        //半径计算小余0直接return
        if(this.radius <= 0){
            return;
        }

        for (var i = 0, iCount = this.meterStyle.intervals.length; i < iCount; i++) {
            this._drawArcForInterval(ctx, this.meterStyle.getInterval(i));
        }

        this._drawMeterTicks(ctx);

        this._drawDisplay(ctx);

        if (this.titleGlyph && this.meterStyle.isShowTitle) {
            this.titleGlyph.bounds = new FR.Chart.Bounds(this.meterMiddleX - this.titleDim.width / 2,
                this.meterMiddleY - this.radius - this.titleDim.height - 12,
                this.titleDim.width, this.titleDim.height);
            this.titleGlyph.draw(ctx);
        }
    },

    /**
     * 画之前 重置
     */
    resetBeforeDraw: function(){
        this.meterStyle.resetAutoDesignIntervals(this.value);
        this.meterStyle.tickSize = this._getTickSize();
        this.resetAutoMax();
    },

    /**
     * 重新计算 大小
     * @private
     */
    resetAutoMax: function(){
        if(this.meterStyle.designType === 0) {
            if (this.value < 0) {
                var tickSize = this.meterStyle.tickSize * Math.pow(10, this.meterStyle.order);
                var startValue = -tickSize * 9;
                this.meterStyle.startValue = startValue;
                this.meterStyle.endValue = 0;

                var meterInterval = this.meterStyle.getInterval(0);
                meterInterval.resetInterval(startValue, startValue + tickSize * 3, "rgb(231, 71, 62)");

                meterInterval = this.meterStyle.getInterval(1);
                meterInterval.resetInterval(startValue + tickSize * 3, startValue + tickSize * 6, "rgb(252, 210, 9)")

                meterInterval = this.meterStyle.getInterval(2);
                meterInterval.resetInterval(startValue + tickSize * 6, 0, "rgb(77, 184, 73)");
            } else {
                this.meterStyle.startValue = 0;
                this.meterStyle.endValue = this.meterStyle.startValue + this.meterStyle.tickSize * 9 * Math.pow(10, this.meterStyle.order);

                var meterInterval = this.meterStyle.getInterval(0);
                meterInterval.resetInterval(this.meterStyle.startValue, this.meterStyle.endValue / 3, "rgb(231, 71, 62)");

                meterInterval = this.meterStyle.getInterval(1);
                meterInterval.resetInterval(this.meterStyle.endValue / 3, this.meterStyle.endValue * 2 / 3, "rgb(252, 210, 9)")

                meterInterval = this.meterStyle.getInterval(2);
                meterInterval.resetInterval(this.meterStyle.endValue * 2 / 3, this.meterStyle.endValue, "rgb(77, 184, 73)");
            }
        }
    },

    /**
     * 返回当前值占用的大小
     * @returns {FR.Chart.Dimension2D} 当前值占用的大小
     * @private
     */
    getCurrentValueDim: function(){
        var value = this.dealWithDoubleValue((this.value / Math.pow(10,this.meterStyle.order)));
        return FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(value, this.meterStyle.valueTextAttr);
    },

    /**
     * 当前值的颜色
     */
    getColor4CurrentValue: function(){
        for(var i = 0, len = this.meterStyle.intervals.length; i < len; i++){
            var interval = this.meterStyle.getInterval(i);
            if(interval.beginValue <= this.value && interval.endValue > this.value){
                return FR.Chart.Color.rgb2Hex(interval.backgroundColor).hex;
            }
        }
        return 'rgb(0,0,0)';
    },

    /**
     * 精确值, 强制parseFloat, 避免字符串的情况.
     * @param doubleValue 刻度值
     * @returns {number} 刻度
     * @private
     */
    dealWithDoubleValue: function(doubleValue){
        return parseFloat(FR.Chart.ChartUtils.accNumber(doubleValue));
    },

    /**
     * 返回标签大小
     * @returns {FR.Chart.Dimension2D} 占用大小
     */
    getMaxTickLableDim: function(){
        var fontText = new FR.Chart.TextAttr(); // 特定的颜色字体 未选中
        var font = new FR.Chart.FRFont();
        font.size = 9;
        font.fontName = "CenturyGothic";
        fontText.frFont = font;
        var order = Math.pow(10,this.meterStyle.order);
        var start = this.dealWithDoubleValue(this.meterStyle.startValue / order);
        var end = this.dealWithDoubleValue(this.meterStyle.endValue / order);
        var tickSize = this.dealWithDoubleValue(this.meterStyle.tickSize);
        var result = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation("", fontText);
        for (var tickValue = start; tickValue <= end; tickValue += tickSize) {
            var tmpValue = this.dealWithDoubleValue(tickValue);

            if (this.meterStyle.unitFormat) {
                tmpValue = FR.contentFormat(tickValue, this.meterStyle.unitFormat);
            } else {
                tmpValue = FR.Chart.ChartUtils.accNumber(tmpValue);
                tmpValue = FR.contentFormat(tickValue, "#.##");
            }

            var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(tmpValue, fontText);
            if(dim.width > result.width){
                result = dim;
            }
        }
        return result;
    },

    /**
     * 返回刻度大小
     * TODO: 当前值小于等于0.001无法正确显示
     * @returns {number} 刻度值大小
     * @private
     */
    _getTickSize: function(){
        var base = 0.09;
        var order = Math.pow(10,this.meterStyle.order);
        var maxValue = this.meterStyle.endValue / order;
        var minValue = this.meterStyle.startValue /order;
        var gap = maxValue - minValue;
        while(gap > 1){
            gap /= 10;
            base *= 10;
        }
        gap = base;
        while(gap < maxValue - minValue){
            gap += base;
        }
        return gap / 9;
    },

    /**
     * 调整获得合适的半径
     * @param meterW meter宽度
     * @param meterH meter高度
     * @private
     */
    _adjustRadius: function(meterW, meterH){

        var angle = this.meterStyle.meterAngle;

        var totalGap = this.getCurrentValueDim().height + this.radius * 0.18;

        if (this.meterMiddleY + totalGap > this.bounds.y + this.bounds.height) {
            meterH = meterH - (this.meterMiddleY + totalGap - this.bounds.y - this.bounds.height);
        }

        if(angle === 360){
            this.radius = Math.min(meterW, meterH) / 2;
        }else{
            if(angle > 180){
                this.radius = Math.min(meterH / (1 + Math.sin((this.meterStyle.meterAngle - 180) / 2 * Math.PI / 180)), meterW / 2);
            }else{
                this.radius = Math.min(meterH, meterW/2);
            }
        }

        this.meterMiddleY = this.bounds.y + this.titleDim.height + this.radius;

        //居中显示
        var center = this.bounds.y + this.bounds.height / 2;
        this.meterMiddleY = Math.max(center, this.meterMiddleY);
    },

    /**
     * 初始化 临时变量 便于后续的计算使用.
     * @private
     */
    _initTransientValue: function () {
        this.titleDim = new FR.Chart.Dimension2D();
        if (this.titleGlyph != null && this.meterStyle.isShowTitle) {
            var title = this.titleGlyph.text;
            this.titleDim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(title, this.meterStyle.titleTextAttr);
        }

        //这步很奇怪，是不是可以删掉？
        this.meterStyle.maxArrowAngle = Math.min(this.meterStyle.maxArrowAngle, this.meterStyle.meterAngle);

        var gapHorizontal = (2 * FR.Chart.MeterStyle.DEFAULT_BORDER_SIZE);
        var gapVertical = (2 * FR.Chart.MeterStyle.DEFAULT_BORDER_SIZE);
        var meterW = this.bounds.width - 8 - gapHorizontal;
        var meterH = this.bounds.height - 8 - gapVertical - this.titleDim.height;

        this.meterMiddleX = this.bounds.width / 2 + this.bounds.x;

        this._calculateMiddleYAndRadius(meterW,meterH);

        this.arcWidth = 0.24 * this.radius;
    },


    _calculateMiddleYAndRadius:function(meterW, meterH){
        if (this.meterStyle.dialShape == FR.Chart.MeterStyle.CIRCLE) {       // 圆形的时候
            this.radius = Math.min(meterW, meterH) / 2;
        } else {
            if (this.meterStyle.meterAngle > 180) {
                this.radius = Math.min(meterH / (1 + Math.sin((this.meterStyle.meterAngle - 180) / 2 * Math.PI / 180)), meterW / 2);
            } else {
                this.radius = Math.min(meterH, meterW / 2);
            }
        }

        this.meterMiddleY = this.bounds.y + this.titleDim.height + this.radius;
        this._adjustRadius(meterW, meterH);
    },

    /**
     * 指针
     * @param ctx 画布
     * @private
     */
    _drawArrow: function (ctx) {
        if(!ctx){     return; } ctx.save();

        if(!this.arrowColor){
            return;
        }

        ctx.fillStyle = this.arrowColor;

        this.value = FR.Chart.ChartUtils.accNumber(this.value);

        if ((this.value >= this.meterStyle.startValue && this.value <= this.meterStyle.endValue)) {
            var pointerEnd = 0.74;
            var pointerOuter = 0.06;
            var pointerInner = 0.04;

            var valueAngle = this._valueToAngle(this.value, this.meterStyle.maxArrowAngle);
            var pointerEndX = this.meterMiddleX + ((pointerEnd * this.radius) * Math.cos(Math.PI * valueAngle / 180));
            var pointerEndY = this.meterMiddleY - ((pointerEnd * this.radius) * Math.sin(Math.PI * valueAngle / 180));

            var pointerLeftX = this.meterMiddleX + this.radius*pointerOuter * Math.cos(Math.PI * (valueAngle - 90) / 180);
            var pointerLeftY = this.meterMiddleY - this.radius*pointerOuter * Math.sin(Math.PI * (valueAngle - 90) / 180);


            var pointerRightX = this.meterMiddleX + this.radius*pointerOuter * Math.cos(Math.PI * (valueAngle + 90) / 180);
            var pointerRightY = this.meterMiddleY - this.radius*pointerOuter * Math.sin(Math.PI * (valueAngle + 90) / 180);

            var pointer = new FR.Chart.GeneralPath();

            pointer.moveTo(pointerEndX, pointerEndY);
            pointer.lineTo(pointerLeftX, pointerLeftY);
            pointer.lineTo(pointerRightX, pointerRightY);
            pointer.closePath();
            pointer.paint(ctx);


            var outerRadius = this.radius * pointerOuter;
            var outerArc = new FR.Chart.Arc2D(this.meterMiddleX, this.meterMiddleY, outerRadius,
                0, 360, FR.Chart.Arc2D.PIE);
            outerArc.paint(ctx);

            ctx.fillStyle = "white";
            var innerRadius = this.radius * pointerInner;
            var inner = new FR.Chart.Arc2D(this.meterMiddleX, this.meterMiddleY, innerRadius,
                0, 360, FR.Chart.Arc2D.PIE);
            inner.paint(ctx);
        }
        ctx.restore();
    },

    /**
     * 获取当前仪表盘占用的Bounds边界
     * @returns {FR.Chart.Bounds} 边界.
     */
    getShape: function () {
        return this.getBounds();
    },

    /**
     * 值转为角度
     * @param value 值
     * @param angleRange 角度范围
     * @returns {number} 角度值
     * @private
     */
    _valueToAngle: function (value, angleRange) {
        value = value - this.meterStyle.startValue;
//		// kunsnat: 同java
        var baseAngle = (angleRange / 2 + 90);
        return (baseAngle - (value / (this.meterStyle.endValue - this.meterStyle.startValue)) * angleRange);
    },

    /**
     * 返回合适的字体大小
     * @param maxDimen 最大占用宽度等
     * @param string 字符串
     * @returns {FR.Chart.FRFont} 字体
     * @private
     */
    _getFont: function (maxDimen, string) {
        var font = new FR.Chart.FRFont();

        font.size = 8;

        var textAttr = new FR.Chart.TextAttr();
        textAttr.frFont = font;

        var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(string, textAttr);

        while (dim.width < maxDimen.width && dim.height < maxDimen.height) {
            font.size += 1;

            textAttr.frFont = font;
            dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(string, textAttr);

            if (font.size > 20) {
                return font;
            }
        }

        return font;
    },

    /**
     * 返回文本占用大小
     * @param text 文本
     * @param ctx 画布
     * @returns {FR.Chart.Bounds} 占用大小
     * @private
     */
    _getTextBounds: function (text, font) {
        if (text == null) {
            return;
        }
        var dim = FR.Chart.GraphHelper.stringDimension(text, font, 0);
        return new FR.Chart.Bounds(0, 0, dim.width, dim.height);
    },

    /**
     * 画仪表盘分区的各个弧形 分段.
     * @param ctx 画布
     * @param interval 分段.
     * @private
     */
    _drawArcForInterval: function (ctx, interval) {
        if(!interval.backgroundColor) {
            return;
        }
        if(!ctx){     return; } ctx.save();

        ctx.strokeStyle = interval.backgroundColor;

        var begin = interval.beginValue < this.meterStyle.startValue ? this.meterStyle.startValue : interval.beginValue;
        var end = interval.endValue > this.meterStyle.endValue ? this.meterStyle.endValue : interval.endValue;

        if (begin > end) {
            return;
        }

        var minAngle = this._valueToAngle(begin, this.meterStyle.maxArrowAngle);
        var maxAngle = this._valueToAngle(end, this.meterStyle.maxArrowAngle);

        ctx.lineWidth = 3;

        var minRadius = this.radius - this.arcWidth;

        for (var i = minRadius; i < this.radius; i++) {
            ctx.beginPath();  // arc  竟然要取余数??
            ctx.arc(this.meterMiddleX, this.meterMiddleY, i, (360 - minAngle) * Math.PI / 180,
                (360 - maxAngle) * Math.PI / 180, false);
            ctx.stroke();
        }

        ctx.restore();
    },

    /**
     * 画出仪表盘刻度
     * @param ctx 画布.
     * @private
     */
    _drawMeterTicks: function (ctx) {
        var order = Math.pow(10,this.meterStyle.order);
        var startValue = this.meterStyle.startValue / order;
        var endValue = this.meterStyle.endValue / order;

        if (this.meterStyle.tickSize > 0) {
            if (startValue > endValue) {
                return;
            }

            if(!ctx){     return; } ctx.save();

            ctx.strokeStyle = this.meterStyle.tickColor;
            ctx.fillStyle = this.meterStyle.tickColor;

            var labelPath = new FR.Chart.BoundsManager();

            for (var tickValue = startValue, index = 0; tickValue <= endValue; tickValue += this.meterStyle.tickSize, index++) {

                if (tickValue != startValue && tickValue != endValue) {
                    this._drawMeterTickLine(ctx, tickValue * order, false, true);
                }

                if (!(this.meterStyle.maxArrowAngle === 360 && tickValue == endValue)) {
                    this._drawMeterTickLabel(ctx, tickValue, labelPath);
                }
            }

            for (var smallTick = startValue; smallTick <= endValue; smallTick += this.meterStyle.tickSize / 5) {
                if ((smallTick - startValue) % this.meterStyle.tickSize === 0) {
                    continue;
                }

                this._drawMeterTickLine(ctx, smallTick * order, false, false);
            }

            ctx.restore();
        }
    },

    /**
     * 画出仪表盘刻度线
     * @param ctx 画布
     * @param value 值
     * @param isStartEnd 是起始结束
     * @param isMain 是否为主刻度
     * @private
     */
    _drawMeterTickLine: function (ctx, value, isStartEnd, isMain) {
        if(!ctx){     return; } ctx.save();

        var valueAngle = this._valueToAngle(value, this.meterStyle.maxArrowAngle);

        var start = this.radius * 0.96;
        var length = this.radius * 0.18;
        if(!isMain && !isStartEnd){
            length = this.radius * 0.1;
        }
        var startX = this.meterMiddleX + start * Math.cos(Math.PI * (valueAngle / 180));
        var startY = this.meterMiddleY - start * Math.sin(Math.PI * (valueAngle / 180));
        var endX = this.meterMiddleX + (start - length) * Math.cos(Math.PI * (valueAngle / 180));
        var endY = this.meterMiddleY - (start - length) * Math.sin(Math.PI * (valueAngle / 180));

        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(endX, endY), new FR.Chart.Point2D(startX, startY));

        ctx.lineWidth = this.radius > 100 ? 1.4 : 1.0;

        ctx.strokeStyle = 'white';
        ctx.lineCap="round";

        line.draw(ctx);

        ctx.restore();
    },

    /**
     * 画出仪表盘刻度标签
     * @param ctx 画布
     * @param value 值
     * @param labelBoundsManager  标签重叠管理
     * @private
     */
    _drawMeterTickLabel: function (ctx, value, labelBoundsManager) {
        if(!ctx){     return; } ctx.save();
        ctx.textBaseline = 'top';
        // kunsnat: 刻度标签是否可见. 颜色和刻度保持一致. 数字格式和单位保持一致.
        if (this.meterStyle.tickLabelsVisible) {
            var order = Math.pow(10,this.meterStyle.order);
            var valueAngle = this._valueToAngle(value * order, this.meterStyle.maxArrowAngle);

            var tickLabel;
            if (this.meterStyle.unitFormat) {
                tickLabel = FR.contentFormat(value, this.meterStyle.unitFormat);
            } else {
                value = FR.Chart.ChartUtils.accNumber(value);
                tickLabel = FR.contentFormat(value, "#.##");
            }

            ctx.strokeStyle = 'rgb(51, 51, 51)';
            ctx.fillStyle = 'rgb(51, 51, 51)';
            var fontSize = 9;
            ctx.font = " " + fontSize + "px Verdana";

            var textAttr = new FR.Chart.TextAttr();
            textAttr.frFont = new FR.Chart.FRFont();
            textAttr.frFont.color = 'rgb(51, 51, 51)';
            textAttr.frFont.size = fontSize;
            textAttr.frFont.fontName = 'Verdana';

            var tickLabelBounds = this._getTextBounds(tickLabel, textAttr.frFont);

            var start = this.radius * 0.72;
            var x = this.meterMiddleX + start * Math.cos(Math.PI * (valueAngle / 180));
            var y = this.meterMiddleY - start * Math.sin(Math.PI * (valueAngle / 180));

            if(valueAngle < 90 || valueAngle > 270){
                x -= tickLabelBounds.width;
            }

            var textAttr = new FR.Chart.TextAttr();
            textAttr.frFont = new FR.Chart.FRFont();
            textAttr.frFont.color = 'rgb(51, 51, 51)';
            textAttr.frFont.size = fontSize;
            textAttr.frFont.fontName = 'Verdana';

            var newBounds = new FR.Chart.Bounds(x, y, tickLabelBounds.width, tickLabelBounds.height);
            if(!labelBoundsManager.isInteract(newBounds)){
                labelBoundsManager.addBounds(newBounds);
                FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, tickLabel, textAttr, newBounds, FR.Chart.currentDivKey);
            }
        }
        ctx.restore();
    },

    /**
     * 画出单位显示.
     * @param ctx 画布
     * @private
     */
    _drawDisplay: function (ctx) {
        if(!ctx){     return; } ctx.save();

        var order = Math.pow(10,this.meterStyle.order);
        var valueStr = this.dealWithDoubleValue(this.value / order);

        var currentDim = this.getCurrentValueDim();
        var displayerWidth = currentDim.width;
        var displayerX = this.meterMiddleX - displayerWidth / 2;
        var displayerY = this.meterMiddleY + 0.06 * this.radius;

        if(this.meterStyle.valueTextAttr && this.meterStyle.valueTextAttr.frFont){
            this.meterStyle.valueTextAttr.frFont.color = this.getColor4CurrentValue();
        }

        var textBounds = new FR.Chart.Bounds(displayerX, displayerY, currentDim.width, currentDim.height);
        FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, valueStr, this.meterStyle.valueTextAttr, textBounds, FR.Chart.currentDivKey);

        var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(this.meterStyle.units, this.meterStyle.unitTextAttr);
        var unitX = this.meterMiddleX - dim.width / 2;
        var unitY = this.meterMiddleY - 0.22 * this.radius - dim.height;

        textBounds = new FR.Chart.Bounds(unitX, unitY, dim.width, dim.height);
        FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, this.meterStyle.units, this.meterStyle.unitTextAttr, textBounds, FR.Chart.currentDivKey);
    },

    /**
     * 获取动态步骤中 每一步的Glyph
     * @param step 步骤
     * @param actionModel 动态方式
     * @returns {Array} 动态步骤数组
     */
    getAnimateGlyph: function (step, actionModel, lastGlyph) {
        var maxValue = this.meterStyle.getInterval(this.meterStyle.intervals.length-1).endValue;
        if(this.value < this.meterStyle.getInterval(0).beginValue || this.value > maxValue){
            return null ;
        }
        var glyph = [];
        var arrowColor = this.getColor4CurrentValue();
        for (var i = 0; i < step; i++) {
            if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL) {
                glyph[i] = this;
                glyph[i].arrowColor = arrowColor;
            } else {
                var meter;
                var val = FR.Chart.ChartEase.easeOutQuint(i + 1, 0, this.value, step);
                if(lastGlyph){
                    var lastValue = lastGlyph.value
                    val = FR.Chart.ChartEase.easeOutQuint(i + 1, lastValue, this.value - lastValue, step);
                }
                if(this.value < 0 && lastGlyph){
                    var minValue = this.meterStyle.getInterval(0).beginValue;
                    val = FR.Chart.ChartEase.easeOutQuint(i + 1, minValue - lastValue, minValue - this.value, step);
                    val = minValue - val;
                }
                if(this.value <0 && !lastGlyph){
                    var minValue = this.meterStyle.getInterval(0).beginValue;
                    val = FR.Chart.ChartEase.easeInBack(i + 1, 0, minValue - this.value, step);
                    val = minValue - val;
                }
                meter = new FR.Chart.Meter(this.titleGlyph, val, this.meterStyle);
                meter.bounds = this.bounds;
                meter.arrowColor = arrowColor;
                glyph[i] = meter;
            }
        }

        return glyph;
    }
});
// Meter.end

//SimpleMeter.Start
/**
 * 简单的仪表盘
 * @type {void|*}
 * @extends FR.Chart.Meter
 */
FR.Chart.SimpleMeter = FR.extend(FR.Chart.Meter,{
    constructor: function (titleGlyph, value, meterStyle) {
        this.initAll();
        FR.Chart.Meter.superclass.constructor.apply(this,[titleGlyph,value,meterStyle])
    },

    _calculateMeterShape:function(){
        this.innerRadius = this.radius*0.88;
    },

    draw: function (ctx) {
        this._initTransientValue();
        this._calculateMeterShape();
        this.drawLight(ctx);
    },

    drawLight:function(ctx){
        if(!ctx){     return; } ctx.save();
        ctx.alpha = 1;
        ctx.strokeStyle = this.valueColor;
        ctx.lineWidth = this.innerRadius*0.1364;
        ctx.lineCap = "round";
        var drawValue = Math.min(this.value, this.meterStyle.endValue);
        drawValue = Math.max(drawValue, this.meterStyle.startValue);
        this.meterExtend = (drawValue - this.meterStyle.startValue)*300/(this.meterStyle.endValue-this.meterStyle.startValue);
        ctx.beginPath();
        ctx.arc(this.meterMiddleX,this.meterMiddleY,this.innerRadius,
            120*Math.PI/180,(120+this.meterExtend)*Math.PI/180,false);
        ctx.stroke();
        ctx.restore;
    },

    initAll:function(){
        this.tickColor ="rgb(51,51,51)";
    },

    drawMeterBottom:function(ctx, needReset){
        if(!ctx){
            return;
        }

        if(needReset){
            this.resetBeforeDraw();
        }

        this._initTransientValue();
        this._calculateMeterShape();
        this._calculateLightColor();
        this.arcWidth = this.radius*0.12;
        //先画背景圆环
        ctx.alpha = 0.5;
        ctx.strokeStyle ="rgb(214,214,214)";
        ctx.lineWidth = this.arcWidth;
        ctx.lineCap = "round";
        ctx.beginPath();  // arc  竟然要取余数??
        ctx.arc(this.meterMiddleX, this.meterMiddleY, this.innerRadius,120* Math.PI / 180,
            60 * Math.PI / 180, false);
        ctx.stroke();
        this._drawStrings(ctx);
        if (this.titleGlyph  && this.meterStyle.isShowTitle) {
            this.titleGlyph.bounds = new FR.Chart.Bounds(this.meterMiddleX - this.titleDim.width / 2,
                this.meterMiddleY - this.radius - this.titleDim.height - 12,
                this.titleDim.width, this.titleDim.height);
            this.titleGlyph.draw(ctx);
        }
    },

    _drawStrings:function(ctx, currentDivKey){
        ctx.strokeStyle = "";
        ctx.lineWidth = 1;
        var valueFont = "30px Gentury Gothic";
        var unitFont= "12px Microsoft YaHei";
        var tickFont = "9px Gentury Gothic";
        //画值
        var order = Math.pow(10,this.meterStyle.order);
        var valueStr = this.dealWithDoubleValue(this.value / order)+"";
        var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(valueStr,this.meterStyle.valueTextAttr);
        var stringX = this.meterMiddleX - dim.width/2.0;
        var stringY = this.meterMiddleY-dim.height/2.0;
        var boun = new FR.Chart.Bounds(stringX,stringY,dim.width,dim.height);
        this.meterStyle.valueTextAttr.frFont.color = this.valueColor;
        FR.Chart.TextUtils.drawLabelWithNoRotation(ctx,valueStr,this.meterStyle.valueTextAttr,boun,currentDivKey);

        //画单位
        var units = this.meterStyle.units;
        var unitDim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(units,this.meterStyle.unitTextAttr);
        var stringX = this.meterMiddleX - unitDim.width/2.0;
        var stringY = stringY - this.radius*0.05-unitDim.height;
        var unitBoun = new FR.Chart.Bounds(stringX,stringY,unitDim.width,unitDim.height);
        FR.Chart.TextUtils.drawLabelWithNoRotation(ctx,units,this.meterStyle.unitTextAttr,unitBoun,currentDivKey);

        //画标签
        ctx.fon = tickFont;
        ctx.fillStyle =this.tickColor;
        var leftX = this.meterMiddleX+this.innerRadius*Math.cos(120* Math.PI / 180);
        var leftY = this.meterMiddleY+this.innerRadius*Math.sin(120*Math.PI/180)-this.radius*0.1;
        var rightX = this.meterMiddleX+this.innerRadius*Math.cos(60*Math.PI/180);
        var rightY = this.meterMiddleY+this.innerRadius*Math.sin(60*Math.PI/180)-this.radius*0.1;

        var order = Math.pow(10,this.meterStyle.order);
        var leftStr = this.dealWithDoubleValue(this.meterStyle.startValue/order);
        ctx.fillText(leftStr,leftX,leftY) ;
        var rightStr = this.dealWithDoubleValue(this.meterStyle.endValue/order);
        var displayer = ctx.measureText(rightStr);
        ctx.fillText(rightStr,rightX-displayer.width,rightY);

    },

    _calculateLightColor:function(){
        for (var i = 0, iCount = this.meterStyle.intervals.length; i < iCount; i++) {
            var beginValue = this._dealBegin(i);
            var endValue = this._dealEnd(i);
            if(beginValue<=this.value && endValue>=this.value){
                this.valueColor = this.meterStyle.getInterval(i).backgroundColor;
                break;
            }
        }
        if(this.value<this.meterStyle.startValue){
            this.valueColor = this.meterStyle.getInterval(0).backgroundColor;
        }
        if(this.value>this.meterStyle.endValue){
            this.valueColor = this.meterStyle.getInterval(this.meterStyle.intervals.length-1).backgroundColor;
        }
    },

    _dealBegin:function(index){
        var interval = this.meterStyle.getInterval(index);
        return interval.beginValue<this.meterStyle.startValue?this.meterStyle.startValue:interval.beginValue;
    },

    _dealEnd:function(index){
        var interval = this.meterStyle.getInterval(index);
        return interval.endValue>this.meterStyle.endValue?this.meterStyle.endValue:interval.endValue;
    },


    _calculateMiddleYAndRadius:function(meterW,meterH){
        this.radius = Math.min(meterW/2.0, meterH/1.886);
        this.meterMiddleY = this.bounds.y + 8 + FR.Chart.MeterStyle.DEFAULT_BORDER_SIZE + this.titleDim.height;
        if(meterW/2.0 > meterH/1.886) {
            this.meterMiddleY += this.radius;
        } else {
            this.meterMiddleY += meterH/2.0;
        }
    },

    /**
     * 获取动态步骤中 每一步的Glyph
     * @param step 步骤
     * @param actionModel 动态方式
     * @returns {Array} 动态步骤数组
     */
    getAnimateGlyph: function (step, actionModel, lastGlyph) {

        var glyph = [];
        for (var i = 0; i < step; i++) {
            if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL) {
                glyph[i] = this;
            } else {
                var meter;
                var val = FR.Chart.ChartEase.easeOutQuint(i + 1, 0, this.value, step);
                if(lastGlyph){
                    var lastValue = lastGlyph.value;
                    val = FR.Chart.ChartEase.easeOutQuint(i + 1, lastValue, this.value - lastValue, step);
                }
                meter = new FR.Chart.SimpleMeter(this.titleGlyph, val, this.meterStyle);
                meter.bounds = this.bounds;
                meter.valueColor = this.valueColor;
                glyph[i] = meter;
            }
        }

        return glyph;
    }

});
// simplemeter.end

// bluemeter.start
/**
 * 简单的仪表盘
 * @type {void|*}
 * @extends FR.Chart.Meter
 */
FR.Chart.BlueMeter = FR.extend(FR.Chart.Meter, {

    /**
     * 画仪表盘
     * @param ctx 画布
     */
    draw: function (ctx) {
        this._initTransientValue();

        this._drawArrow(ctx);
    },

    drawMeterBottom:function(ctx, needReset){
        if(!ctx){
            return;
        }
        if(needReset){
            this.resetBeforeDraw();
        }
        this._initTransientValue();

        this.drawBackground(ctx);

        for (var i = 0, iCount = this.meterStyle.intervals.length; i < iCount; i++) {
            this._drawArcForInterval(ctx, this.meterStyle.getInterval(i), i);
        }

        this._drawMeterTicks(ctx);

        this.drawDisplay(ctx);

        if (this.titleGlyph  && this.meterStyle.isShowTitle ) {
            this.titleGlyph.bounds = new FR.Chart.Bounds(this.meterMiddleX - this.titleDim.width / 2,
                this.meterMiddleY - this.radius - this.titleDim.height - 12,
                this.titleDim.width, this.titleDim.height);
            this.titleGlyph.draw(ctx);
        }
    },

    /**
     * 初始化 临时变量 便于后续的计算使用.
     * @private
     */
    _initTransientValue: function () {
        this.meterStyle.maxArrowAngle = 300;
        FR.Chart.BlueMeter.superclass._initTransientValue.apply(this, []);
    },

    drawBackground: function (ctx) {
        if(!ctx){     return; } ctx.save();

        var arrowGradient = ctx.createLinearGradient(
            this.meterMiddleX - this.radius, this.meterMiddleY - this.radius,
            this.meterMiddleX + this.radius, this.meterMiddleY + this.radius
        );

        arrowGradient.addColorStop(0, 'rgb(234,231,227)');
        arrowGradient.addColorStop(0.2, 'rgb(154,149,154)');
        arrowGradient.addColorStop(0.5, 'rgb(255,255,255)');
        arrowGradient.addColorStop(0.8, 'rgb(142,136,139)');
        arrowGradient.addColorStop(1, 'rgb(181,181,182)');

        ctx.fillStyle = arrowGradient;

        //画阴影
        ctx.shadowBlur = 5;
        ctx.shadowOffsetX = 2;
        ctx.shadowOffsetY = 4;
        ctx.shadowColor = 'rgba(0,0,0,0.3)';

        var arc = new FR.Chart.Arc2D(this.meterMiddleX, this.meterMiddleY, this.radius, 0, 360, false);
        arc.paint(ctx);

        var newRad = this.radius * 0.9;
        var radGradient = ctx.createRadialGradient(this.meterMiddleX, this.meterMiddleY, 0, this.meterMiddleX, this.meterMiddleY, newRad);

        radGradient.addColorStop(0.0, 'rgb(20, 134, 255)');
        radGradient.addColorStop(0.7, 'rgb(0, 59, 128)');
        radGradient.addColorStop(1.0, 'rgb(0, 35, 64)');
        ctx.fillStyle = radGradient;

        var newArc = new FR.Chart.Arc2D(this.meterMiddleX, this.meterMiddleY, newRad, 0, 360, false);
        newArc.paint(ctx);

        ctx.restore();
    },

    _drawArcForInterval : function(ctx, interval, index) {
        if(!ctx){     return; } ctx.save();

        ctx.strokeStyle = interval.backgroundColor;

        var begin = interval.beginValue < this.meterStyle.startValue ? this.meterStyle.startValue : interval.beginValue;
        var end = interval.endValue > this.meterStyle.endValue ? this.meterStyle.endValue : interval.endValue;

        if (begin > end) {
            return;
        }

        var minAngle = this._valueToAngle(begin, this.meterStyle.maxArrowAngle);
        var maxAngle = this._valueToAngle(end, this.meterStyle.maxArrowAngle);

        var arcWidth = this.radius * 0.07;
        var arcRadius = this.radius * 0.85 - arcWidth * 0.5;

        ctx.lineWidth = arcWidth; //kunsnat: 顺序和java 不同, 所以 cap圆的也不同.
        if(index === 0) {
            ctx.lineCap = "round";
        } else {
            ctx.lineCap = "square";
        }

        var minPie = (360 - minAngle) * Math.PI / 180;
        var maxPie = (360 - maxAngle) * Math.PI / 180;
        var centerPie = (minPie + maxPie) / 2;

        ctx.beginPath();
        ctx.arc(this.meterMiddleX, this.meterMiddleY, arcRadius, minPie, centerPie, false);
        ctx.stroke();

        if(index === this.meterStyle.intervals.length - 1) {
            ctx.lineCap = "round";
        } else {
            ctx.lineCap = "square";
        }

        ctx.beginPath();
        ctx.arc(this.meterMiddleX, this.meterMiddleY, arcRadius, centerPie, maxPie, false);
        ctx.stroke();

        ctx.restore();
    },

    /**
     * 画出仪表盘刻度线
     * @param ctx 画布
     * @param value 值
     * @param isStartEnd 是起始结束
     * @param isMain 是否为主刻度
     * @private
     */
    _drawMeterTickLine: function (ctx, value, isStartEnd, isMain) {
        if(!ctx){     return; } ctx.save();

        var valueAngle = this._valueToAngle(value, this.meterStyle.maxArrowAngle);
        var start = this.radius * 0.75;
        var length = this.radius * 0.12;
        if(!isMain && !isStartEnd){
            length = this.radius * 0.08;
        }
        var startX = this.meterMiddleX + start * Math.cos(Math.PI * (valueAngle / 180));
        var startY = this.meterMiddleY - start * Math.sin(Math.PI * (valueAngle / 180));
        var endX = this.meterMiddleX + (start - length) * Math.cos(Math.PI * (valueAngle / 180));
        var endY = this.meterMiddleY - (start - length) * Math.sin(Math.PI * (valueAngle / 180));

        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(endX, endY), new FR.Chart.Point2D(startX, startY));

        ctx.lineWidth = 2;
        ctx.lineCap = 'round';

        ctx.strokeStyle = 'white';
        line.draw(ctx);

        ctx.restore();
    },

    /**
     * 画出仪表盘刻度标签
     * @param ctx 画布
     * @param value 值
     * @param labelBoundsManager  标签重叠管理
     * @private
     */
    _drawMeterTickLabel: function (ctx, value, labelBoundsManager) {
        if(!ctx){     return; } ctx.save();

        // kunsnat: 刻度标签是否可见. 颜色和刻度保持一致. 数字格式和单位保持一致.
        if (this.meterStyle.tickLabelsVisible) {
            var order = Math.pow(10, this.meterStyle.order);
            var valueAngle = this._valueToAngle(value * order, this.meterStyle.maxArrowAngle);

            var tickLabel;
            if (this.meterStyle.unitFormat) {
                tickLabel = FR.contentFormat(value, this.meterStyle.unitFormat);
            } else {
                value = FR.Chart.ChartUtils.accNumber(value);
                tickLabel = FR.contentFormat(value, "#.##");
            }

            ctx.fillStyle = 'white';
            var fontSize = 9;

            ctx.font = " " + fontSize + "px CenturyGothic";

            var textattr = new FR.Chart.TextAttr();
            textattr.frFont = new FR.Chart.FRFont();
            textattr.frFont.color = 'white';
            textattr.frFont.size = 9;
            textattr.frFont.fontName = 'CenturyGothic';

            var tickLabelBounds = this._getTextBounds(tickLabel, textattr.frFont);

            var x = this.meterMiddleX + this.radius * Math.cos(Math.PI * (valueAngle / 180)) * 0.5 - tickLabelBounds.width * 0.5;
            var y = this.meterMiddleY - this.radius * Math.sin(Math.PI * (valueAngle / 180)) * 0.5 - tickLabelBounds.height * 0.5;

            var newBounds = new FR.Chart.Bounds(x, y, tickLabelBounds.width, tickLabelBounds.height);
            if (!labelBoundsManager.isInteract(newBounds)) {
                labelBoundsManager.addBounds(newBounds);
                ctx.notUseDivText = true; // kunsnat: 保证 指针在刻度标签上.
                FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, tickLabel, textattr, newBounds, FR.Chart.currentDivKey);
                ctx.notUseDivText = false;
            }
        }
        ctx.restore();
    },

    drawDisplay : function(ctx) {
        var valueString = "" + this.dealWithDoubleValue(this.value / Math.pow(10, this.meterStyle.order));

        var valueTextAttr = this.meterStyle.valueTextAttr;
        valueTextAttr.frFont.color = 'white';
        var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(valueString, valueTextAttr);

        var displayX = this.meterMiddleX - dim.width / 2;
        var displayY = this.meterMiddleY;
        //画数字
        var boun = new FR.Chart.Bounds(displayX, displayY, dim.width, dim.height);
        FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, valueString, valueTextAttr, boun, FR.Chart.currentDivKey);

        //画单位
        var unitTextAttr = this.meterStyle.unitTextAttr;
        unitTextAttr.frFont.color = 'white';

        var units = this.meterStyle.units;
        var unitDim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(units, unitTextAttr);
        var unitX = (this.meterMiddleX - unitDim.width / 2);
        var unitY = (this.meterMiddleY - unitDim.height);
        var unitBounds = new FR.Chart.Bounds(unitX, unitY, unitDim.width, unitDim.height);
        FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, units, unitTextAttr, unitBounds, FR.Chart.currentDivKey);
    },

    _drawArrow : function(ctx) {
        if(this.value >= this.meterStyle.startValue && this.value <= this.meterStyle.endValue) {
            if(!ctx){     return; } ctx.save();

            ctx.fillStyle = 'white'; // 白色圆环
            ctx.globalAlpha = 0.25;

            var arrowRad = this.radius * 0.44;
            var arrowArc = new FR.Chart.Arc2D(this.meterMiddleX, this.meterMiddleY, arrowRad, 0, 360, false);
            arrowArc.paint(ctx);

            ctx.globalAlpha = 1;

            var leftPointer = new FR.Chart.GeneralPath(); // 指针
            var rightPointer = new FR.Chart.GeneralPath();
            var pointer = this.getPointerPath(leftPointer, rightPointer);

            ctx.fillStyle = 'rgb(242, 150, 0)';
            leftPointer.paint(ctx);

            ctx.fillStyle = 'rgb(216, 110, 30)';
            rightPointer.paint(ctx);

            ctx.fillStyle = 'black';
            ctx.globalAlpha = 0.5;
            pointer.draw(ctx);

            var arrowOutRad = this.radius * 0.36;// 覆盖层级.  盖住指针进入圆形的部分   内部蓝色环
            var arc = new FR.Chart.Arc2D(this.meterMiddleX, this.meterMiddleY, arrowOutRad, 0, 360, false);
            ctx.fillStyle = 'white';
            ctx.globalAlpha = 1;
            arc.paint(ctx);

            var radGradient = ctx.createRadialGradient(this.meterMiddleX, this.meterMiddleY, 0, this.meterMiddleX, this.meterMiddleY, arrowOutRad);
            radGradient.addColorStop(0, 'rgb(20,134,255)');
            radGradient.addColorStop(1, 'rgb(0, 90, 238)');
            ctx.fillStyle = radGradient;
            arc.paint(ctx);

            ctx.strokeStyle = 'rgb(16, 68, 160)';
            ctx.lineWidth = 2;

            arc.draw(ctx);

            ctx.restore();
        }
    },

    getPointerPath : function(left, right) {

        var ARROW_INNER_R = 0.12;
        var ARROW_LENGTH = 0.72;

        var PIE = 180;
        var angle = -90 / 180.0 * Math.PI;
        var actionValue = this.value;
        var valueAngle = this._valueToAngle(actionValue, this.meterStyle.maxArrowAngle);
        var aPointX = this.meterMiddleX + this.radius * ARROW_INNER_R * Math.cos(Math.PI * (valueAngle / PIE) - angle);
        var aPointY = this.meterMiddleY - this.radius * ARROW_INNER_R * Math.sin(Math.PI * (valueAngle / PIE) - angle);
        var bPointX = this.meterMiddleX + this.radius * ARROW_INNER_R * Math.cos(Math.PI * (valueAngle / PIE) + angle);
        var bPointY = this.meterMiddleY - this.radius * ARROW_INNER_R * Math.sin(Math.PI * (valueAngle / PIE) + angle);
        var startX = this.meterMiddleX + ((ARROW_LENGTH * this.radius) * Math.cos(Math.PI * valueAngle / PIE));
        var startY = this.meterMiddleY - ((ARROW_LENGTH * this.radius) * Math.sin(Math.PI * valueAngle / PIE));

        left.moveTo(startX, startY);
        left.lineTo(aPointX, aPointY);
        left.lineTo((bPointX + aPointX)/2, (bPointY + aPointY)/2);
        left.closePath();
        right.moveTo(startX, startY);
        right.lineTo(bPointX, bPointY);
        right.lineTo((bPointX + aPointX)/2, (bPointY + aPointY)/2);
        right.closePath();
        var pointer = new FR.Chart.GeneralPath();
        pointer.moveTo(startX, startY);
        pointer.lineTo(aPointX, aPointY);
        pointer.lineTo(bPointX, bPointY);
        pointer.closePath();
        return pointer;
    },

    _calculateMiddleYAndRadius : function(meterW, meterH){
        this.radius = Math.min(meterW, meterH)/2.0;
        this.meterMiddleY = this.bounds.y + 8 + FR.Chart.MeterStyle.DEFAULT_BORDER_SIZE + this.titleDim.height;
        if(meterW > meterH) {
            this.meterMiddleY += this.radius;
        } else {
            this.meterMiddleY += meterH/2.0;
        }
    },

    /**
     * 获取动态步骤中 每一步的Glyph
     * @param step 步骤
     * @param actionModel 动态方式
     * @returns {Array} 动态步骤数组
     */
    getAnimateGlyph: function (step, actionModel, lastGlyph) {
        var maxValue = this.meterStyle.getInterval(this.meterStyle.intervals.length-1).endValue;
        if(this.value < this.meterStyle.getInterval(0).beginValue || this.value > maxValue){
            return null ;
        }

        var glyph = [];
        for (var i = 0; i < step; i++) {
            if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL) {
                glyph[i] = this;
            } else {
                var meter;
                var val = FR.Chart.ChartEase.easeOutQuint(i + 1, 0, this.value, step);
                if(lastGlyph){
                    var lastValue = lastGlyph.value;
                    val = FR.Chart.ChartEase.easeOutQuint(i + 1, lastValue, this.value - lastValue, step);
                }
                meter = new FR.Chart.BlueMeter(this.titleGlyph, val, this.meterStyle);
                meter.bounds = this.bounds;
                glyph[i] = meter;
            }
        }

        return glyph;
    }
})

// bluemeter.end

FR.Chart.IconDemo = function(bounds, state){
    this.bounds = bounds;
    this.state = FR.pick(state, FR.Chart.ChartIconState.STATE_NORMAL);
    this.normalStyle = 'rgb(102,102,102)';
    this.selectedStyle ='rgb(41,158,255)';
    this.currentStyle = this.normalStyle;
};

$.extend(FR.Chart.IconDemo.prototype,{
    bounds: new FR.Chart.Bounds(),
    state:FR.Chart.ChartIconState.STATE_NORMAL,

    draw:function(ctx){
        if(this.state === FR.Chart.ChartIconState.STATE_SELECT || this.state === FR.Chart.ChartIconState.STATE_OVER){
            this.currentStyle =this.selectedStyle;
        }else{
            this.currentStyle = this.normalStyle;
        }
        if(!ctx){     return; } ctx.save();
        if(this.state === FR.Chart.ChartIconState.STATE_OVER){
            ctx.globalAlpha = 0.6;
        }
        this.paint(ctx);
        ctx.restore();

    },

    paint:function(ctx){

    }

});



//柱形图小图标
FR.Chart.ColumnSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){

        ctx.fillStyle = this.currentStyle;

        var firstRect = new FR.Chart.Bounds(this.bounds.x,this.bounds.y+11,6,11);
        firstRect.paint(ctx);

        var secondRect = new FR.Chart.Bounds(this.bounds.x+8,this.bounds.y,6,22);
        secondRect.paint(ctx);

        var thirdRect = new FR.Chart.Bounds(this.bounds.x+16,this.bounds.y+5,6,17);
        thirdRect.paint(ctx);


    }

});

//折线图小图标
FR.Chart.LineSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){

        ctx.strokeStyle = this.currentStyle;
        ctx.lineCap = 'round';
        var line = new FR.Chart.GeneralPath();
        line.moveTo(this.bounds.x+1,this.bounds.y+15);
        line.lineTo(this.bounds.x+7,this.bounds.y+7);
        line.lineTo(this.bounds.x+15,this.bounds.y+14);
        line.lineTo(this.bounds.x+21,this.bounds.y+3);
        line.draw(ctx,2);

    }
});


//条形图小图标
FR.Chart.BarSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){


        ctx.fillStyle = this.currentStyle;

        var firstRect = new FR.Chart.Bounds(this.bounds.x,this.bounds.y,11,6);
        firstRect.paint(ctx);

        var secondRect = new FR.Chart.Bounds(this.bounds.x,this.bounds.y+8,22,6);
        secondRect.paint(ctx);

        var thirdRect = new FR.Chart.Bounds(this.bounds.x,this.bounds.y+16,17,6);
        thirdRect.paint(ctx);

    }
});


//饼图小图标
FR.Chart.PieSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){

        ctx.fillStyle = this.currentStyle;

        var firstPath = new FR.Chart.Arc2D(this.bounds.x+11,this.bounds.y+11,10, 0, 90);
        firstPath.paint(ctx);

        ctx.moveTo(this.bounds.x+10,this.bounds.y+1);
        ctx.arc(this.bounds.x+11,this.bounds.y+11,10,264.26*Math.PI/180,137.87*Math.PI/180, 1);
        ctx.lineTo(this.bounds.x+10,this.bounds.y+11);
        ctx.closePath();
        ctx.fill();

        ctx.moveTo(this.bounds.x+21,this.bounds.y+12);
        ctx.arc(this.bounds.x+11,this.bounds.y+11,10,5.71*Math.PI/180, 132.22*Math.PI/180, 0);
        ctx.lineTo(this.bounds.x+11,this.bounds.y+12);
        ctx.closePath();
        ctx.fill();

    }
});


//面积图小图标
FR.Chart.AreaSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){

        ctx.fillStyle = this.currentStyle;

        var path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x,this.bounds.y+12);
        path.lineTo(this.bounds.x+3,this.bounds.y+13);
        path.lineTo(this.bounds.x+11,this.bounds.y+8);
        path.lineTo(this.bounds.x+16,this.bounds.y+10);
        path.lineTo(this.bounds.x+22,this.bounds.y+2);
        path.lineTo(this.bounds.x+22,this.bounds.y+22);
        path.lineTo(this.bounds.x,this.bounds.y+22);
        path.closePath();
        path.paint(ctx);

    }
});


//散点图小图标
FR.Chart.ScatterSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){
        ctx.fillStyle = this.currentStyle;

        var arc = new FR.Chart.Arc2D(this.bounds.x+4, this.bounds.y+3, 2, 0, 360);
        arc.paint(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+10, this.bounds.y+5, 2, 0, 360);
        arc.paint(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+19, this.bounds.y+4, 2, 0, 360);
        arc.paint(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+3, this.bounds.y+11, 2, 0, 360);
        arc.paint(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+13, this.bounds.y+10, 2, 0, 360);
        arc.paint(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+18, this.bounds.y+11, 2, 0, 360);
        arc.paint(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+5, this.bounds.y+19, 2, 0, 360);
        arc.paint(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+9, this.bounds.y+14, 2, 0, 360);
        arc.paint(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+15, this.bounds.y+5, 2, 0, 360);
        arc.paint(ctx);

    }
});

//气泡图小图标
FR.Chart.BubbleSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){

        ctx.strokeStyle = this.currentStyle;

        var arc = new FR.Chart.Arc2D(this.bounds.x+4, this.bounds.y+4, 2, 0, 360);
        arc.draw(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+13, this.bounds.y+4, 1, 0, 360);
        arc.draw(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+10, this.bounds.y+12, 4, 0, 360);
        arc.draw(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+18, this.bounds.y+9, 1, 0, 360);
        arc.draw(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+4, this.bounds.y+18, 1, 0, 360);
        arc.draw(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+17, this.bounds.y+17, 2, 0, 360);
        arc.draw(ctx);


    }
});


//雷达图小图标
FR.Chart.RadarSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){

        this.paintBackgroung(ctx);

        ctx.strokeStyle = this.currentStyle;

        var grayPath = new FR.Chart.GeneralPath();
        grayPath.moveTo(this.bounds.x+3,this.bounds.y+9);
        grayPath.lineTo(this.bounds.x+11,this.bounds.y+6);
        grayPath.lineTo(this.bounds.x+19,this.bounds.y+9);
        grayPath.lineTo(this.bounds.x+16,this.bounds.y+22);
        grayPath.lineTo(this.bounds.x+8,this.bounds.y+16);
        grayPath.closePath();
        grayPath.draw(ctx,1);

    },

    paintBackgroung:function(ctx){

        ctx.strokeStyle ='rgb(204,204,204)' ;

        var grayPath = new FR.Chart.GeneralPath();
        grayPath.moveTo(this.bounds.x+11,this.bounds.y);
        grayPath.lineTo(this.bounds.x+22,this.bounds.y+8);
        grayPath.lineTo(this.bounds.x+18,this.bounds.y+22);
        grayPath.lineTo(this.bounds.x+4,this.bounds.y+22);
        grayPath.lineTo(this.bounds.x,this.bounds.y+8);
        grayPath.closePath();
        grayPath.draw(ctx,1);

        grayPath =new FR.Chart.GeneralPath();
        grayPath.moveTo(this.bounds.x+11,this.bounds.y+5);
        grayPath.lineTo(this.bounds.x+17,this.bounds.y+10);
        grayPath.lineTo(this.bounds.x+15,this.bounds.y+17);
        grayPath.lineTo(this.bounds.x+7,this.bounds.y+17);
        grayPath.lineTo(this.bounds.x+5,this.bounds.y+10);
        grayPath.closePath();
        grayPath.draw(ctx,1);

        grayPath =new FR.Chart.GeneralPath();
        grayPath.moveTo(this.bounds.x+11,this.bounds.y+11);
        grayPath.lineTo(this.bounds.x+11,this.bounds.y);
        grayPath.draw(ctx,1);

        grayPath =new FR.Chart.GeneralPath();
        grayPath.moveTo(this.bounds.x+11,this.bounds.y+11);
        grayPath.lineTo(this.bounds.x+22,this.bounds.y+8);
        grayPath.draw(ctx,1);

        grayPath =new FR.Chart.GeneralPath();
        grayPath.moveTo(this.bounds.x+11,this.bounds.y+11);
        grayPath.lineTo(this.bounds.x+18,this.bounds.y+22);
        grayPath.draw(ctx,1);

        grayPath =new FR.Chart.GeneralPath();
        grayPath.moveTo(this.bounds.x+11,this.bounds.y+11);
        grayPath.lineTo(this.bounds.x+4,this.bounds.y+22);
        grayPath.draw(ctx,1);

        grayPath =new FR.Chart.GeneralPath();
        grayPath.moveTo(this.bounds.x+11,this.bounds.y+11);
        grayPath.lineTo(this.bounds.x,this.bounds.y+8);
        grayPath.draw(ctx,1);

    }
});

//股价图小图标
FR.Chart.StockSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){

        ctx.fillStyle = this.currentStyle;

        var rect = new FR.Chart.Bounds(this.bounds.x+2,this.bounds.y+4,2,14);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x,this.bounds.y+8,6,6);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+10,this.bounds.y,2,22);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+8,this.bounds.y+4,6,14);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+18,this.bounds.y+1,2,17);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+16,this.bounds.y+5,6,9);
        rect.paint(ctx);


    }

});

//仪表盘小图标
FR.Chart.MeterSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){

        ctx.fillStyle = this.currentStyle;
        ctx.strokeStyle = this.currentStyle;

        ctx.lineWidth = 2;
        var arc = new FR.Chart.Arc2D(this.bounds.x+11, this.bounds.y+11, 9, 0, 360);
        arc.draw(ctx);

        var path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x+11, this.bounds.y+9);
        path.lineTo(this.bounds.x+13, this.bounds.y+10);
        path.lineTo(this.bounds.x+15, this.bounds.y+5);
        path.closePath();
        path.paint(ctx);

        arc = new FR.Chart.Arc2D(this.bounds.x+11, this.bounds.y+11, 3, 0, 360);
        arc.paint(ctx);

        ctx.lineWidth = 1;
        ctx.lineCap = 'round';
        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(this.bounds.x+11, this.bounds.y+2)
            ,new FR.Chart.Point2D(this.bounds.x+11, this.bounds.y+5));
        line.draw(ctx,FR.Chart.Constants.LINE_THIN);

        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(this.bounds.x+16, this.bounds.y+8)
            ,new FR.Chart.Point2D(this.bounds.x+18, this.bounds.y+7));
        line.draw(ctx,FR.Chart.Constants.LINE_THIN);

        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(this.bounds.x+16, this.bounds.y+13)
            ,new FR.Chart.Point2D(this.bounds.x+19, this.bounds.y+14));
        line.draw(ctx,FR.Chart.Constants.LINE_THIN);

        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(this.bounds.x+3, this.bounds.y+14)
            ,new FR.Chart.Point2D(this.bounds.x+6, this.bounds.y+13));
        line.draw(ctx,FR.Chart.Constants.LINE_THIN);

        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(this.bounds.x+4, this.bounds.y+7)
            ,new FR.Chart.Point2D(this.bounds.x+6, this.bounds.y+8));
        line.draw(ctx,FR.Chart.Constants.LINE_THIN);


    }

});


//全距图小图标
FR.Chart.RangeSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){


        ctx.fillStyle = 'rgb(204,204,204)';
        var background = new FR.Chart.Bounds(this.bounds.x,this.bounds.y+5,22,11);
        background.paint(ctx);

        ctx.strokeStyle = 'rgb(153,153,153)';
        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(this.bounds.x, this.bounds.y+10)
            ,new FR.Chart.Point2D(this.bounds.x+22, this.bounds.y+10));
        line.draw(ctx,1);

        ctx.fillStyle = this.currentStyle;
        var rect = new FR.Chart.Bounds(this.bounds.x+3,this.bounds.y+5,2,11);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+10,this.bounds.y+5,2,11);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+14,this.bounds.y+5,2,11);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+18,this.bounds.y+5,2,11);
        rect.paint(ctx);

    }
});

//组合图小图标
FR.Chart.CustomSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){

        ctx.strokeStyle = this.currentStyle;
        var line = new FR.Chart.GeneralPath();
        line.moveTo(this.bounds.x+3,this.bounds.y+12);
        line.lineTo(this.bounds.x+7,this.bounds.y+6);
        line.lineTo(this.bounds.x+13,this.bounds.y+8);
        line.lineTo(this.bounds.x+19,this.bounds.y+1);
        line.draw(ctx, FR.Chart.Constants.LINE_MEDIUM);


        ctx.fillStyle = this.currentStyle;
        var rect = new FR.Chart.Bounds(this.bounds.x,this.bounds.y+15,6,7);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+8,this.bounds.y+11,6,11);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+16,this.bounds.y+8,6,14);
        rect.paint(ctx);

    }
});


//甘特图小图标
FR.Chart.GanttSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){;

        ctx.fillStyle = this.currentStyle;
        var rect = new FR.Chart.Bounds(this.bounds.x,this.bounds.y,6,6);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+5,this.bounds.y+8,8,6);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+12,this.bounds.y+16,6,6);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+5,this.bounds.y,1,8);
        rect.paint(ctx);

        rect = new FR.Chart.Bounds(this.bounds.x+12,this.bounds.y+8,1,8);
        rect.paint(ctx);

        ctx.fillStyle = 'rgb(204,204,204)';
        rect = new FR.Chart.Bounds(this.bounds.x+18,this.bounds.y+16,4,6);
        rect.paint(ctx);

    }
});

//圆环图小图标
FR.Chart.DonutSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){

        ctx.fillStyle = this.currentStyle;

        ctx.beginPath();
        ctx.moveTo(this.bounds.x+11,this.bounds.y+1);
        ctx.arc(this.bounds.x+11,this.bounds.y+11,10,270*Math.PI/180,360*Math.PI/180,0);
        ctx.lineTo(this.bounds.x+16,this.bounds.y+11);
        ctx.arc(this.bounds.x+11,this.bounds.y+11,5,360*Math.PI/180,270*Math.PI/180,1);
        ctx.closePath();
        ctx.fill();

        ctx.moveTo(this.bounds.x+10,this.bounds.y+1);
        ctx.arc(this.bounds.x+11,this.bounds.y+11,10,264.26*Math.PI/180,137.87*Math.PI/180,1);
        ctx.lineTo(this.bounds.x+7,this.bounds.y+14);
        ctx.arc(this.bounds.x+11,this.bounds.y+11,5,140.22*Math.PI/180,258.64*Math.PI/180,0);
        ctx.closePath();
        ctx.fill();

        ctx.moveTo(this.bounds.x+21,this.bounds.y+12);
        ctx.arc(this.bounds.x+11,this.bounds.y+11,10,5.74*Math.PI/180,132.13*Math.PI/180,0);
        ctx.lineTo(this.bounds.x+8,this.bounds.y+15);
        ctx.arc(this.bounds.x+11,this.bounds.y+11,5,129.29*Math.PI/180,11.53*Math.PI/180,1);
        ctx.closePath();
        ctx.fill();

    }
});


//漏斗图小图标
FR.Chart.FunnelSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){

        ctx.fillStyle = this.currentStyle;

        var path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x,this.bounds.y);
        path.lineTo(this.bounds.x+22,this.bounds.y);
        path.lineTo(this.bounds.x+21,this.bounds.y+8);
        path.lineTo(this.bounds.x+1,this.bounds.y+8);
        path.closePath();
        path.paint(ctx);

        path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x+1,this.bounds.y+9);
        path.lineTo(this.bounds.x+21,this.bounds.y+9);
        path.lineTo(this.bounds.x+19,this.bounds.y+16);
        path.lineTo(this.bounds.x+3,this.bounds.y+16);
        path.closePath();
        path.paint(ctx);

        path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x+3,this.bounds.y+17);
        path.lineTo(this.bounds.x+19,this.bounds.y+17);
        path.lineTo(this.bounds.x+14,this.bounds.y+22);
        path.lineTo(this.bounds.x+8,this.bounds.y+22);
        path.closePath();
        path.paint(ctx);

    }
});



//地图小图标,美国地图
FR.Chart.MapSmallIcon = FR.extend(FR.Chart.IconDemo,{

    paint:function(ctx){

        ctx.fillStyle = this.currentStyle;
        var path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x+1,this.bounds.y+12);
        path.lineTo(this.bounds.x+2,this.bounds.y+7);
        path.lineTo(this.bounds.x+3,this.bounds.y+5);
        path.lineTo(this.bounds.x+5,this.bounds.y+6);
        path.lineTo(this.bounds.x+14,this.bounds.y+6);
        path.lineTo(this.bounds.x+16,this.bounds.y+8);
        path.lineTo(this.bounds.x+17,this.bounds.y+8);
        path.lineTo(this.bounds.x+20,this.bounds.y+5);
        path.lineTo(this.bounds.x+21,this.bounds.y+6);
        path.lineTo(this.bounds.x+20,this.bounds.y+7);
        path.lineTo(this.bounds.x+20,this.bounds.y+12);
        path.lineTo(this.bounds.x+18,this.bounds.y+14);
        path.lineTo(this.bounds.x+19,this.bounds.y+16);
        path.lineTo(this.bounds.x+18,this.bounds.y+17);
        path.lineTo(this.bounds.x+17,this.bounds.y+15);
        path.lineTo(this.bounds.x+15,this.bounds.y+15);
        path.lineTo(this.bounds.x+14,this.bounds.y+16);
        path.lineTo(this.bounds.x+12,this.bounds.y+16);
        path.lineTo(this.bounds.x+11,this.bounds.y+18);
        path.lineTo(this.bounds.x+9,this.bounds.y+16);
        path.lineTo(this.bounds.x+7,this.bounds.y+15);
        path.lineTo(this.bounds.x+5,this.bounds.y+15);
        path.lineTo(this.bounds.x+3,this.bounds.y+14);
        path.closePath();
        path.paint(ctx);

        path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x+4,this.bounds.y+16);
        path.lineTo(this.bounds.x+6,this.bounds.y+16);
        path.lineTo(this.bounds.x+6,this.bounds.y+17);
        path.lineTo(this.bounds.x+4,this.bounds.y+17);
        path.closePath();
        path.paint(ctx);

    }
});


//GIS地图小图标
FR.Chart.GisMapSmallIcon = FR.extend(FR.Chart.IconDemo,{

    draw:function(ctx){
        if(!ctx){     return; } ctx.save();
        this.drawBackground(ctx);

        var outerColor = 'rgba(77,77,77,0.9)';
        var innerColor = 'rgba(255,255,255,0.8)';
        if(this.state === FR.Chart.ChartIconState.STATE_SELECT || this.state === FR.Chart.ChartIconState.STATE_OVER){
            outerColor = 'rgba(237,30,121,0.9)';
            innerColor = 'rgba(255,255,255,0.8)';
        }

        if(this.state === FR.Chart.ChartIconState.STATE_OVER){
            ctx.globalAlpha = 0.6;
        }
        this.paint(ctx,innerColor,outerColor);
        ctx.restore();
    },

    drawBackground:function(ctx){
        ctx.fillStyle = 'rgb(204,204,204)';
        var path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x+1,this.bounds.y+2);
        path.lineTo(this.bounds.x+21,this.bounds.y+5);
        path.lineTo(this.bounds.x+21,this.bounds.y+7);
        path.lineTo(this.bounds.x+1,this.bounds.y+4);
        path.closePath();
        path.paint(ctx);

        path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x+1,this.bounds.y+9);
        path.lineTo(this.bounds.x+21,this.bounds.y+12);
        path.lineTo(this.bounds.x+21,this.bounds.y+14);
        path.lineTo(this.bounds.x+1,this.bounds.y+11);
        path.closePath();
        path.paint(ctx);

        path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x+6,this.bounds.y+1);
        path.lineTo(this.bounds.x+8,this.bounds.y+1);
        path.lineTo(this.bounds.x+5,this.bounds.y+21);
        path.lineTo(this.bounds.x+3,this.bounds.y+21);
        path.closePath();
        path.paint(ctx);

    },

    paint:function(ctx,innerColor,outerColor){
        ctx.fillStyle = outerColor;

        var path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x+7,this.bounds.y+13);
        path.lineTo(this.bounds.x+11,this.bounds.y+14);
        path.lineTo(this.bounds.x+15,this.bounds.y+13);
        path.lineTo(this.bounds.x+11,this.bounds.y+17);
        path.closePath();
        path.paint(ctx);

        var arc = new FR.Chart.Arc2D(this.bounds.x+11,this.bounds.y+10, 5, 0, 360);
        arc.paint(ctx);


        ctx.fillStyle = innerColor;
        arc = new FR.Chart.Arc2D(this.bounds.x+11,this.bounds.y+10, 3, 0, 360);
        arc.paint(ctx);

    }

});

FR.Chart.MapReturnSmallIcon = FR.extend(FR.Chart.IconDemo,{
    draw:function(ctx){
        if(!ctx){     return; } ctx.save();
        ctx.fillStyle = 'rgb(41,158,255)';
        ctx.beginPath();
        ctx.moveTo(this.bounds.x+12,this.bounds.y+5);
        ctx.arc(this.bounds.x+12,this.bounds.y+13,8, 270*Math.PI/180, 90*Math.PI/180, 0);
        ctx.arc(this.bounds.x+12,this.bounds.y+17,4, 90*Math.PI/180, 270*Math.PI/180, 1);
        ctx.lineTo(this.bounds.x+9,this.bounds.y+13);
        ctx.lineTo(this.bounds.x+9,this.bounds.y+16);
        ctx.lineTo(this.bounds.x+2,this.bounds.y+9);
        ctx.lineTo(this.bounds.x+9,this.bounds.y+2);
        ctx.lineTo(this.bounds.x+9,this.bounds.y+5);
        ctx.closePath();
        ctx.fill();
        ctx.restore();
    }
});


//刷新小图标
FR.Chart.RefreshSmallIcon = FR.extend(FR.Chart.IconDemo,{

    draw:function(ctx){
        if(!ctx){     return; } ctx.save();

        ctx.fillStyle = 'rgb(41,158,255)';
        ctx.moveTo(this.bounds.x+4,this.bounds.y+4);
        ctx.arc(this.bounds.x+11,this.bounds.y+11,10, 225*Math.PI/180,90*Math.PI/180,1);
        ctx.lineTo(this.bounds.x+11,this.bounds.y+19);
        ctx.arc(this.bounds.x+11,this.bounds.y+11,8,90*Math.PI/180, 225*Math.PI/180, 0);
        ctx.closePath();
        ctx.fill();

        var path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x+1,this.bounds.y+1);
        path.lineTo(this.bounds.x+8,this.bounds.y+8);
        path.lineTo(this.bounds.x+8,this.bounds.y+1);
        path.closePath();
        path.paint(ctx);

        ctx.moveTo(this.bounds.x+11,this.bounds.y+3);
        ctx.arc(this.bounds.x+11,this.bounds.y+11,8, 270*Math.PI/180,45*Math.PI/180,0);
        ctx.lineTo(this.bounds.x+18,this.bounds.y+18);
        ctx.arc(this.bounds.x+11,this.bounds.y+11,10,45*Math.PI/180, 270*Math.PI/180,1);
        ctx.closePath();
        ctx.fill();

        var path = new FR.Chart.GeneralPath();
        path.moveTo(this.bounds.x+14,this.bounds.y+14);
        path.lineTo(this.bounds.x+21,this.bounds.y+21);
        path.lineTo(this.bounds.x+14,this.bounds.y+21);
        path.closePath();
        path.paint(ctx);

        ctx.restore();
    }

});

// timeSwitchButton.start
/**
 * 时间qiehuan1按钮
 *     @example
 *     var button = new FR.Chart.TimeSwitchButton(x, y);
 *
 * @class FR.Chart.TimeSwitchButton
 * @param x 位置x
 * @param y 位置y
 *
 * @property {Boolean} onPress 是否鼠标按压状态
 * @property {Boolean} isDisable 按钮是否可用
 * @property {Boolean} onMove 是否鼠标悬浮状态
 * @constructor
 */
FR.Chart.TimeSwitchButton = function (x, y) {
    this.time = "1";
    this.unit = FR.Chart.Constants.YEAR;
    this.isGlobal = false;//是否是全部按钮，默认不是
    this.isPress = false;
    this.isOver = false;
};

$.extend(FR.Chart.TimeSwitchButton.prototype, {

    /**
     * 画出图例按钮
     * @param ctx 画布
     */
    draw: function (ctx) {
        if (!this.bounds) {
            return;
        }
        if(!ctx){     return; } ctx.save();
        ctx.fillStyle =this.isPress?'rgb(41,171,226)':(this.isOver?'rgb(41,171,226)':'rgb(122,218,244)');

        var x = Math.round(this.bounds.x), y = Math.round(this.bounds.y),
            height = Math.round(this.bounds.height), width = Math.round(this.bounds.width);
        var arcLen = 4;
        ctx.beginPath();
        ctx.moveTo(x, y + arcLen);
        ctx.lineTo(x, y + height - arcLen);
        ctx.quadraticCurveTo(x, y + height, x + arcLen, y + height);
        ctx.lineTo(x+ width - arcLen,y+height);
        ctx.quadraticCurveTo(x+width,y+height,x+width,y+height - arcLen);
        ctx.lineTo(x+ width,y+arcLen);
        ctx.quadraticCurveTo(x+width,y,x+width-arcLen,y);
        ctx.lineTo(x+arcLen,y);
        ctx.quadraticCurveTo(x,y,x, y + arcLen);
        ctx.closePath();
        ctx.fill();

        this._drawButtonText(ctx);
        ctx.restore();
    },

    //获取单位的字符串值
    getUnitString:function(unit){
        if(unit === FR.Chart.Constants.YEAR){
            return FR.i18nText("Fine-Dec_Chart_TimeUnit_Year");
        }else if(unit === FR.Chart.Constants.MONTH){
            return FR.i18nText("Fine-Dec_Chart_TimeUnit_Month");
        }else if(unit === FR.Chart.Constants.DAY){
            return FR.i18nText("Fine-Dec_Chart_TimeUnit_Day");
        }else if(unit === FR.Chart.Constants.Hour){
            return FR.i18nText("Fine-Dec_Chart_TimeUnit_Hour");
        }else if(unit === FR.Chart.Constants.Minute){
            return FR.i18nText("Fine-Dec_Chart_TimeUnit_Minute");
        }else if(unit === FR.Chart.Constants.Second){
            return FR.i18nText("Fine-Dec_Chart_TimeUnit_Second");
        }
        return FR.i18nText("Fine-Dec_Chart_TimeUnit_Year");
    },


    getTextAttr:function(){
        var textAttr = new FR.Chart.TextAttr();
        textAttr.frFont =new FR.Chart.FRFont();
        textAttr.frFont.color = "white";
        textAttr.frFont.size = 9;
        textAttr.horizontalAlignment =FR.Chart.Constants.CENTER;
        textAttr.verticalAlignment =FR.Chart.Constants.CENTER ;
        return textAttr;
    },

    /**
     * 画出按钮文字
     * @param ctx 画布
     * @private
     */
    _drawButtonText: function (ctx) {
        if (this.bounds) {
            var textGlyph = new FR.Chart.TextGlyph();
            textGlyph.textAttr = this.getTextAttr();
            textGlyph.text =this.isGlobal?FR.i18nText("Fine-Dec_Chart_TimeSwitch_Global"):this.time+this.getUnitString(this.unit);
            var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(textGlyph.text, textGlyph.textAttr);
            var diffY = (this.bounds.height-dim.height)/2.0-2;
            var diffX =10+(this.bounds.width-dim.width)/2.0;
            var x = Math.round(this.bounds.x), y = Math.round(this.bounds.y);
            textGlyph.bounds = new FR.Chart.Bounds(x+diffX,y+diffY,dim.width,dim.height);
            textGlyph.position = FR.Chart.Constants.CENTER;
            ctx.notUseDivText = true;
            textGlyph.draw(ctx);
            ctx.notUseDivText = false;
        }
    },

    /**
     * 判断是否包含坐标(x,y)
     * @param x 坐标x
     * @param y 坐标y
     * @returns {Boolean} 判断是否包含(x,y)
     */
    containsPoint: function (x, y) {
        return this.bounds == null ? false : this.bounds.containsPoint(x, y);
    }
});

$.extend(FR.Chart.TimeSwitchButton,{

});

//TimeSwitchButton.end